@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <dfb_types.h>
 
 #include <stdio.h>
@@ -47,7 +50,7 @@
 #include <core/screens.h>
 #include <core/state.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <gfx/convert.h>
 
@@ -85,8 +88,8 @@ DFB_GRAPHICS_DRIVER( ati128 )
 
 
 /* macro for S12.0 and S14.0 format */
-#define S12(val) (((__u16)((__s16)(val)))&0x3fff)
-#define S14(val) (((__u16)((__s16)(val)))&0x3fff)
+#define S12(val) (((u16)((s16)(val)))&0x3fff)
+#define S14(val) (((u16)((s16)(val)))&0x3fff)
 
 /** CARD FUNCTIONS **/
 static bool ati128FillRectangle( void *drv, void *dev, DFBRectangle *rect );
@@ -96,12 +99,14 @@ static bool ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
 
 /* required implementations */
 
-static void ati128EngineSync( void *drv, void *dev )
+static DFBResult ati128EngineSync( void *drv, void *dev )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
 
      ati128_waitidle( adrv, adev );
+
+     return DFB_OK;
 }
 
 static bool ati128_check_blend( CardState *state )
@@ -115,7 +120,7 @@ static bool ati128_check_blend( CardState *state )
 static void ati128CheckState( void *drv, void *dev,
                               CardState *state, DFBAccelerationMask accel )
 {
-     switch (state->destination->format) {
+     switch (state->destination->config.format) {
           case DSPF_RGB332:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
@@ -147,14 +152,14 @@ static void ati128CheckState( void *drv, void *dev,
      if (!(accel & ~ATI128_SUPPORTED_BLITTINGFUNCTIONS) &&
          !(state->blittingflags & ~ATI128_SUPPORTED_BLITTINGFLAGS)  &&
          state->source &&
-         state->source->width  >= 8 &&
-         state->source->height >= 8 )
+         state->source->config.size.w >= 8 &&
+         state->source->config.size.h >= 8 )
      {
           if (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL &&
               !ati128_check_blend( state ))
                return;
 
-          switch (state->source->format) {
+          switch (state->source->config.format) {
                case DSPF_RGB332:
                case DSPF_ARGB1555:
                case DSPF_RGB16:
@@ -176,22 +181,22 @@ static void ati128SetState( void *drv, void *dev,
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
 
-     if (state->modified & SMF_SOURCE)
+     if (state->mod_hw & SMF_SOURCE)
          adev->v_source = 0;
 
-     if (state->modified & SMF_DESTINATION)
+     if (state->mod_hw & SMF_DESTINATION)
           adev->v_destination = adev->v_color = 0;
 
-     if (state->modified & SMF_COLOR)
+     if (state->mod_hw & SMF_COLOR)
           adev->v_color = 0;
 
-     if (state->modified & SMF_SRC_COLORKEY)
+     if (state->mod_hw & SMF_SRC_COLORKEY)
           adev->v_src_colorkey = 0;
 
-     if (state->modified & SMF_BLITTING_FLAGS)
+     if (state->mod_hw & SMF_BLITTING_FLAGS)
           adev->v_blittingflags = 0;
 
-     if (state->modified & (SMF_SRC_BLEND | SMF_DST_BLEND))
+     if (state->mod_hw & (SMF_SRC_BLEND | SMF_DST_BLEND))
           adev->v_blending_function = 0;
 
      ati128_set_destination( adrv, adev, state);
@@ -229,17 +234,17 @@ static void ati128SetState( void *drv, void *dev,
                break;
      }
 
-     if (state->modified & SMF_CLIP)
+     if (state->mod_hw & SMF_CLIP)
           ati128_set_clip( adrv, adev, state);
 
-     state->modified = 0;
+     state->mod_hw = 0;
 }
 
 static bool ati128FillRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      ati128_waitfifo( adrv, adev, 5 );
      /* set the destination datatype */
@@ -260,11 +265,11 @@ static bool ati128FillBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
+     u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
      ati128_waitidle( adrv, adev );
-     *((__u32*)  dfb_gfxcard_memory_virtual(NULL,fts)  ) = adev->fake_texture_color;
+     *((u32*)  dfb_gfxcard_memory_virtual(NULL,fts)  ) = adev->fake_texture_color;
      ati128_waitidle( adrv, adev );
 
      ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
@@ -301,7 +306,7 @@ static bool ati128DrawRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      ati128_waitfifo( adrv, adev, 3 );
      /* set the destination datatype */
@@ -331,12 +336,12 @@ static bool ati128DrawBlendRectangle( void *drv, void *dev, DFBRectangle *rect )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
+     u32 fts = adev->ATI_fake_texture_src + (adev->fake_texture_number & 7)*4;
 
      ati128_waitidle( adrv, adev );
-     *((__u32*)  dfb_gfxcard_memory_virtual(NULL,fts)  ) = adev->fake_texture_color;
+     *((u32*)  dfb_gfxcard_memory_virtual(NULL,fts)  ) = adev->fake_texture_color;
      ati128_waitidle( adrv, adev );
 
      ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
@@ -386,7 +391,7 @@ static bool ati128DrawLine( void *drv, void *dev, DFBRegion *line )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      int dx, dy;
      int small, large;
@@ -453,12 +458,12 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 src = 0;
+     u32 src = 0;
 
-     __u32 scalex = (__u32)(((double)sr->w/(double)dr->w) * 65536);
-     __u32 scaley = (__u32)(((double)sr->h/(double)dr->h) * 65536);
+     u32 scalex = (u32)(((double)sr->w/(double)dr->w) * 65536);
+     u32 scaley = (u32)(((double)sr->h/(double)dr->h) * 65536);
 
      ati128_waitfifo( adrv, adev, 9 );
 
@@ -466,16 +471,16 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
 
      ati128_out32( mmio, CLR_CMP_CNTL, adev->ATI_color_compare );
 
-     switch (adev->source->format) {
+     switch (adev->source->config.format) {
           case DSPF_RGB332:
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_8BPP_RGB332 );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>3);
+                             adev->src->pitch >>3);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x;
+                     adev->src->pitch + sr->x;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -484,11 +489,11 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_15BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>4);
+                             adev->src->pitch >>4);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*2;
+                     adev->src->pitch + sr->x*2;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -497,11 +502,11 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_16BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>4);
+                             adev->src->pitch >>4);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*2;
+                     adev->src->pitch + sr->x*2;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -510,11 +515,11 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_24BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>3);
+                             adev->src->pitch >>3);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*3;
+                     adev->src->pitch + sr->x*3;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -523,11 +528,11 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>5);
+                             adev->src->pitch >>5);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*4;
+                     adev->src->pitch + sr->x*4;
 
                ati128_out32( mmio, TEX_CNTL, 0);
 
@@ -536,11 +541,11 @@ static bool ati128StretchBlit( void *drv, void *dev, DFBRectangle *sr, DFBRectan
                ati128_out32( mmio, SCALE_3D_DATATYPE, DST_32BPP );
 
                ati128_out32( mmio, SCALE_PITCH,
-                             adev->source->front_buffer->video.pitch >>5);
+                             adev->src->pitch >>5);
 
-               src = adev->source->front_buffer->video.offset +
+               src = adev->src->offset +
                      sr->y *
-                     adev->source->front_buffer->video.pitch + sr->x*4;
+                     adev->src->pitch + sr->x*4;
 
                if (adev->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
                     ati128_out32( mmio, TEX_CNTL, TEX_CNTL_ALPHA_EN_ON );
@@ -593,11 +598,11 @@ static bool ati128Blit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy
 {
      ATI128DriverData *adrv = (ATI128DriverData*) drv;
      ATI128DeviceData *adev = (ATI128DeviceData*) dev;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
-     __u32 dir_cmd = 0;
+     u32 dir_cmd = 0;
 
-     if ((adev->source->format != adev->destination->format) ||
+     if ((adev->source->config.format != adev->destination->config.format) ||
          (adev->blittingflags & DSBLIT_BLEND_ALPHACHANNEL))
      {
           DFBRectangle sr = { rect->x, rect->y, rect->w, rect->h };
@@ -654,7 +659,7 @@ static bool ati128Blit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy
 /* exported symbols */
 
 static int
-driver_probe( GraphicsDevice *device )
+driver_probe( CoreGraphicsDevice *device )
 {
      switch (dfb_gfxcard_get_accelerator( device )) {
           case FB_ACCEL_ATI_RAGE128:          /* ATI Rage 128 */
@@ -666,7 +671,7 @@ driver_probe( GraphicsDevice *device )
 
 
 static void
-driver_get_info( GraphicsDevice     *device,
+driver_get_info( CoreGraphicsDevice *device,
                  GraphicsDriverInfo *info )
 {
      /* fill driver info structure */
@@ -676,7 +681,7 @@ driver_get_info( GraphicsDevice     *device,
 
      snprintf( info->vendor,
                DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 2;
@@ -686,7 +691,7 @@ driver_get_info( GraphicsDevice     *device,
 }
 
 static DFBResult
-driver_init_driver( GraphicsDevice      *device,
+driver_init_driver( CoreGraphicsDevice  *device,
                     GraphicsDeviceFuncs *funcs,
                     void                *driver_data,
                     void                *device_data,
@@ -694,7 +699,7 @@ driver_init_driver( GraphicsDevice      *device,
 {
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
 
-     adrv->mmio_base = (volatile __u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
+     adrv->mmio_base = (volatile u8*) dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!adrv->mmio_base)
           return DFB_IO;
 
@@ -717,14 +722,14 @@ driver_init_driver( GraphicsDevice      *device,
 
 
 static DFBResult
-driver_init_device( GraphicsDevice     *device,
+driver_init_device( CoreGraphicsDevice *device,
                     GraphicsDeviceInfo *device_info,
                     void               *driver_data,
                     void               *device_data )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
      ATI128DeviceData *adev = (ATI128DeviceData*) device_data;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      /* fill device info */
      snprintf( device_info->name,
@@ -773,13 +778,13 @@ driver_init_device( GraphicsDevice     *device,
 }
 
 static void
-driver_close_device( GraphicsDevice *device,
-                     void           *driver_data,
-                     void           *device_data )
+driver_close_device( CoreGraphicsDevice *device,
+                     void               *driver_data,
+                     void               *device_data )
 {
      ATI128DeviceData *adev = (ATI128DeviceData*) device_data;
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
-     volatile __u8    *mmio = adrv->mmio_base;
+     volatile u8      *mmio = adrv->mmio_base;
 
      D_DEBUG( "DirectFB/ATI128: FIFO Performance Monitoring:\n" );
      D_DEBUG( "DirectFB/ATI128:  %9d ati128_waitfifo calls\n",
@@ -828,8 +833,8 @@ driver_close_device( GraphicsDevice *device,
 }
 
 static void
-driver_close_driver( GraphicsDevice *device,
-                     void           *driver_data )
+driver_close_driver( CoreGraphicsDevice *device,
+                     void               *driver_data )
 {
      ATI128DriverData *adrv = (ATI128DriverData*) driver_data;
 
