@@ -7,6 +7,8 @@
    version 2 of the License, or (at your option) any later version.
 */
 
+#include <config.h>
+
 #include "unichrome.h"
 #include "uc_overlay.h"
 #include "vidregs.h"
@@ -26,7 +28,7 @@ uc_ovl_remove(CoreLayer *layer,
               void      *region_data);
 
 
-static int uc_ovl_datasize()
+static int uc_ovl_datasize( void )
 {
     return sizeof(UcOverlayData);
 }
@@ -102,7 +104,8 @@ uc_ovl_set_region( CoreLayer                  *layer,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
-                   CorePalette                *palette )
+                   CorePalette                *palette,
+                   CoreSurfaceBufferLock      *lock )
 {
     UcDriverData*  ucdrv = (UcDriverData*) driver_data;
     UcOverlayData* ucovl = (UcOverlayData*) layer_data;
@@ -125,8 +128,9 @@ uc_ovl_set_region( CoreLayer                  *layer,
 
     ucovl->deinterlace = config->options & DLOP_DEINTERLACING;
     ucovl->surface     = surface;
+    ucovl->lock        = lock;
 
-    return uc_ovl_update(ucdrv, ucovl, UC_OVL_CHANGE, surface);
+    return uc_ovl_update(ucdrv, ucovl, UC_OVL_CHANGE, surface, lock);
 }
 
 
@@ -138,7 +142,7 @@ uc_ovl_remove(CoreLayer *layer,
 {
     UcDriverData*  ucdrv = (UcDriverData*) driver_data;
     UcOverlayData* ucovl = (UcOverlayData*) layer_data;
-    volatile __u8* vio = ucdrv->hwregs;
+    volatile u8*   vio   = ucdrv->hwregs;
 
     ucovl->v1.isenabled = false;
 
@@ -218,12 +222,13 @@ uc_ovl_test_region(CoreLayer                  *layer,
 
 
 static DFBResult
-uc_ovl_flip_region( CoreLayer           *layer,
-                    void                *driver_data,
-                    void                *layer_data,
-                    void                *region_data,
-                    CoreSurface         *surface,
-                    DFBSurfaceFlipFlags  flags )
+uc_ovl_flip_region( CoreLayer             *layer,
+                    void                  *driver_data,
+                    void                  *layer_data,
+                    void                  *region_data,
+                    CoreSurface           *surface,
+                    DFBSurfaceFlipFlags    flags,
+                    CoreSurfaceBufferLock *lock)
 {
     //printf("Entering %s ... \n", __PRETTY_FUNCTION__);
 
@@ -235,11 +240,12 @@ uc_ovl_flip_region( CoreLayer           *layer,
         !dfb_config->pollvsync_after)
         dfb_layer_wait_vsync( layer );
 
-    dfb_surface_flip_buffers(surface, false);
+    dfb_surface_flip(surface, false);
 
     ucovl->field = 0;
+    ucovl->lock = lock;
 
-    ret = uc_ovl_update(ucdrv, ucovl, UC_OVL_FLIP, surface);
+    ret = uc_ovl_update(ucdrv, ucovl, UC_OVL_FLIP, surface, lock);
     if (ret)
         return ret;
 
@@ -297,18 +303,18 @@ uc_ovl_set_input_field( CoreLayer *layer,
 
      ucovl->field = field;
 
-     return uc_ovl_update(ucdrv, ucovl, UC_OVL_FIELD, ucovl->surface);
+     return uc_ovl_update(ucdrv, ucovl, UC_OVL_FIELD, ucovl->surface, ucovl->lock);
 }
 
 DisplayLayerFuncs ucOverlayFuncs = {
-    LayerDataSize:      uc_ovl_datasize,
-    InitLayer:          uc_ovl_init_layer,
-    SetRegion:          uc_ovl_set_region,
-    RemoveRegion:       uc_ovl_remove,
-    TestRegion:         uc_ovl_test_region,
-    FlipRegion:         uc_ovl_flip_region,
-    GetLevel:           uc_ovl_get_level,
-    SetLevel:           uc_ovl_set_level,
-    SetInputField:      uc_ovl_set_input_field,
-    SetColorAdjustment: uc_ovl_set_adjustment
+    .LayerDataSize      = uc_ovl_datasize,
+    .InitLayer          = uc_ovl_init_layer,
+    .SetRegion          = uc_ovl_set_region,
+    .RemoveRegion       = uc_ovl_remove,
+    .TestRegion         = uc_ovl_test_region,
+    .FlipRegion         = uc_ovl_flip_region,
+    .GetLevel           = uc_ovl_get_level,
+    .SetLevel           = uc_ovl_set_level,
+    .SetInputField      = uc_ovl_set_input_field,
+    .SetColorAdjustment = uc_ovl_set_adjustment,
 };
