@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -137,7 +137,8 @@ osdSetRegion( CoreLayer                  *layer,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags  updated,
               CoreSurface                *surface,
-              CorePalette                *palette )
+              CorePalette                *palette,
+              CoreSurfaceBufferLock      *lock )
 {
      DFBResult     ret;
      UcDriverData *ucdrv = (UcDriverData*) driver_data;
@@ -146,7 +147,7 @@ osdSetRegion( CoreLayer                  *layer,
      ret = ucOldPrimaryFuncs.SetRegion( layer, ucOldPrimaryDriverData,
                                         layer_data, region_data,
                                         config, updated, surface,
-                                        palette );
+                                        palette, lock );
      if (ret)
           return ret;
 
@@ -167,9 +168,9 @@ osdSetRegion( CoreLayer                  *layer,
 }
 
 DisplayLayerFuncs ucPrimaryFuncs = {
-     InitLayer:          osdInitLayer,
+     .InitLayer  = osdInitLayer,
 
-     TestRegion:         osdTestRegion,
-     SetRegion:          osdSetRegion
+     .TestRegion = osdTestRegion,
+     .SetRegion  = osdSetRegion,
 };
 
