@@ -74,9 +74,9 @@ later versions on an EPIA-M10000.
 #include <direct/messages.h>
 
 #include <core/coretypes.h>
+#include <core/core.h>
 #include <core/gfxcard.h>
 #include <core/graphics_driver.h>
-#include <core/surfacemanager.h>
 #include <core/system.h>
 #include <core/screens.h>
 
@@ -138,7 +138,7 @@ static void uc_dump_vq(UcDeviceData *ucdev)
 
 /** Allocate memory for the virtual queue. */
 
-static DFBResult uc_alloc_vq(GraphicsDevice *device, UcDeviceData *ucdev)
+static DFBResult uc_alloc_vq(CoreGraphicsDevice *device, UcDeviceData *ucdev)
 {
      if (ucdev->vq_start) return DFB_OK;
 
@@ -164,7 +164,7 @@ static DFBResult uc_alloc_vq(GraphicsDevice *device, UcDeviceData *ucdev)
  * @param enable    enable VQ if true (else disable it.)
  */
 
-DFBResult uc_init_2d_engine(GraphicsDevice *device, UcDeviceData *ucdev, UcDriverData *ucdrv, bool enable)
+static DFBResult uc_init_2d_engine(CoreGraphicsDevice *device, UcDeviceData *ucdev, UcDriverData *ucdrv, bool enable)
 {
      DFBResult result = DFB_OK;
      volatile u8* hwregs = ucdrv->hwregs;
@@ -248,7 +248,7 @@ DFBResult uc_init_2d_engine(GraphicsDevice *device, UcDeviceData *ucdev, UcDrive
      return result;
 }
 
-void uc_init_3d_engine(volatile u8* hwregs, int hwrev, bool init_all)
+static void uc_init_3d_engine(volatile u8* hwregs, int hwrev, bool init_all)
 {
      u32 i;
 
@@ -340,7 +340,7 @@ static void uc_after_set_var(void* drv, void* dev)
 
 /** Wait until the engine is idle. */
 
-static void uc_engine_sync(void* drv, void* dev)
+static DFBResult uc_engine_sync(void* drv, void* dev)
 {
      UcDriverData* ucdrv = (UcDriverData*) drv;
      UcDeviceData* ucdev = (UcDeviceData*) dev;
@@ -365,12 +365,14 @@ static void uc_engine_sync(void* drv, void* dev)
 
      ucdev->idle_waitcycles += loop;
      ucdev->must_wait = 0;
+
+     return DFB_OK;
 }
 
 
 // DirectFB interfacing functions --------------------------------------------
 
-static int driver_probe(GraphicsDevice *device)
+static int driver_probe(CoreGraphicsDevice *device)
 {
      struct stat s;
 
@@ -382,7 +384,7 @@ static int driver_probe(GraphicsDevice *device)
      return stat(UNICHROME_DEVICE, &s) + 1;
 }
 
-static void driver_get_info(GraphicsDevice* device,
+static void driver_get_info(CoreGraphicsDevice* device,
                             GraphicsDriverInfo* info)
 {
      // Fill in driver info structure.
@@ -411,7 +413,7 @@ static void driver_get_info(GraphicsDevice* device,
 }
 
 
-static DFBResult driver_init_driver(GraphicsDevice* device,
+static DFBResult driver_init_driver(CoreGraphicsDevice* device,
                                     GraphicsDeviceFuncs* funcs,
                                     void* driver_data,
                                     void* device_data,
@@ -438,7 +440,7 @@ static DFBResult driver_init_driver(GraphicsDevice* device,
           ucdrv->file = fd;
 
           ucdrv->hwregs = mmap(NULL, 0x1000000, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
-          if ((int) ucdrv->hwregs == -1)
+          if (ucdrv->hwregs == MAP_FAILED)
                return DFB_IO;
      }
 
@@ -480,7 +482,7 @@ static DFBResult driver_init_driver(GraphicsDevice* device,
      return DFB_OK;
 }
 
-static DFBResult driver_init_device(GraphicsDevice* device,
+static DFBResult driver_init_device(CoreGraphicsDevice* device,
                                     GraphicsDeviceInfo* device_info,
                                     void* driver_data,
                                     void* device_data)
@@ -522,7 +524,7 @@ static DFBResult driver_init_device(GraphicsDevice* device,
      return DFB_OK;
 }
 
-static void driver_close_device(GraphicsDevice *device,
+static void driver_close_device(CoreGraphicsDevice *device,
                                 void *driver_data, void *device_data)
 {
      UcDriverData* ucdrv = (UcDriverData*) driver_data;
@@ -534,7 +536,7 @@ static void driver_close_device(GraphicsDevice *device,
      uc_init_2d_engine(device, ucdev, ucdrv, false);
 }
 
-static void driver_close_driver(GraphicsDevice* device, void* driver_data)
+static void driver_close_driver(CoreGraphicsDevice* device, void* driver_data)
 {
      UcDriverData* ucdrv = (UcDriverData*) driver_data;
 
