@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,11 +26,13 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <directfb.h>
 
 #include <core/coretypes.h>
 #include <core/layers.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include "cyber5k.h"
 #include "cyber5k_alpha.h"
@@ -42,7 +45,8 @@ typedef struct {
 static void udl_set_all     ( CyberDriverData        *cdrv,
                               CyberUnderlayLayerData *cudl,
                               CoreLayerRegionConfig  *config,
-                              CoreSurface            *surface );
+                              CoreSurface            *surface,
+                              CoreSurfaceBufferLock  *lock );
 static void udl_set_location( CyberDriverData        *cdrv,
                               CyberUnderlayLayerData *cudl,
                               CoreLayerRegionConfig  *config,
@@ -53,7 +57,7 @@ static void udl_set_location( CyberDriverData        *cdrv,
 /**********************/
 
 static int
-udlLayerDataSize()
+udlLayerDataSize( void )
 {
      return sizeof(CyberUnderlayLayerData);
 }
@@ -153,7 +157,8 @@ udlSetRegion( CoreLayer                  *layer,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags  updated,
               CoreSurface                *surface,
-              CorePalette                *palette )
+              CorePalette                *palette,
+              CoreSurfaceBufferLock      *lock )
 {
      CyberDriverData        *cdrv = (CyberDriverData*) driver_data;
      CyberUnderlayLayerData *cudl = (CyberUnderlayLayerData*) layer_data;
@@ -162,7 +167,7 @@ udlSetRegion( CoreLayer                  *layer,
      cudl->config = *config;
 
      /* set up layer */
-     udl_set_all( cdrv, cudl, config, surface );
+     udl_set_all( cdrv, cudl, config, surface, lock );
 
      return DFB_OK;
 }
@@ -182,32 +187,33 @@ udlRemoveRegion( CoreLayer *layer,
 }
 
 static DFBResult
-udlFlipRegion( CoreLayer           *layer,
-               void                *driver_data,
-               void                *layer_data,
-               void                *region_data,
-               CoreSurface         *surface,
-               DFBSurfaceFlipFlags  flags )
+udlFlipRegion( CoreLayer             *layer,
+               void                  *driver_data,
+               void                  *layer_data,
+               void                  *region_data,
+               CoreSurface           *surface,
+               DFBSurfaceFlipFlags    flags,
+               CoreSurfaceBufferLock *lock )
 {
      CyberDriverData        *cdrv = (CyberDriverData*) driver_data;
      CyberUnderlayLayerData *cudl = (CyberUnderlayLayerData*) layer_data;
 
-     dfb_surface_flip_buffers( surface, false );
+     dfb_surface_flip( surface, false );
 
-     udl_set_all( cdrv, cudl, &cudl->config, surface );
+     udl_set_all( cdrv, cudl, &cudl->config, surface, lock );
 
      return DFB_OK;
 }
 
 
 DisplayLayerFuncs cyberUnderlayFuncs = {
-     LayerDataSize:      udlLayerDataSize,
-     InitLayer:          udlInitLayer,
+     .LayerDataSize = udlLayerDataSize,
+     .InitLayer     = udlInitLayer,
 
-     TestRegion:         udlTestRegion,
-     SetRegion:          udlSetRegion,
-     RemoveRegion:       udlRemoveRegion,
-     FlipRegion:         udlFlipRegion
+     .TestRegion    = udlTestRegion,
+     .SetRegion     = udlSetRegion,
+     .RemoveRegion  = udlRemoveRegion,
+     .FlipRegion    = udlFlipRegion,
 };
 
 
@@ -216,12 +222,11 @@ DisplayLayerFuncs cyberUnderlayFuncs = {
 static void udl_set_all( CyberDriverData        *cdrv,
                          CyberUnderlayLayerData *cudl,
                          CoreLayerRegionConfig  *config,
-                         CoreSurface            *surface )
+                         CoreSurface            *surface,
+                         CoreSurfaceBufferLock  *lock )
 {
-     SurfaceBuffer *front = surface->front_buffer;
-
      /* set the pixel format */
-     switch (surface->format) {
+     switch (surface->config.format) {
           case DSPF_RGB332:
                cyber_set_overlay_format (OVERLAY_RGB8);
                break;
@@ -255,8 +260,8 @@ static void udl_set_all( CyberDriverData        *cdrv,
      cyber_set_overlay_mode( OVERLAY_WINDOWKEY );
 
      /* set address */
-     cyber_set_overlay_srcaddr( front->video.offset, 0, 0,
-                                surface->width, front->video.pitch );
+     cyber_set_overlay_srcaddr( lock->offset, 0, 0,
+                                surface->config.size.w, lock->pitch );
 
      /* set location and scaling */
      udl_set_location( cdrv, cudl, config, surface );
@@ -288,9 +293,9 @@ static void udl_set_location( CyberDriverData        *cdrv,
                                config->dest.y + config->dest.h - 1 );
 
      /* set scaling */
-     cyber_set_overlay_scale( surface->height == 576 ? /* HACK: support interlaced video */
+     cyber_set_overlay_scale( surface->config.size.h == 576 ? /* HACK: support interlaced video */
                               OVERLAY_BOBMODE : OVERLAY_WEAVEMODE,
-                              surface->width, config->dest.w,
-                              surface->height, config->dest.h );
+                              surface->config.size.w, config->dest.w,
+                              surface->config.size.h, config->dest.h );
 }
 
