@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,13 +26,15 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 
 #include <direct/messages.h>
 
 #include <core/coredefs.h>
 #include <core/layers.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
 
 #include <gfx/convert.h>
@@ -48,35 +51,37 @@ typedef struct {
 
      /* overlay registers */
      struct {
-          __u32 overlay_Y_X_START;
-          __u32 overlay_Y_X_END;
-          __u32 overlay_GRAPHICS_KEY_CLR;
-          __u32 overlay_GRAPHICS_KEY_MSK;
-          __u32 overlay_VIDEO_KEY_CLR;
-          __u32 overlay_VIDEO_KEY_MSK;
-          __u32 overlay_KEY_CNTL;
-          __u32 overlay_SCALE_INC;
-          __u32 overlay_SCALE_CNTL;
-          __u32 scaler_HEIGHT_WIDTH;
-          __u32 scaler_BUF_PITCH;
-          __u32 scaler_BUF0_OFFSET;
-          __u32 scaler_BUF1_OFFSET;
-          __u32 scaler_BUF0_OFFSET_U;
-          __u32 scaler_BUF0_OFFSET_V;
-          __u32 scaler_BUF1_OFFSET_U;
-          __u32 scaler_BUF1_OFFSET_V;
-          __u32 video_FORMAT;
-          __u32 capture_CONFIG;
+          u32 overlay_Y_X_START;
+          u32 overlay_Y_X_END;
+          u32 overlay_GRAPHICS_KEY_CLR;
+          u32 overlay_GRAPHICS_KEY_MSK;
+          u32 overlay_VIDEO_KEY_CLR;
+          u32 overlay_VIDEO_KEY_MSK;
+          u32 overlay_KEY_CNTL;
+          u32 overlay_SCALE_INC;
+          u32 overlay_SCALE_CNTL;
+          u32 scaler_HEIGHT_WIDTH;
+          u32 scaler_BUF_PITCH;
+          u32 scaler_BUF0_OFFSET;
+          u32 scaler_BUF1_OFFSET;
+          u32 scaler_BUF0_OFFSET_U;
+          u32 scaler_BUF0_OFFSET_V;
+          u32 scaler_BUF1_OFFSET_U;
+          u32 scaler_BUF1_OFFSET_V;
+          u32 video_FORMAT;
+          u32 capture_CONFIG;
      } regs;
 } Mach64OverlayLayerData;
 
 static void ov_reset( Mach64DriverData *mdrv );
 static void ov_set_regs( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 static void ov_calc_regs( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
-                          CoreLayerRegionConfig *config, CoreSurface *surface );
+                          CoreLayerRegionConfig *config, CoreSurface *surface,
+                          CoreSurfaceBufferLock  *lock );
 static void ov_set_buffer( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 static void ov_calc_buffer( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
-                            CoreLayerRegionConfig *config, CoreSurface *surface );
+                            CoreLayerRegionConfig *config, CoreSurface *surface,
+                            CoreSurfaceBufferLock  *lock );
 static void ov_set_colorkey( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 static void ov_calc_colorkey( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov,
                               CoreLayerRegionConfig *config );
@@ -90,7 +95,7 @@ static void ov_set_field( Mach64DriverData *mdrv, Mach64OverlayLayerData *mov );
 /**********************/
 
 static int
-ovLayerDataSize()
+ovLayerDataSize( void )
 {
      return sizeof(Mach64OverlayLayerData);
 }
@@ -196,6 +201,7 @@ ovTestRegion( CoreLayer                  *layer,
 
      /* check pixel format */
      switch (config->format) {
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
@@ -250,7 +256,8 @@ ovSetRegion( CoreLayer                  *layer,
              CoreLayerRegionConfig      *config,
              CoreLayerRegionConfigFlags  updated,
              CoreSurface                *surface,
-             CorePalette                *palette )
+             CorePalette                *palette,
+             CoreSurfaceBufferLock      *lock )
 {
      Mach64DriverData       *mdrv = (Mach64DriverData*) driver_data;
      Mach64OverlayLayerData *mov  = (Mach64OverlayLayerData*) layer_data;
@@ -262,8 +269,8 @@ ovSetRegion( CoreLayer                  *layer,
           ov_reset( mdrv );
 
      if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT | CLRCF_SOURCE | CLRCF_DEST | CLRCF_OPTIONS)) {
-          ov_calc_buffer( mdrv, mov, config, surface );
-          ov_calc_regs( mdrv, mov, config, surface );
+          ov_calc_buffer( mdrv, mov, config, surface, lock );
+          ov_calc_regs( mdrv, mov, config, surface, lock );
           ov_set_buffer( mdrv, mov );
           ov_set_regs( mdrv, mov );
      }
@@ -292,7 +299,7 @@ ovRemoveRegion( CoreLayer *layer,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 2 );
 
@@ -309,21 +316,22 @@ ovRemoveRegion( CoreLayer *layer,
 }
 
 static DFBResult
-ovFlipRegion( CoreLayer           *layer,
-              void                *driver_data,
-              void                *layer_data,
-              void                *region_data,
-              CoreSurface         *surface,
-              DFBSurfaceFlipFlags  flags )
+ovFlipRegion( CoreLayer             *layer,
+              void                  *driver_data,
+              void                  *layer_data,
+              void                  *region_data,
+              CoreSurface           *surface,
+              DFBSurfaceFlipFlags    flags,
+              CoreSurfaceBufferLock *lock )
 {
      Mach64DriverData       *mdrv = (Mach64DriverData*) driver_data;
      Mach64OverlayLayerData *mov  = (Mach64OverlayLayerData*) layer_data;
 
-     dfb_surface_flip_buffers( surface, false );
-
-     ov_calc_buffer( mdrv, mov, &mov->config, surface );
+     ov_calc_buffer( mdrv, mov, &mov->config, surface, lock );
      ov_set_buffer( mdrv, mov );
 
+     dfb_surface_flip( surface, false );
+
      return DFB_OK;
 }
 
@@ -335,7 +343,7 @@ ovSetColorAdjustment( CoreLayer          *layer,
 {
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      if (mdev->chip < CHIP_3D_RAGE_PRO)
           return DFB_UNSUPPORTED;
@@ -368,15 +376,15 @@ ovSetInputField( CoreLayer          *layer,
 }
 
 DisplayLayerFuncs mach64OverlayFuncs = {
-     LayerDataSize:      ovLayerDataSize,
-     InitLayer:          ovInitLayer,
+     .LayerDataSize      = ovLayerDataSize,
+     .InitLayer          = ovInitLayer,
 
-     TestRegion:         ovTestRegion,
-     SetRegion:          ovSetRegion,
-     RemoveRegion:       ovRemoveRegion,
-     FlipRegion:         ovFlipRegion,
-     SetColorAdjustment: ovSetColorAdjustment,
-     SetInputField:      ovSetInputField,
+     .TestRegion         = ovTestRegion,
+     .SetRegion          = ovSetRegion,
+     .RemoveRegion       = ovRemoveRegion,
+     .FlipRegion         = ovFlipRegion,
+     .SetColorAdjustment = ovSetColorAdjustment,
+     .SetInputField      = ovSetInputField,
 };
 
 /* internal */
@@ -384,7 +392,7 @@ DisplayLayerFuncs mach64OverlayFuncs = {
 static void ov_reset( Mach64DriverData *mdrv )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      if (mdev->chip >= CHIP_3D_RAGE_PRO) {
           mach64_waitfifo( mdrv, mdev, 6 );
@@ -414,7 +422,7 @@ static void ov_set_regs( Mach64DriverData       *mdrv,
                          Mach64OverlayLayerData *mov )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, (mdev->chip >= CHIP_264VT3) ? 6 : 7 );
 
@@ -436,7 +444,7 @@ static void ov_set_buffer( Mach64DriverData       *mdrv,
                            Mach64OverlayLayerData *mov )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, (mdev->chip >= CHIP_3D_RAGE_PRO) ? 6 : 2 );
 
@@ -460,7 +468,7 @@ static void ov_set_colorkey( Mach64DriverData       *mdrv,
                              Mach64OverlayLayerData *mov )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 5 );
 
@@ -475,7 +483,7 @@ static void ov_set_opacity( Mach64DriverData       *mdrv,
                             Mach64OverlayLayerData *mov )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 1 );
 
@@ -486,7 +494,7 @@ static void ov_set_field( Mach64DriverData       *mdrv,
                           Mach64OverlayLayerData *mov )
 {
      Mach64DeviceData *mdev = mdrv->device_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      mach64_waitfifo( mdrv, mdev, 1 );
 
@@ -496,21 +504,21 @@ static void ov_set_field( Mach64DriverData       *mdrv,
 static void ov_calc_regs( Mach64DriverData       *mdrv,
                           Mach64OverlayLayerData *mov,
                           CoreLayerRegionConfig  *config,
-                          CoreSurface            *surface )
+                          CoreSurface            *surface,
+                          CoreSurfaceBufferLock  *lock )
 {
      Mach64DeviceData *mdev   = mdrv->device_data;
-     volatile __u8    *mmio   = mdrv->mmio_base;
-     SurfaceBuffer    *buffer = surface->front_buffer;
+     volatile u8      *mmio   = mdrv->mmio_base;
      VideoMode        *mode   = dfb_system_current_mode();
      int               yres   = mode->yres;
-     int               pitch  = buffer->video.pitch / DFB_BYTES_PER_PIXEL( surface->format );
+     unsigned int      pitch  = lock->pitch / DFB_BYTES_PER_PIXEL( surface->config.format );
      DFBRectangle      source = config->source;
      DFBRectangle      dest   = config->dest;
 
      DFBRegion dst;
      int h_inc, v_inc;
-     __u32 lcd_gen_ctrl, vert_stretching;
-     __u8 ecp_div;
+     u32 lcd_gen_ctrl, vert_stretching;
+     u8 ecp_div;
 
      if (mode->doubled) {
           dest.y *= 2;
@@ -547,7 +555,8 @@ static void ov_calc_regs( Mach64DriverData       *mdrv,
      else
           v_inc = (source.h << 12) / dest.h;
 
-     switch (surface->format) {
+     switch (surface->config.format) {
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mov->regs.video_FORMAT = SCALER_IN_RGB15;
                break;
@@ -582,13 +591,13 @@ static void ov_calc_regs( Mach64DriverData       *mdrv,
 static void ov_calc_buffer( Mach64DriverData       *mdrv,
                             Mach64OverlayLayerData *mov,
                             CoreLayerRegionConfig  *config,
-                            CoreSurface            *surface )
+                            CoreSurface            *surface,
+                            CoreSurfaceBufferLock  *lock )
 {
-     SurfaceBuffer *buffer = surface->front_buffer;
-     int            pitch  = buffer->video.pitch;
+     unsigned int   pitch  = lock->pitch;
      DFBRectangle   source = config->source;
 
-     __u32 offset, offset_u, offset_v;
+     u32 offset, offset_u, offset_v;
      int cropleft, croptop;
 
      if (config->options & DLOP_DEINTERLACING) {
@@ -607,13 +616,13 @@ static void ov_calc_buffer( Mach64DriverData       *mdrv,
      if (config->dest.y < 0)
           croptop  += -config->dest.y * source.h / config->dest.h;
 
-     switch (surface->format) {
+     switch (surface->config.format) {
           case DSPF_I420:
                cropleft &= ~15;
                croptop  &= ~1;
 
-               offset_u  = buffer->video.offset + surface->height * buffer->video.pitch;
-               offset_v  = offset_u + surface->height/2 * buffer->video.pitch/2;
+               offset_u  = lock->offset + surface->config.size.h * lock->pitch;
+               offset_v  = offset_u + surface->config.size.h/2 * lock->pitch/2;
                offset_u += croptop/2 * pitch/2 + cropleft/2;
                offset_v += croptop/2 * pitch/2 + cropleft/2;
                break;
@@ -622,8 +631,8 @@ static void ov_calc_buffer( Mach64DriverData       *mdrv,
                cropleft &= ~15;
                croptop  &= ~1;
 
-               offset_v  = buffer->video.offset + surface->height * buffer->video.pitch;
-               offset_u  = offset_v + surface->height/2 * buffer->video.pitch/2;
+               offset_v  = lock->offset + surface->config.size.h * lock->pitch;
+               offset_u  = offset_v + surface->config.size.h/2 * lock->pitch/2;
                offset_v += croptop/2 * pitch/2 + cropleft/2;
                offset_u += croptop/2 * pitch/2 + cropleft/2;
                break;
@@ -634,19 +643,19 @@ static void ov_calc_buffer( Mach64DriverData       *mdrv,
                break;
      }
 
-     offset  = buffer->video.offset;
-     offset += croptop * pitch + cropleft * DFB_BYTES_PER_PIXEL( surface->format );
+     offset  = lock->offset;
+     offset += croptop * pitch + cropleft * DFB_BYTES_PER_PIXEL( surface->config.format );
 
      mov->regs.scaler_BUF0_OFFSET   = offset;
      mov->regs.scaler_BUF0_OFFSET_U = offset_u;
      mov->regs.scaler_BUF0_OFFSET_V = offset_v;
 
-     mov->regs.scaler_BUF1_OFFSET   = offset   + buffer->video.pitch;
-     mov->regs.scaler_BUF1_OFFSET_U = offset_u + buffer->video.pitch/2;
-     mov->regs.scaler_BUF1_OFFSET_V = offset_v + buffer->video.pitch/2;
+     mov->regs.scaler_BUF1_OFFSET   = offset   + lock->pitch;
+     mov->regs.scaler_BUF1_OFFSET_U = offset_u + lock->pitch/2;
+     mov->regs.scaler_BUF1_OFFSET_V = offset_v + lock->pitch/2;
 }
 
-static __u32 ovColorKey[] = {
+static u32 ovColorKey[] = {
      VIDEO_KEY_FN_TRUE      | GRAPHICS_KEY_FN_TRUE  | OVERLAY_CMP_MIX_OR, /* 0 */
      VIDEO_KEY_FN_NOT_EQUAL | GRAPHICS_KEY_FN_FALSE | OVERLAY_CMP_MIX_OR, /* DLOP_SRC_COLORKEY */
      VIDEO_KEY_FN_FALSE     | GRAPHICS_KEY_FN_EQUAL | OVERLAY_CMP_MIX_OR, /* DLOP_DST_COLORKEY */
@@ -674,28 +683,27 @@ static void ov_calc_colorkey( Mach64DriverData       *mdrv,
                                                                   config->dst_key.g,
                                                                   config->dst_key.b );
                break;
+          case DSPF_RGB555:
+               mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_RGB555( config->dst_key.r,
+                                                                  config->dst_key.g,
+                                                                  config->dst_key.b );
+               break;
           case DSPF_ARGB1555:
-               mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_ARGB1555( config->dst_key.a,
-                                                                    config->dst_key.r,
-                                                                    config->dst_key.g,
-                                                                    config->dst_key.b );
+               mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_RGB555( config->dst_key.r,
+                                                                  config->dst_key.g,
+                                                                  config->dst_key.b );
                break;
           case DSPF_RGB16:
                mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_RGB16( config->dst_key.r,
                                                                  config->dst_key.g,
                                                                  config->dst_key.b );
                break;
+          case DSPF_ARGB:
           case DSPF_RGB32:
                mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_RGB32( config->dst_key.r,
                                                                  config->dst_key.g,
                                                                  config->dst_key.b );
                break;
-          case DSPF_ARGB:
-               mov->regs.overlay_GRAPHICS_KEY_CLR = PIXEL_ARGB( config->dst_key.a,
-                                                                config->dst_key.r,
-                                                                config->dst_key.g,
-                                                                config->dst_key.b );
-               break;
           default:
                D_BUG( "unexpected pixelformat" );
      }
