@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,16 +26,16 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <linux/fb.h>
+#include <config.h>
+
+#include <fbdev/fb.h>
 
 #include <math.h>
 
 #include <directfb.h>
 
 #include <direct/messages.h>
-#include <direct/util.h>
-
-#include <core/accelerators.h>
+#include <direct/types.h>
 
 #include "regs.h"
 #include "mmio.h"
@@ -47,12 +48,12 @@
 #define CEIL(x) my_ceil(x)
 #define FLOOR(x) my_floor(x)
 #else
-#define RINT(x) ((__s32)(x))
-#define CEIL(x) ((__s32)ceil(x))
-#define FLOOR(x) ((__s32)floor(x))
+#define RINT(x) ((s32)(x))
+#define CEIL(x) ((s32)ceil(x))
+#define FLOOR(x) ((s32)floor(x))
 #endif
 
-
+#ifdef ARCH_X86
 static inline long
 my_rint(const float x)
 {
@@ -95,13 +96,13 @@ my_floor(const float x)
                        : "m" (cwtmp), "m" (cw), "t" (arg));
      return value;
 }
-
+#endif
 
 #define F2COL(x) (RINT(x) & 0x00ffffff)
 
-#define mgaF1800(x) (((__s32) (x)) & 0x0003ffff)
-#define mgaF2400(x) (((__s32) (x)) & 0x00ffffff)
-#define mgaF2200(x) (((__s32) (x)) & 0x003fffff)
+#define mgaF1800(x) (((s32) (x)) & 0x0003ffff)
+#define mgaF2400(x) (((s32) (x)) & 0x00ffffff)
+#define mgaF2200(x) (((s32) (x)) & 0x003fffff)
 
 #define OUTREG(r,d)  do { mga_out32( mmio, d, r ); } while (0)
 
@@ -216,7 +217,7 @@ texture_triangle( MatroxDriverData *mdrv, MatroxDeviceData *mdev,
      int         Shape; /* 1 = Top half, 2 = bottom half, 3 = top+bottom */
 //     float       bf = mga_bf_sign;
 
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8 *mmio = mdrv->mmio_base;
 
 /* find the order of the 3 vertices along the Y axis */
      {
@@ -476,7 +477,7 @@ texture_triangle( MatroxDriverData *mdrv, MatroxDeviceData *mdev,
           }
 
           {
-               __u32 sgn = 0;
+               u32 sgn = 0;
 
                mga_waitfifo( mdrv, mdev, 9 );
 
@@ -491,8 +492,8 @@ texture_triangle( MatroxDriverData *mdrv, MatroxDeviceData *mdev,
                }
 
                mga_out32( mmio, sgn,                            SGN );
-               mga_out32( mmio, ((__u32)(eLeft->sx) & 0xFFFF) | ((__u32)(eRight->sx) << 16), FXBNDRY );
-               mga_out32( mmio, lines     | ((__u32)(eLeft->sy)  << 16), YDSTLEN | EXECUTE );
+               mga_out32( mmio, ((u32)(eLeft->sx) & 0xFFFF) | ((u32)(eRight->sx) << 16), FXBNDRY );
+               mga_out32( mmio, lines     | ((u32)(eLeft->sy)  << 16), YDSTLEN | EXECUTE );
 
                if (Shape != 3) { /* has only one half? */
                     return;
@@ -535,8 +536,8 @@ matroxTextureTriangles( void *drv, void *dev,
      int               i;
      MatroxDriverData *mdrv = (MatroxDriverData*) drv;
      MatroxDeviceData *mdev = (MatroxDeviceData*) dev;
-     volatile __u8    *mmio = mdrv->mmio_base;
-     __u32             dwgctl;
+     volatile u8      *mmio = mdrv->mmio_base;
+     u32               dwgctl;
 
      float wScale;
 
