@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,13 +26,9 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
+#include <config.h>
 
-#include <sys/mman.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
+#include <stdio.h>
 
 #include <fbdev/fb.h>
 
@@ -39,19 +36,12 @@
 
 #include <direct/messages.h>
 
-#include <fusion/shmalloc.h>
-
 #include <core/coredefs.h>
 #include <core/coretypes.h>
-#include <core/gfxcard.h>
-#include <core/layer_control.h>
 #include <core/layers.h>
 #include <core/screen.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
-#include <core/windows.h>
-
-#include <direct/mem.h>
 
 #include <misc/util.h>
 
@@ -65,36 +55,36 @@ typedef struct {
      /* Stored registers */
      struct {
           /* BES */
-          __u32 besGLOBCTL;
-          __u32 besA1ORG;
-          __u32 besA2ORG;
-          __u32 besA1CORG;
-          __u32 besA2CORG;
-          __u32 besA1C3ORG;
-          __u32 besA2C3ORG;
-          __u32 besCTL;
+          u32 besGLOBCTL;
+          u32 besA1ORG;
+          u32 besA2ORG;
+          u32 besA1CORG;
+          u32 besA2CORG;
+          u32 besA1C3ORG;
+          u32 besA2C3ORG;
+          u32 besCTL;
 
-          __u32 besCTL_field;
+          u32 besCTL_field;
 
-          __u32 besHCOORD;
-          __u32 besVCOORD;
+          u32 besHCOORD;
+          u32 besVCOORD;
 
-          __u32 besHSRCST;
-          __u32 besHSRCEND;
-          __u32 besHSRCLST;
+          u32 besHSRCST;
+          u32 besHSRCEND;
+          u32 besHSRCLST;
 
-          __u32 besPITCH;
+          u32 besPITCH;
 
-          __u32 besV1WGHT;
-          __u32 besV2WGHT;
+          u32 besV1WGHT;
+          u32 besV2WGHT;
 
-          __u32 besV1SRCLST;
-          __u32 besV2SRCLST;
+          u32 besV1SRCLST;
+          u32 besV2SRCLST;
 
-          __u32 besVISCAL;
-          __u32 besHISCAL;
+          u32 besVISCAL;
+          u32 besHISCAL;
 
-          __u8  xKEYOPMODE;
+          u8    xKEYOPMODE;
      } regs;
 } MatroxBesLayerData;
 
@@ -102,7 +92,7 @@ static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
                           bool onsync );
 static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
                            CoreLayerRegionConfig *config, CoreSurface *surface,
-                           bool front );
+                           CoreSurfaceBufferLock *lock );
 
 #define BES_SUPPORTED_OPTIONS   (DLOP_DEINTERLACING | DLOP_DST_COLORKEY)
 
@@ -110,7 +100,7 @@ static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
 /**********************/
 
 static int
-besLayerDataSize()
+besLayerDataSize( void )
 {
      return sizeof(MatroxBesLayerData);
 }
@@ -124,7 +114,7 @@ besInitLayer( CoreLayer                  *layer,
               DFBColorAdjustment         *adjustment )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
-     volatile __u8      *mmio = mdrv->mmio_base;
+     volatile u8        *mmio = mdrv->mmio_base;
 
      /* set capabilities and type */
      description->caps = DLCAPS_SCREEN_LOCATION | DLCAPS_SURFACE |
@@ -159,6 +149,8 @@ besInitLayer( CoreLayer                  *layer,
           mga_out32( mmio, 0x80, BESLUMACTL );
      }
 
+     /* make sure BES registers get updated (besvcnt) */
+     mga_out32( mmio, 0, BESGLOBCTL );
      /* disable backend scaler */
      mga_out32( mmio, 0, BESCTL );
 
@@ -192,10 +184,17 @@ besTestRegion( CoreLayer                  *layer,
      if (config->options & ~BES_SUPPORTED_OPTIONS)
           fail |= CLRCF_OPTIONS;
 
+     if (config->surface_caps & ~(DSCAPS_INTERLACED | DSCAPS_SEPARATED))
+          fail |= CLRCF_SURFACE_CAPS;
+
      if (config->options & DLOP_DEINTERLACING) {
           /* make sure BESPITCH < 4096 */
-          max_width = mdev->g450_matrox ? 2048 - 128 : 1024;
+          if (mdev->g450_matrox && !(config->surface_caps & DSCAPS_SEPARATED))
+               max_width = 2048 - 128;
           max_height = 2048;
+     } else {
+          if (config->surface_caps & DSCAPS_SEPARATED)
+               fail |= CLRCF_SURFACE_CAPS;
      }
 
      switch (config->format) {
@@ -208,6 +207,7 @@ besTestRegion( CoreLayer                  *layer,
           case DSPF_RGB32:
                if (!mdev->g450_matrox)
                     max_width = 512;
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_UYVY:
@@ -258,11 +258,12 @@ besSetRegion( CoreLayer                  *layer,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags  updated,
               CoreSurface                *surface,
-              CorePalette                *palette )
+              CorePalette                *palette,
+              CoreSurfaceBufferLock      *lock )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
      MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
-     volatile __u8      *mmio = mdrv->mmio_base;
+     volatile u8        *mmio = mdrv->mmio_base;
 
      /* remember configuration */
      mbes->config = *config;
@@ -271,15 +272,16 @@ besSetRegion( CoreLayer                  *layer,
      if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT |
                     CLRCF_OPTIONS | CLRCF_DEST | CLRCF_OPACITY | CLRCF_SOURCE))
      {
-          bes_calc_regs( mdrv, mbes, config, surface, true );
+          bes_calc_regs( mdrv, mbes, config, surface, lock );
           bes_set_regs( mdrv, mbes, true );
      }
 
      /* set color key */
      if (updated & CLRCF_DSTKEY) {
-          DFBColor key = config->dst_key;
+          DFBColorKey key = config->dst_key;
 
           switch (dfb_primary_layer_pixelformat()) {
+               case DSPF_RGB555:
                case DSPF_ARGB1555:
                     key.r >>= 3;
                     key.g >>= 3;
@@ -311,28 +313,32 @@ besRemoveRegion( CoreLayer *layer,
                  void      *region_data )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
+     volatile u8        *mmio = mdrv->mmio_base;
 
+     /* make sure BES registers get updated (besvcnt) */
+     mga_out32( mmio, 0, BESGLOBCTL );
      /* disable backend scaler */
-     mga_out32( mdrv->mmio_base, 0, BESCTL );
+     mga_out32( mmio, 0, BESCTL );
 
      return DFB_OK;
 }
 
 static DFBResult
-besFlipRegion( CoreLayer           *layer,
-               void                *driver_data,
-               void                *layer_data,
-               void                *region_data,
-               CoreSurface         *surface,
-               DFBSurfaceFlipFlags  flags )
+besFlipRegion( CoreLayer             *layer,
+               void                  *driver_data,
+               void                  *layer_data,
+               void                  *region_data,
+               CoreSurface           *surface,
+               DFBSurfaceFlipFlags    flags,
+               CoreSurfaceBufferLock *lock )
 {
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
      MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
 
-     bes_calc_regs( mdrv, mbes, &mbes->config, surface, false );
+     bes_calc_regs( mdrv, mbes, &mbes->config, surface, lock );
      bes_set_regs( mdrv, mbes, flags & DSFLIP_ONSYNC );
 
-     dfb_surface_flip_buffers( surface, false );
+     dfb_surface_flip( surface, false );
 
      if (flags & DSFLIP_WAIT)
           dfb_screen_wait_vsync( mdrv->primary );
@@ -347,13 +353,13 @@ besSetColorAdjustment( CoreLayer          *layer,
                        DFBColorAdjustment *adj )
 {
      MatroxDriverData *mdrv = (MatroxDriverData*) driver_data;
-     volatile __u8    *mmio = mdrv->mmio_base;
+     volatile u8      *mmio = mdrv->mmio_base;
 
      if (mdrv->accelerator == FB_ACCEL_MATROX_MGAG200)
           return DFB_UNSUPPORTED;
 
      mga_out32( mmio, (adj->contrast >> 8) |
-                      ((__u8)(((int)adj->brightness >> 8) - 128)) << 16,
+                      ((u8)(((int)adj->brightness >> 8) - 128)) << 16,
                 BESLUMACTL );
 
      return DFB_OK;
@@ -378,16 +384,16 @@ besSetInputField( CoreLayer *layer,
 }
 
 DisplayLayerFuncs matroxBesFuncs = {
-     LayerDataSize:      besLayerDataSize,
-     InitLayer:          besInitLayer,
+     .LayerDataSize      = besLayerDataSize,
+     .InitLayer          = besInitLayer,
 
-     TestRegion:         besTestRegion,
-     SetRegion:          besSetRegion,
-     RemoveRegion:       besRemoveRegion,
-     FlipRegion:         besFlipRegion,
+     .TestRegion         = besTestRegion,
+     .SetRegion          = besSetRegion,
+     .RemoveRegion       = besRemoveRegion,
+     .FlipRegion         = besFlipRegion,
 
-     SetColorAdjustment: besSetColorAdjustment,
-     SetInputField:      besSetInputField
+     .SetColorAdjustment = besSetColorAdjustment,
+     .SetInputField      = besSetInputField,
 };
 
 
@@ -397,7 +403,7 @@ static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
                           bool onsync )
 {
      int            line = 0;
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile u8   *mmio = mdrv->mmio_base;
      VideoMode     *mode = dfb_system_current_mode();
 
      if (!mode) {
@@ -449,15 +455,14 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
                            MatroxBesLayerData    *mbes,
                            CoreLayerRegionConfig *config,
                            CoreSurface           *surface,
-                           bool                   front )
+                           CoreSurfaceBufferLock *lock )
 {
      MatroxDeviceData *mdev = mdrv->device_data;
      int cropleft, cropright, croptop, cropbot, croptop_uv;
-     int pitch, tmp, hzoom, intrep, field_height;
+     int pitch, tmp, hzoom, intrep, field_height, field_offset;
      DFBRectangle   source, dest;
      DFBRegion      dst;
      bool           visible;
-     SurfaceBuffer *buffer = front ? surface->front_buffer : surface->back_buffer;
      VideoMode     *mode   = dfb_system_current_mode();
 
      if (!mode) {
@@ -471,18 +476,19 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
      source = config->source;
      dest   = config->dest;
 
-     if (!mdev->g450_matrox && (surface->format == DSPF_RGB32 || surface->format == DSPF_ARGB))
+     if (!mdev->g450_matrox && (surface->config.format == DSPF_RGB32 || surface->config.format == DSPF_ARGB))
           dest.w = source.w;
 
-     pitch = buffer->video.pitch;
+     pitch = lock->pitch;
 
-     field_height = surface->height;
+     field_height = surface->config.size.h;
 
      if (config->options & DLOP_DEINTERLACING) {
           field_height /= 2;
           source.y /= 2;
           source.h /= 2;
-          pitch    *= 2;
+          if (!(surface->config.caps & DSCAPS_SEPARATED))
+               pitch *= 2;
      } else
           mbes->regs.besCTL_field = 0;
 
@@ -508,15 +514,15 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
 
      /* scale crop values to source dimensions */
      if (cropleft)
-          cropleft = ((__u64) (source.w << 16) * cropleft / dest.w) & ~0x3;
+          cropleft = ((u64) (source.w << 16) * cropleft / dest.w) & ~0x3;
      if (croptop)
-          croptop = ((__u64) (source.h << 16) * croptop / dest.h) & ~0x3;
+          croptop = ((u64) (source.h << 16) * croptop / dest.h) & ~0x3;
      if (cropright)
-          cropright = ((__u64) (source.w << 16) * cropright / dest.w) & ~0x3;
+          cropright = ((u64) (source.w << 16) * cropright / dest.w) & ~0x3;
      if (cropbot)
-          cropbot = ((__u64) (source.h << 16) * cropbot / dest.h) & ~0x3;
+          cropbot = ((u64) (source.h << 16) * cropbot / dest.h) & ~0x3;
      if (croptop_uv)
-          croptop_uv = ((__u64) ((source.h/2) << 16) * croptop_uv / dest.h) & ~0x3;
+          croptop_uv = ((u64) ((source.h/2) << 16) * croptop_uv / dest.h) & ~0x3;
 
      /* should horizontal zoom be used? */
      if (mdev->g450_matrox)
@@ -531,7 +537,7 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
      mbes->regs.besCTL = (config->opacity && visible) ? BESEN : 0;
 
      /* pixel format settings */
-     switch (surface->format) {
+     switch (surface->config.format) {
           case DSPF_YV12:
                mbes->regs.besGLOBCTL |= BESCORDER;
                /* fall through */
@@ -559,6 +565,7 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
                mbes->regs.besCTL     |= BESHFEN | BESVFEN | BESCUPS;
                break;
 
+          case DSPF_RGB555:
           case DSPF_ARGB1555:
                mbes->regs.besGLOBCTL |= BESRGB15;
                break;
@@ -577,41 +584,54 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
                return;
      }
 
-     if (surface->width > 1024)
+     if (surface->config.size.w > 1024)
           mbes->regs.besCTL &= ~BESVFEN;
 
      mbes->regs.besGLOBCTL |= 3*hzoom | (mode->yres & 0xFFF) << 16;
 
-     mbes->regs.besPITCH = pitch / DFB_BYTES_PER_PIXEL(surface->format);
+     mbes->regs.besPITCH = pitch / DFB_BYTES_PER_PIXEL(surface->config.format);
 
      /* buffer offsets */
-     mbes->regs.besA1ORG = buffer->video.offset +
+
+     field_offset = lock->pitch;
+     if (surface->config.caps & DSCAPS_SEPARATED)
+          field_offset *= surface->config.size.h / 2;
+
+     mbes->regs.besA1ORG = lock->offset +
                            pitch * (source.y + (croptop >> 16));
      mbes->regs.besA2ORG = mbes->regs.besA1ORG +
-                           buffer->video.pitch;
+                           field_offset;
 
-     switch (surface->format) {
+     switch (surface->config.format) {
           case DSPF_NV12:
           case DSPF_NV21:
-               mbes->regs.besA1CORG  = buffer->video.offset +
-                                       surface->height * buffer->video.pitch +
+               field_offset = lock->pitch;
+               if (surface->config.caps & DSCAPS_SEPARATED)
+                    field_offset *= surface->config.size.h / 4;
+
+               mbes->regs.besA1CORG  = lock->offset +
+                                       surface->config.size.h * lock->pitch +
                                        pitch * (source.y/2 + (croptop_uv >> 16));
                mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
-                                       buffer->video.pitch;
+                                       field_offset;
                break;
 
           case DSPF_I420:
           case DSPF_YV12:
-               mbes->regs.besA1CORG  = buffer->video.offset +
-                                       surface->height * buffer->video.pitch +
+               field_offset = lock->pitch / 2;
+               if (surface->config.caps & DSCAPS_SEPARATED)
+                    field_offset *= surface->config.size.h / 4;
+
+               mbes->regs.besA1CORG  = lock->offset +
+                                       surface->config.size.h * lock->pitch +
                                        pitch/2 * (source.y/2 + (croptop_uv >> 16));
                mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
-                                       buffer->video.pitch/2;
+                                       field_offset;
 
                mbes->regs.besA1C3ORG = mbes->regs.besA1CORG +
-                                       surface->height/2 * buffer->video.pitch/2;
+                                       surface->config.size.h/2 * lock->pitch/2;
                mbes->regs.besA2C3ORG = mbes->regs.besA1C3ORG +
-                                       buffer->video.pitch/2;
+                                       field_offset;
                break;
 
           default:
@@ -623,7 +643,7 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
 
      mbes->regs.besHSRCST   = (source.x << 16) + cropleft;
      mbes->regs.besHSRCEND  = ((source.x + source.w - 1) << 16) - cropright;
-     mbes->regs.besHSRCLST  = (surface->width - 1) << 16;
+     mbes->regs.besHSRCLST  = (surface->config.size.w - 1) << 16;
 
      /* vertical starting weights */
      tmp = croptop & 0xfffc;
@@ -647,7 +667,7 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
           field_height - 1 - source.y - (croptop >> 16);
 
      /* horizontal scaling */
-     if (!mdev->g450_matrox && (surface->format == DSPF_RGB32 || surface->format == DSPF_ARGB)) {
+     if (!mdev->g450_matrox && (surface->config.format == DSPF_RGB32 || surface->config.format == DSPF_ARGB)) {
           mbes->regs.besHISCAL   = 0x20000 << hzoom;
           mbes->regs.besHSRCST  *= 2;
           mbes->regs.besHSRCEND *= 2;
