@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -60,14 +60,14 @@ static void crtc2_wait_vsync( MatroxDriverData *mdrv );
 /**************************************************************************************************/
 
 static int
-crtc2ScreenDataSize()
+crtc2ScreenDataSize( void )
 {
      return sizeof(MatroxCrtc2ScreenData);
 }
 
 static DFBResult
 crtc2InitScreen( CoreScreen           *screen,
-                 GraphicsDevice       *device,
+                 CoreGraphicsDevice   *device,
                  void                 *driver_data,
                  void                 *screen_data,
                  DFBScreenDescription *description )
@@ -249,17 +249,17 @@ crtc2GetScreenSize( CoreScreen *screen,
 }
 
 ScreenFuncs matroxCrtc2ScreenFuncs = {
-     ScreenDataSize:     crtc2ScreenDataSize,
-     InitScreen:         crtc2InitScreen,
-     InitEncoder:        crtc2InitEncoder,
-     InitOutput:         crtc2InitOutput,
-     SetPowerMode:       crtc2SetPowerMode,
-     WaitVSync:          crtc2WaitVSync,
-     TestEncoderConfig:  crtc2TestEncoderConfig,
-     SetEncoderConfig:   crtc2SetEncoderConfig,
-     TestOutputConfig:   crtc2TestOutputConfig,
-     SetOutputConfig:    crtc2SetOutputConfig,
-     GetScreenSize:      crtc2GetScreenSize
+     .ScreenDataSize    = crtc2ScreenDataSize,
+     .InitScreen        = crtc2InitScreen,
+     .InitEncoder       = crtc2InitEncoder,
+     .InitOutput        = crtc2InitOutput,
+     .SetPowerMode      = crtc2SetPowerMode,
+     .WaitVSync         = crtc2WaitVSync,
+     .TestEncoderConfig = crtc2TestEncoderConfig,
+     .SetEncoderConfig  = crtc2SetEncoderConfig,
+     .TestOutputConfig  = crtc2TestOutputConfig,
+     .SetOutputConfig   = crtc2SetOutputConfig,
+     .GetScreenSize     = crtc2GetScreenSize,
 };
 
 /**************************************************************************************************/
