@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -122,6 +122,7 @@ typedef struct {
      u32                     src_height;
      bool                    src_system;
      bool                    src_interlaced;
+     CoreSurfaceBufferLock  *src_lock;
 
      DFBRectangle            clip;
 
@@ -131,6 +132,8 @@ typedef struct {
      DFBSurfaceDrawingFlags  drawingflags;
      DFBSurfaceBlittingFlags blittingflags;
      
+     const s32              *matrix;
+     
      /* NVRectangle/NVTriangle/NVLine registers */
      u32                     drawing_operation; // SetOperation
      
@@ -152,7 +155,7 @@ typedef struct {
      /* 3D stuff */
      bool                    enabled_3d;       // 3d engine enabled
      u32                     buf_offset[2];    // reserved buffers
-     SurfaceBuffer          *src_texture;      // current source for TextureTriangles
+     CoreSurfaceBuffer      *src_texture;      // current source for TextureTriangles
      u32                     max_texture_size;
      
      struct {
@@ -211,7 +214,7 @@ enum {
 };
 
 typedef struct {
-     GraphicsDevice         *device;
+     CoreGraphicsDevice     *device;
      NVidiaDeviceData       *device_data;
 
      volatile void          *fb_base;
