@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -31,7 +31,7 @@
 #include <math.h>
 
 #include <core/coredefs.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/gfxcard.h>
 
 #include <core/layers.h>
@@ -51,19 +51,27 @@
 
 
 typedef struct {
-     CoreLayerRegionConfig config;
-     CoreSurface*          videoSurface;
-     short                 brightness;
-     short                 contrast;
-     short                 hue;
-     short                 saturation;
-     int                   field;
+     CoreLayerRegionConfig  config;
+     CoreSurface           *videoSurface;
+     CoreSurfaceBufferLock *lock;
+     
+     short                  brightness;
+     short                  contrast;
+     short                  hue;
+     short                  saturation;
+     int                    field;
 
      struct {
           u32 BUFFER;
           u32 STOP;
+          u32 UVBASE_0;
+          u32 UVBASE_1;
+          u32 UVOFFSET_0;
+          u32 UVOFFSET_1;
           u32 BASE_0;
           u32 BASE_1;
+          u32 OFFSET_0;
+          u32 OFFSET_1;
           u32 SIZE_IN_0;
           u32 SIZE_IN_1;
           u32 POINT_IN_0;
@@ -102,7 +110,7 @@ static void ov0_set_csc     ( NVidiaDriverData       *nvdrv,
 
 
 static int
-ov0LayerDataSize()
+ov0LayerDataSize( void )
 {
      return sizeof(NVidiaOverlayLayerData);
 }
@@ -182,7 +190,9 @@ ov0TestRegion(CoreLayer                  *layer,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags *failed )
 {
-     CoreLayerRegionConfigFlags fail = 0;
+     NVidiaDriverData           *nvdrv = driver_data;
+     NVidiaDeviceData           *nvdev = nvdrv->device_data;
+     CoreLayerRegionConfigFlags  fail  = CLRCF_NONE;
 
 
      /* check for unsupported options */
@@ -190,15 +200,28 @@ ov0TestRegion(CoreLayer                  *layer,
           fail |= CLRCF_OPTIONS;
 
      /* check buffermode */
-     if (config->buffermode == DLBM_WINDOWS)
-          fail |= CLRCF_BUFFERMODE;
+     switch (config->buffermode) {
+          case DLBM_FRONTONLY:
+          case DLBM_BACKSYSTEM:
+          case DLBM_BACKVIDEO:
+          case DLBM_TRIPLE:
+               break;
+
+          default:
+               fail |= CLRCF_BUFFERMODE;
+               break;
+     }
 
      /* check pixel format */
-     switch (config->format) {
+     switch (config->format) { 
           case DSPF_YUY2:
           case DSPF_UYVY:
-          case DSPF_I420:
-          case DSPF_YV12:
+               break;
+
+          case DSPF_NV12:
+          /*case DSPF_NV21:*/
+               if (nvdev->arch < NV_ARCH_30)
+                    fail |= CLRCF_FORMAT;
                break;
 
           default:
@@ -233,13 +256,17 @@ ov0SetRegion( CoreLayer                  *layer,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags  updated,
               CoreSurface                *surface,
-              CorePalette                *palette )
+              CorePalette                *palette,
+              CoreSurfaceBufferLock      *lock )
 {
      NVidiaDriverData       *nvdrv = (NVidiaDriverData*) driver_data;
      NVidiaOverlayLayerData *nvov0 = (NVidiaOverlayLayerData*) layer_data;
      
      /* remember configuration */
      nvov0->config = *config;
+     
+     nvov0->videoSurface = surface;
+     nvov0->lock = lock;
 
      /* set configuration */
      if (updated & (CLRCF_WIDTH  | CLRCF_HEIGHT | CLRCF_FORMAT  |
@@ -257,289 +284,21 @@ ov0SetRegion( CoreLayer                  *layer,
 }
 
 static DFBResult
-ov0AllocateSurface( CoreLayer              *layer,
-                    void                   *driver_data,
-                    void                   *layer_data,
-                    void                   *region_data,
-                    CoreLayerRegionConfig  *config,
-                    CoreSurface           **surface )
-{
-     NVidiaOverlayLayerData *nvov0        = (NVidiaOverlayLayerData*) layer_data;
-     DFBSurfaceCapabilities  caps         = DSCAPS_VIDEOONLY;
-     CoreSurfacePolicy       front_policy = CSP_VIDEOONLY;
-     CoreSurfacePolicy       back_policy  = CSP_VIDEOONLY;
-     DFBResult               result;
-      
-     switch (config->buffermode) {
-          case DLBM_FRONTONLY:
-               break;
-               
-          case DLBM_BACKSYSTEM:
-               back_policy = CSP_SYSTEMONLY;
-               /* fall through */
-          case DLBM_BACKVIDEO:
-               caps |= DSCAPS_DOUBLE;
-               break;
-
-          case DLBM_TRIPLE:
-               caps |= DSCAPS_TRIPLE;
-               break;
-
-          default:
-               D_BUG("unknown buffermode");
-               return DFB_BUG;
-     }
-
-     switch (config->format) {
-          case DSPF_YUY2:
-          case DSPF_UYVY:
-               break;
-               
-          case DSPF_I420:
-          case DSPF_YV12:
-               if (config->buffermode == DLBM_BACKSYSTEM)
-                    caps &= ~DSCAPS_DOUBLE;
-               break;
-               
-          default:
-               D_BUG( "unexpected pixelformat" );
-               return DFB_BUG;
-     }
-
-     if (DFB_PLANAR_PIXELFORMAT( config->format )) {
-          result = dfb_surface_create( layer->core, 
-                                       config->width, config->height,
-                                       DSPF_YUY2, CSP_VIDEOONLY, caps,
-                                       NULL, &nvov0->videoSurface );
-          
-          if (result == DFB_OK) {
-               result = dfb_surface_create( layer->core,
-                                            config->width, config->height,
-                                            config->format, CSP_SYSTEMONLY,
-                                            DSCAPS_SYSTEMONLY, NULL, surface );
-          }
-     } else {
-          result = dfb_surface_create( layer->core,
-                                       config->width, config->height,
-                                       config->format, CSP_VIDEOONLY, caps, 
-                                       NULL, surface );
-          
-          if (result == DFB_OK) {
-               dfb_surface_ref( *surface );
-               nvov0->videoSurface = *surface;
-
-               if (front_policy != back_policy)
-                    result = dfb_surface_reconfig( *surface,
-                                                   front_policy, back_policy );
-          }
-     }
-
-     if (config->options & DLOP_DEINTERLACING)
-          (*surface)->caps |= DSCAPS_INTERLACED;
-          
-     return result;
-}
-
-static DFBResult
-ov0ReallocateSurface( CoreLayer             *layer,
-                      void                  *driver_data,
-                      void                  *layer_data,
-                      void                  *region_data,
-                      CoreLayerRegionConfig *config,
-                      CoreSurface           *surface )
-{
-     NVidiaOverlayLayerData *nvov0        = (NVidiaOverlayLayerData*) layer_data;
-     DFBSurfaceCapabilities  caps         = DSCAPS_VIDEOONLY; 
-     CoreSurfacePolicy       front_policy = CSP_VIDEOONLY;
-     CoreSurfacePolicy       back_policy  = CSP_VIDEOONLY;
-     DFBResult               result;
-
-     switch (config->buffermode) {
-          case DLBM_FRONTONLY:
-               break;
-               
-          case DLBM_BACKSYSTEM:
-               back_policy = CSP_SYSTEMONLY;
-               /* fall through */
-          case DLBM_BACKVIDEO:
-               caps |=  DSCAPS_DOUBLE;
-               break;
-
-          case DLBM_TRIPLE:
-               caps |=  DSCAPS_TRIPLE;
-               break;
-
-          default:
-               D_BUG("unknown buffermode");
-               return DFB_BUG;
-     }
-     
-     switch (config->format) {
-          case DSPF_YUY2:
-          case DSPF_UYVY:
-               surface->caps = caps;
-               break;
-               
-          case DSPF_I420:
-          case DSPF_YV12:
-               front_policy  = CSP_SYSTEMONLY;
-               back_policy   = CSP_SYSTEMONLY;
-               surface->caps = DSCAPS_SYSTEMONLY;
-               if (config->buffermode == DLBM_BACKSYSTEM)
-                    caps &= ~DSCAPS_DOUBLE;
-               break;
-
-          default:
-               D_BUG("unexpected pixelformat");
-               return DFB_BUG;
-     }
-
-     dfb_surface_unref( nvov0->videoSurface );
-     nvov0->videoSurface = NULL;
-
-     result = dfb_surface_reconfig( surface, front_policy, back_policy );
-     if (result == DFB_OK)     
-          result = dfb_surface_reformat( layer->core, surface, config->width,
-                                         config->height, config->format );
-     if (result != DFB_OK)
-          return result;
-
-     if (DFB_PLANAR_PIXELFORMAT( config->format )) {
-          result = dfb_surface_create( layer->core,
-                                       config->width, config->height,
-                                       DSPF_YUY2, CSP_VIDEOONLY, caps,
-                                       NULL, &nvov0->videoSurface );
-     } else {
-          dfb_surface_ref( surface );
-          nvov0->videoSurface = surface;
-     }
-
-     if (config->options & DLOP_DEINTERLACING)
-          surface->caps |= DSCAPS_INTERLACED;
-     
-     return result;
-}
-
-static DFBResult
-ov0DeallocateSurface( CoreLayer   *layer,
-                      void        *driver_data,
-                      void        *layer_data,
-                      void        *region_data,
-                      CoreSurface *surface )
-{
-     NVidiaOverlayLayerData *nvov0 = (NVidiaOverlayLayerData*) layer_data;
-     
-     if (nvov0->videoSurface)
-          dfb_surface_unref( nvov0->videoSurface );
-     dfb_surface_unref( surface );
-     
-     return DFB_OK;
-}
-
-static void
-ov0CopyData420
-(
-     u8 *src1,
-     u8 *src2,
-     u8 *src3,
-     u8 *dst1,
-     int   srcPitch,
-     int   srcPitch2,
-     int   dstPitch,
-     int   h,
-     int   w
-)
-{
-     u32 *dst;
-     u8 *s1, *s2, *s3;
-     int i, j;
-
-     w >>= 1;
-
-     for(j = 0; j < h; j++) {
-          dst = (u32 *)dst1;
-          s1 = src1;  s2 = src2;  s3 = src3;
-          i = w;
-          while(i > 4) {
-#ifdef WORDS_BIGENDIAN
-               dst[0] = (s1[0] << 24) | (s1[1] << 8) | (s3[0] << 16) | s2[0];
-               dst[1] = (s1[2] << 24) | (s1[3] << 8) | (s3[1] << 16) | s2[1];
-               dst[2] = (s1[4] << 24) | (s1[5] << 8) | (s3[2] << 16) | s2[2];
-               dst[3] = (s1[6] << 24) | (s1[7] << 8) | (s3[3] << 16) | s2[3];
-#else
-               dst[0] = s1[0] | (s1[1] << 16) | (s3[0] << 8) | (s2[0] << 24);
-               dst[1] = s1[2] | (s1[3] << 16) | (s3[1] << 8) | (s2[1] << 24);
-               dst[2] = s1[4] | (s1[5] << 16) | (s3[2] << 8) | (s2[2] << 24);
-               dst[3] = s1[6] | (s1[7] << 16) | (s3[3] << 8) | (s2[3] << 24);
-#endif
-               dst += 4; s2 += 4; s3 += 4; s1 += 8;
-               i -= 4;
-          }
-
-          while(i--) {
-#ifdef WORDS_BIGENDIAN
-               dst[0] = (s1[0] << 24) | (s1[1] << 8) | (s3[0] << 16) | s2[0];
-#else
-               dst[0] = s1[0] | (s1[1] << 16) | (s3[0] << 8) | (s2[0] << 24);
-#endif
-               dst++; s2++; s3++;
-               s1 += 2;
-          }
-
-          dst1 += dstPitch;
-          src1 += srcPitch;
-          if(j & 1) {
-               src2 += srcPitch2;
-               src3 += srcPitch2;
-          }
-     }
-}
-
-static DFBResult
-ov0FlipRegion ( CoreLayer           *layer,
-                void                *driver_data,
-                void                *layer_data,
-                void                *region_data,
-                CoreSurface         *surface,
-                DFBSurfaceFlipFlags  flags )
+ov0FlipRegion ( CoreLayer             *layer,
+                void                  *driver_data,
+                void                  *layer_data,
+                void                  *region_data,
+                CoreSurface           *surface,
+                DFBSurfaceFlipFlags    flags,
+                CoreSurfaceBufferLock *lock )
 {
      NVidiaDriverData       *nvdrv = (NVidiaDriverData*) driver_data;
      NVidiaOverlayLayerData *nvov0 = (NVidiaOverlayLayerData*) layer_data;
-     
-     dfb_surface_flip_buffers( nvov0->videoSurface, false );
+    
+     nvov0->videoSurface = surface;
+     nvov0->lock = lock;
 
-     if (DFB_PLANAR_PIXELFORMAT( surface->format )) {  
-          SurfaceBuffer *data_buffer;
-          SurfaceBuffer *video_buffer;
-          u32 srcPitch, srcPitch2, dstPitch, s2offset, s3offset, tmp;
-          u32 width  = surface->width;
-          u32 height = surface->height;
-          u8 *dstStart;
-          u8 *buf;
-          
-          data_buffer  = surface->front_buffer;
-          video_buffer = nvov0->videoSurface->front_buffer;
-
-          buf      = data_buffer->system.addr;
-          dstStart = dfb_gfxcard_memory_virtual( nvdrv->device, 
-                                                 video_buffer->video.offset);
-          
-          dstPitch  = video_buffer->video.pitch;
-          srcPitch  = data_buffer->system.pitch;
-          srcPitch2 = srcPitch >> 1;
-
-          s2offset = srcPitch * height;
-          s3offset = (srcPitch2 * (height >> 1)) + s2offset;
-          if (nvov0->config.format == DSPF_I420) {
-               tmp = s2offset;
-               s2offset = s3offset;
-               s3offset = tmp;
-          }
-          
-          ov0CopyData420( buf, buf + s2offset,
-                          buf + s3offset, dstStart, srcPitch, srcPitch2,
-                          dstPitch, height, width );
-     }
+     dfb_surface_flip( nvov0->videoSurface, false );
 
      ov0_calc_regs( nvdrv, nvov0, &nvov0->config, CLRCF_SURFACE );
      ov0_set_regs( nvdrv, nvov0, CLRCF_SURFACE );
@@ -550,53 +309,6 @@ ov0FlipRegion ( CoreLayer           *layer,
      return DFB_OK;
 }
 
-static DFBResult
-ov0UpdateRegion ( CoreLayer           *layer,
-                  void                *driver_data,
-                  void                *layer_data,
-                  void                *region_data,
-                  CoreSurface         *surface,
-                  const DFBRegion     *update )
-{ 
-     NVidiaDriverData       *nvdrv = (NVidiaDriverData*) driver_data;
-     NVidiaOverlayLayerData *nvov0 = (NVidiaOverlayLayerData*) layer_data;
-
-     if (DFB_PLANAR_PIXELFORMAT( surface->format )) {  
-          SurfaceBuffer *data_buffer;
-          SurfaceBuffer *video_buffer;
-          u32 srcPitch, srcPitch2, dstPitch, s2offset, s3offset, tmp;
-          u32 width  = surface->width;
-          u32 height = surface->height;
-          u8 *dstStart;
-          u8 *buf;
-          
-          data_buffer  = surface->front_buffer;
-          video_buffer = nvov0->videoSurface->front_buffer;
-
-          buf      = data_buffer->system.addr;
-          dstStart = dfb_gfxcard_memory_virtual( nvdrv->device,
-                                                 video_buffer->video.offset );
-          
-          dstPitch  = video_buffer->video.pitch;
-          srcPitch  = data_buffer->system.pitch;
-          srcPitch2 = srcPitch >> 1;
-
-          s2offset = srcPitch * height;
-          s3offset = (srcPitch2 * (height >> 1)) + s2offset;
-          if(nvov0->config.format == DSPF_I420) {
-               tmp = s2offset;
-               s2offset = s3offset;
-               s3offset = tmp;
-          }
-          
-          ov0CopyData420( buf, buf + s2offset,
-                          buf + s3offset, dstStart, srcPitch, srcPitch2,
-                          dstPitch, height, width );
-     }
-     
-     return DFB_OK;
-}
-
 static DFBResult
 ov0SetColorAdjustment( CoreLayer          *layer,
                        void               *driver_data,
@@ -656,12 +368,8 @@ DisplayLayerFuncs nvidiaOverlayFuncs = {
      .RemoveRegion       = ov0Remove,
      .TestRegion         = ov0TestRegion,
      .FlipRegion         = ov0FlipRegion,
-     .UpdateRegion       = ov0UpdateRegion,
      .SetColorAdjustment = ov0SetColorAdjustment,
      .SetInputField      = ov0SetInputField,
-     .AllocateSurface    = ov0AllocateSurface,
-     .DeallocateSurface  = ov0DeallocateSurface,
-     .ReallocateSurface  = ov0ReallocateSurface
 };
 
 
@@ -674,8 +382,16 @@ static void ov0_set_regs( NVidiaDriverData           *nvdrv,
      volatile u8 *mmio = nvdrv->mmio_base;
      
      if (flags & CLRCF_SURFACE) {
+          if (DFB_PLANAR_PIXELFORMAT(nvov0->config.format)) { 
+               nv_out32( mmio, PVIDEO_UVBASE_0, nvov0->regs.UVBASE_0 );
+               nv_out32( mmio, PVIDEO_UVBASE_1, nvov0->regs.UVBASE_1 );
+               nv_out32( mmio, PVIDEO_UVOFFSET_0, nvov0->regs.UVOFFSET_0 );
+               nv_out32( mmio, PVIDEO_UVOFFSET_1, nvov0->regs.UVOFFSET_1 );
+          }
           nv_out32( mmio, PVIDEO_BASE_0, nvov0->regs.BASE_0 );
           nv_out32( mmio, PVIDEO_BASE_1, nvov0->regs.BASE_1 );
+          nv_out32( mmio, PVIDEO_OFFSET_0, nvov0->regs.OFFSET_0 );
+          nv_out32( mmio, PVIDEO_OFFSET_1, nvov0->regs.OFFSET_1 );
      }
      if (flags & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_OPTIONS)) {
           nv_out32( mmio, PVIDEO_SIZE_IN_0, nvov0->regs.SIZE_IN_0 );
@@ -766,14 +482,17 @@ ov0_calc_regs( NVidiaDriverData           *nvdrv,
      }
      
      if (flags & (CLRCF_SURFACE | CLRCF_FORMAT | CLRCF_OPTIONS)) {
-          SurfaceBuffer *buffer = nvov0->videoSurface->front_buffer;
-          u32            format;
+          CoreSurfaceBufferLock *lock = nvov0->lock;
+          u32                    format;
           
           if (config->options & DLOP_DEINTERLACING)
-               format = (buffer->video.pitch*2) & PVIDEO_FORMAT_PITCH_MSK;
+               format = (lock->pitch*2) & PVIDEO_FORMAT_PITCH_MSK;
           else
-               format =  buffer->video.pitch    & PVIDEO_FORMAT_PITCH_MSK;
-     
+               format =  lock->pitch    & PVIDEO_FORMAT_PITCH_MSK;
+               
+          if (DFB_PLANAR_PIXELFORMAT(config->format))
+               format |= PVIDEO_FORMAT_PLANAR_NV;
+    
           if (config->format == DSPF_UYVY)
                format |= PVIDEO_FORMAT_COLOR_YB8CR8YA8CB8;
           else
@@ -783,11 +502,16 @@ ov0_calc_regs( NVidiaDriverData           *nvdrv,
                format |= PVIDEO_FORMAT_DISPLAY_COLOR_KEY_EQUAL;
                
           /* Use Buffer 0 for Odd field */
-          nvov0->regs.BASE_0   = (nvdev->fb_offset +
-                                  buffer->video.offset) & PVIDEO_BASE_MSK;
+          nvov0->regs.OFFSET_0   = (nvdev->fb_offset + lock->offset) & PVIDEO_OFFSET_MSK;
           /* Use Buffer 1 for Even field */
-          nvov0->regs.BASE_1   = nvov0->regs.BASE_0 + buffer->video.pitch;
-          nvov0->regs.FORMAT_0 =
+          nvov0->regs.OFFSET_1   = nvov0->regs.OFFSET_0 + lock->pitch;
+          if (DFB_PLANAR_PIXELFORMAT(config->format)) {
+               CoreSurface *surface = nvov0->videoSurface;
+               nvov0->regs.UVOFFSET_0 = (nvov0->regs.OFFSET_0 + 
+                                         lock->pitch * surface->config.size.h) & PVIDEO_UVOFFSET_MSK;
+               nvov0->regs.UVOFFSET_1 = nvov0->regs.UVOFFSET_0 + lock->pitch;
+          }
+          nvov0->regs.FORMAT_0 = 
           nvov0->regs.FORMAT_1 = format;
      }
  
