@@ -26,7 +26,7 @@
 #include <directfb.h>
 
 #include <core/coredefs.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/gfxcard.h>
 #include <core/system.h>
 #include <core/screen.h>
@@ -45,7 +45,7 @@
 
 static DFBResult
 crtc1InitScreen( CoreScreen           *screen,
-                 GraphicsDevice       *device,
+                 CoreGraphicsDevice   *device,
                  void                 *driver_data,
                  void                 *screen_data,
                  DFBScreenDescription *description )
@@ -98,7 +98,7 @@ crtc1WaitVSync( CoreScreen *screen,
                
                i++;
                if ((i % 2000) == 0) {
-                    struct timespec ts = {0,0}; 
+                    struct timespec ts = { 0, 10000 }; 
                     nanosleep( &ts, NULL );
                }
           }
@@ -157,28 +157,26 @@ void        *OldPrimaryScreenDriverData;
 /*************************** Primary Layer hooks ******************************/
 
 static DFBResult
-fb0FlipRegion( CoreLayer           *layer,
-               void                *driver_data,
-               void                *layer_data,
-               void                *region_data,
-               CoreSurface         *surface,
-               DFBSurfaceFlipFlags  flags )
+fb0FlipRegion( CoreLayer             *layer,
+               void                  *driver_data,
+               void                  *layer_data,
+               void                  *region_data,
+               CoreSurface           *surface,
+               DFBSurfaceFlipFlags    flags,
+               CoreSurfaceBufferLock *lock )
 {
      NVidiaDriverData *nvdrv  = (NVidiaDriverData*) driver_data;
      NVidiaDeviceData *nvdev  = nvdrv->device_data;
-     SurfaceBuffer    *buffer = surface->back_buffer;
      u32               offset;
 
-     dfb_gfxcard_sync();
+     dfb_surface_flip( surface, false );
      
-     offset = (buffer->video.offset + nvdev->fb_offset) & ~3;
+     offset = (lock->offset + nvdev->fb_offset) & ~3;
      nv_out32( nvdrv->mmio_base, PCRTC_START, offset );
 
      if (flags & DSFLIP_WAIT)
           dfb_layer_wait_vsync( layer );
 
-     dfb_surface_flip_buffers( surface, false );
-
      return DFB_OK;
 }
 
