@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -47,7 +47,7 @@
 #include <core/coretypes.h>
 
 #include <core/state.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/gfxcard.h>
 
 #include <gfx/convert.h>
@@ -105,25 +105,23 @@ static inline void savage4_validate_gbd( Savage4DriverData *sdrv,
 {
      u32 BitmapDescriptor;
      CoreSurface *destination;
-     SurfaceBuffer *buffer;
      int bpp;
 
      if (sdev->v_gbd)
           return;
 
      destination = state->destination;
-     buffer = destination->back_buffer;
-     bpp = DFB_BYTES_PER_PIXEL(destination->format);
+     bpp = DFB_BYTES_PER_PIXEL(destination->config.format);
      
      BitmapDescriptor = BCI_BD_BW_DISABLE | 8 | 1;
      BCI_BD_SET_BPP( BitmapDescriptor, bpp * 8 );
-     BCI_BD_SET_STRIDE( BitmapDescriptor, buffer->video.pitch / bpp );
+     BCI_BD_SET_STRIDE( BitmapDescriptor, state->dst.pitch / bpp );
 
      /* strange effects if we don't wait here for the Savage4 being idle */
      savage4_waitidle( sdrv, sdev );
 
      BCI_SEND( BCI_CMD_SETREG | (1 << 16) | BCI_GBD1 );
-     BCI_SEND( buffer->video.offset );
+     BCI_SEND( state->dst.offset );
 
      BCI_SEND( BCI_CMD_SETREG | (1 << 16) | BCI_GBD2 );
      BCI_SEND( BitmapDescriptor );
@@ -137,25 +135,23 @@ static inline void savage4_validate_pbd( Savage4DriverData *sdrv,
 {
      u32 BitmapDescriptor;
      CoreSurface *source;
-     SurfaceBuffer *buffer;
      int bpp;
 
      if (sdev->v_pbd)
           return;
 
      source = state->source;
-     buffer = source->front_buffer;
-     bpp = DFB_BYTES_PER_PIXEL(source->format);
+     bpp = DFB_BYTES_PER_PIXEL(source->config.format);
      
      BitmapDescriptor = BCI_BD_BW_DISABLE;
      BCI_BD_SET_BPP( BitmapDescriptor, bpp * 8 );
-     BCI_BD_SET_STRIDE( BitmapDescriptor, buffer->video.pitch / bpp );
+     BCI_BD_SET_STRIDE( BitmapDescriptor, state->src.pitch / bpp );
 
      
      savage4_waitidle( sdrv, sdev );
 
      BCI_SEND( BCI_CMD_SETREG | (1 << 16) | BCI_PBD1 );
-     BCI_SEND( buffer->video.offset );
+     BCI_SEND( state->src.offset );
 
      BCI_SEND( BCI_CMD_SETREG | (1 << 16) | BCI_PBD2 );
      BCI_SEND( BitmapDescriptor );
@@ -169,7 +165,7 @@ static inline void savage4_validate_color( Savage4DeviceData *sdev,
      if (sdev->v_color)
           return;
      
-     switch (state->destination->format) {
+     switch (state->destination->config.format) {
           case DSPF_A8:
                sdev->Fill_Color = state->color.a;
                break;
@@ -226,7 +222,7 @@ static void savage4CheckState( void *drv, void *dev,
                                CardState *state, DFBAccelerationMask accel )
 {
      SVGDBG("savage4checkstate\n");
-     switch (state->destination->format) {
+     switch (state->destination->config.format) {
           case DSPF_ARGB1555:
           case DSPF_RGB16:
           case DSPF_RGB32:
@@ -244,7 +240,7 @@ static void savage4CheckState( void *drv, void *dev,
           state->accel |= SAVAGE4_DRAWING_FUNCTIONS;
      }
      else {
-          if (state->source->format != state->destination->format)
+          if (state->source->config.format != state->destination->config.format)
                return;
 
           if (state->blittingflags & ~SAVAGE4_BLITTING_FLAGS)
@@ -262,13 +258,13 @@ static void savage4SetState( void *drv, void *dev,
      Savage4DeviceData *sdev = (Savage4DeviceData*) dev;
      
      SVGDBG("savage4setstate\n");
-     if (state->modified) {
-          if (state->modified & SMF_DESTINATION)
+     if (state->mod_hw) {
+          if (state->mod_hw & SMF_DESTINATION)
                sdev->v_gbd = sdev->v_color = 0;
-          else if (state->modified & SMF_COLOR)
+          else if (state->mod_hw & SMF_COLOR)
                sdev->v_color = 0;
 
-          if (state->modified & SMF_SOURCE)
+          if (state->mod_hw & SMF_SOURCE)
                sdev->v_pbd = 0;
      }
      
@@ -296,7 +292,7 @@ static void savage4SetState( void *drv, void *dev,
                return;
      }
 
-     if (state->modified & SMF_BLITTING_FLAGS) {
+     if (state->mod_hw & SMF_BLITTING_FLAGS) {
           if (state->blittingflags & DSBLIT_SRC_COLORKEY)
                sdev->Cmd_Src_Transparent = BCI_CMD_SRC_TRANSPARENT |
                                            BCI_CMD_SEND_COLOR;
@@ -304,13 +300,13 @@ static void savage4SetState( void *drv, void *dev,
                sdev->Cmd_Src_Transparent = 0;
      }
 
-     if (state->modified & SMF_CLIP)
+     if (state->mod_hw & SMF_CLIP)
           savage4_set_clip( sdrv, sdev, &state->clip );
      
-     if (state->modified & SMF_SRC_COLORKEY)
+     if (state->mod_hw & SMF_SRC_COLORKEY)
           sdev->src_colorkey = state->src_colorkey;
 
-     state->modified = 0;
+     state->mod_hw = 0;
 }
 
 
@@ -487,7 +483,7 @@ static void savage4AfterSetVar( void *drv, void *dev )
 /* exported symbols */
 
 void
-savage4_get_info( GraphicsDevice     *device,
+savage4_get_info( CoreGraphicsDevice *device,
                   GraphicsDriverInfo *info )
 {
      SVGDBG("savage4getinfo\n");
@@ -499,7 +495,7 @@ savage4_get_info( GraphicsDevice     *device,
 }
 
 DFBResult
-savage4_init_driver( GraphicsDevice      *device,
+savage4_init_driver( CoreGraphicsDevice  *device,
                      GraphicsDeviceFuncs *funcs,
                      void                *driver_data )
 {
@@ -528,7 +524,7 @@ savage4_init_driver( GraphicsDevice      *device,
 }
 
 DFBResult
-savage4_init_device( GraphicsDevice     *device,
+savage4_init_device( CoreGraphicsDevice *device,
                      GraphicsDeviceInfo *device_info,
                      void               *driver_data,
                      void               *device_data )
@@ -587,16 +583,16 @@ savage4_init_device( GraphicsDevice     *device,
 }
 
 void
-savage4_close_device( GraphicsDevice *device,
-                      void           *driver_data,
-                      void           *device_data )
+savage4_close_device( CoreGraphicsDevice *device,
+                      void               *driver_data,
+                      void               *device_data )
 {
      SVGDBG("savage4closedevice\n");
 }
 
 void
-savage4_close_driver( GraphicsDevice *device,
-                      void           *driver_data )
+savage4_close_driver( CoreGraphicsDevice *device,
+                      void               *driver_data )
 {
      SVGDBG("savage4closedriver\n");
 }
