@@ -7,6 +7,8 @@
    version 2 of the License, or (at your option) any later version.
 */
 
+#include <config.h>
+
 #include <direct/messages.h>
 
 #include <gfx/convert.h>
@@ -94,8 +96,8 @@ void uc_flush_texture_cache(void* drv, void* dev)
  * @note This is actually a 1-pixel high or wide rectangular color fill.
  */
 
-inline void uc_draw_hv_line(struct uc_fifo* fifo,
-                            int x, int y, int len, int hv, int rop)
+static inline void uc_draw_hv_line(struct uc_fifo* fifo,
+                                   int x, int y, int len, int hv, int rop)
 {
     UC_FIFO_ADD_2D(fifo, VIA_REG_DSTPOS, ((RS16(y) << 16) | RS16(x)));
     UC_FIFO_ADD_2D(fifo, VIA_REG_DIMENSION, len << (hv ? 16 : 0));
@@ -256,8 +258,8 @@ static bool uc_blit_planar(void* drv, void* dev, DFBRectangle* rect, int dx, int
 {
     UC_ACCEL_BEGIN()
 
-    int uv_dst_offset = ucdev->dst_offset + (ucdev->dst_pitch * rect->h);
-    int uv_src_offset = ucdev->src_offset + (ucdev->src_pitch * rect->h);
+    int uv_dst_offset = ucdev->dst_offset + (ucdev->dst_pitch * ucdev->dst_height);
+    int uv_src_offset = ucdev->src_offset + (ucdev->src_pitch * ucdev->src_height);
     
     int uv_dst_pitch = ucdev->dst_pitch / 2;
     int uv_src_pitch = ucdev->src_pitch / 2;
@@ -290,8 +292,8 @@ static bool uc_blit_planar(void* drv, void* dev, DFBRectangle* rect, int dx, int
     
     // now for the second chrominance plane
     
-    uv_src_offset += uv_src_pitch * rect2.h;
-    uv_dst_offset += uv_dst_pitch * rect2.h;
+    uv_src_offset += uv_src_pitch * ucdev->src_height/2;
+    uv_dst_offset += uv_dst_pitch * ucdev->dst_height/2;
     
     UC_FIFO_PREPARE ( fifo, 6 );
     UC_FIFO_ADD_HDR( fifo, HC_ParaType_NotTex << 16 );
@@ -440,10 +442,6 @@ bool uc_fill_triangle(void* drv, void* dev, DFBTriangle* tri)
 bool uc_blit_3d(void* drv, void* dev,
                 DFBRectangle* rect, int dx, int dy)
 {
-    // TODO: Write separate blit function to save some overhead.
-
-    // Hmm, I don't think we can save anything beyond a few CPU cycles. -- dok
-
     DFBRectangle dest = {dx, dy, rect->w, rect->h};
     return uc_stretch_blit(drv, dev, rect, &dest);
 }
