@@ -1,10 +1,13 @@
 #ifndef __DIRECTFB_KEYNAMES_H__
 #define __DIRECTFB_KEYNAMES_H__
 
-#define DirectFBKeySymbolNames(Identifier) struct DFBKeySymbolName { \
-     DFBInputDeviceKeySymbol symbol; \
-     const char *name; \
-} Identifier[] = { \
+
+struct DFBKeySymbolName {
+     DFBInputDeviceKeySymbol symbol;
+     const char *name;
+};
+
+#define DirectFBKeySymbolNames(Identifier) struct DFBKeySymbolName Identifier[] = { \
      { DIKS_BACKSPACE, "BACKSPACE" }, \
      { DIKS_TAB, "TAB" }, \
      { DIKS_RETURN, "RETURN" }, \
@@ -351,10 +354,13 @@
      { DIKS_NULL, "NULL" } \
 };
 
-#define DirectFBKeyIdentifierNames(Identifier) struct DFBKeyIdentifierName { \
-     DFBInputDeviceKeyIdentifier identifier; \
-     const char *name; \
-} Identifier[] = { \
+
+struct DFBKeyIdentifierName {
+     DFBInputDeviceKeyIdentifier identifier;
+     const char *name;
+};
+
+#define DirectFBKeyIdentifierNames(Identifier) struct DFBKeyIdentifierName Identifier[] = { \
      { DIKI_A, "A" }, \
      { DIKI_B, "B" }, \
      { DIKI_C, "C" }, \
