@@ -25,6 +25,8 @@
 
  */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -321,6 +323,8 @@ static void *GunzeTouchEventThread(DirectThread *thread, void *driver_data)
             /* if whole packet collected, decode it */
             if (priv->gunBytes == priv->gunDlen) 
             {
+                DFBInputEvent evt;
+
                 priv->gunBytes = 0;
                 if (priv->gunDlen == GUNZE_SERIAL_DLEN) 
                 {
@@ -380,7 +384,6 @@ static void *GunzeTouchEventThread(DirectThread *thread, void *driver_data)
                     priv->flags &= ~FLAG_WAS_UP;
 
                 /* Now send events */
-                DFBInputEvent evt;
 
                 /* only post new x/y if different from previous tap */
                 if ( (priv->gunPrevX != x) || (priv->gunPrevY != y) )
@@ -500,6 +503,10 @@ static DFBResult driver_open_device(CoreInputDevice *device,
     }
 
     data = D_CALLOC (1, sizeof(GunzeTData));
+    if (!data) {
+         close (fd);
+         return D_OOM ();
+    }
 
     data->fd = fd;
     data->device = device;
