@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -134,7 +134,7 @@ joystickEventThread( DirectThread *thread, void *driver_data )
 /* exported symbols */
 
 static int
-driver_get_available()
+driver_get_available( void )
 {
      int  i, fd;
      int  joy_count = 0;
@@ -232,6 +232,10 @@ driver_open_device( CoreInputDevice      *device,
 
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(JoystickData) );
+     if (!data) {
+          close( fd );
+          return D_OOM();
+     }
 
      data->fd     = fd;
      data->device = device;
