@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -28,11 +28,6 @@
 
 #include <config.h>
 
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
-#include <fcntl.h>
-
 
 #include <linux/version.h>
 
@@ -41,6 +36,21 @@ typedef unsigned long kernel_ulong_t;
 #define BITS_PER_LONG    (sizeof(long)*8)
 #endif
 
+#include <linux/input.h>
+
+#ifndef KEY_OK
+/* Linux kernel 2.5.42+ defines additional keys in linux/input.h */
+#include "input_fake.h"
+#endif
+
+#ifndef EV_CNT
+#define EV_CNT (EV_MAX+1)
+#define KEY_CNT (KEY_MAX+1)
+#define REL_CNT (REL_MAX+1)
+#define ABS_CNT (ABS_MAX+1)
+#define LED_CNT (LED_MAX+1)
+#endif
+
 /* compat defines for older kernel like 2.4.x */
 #ifndef EV_SYN
 #define EV_SYN			0x00
@@ -51,12 +61,6 @@ typedef unsigned long kernel_ulong_t;
 #define BTN_TOOL_TRIPLETAP	0x14e
 #endif
 
-#include <linux/input.h>
-#ifndef KEY_OK
-/* Linux kernel 2.5.42+ defines additional keys in linux/input.h */
-#include "input_fake.h"
-#endif
-
 #ifndef EVIOCGLED
 #define EVIOCGLED(len) _IOC(_IOC_READ, 'E', 0x19, len)
 #endif
@@ -65,6 +69,14 @@ typedef unsigned long kernel_ulong_t;
 #define EVIOCGRAB _IOW('E', 0x90, int)
 #endif
 
+#include <linux/keyboard.h>
+
+
+#include <string.h>
+#include <unistd.h>
+#include <errno.h>
+#include <fcntl.h>
+
 #include <dfb_types.h>
 
 #include <sys/types.h>
@@ -73,8 +85,7 @@ typedef unsigned long kernel_ulong_t;
 #include <sys/kd.h>
 #include <stdlib.h>
 
-
-#include <linux/keyboard.h>
+#define DFB_INPUTDRIVER_HAS_AXIS_INFO
 
 #include <directfb.h>
 #include <directfb_keyboard.h>
@@ -115,11 +126,11 @@ DFB_INPUT_DRIVER( linux_input )
 /* compat for 2.4.x kernel - just a compile fix */
 #ifndef HAVE_INPUT_ABSINFO
 struct input_absinfo {
-        __s32 value;
-        __s32 minimum;
-        __s32 maximum;
-        __s32 fuzz;
-        __s32 flat;
+        s32 value;
+        s32 minimum;
+        s32 maximum;
+        s32 fuzz;
+        s32 flat;
 };
 #endif
 
@@ -134,7 +145,7 @@ typedef struct {
      int                      fd;
 
      bool                     has_leds;
-     unsigned long            led_state[NBITS(LED_MAX)];
+     unsigned long            led_state[NBITS(LED_CNT)];
      DFBInputDeviceLockState  locks;
 
 #ifdef LINUX_INPUT_USE_FBDEV
@@ -151,10 +162,10 @@ typedef struct {
 #define MAX_LINUX_INPUT_DEVICES 16
 
 static int num_devices = 0;
-static int device_nums[MAX_LINUX_INPUT_DEVICES];
+static char *device_names[MAX_LINUX_INPUT_DEVICES];
 
 
-static
+static const
 int basic_keycodes [] = {
      DIKI_UNKNOWN, DIKI_ESCAPE, DIKI_1, DIKI_2, DIKI_3, DIKI_4, DIKI_5,
      DIKI_6, DIKI_7, DIKI_8, DIKI_9, DIKI_0, DIKI_MINUS_SIGN,
@@ -250,13 +261,16 @@ int basic_keycodes [] = {
      /*KEY_KPLEFTPAREN,*/ DIKI_UNKNOWN,
      /*KEY_KPRIGHTPAREN,*/ DIKI_UNKNOWN,
 
-     DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
-     DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
+     /* unused codes 181-182: */
+     DIKI_UNKNOWN, DIKI_UNKNOWN,
 
-     DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
-     DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
+     DFB_FUNCTION_KEY(13), DFB_FUNCTION_KEY(14), DFB_FUNCTION_KEY(15),
+     DFB_FUNCTION_KEY(16), DFB_FUNCTION_KEY(17), DFB_FUNCTION_KEY(18),
+     DFB_FUNCTION_KEY(19), DFB_FUNCTION_KEY(20), DFB_FUNCTION_KEY(21),
+     DFB_FUNCTION_KEY(22), DFB_FUNCTION_KEY(23), DFB_FUNCTION_KEY(24),
 
-     DIKI_UNKNOWN,
+     /* unused codes 195-199: */
+     DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN, DIKI_UNKNOWN,
 
      /* KEY_PLAYCD, KEY_PAUSECD */
      DIKS_PLAY, DIKS_PAUSE,
@@ -299,7 +313,7 @@ int basic_keycodes [] = {
      /* KEY_MEDIA          */  DIKI_UNKNOWN,
 };
 
-static
+static const
 int ext_keycodes [] = {
      DIKS_OK, DIKS_SELECT, DIKS_GOTO, DIKS_CLEAR, DIKS_POWER2, DIKS_OPTION,
      DIKS_INFO, DIKS_TIME, DIKS_VENDOR, DIKS_ARCHIVE, DIKS_PROGRAM,
@@ -338,15 +352,15 @@ static void
 touchpad_fsm_init( struct touchpad_fsm_state *state );
 static int
 touchpad_fsm( struct touchpad_fsm_state *state,
-              struct input_event        *levt,
+              const struct input_event  *levt,
               DFBInputEvent             *devt );
 
-static int
-timeout_passed( struct timeval *timeout, struct timeval *current );
-static int
-timeout_is_set( struct timeval *timeout );
+static bool
+timeout_passed( const struct timeval *timeout, const struct timeval *current );
+static bool
+timeout_is_set( const struct timeval *timeout );
 static void
-timeout_sub( struct timeval *timeout, struct timeval *sub );
+timeout_sub( struct timeval *timeout, const struct timeval *sub );
 
 /*
  * Translates a Linux input keycode into a DirectFB keycode.
@@ -556,7 +570,7 @@ keyboard_get_identifier( int code, unsigned short value )
 }
 
 static unsigned short
-keyboard_read_value( LinuxInputData *data,
+keyboard_read_value( const LinuxInputData *data,
                      unsigned char table, unsigned char index )
 {
      struct kbentry entry;
@@ -578,28 +592,30 @@ keyboard_read_value( LinuxInputData *data,
 /*
  * Translates key and button events.
  */
-static int
-key_event( struct input_event *levt,
-           DFBInputEvent      *devt )
+static bool
+key_event( const struct input_event *levt,
+           DFBInputEvent            *devt )
 {
-     /* map touchscreen and smartpad events to button mouse */
-     if (levt->code == BTN_TOUCH || levt->code == BTN_TOOL_FINGER)
-          levt->code = BTN_MOUSE;
+     int code = levt->code;
 
-     if ((levt->code >= BTN_MOUSE && levt->code < BTN_JOYSTICK) || levt->code == BTN_TOUCH) {
+     /* map touchscreen and smartpad events to button mouse */
+     if (code == BTN_TOUCH || code == BTN_TOOL_FINGER)
+          code = BTN_MOUSE;
+
+     if ((code >= BTN_MOUSE && code < BTN_JOYSTICK) || code == BTN_TOUCH) {
           /* ignore repeat events for buttons */
           if (levt->value == 2)
-               return 0;
+               return false;
 
           devt->type   = levt->value ? DIET_BUTTONPRESS : DIET_BUTTONRELEASE;
           /* don't set DIEF_BUTTONS, it will be set by the input core */
-          devt->button = DIBI_FIRST + levt->code - BTN_MOUSE;
+          devt->button = DIBI_FIRST + code - BTN_MOUSE;
      }
      else {
-          int key = translate_key( levt->code );
+          int key = translate_key( code );
 
           if (key == DIKI_UNKNOWN)
-               return 0;
+               return false;
 
           devt->type = levt->value ? DIET_KEYPRESS : DIET_KEYRELEASE;
 
@@ -613,21 +629,21 @@ key_event( struct input_event *levt,
           }
 
           devt->flags |= DIEF_KEYCODE;
-          devt->key_code = levt->code;
+          devt->key_code = code;
      }
 
      if (levt->value == 2)
           devt->flags |= DIEF_REPEAT;
 
-     return 1;
+     return true;
 }
 
 /*
  * Translates relative axis events.
  */
-static int
-rel_event( struct input_event *levt,
-           DFBInputEvent      *devt )
+static bool
+rel_event( const struct input_event *levt,
+           DFBInputEvent            *devt )
 {
      switch (levt->code) {
           case REL_X:
@@ -648,7 +664,7 @@ rel_event( struct input_event *levt,
 
           default:
                if (levt->code > REL_MAX || levt->code > DIAI_LAST)
-                    return 0;
+                    return false;
                devt->axis = levt->code;
                devt->axisrel = levt->value;
      }
@@ -656,15 +672,15 @@ rel_event( struct input_event *levt,
      devt->type    = DIET_AXISMOTION;
      devt->flags  |= DIEF_AXISREL;
 
-     return 1;
+     return true;
 }
 
 /*
  * Translates absolute axis events.
  */
-static int
-abs_event( struct input_event *levt,
-           DFBInputEvent      *devt )
+static bool
+abs_event( const struct input_event *levt,
+           DFBInputEvent            *devt )
 {
      switch (levt->code) {
           case ABS_X:
@@ -682,7 +698,7 @@ abs_event( struct input_event *levt,
 
           default:
                if (levt->code >= ABS_PRESSURE || levt->code > DIAI_LAST)
-                    return 0;
+                    return false;
                devt->axis = levt->code;
      }
 
@@ -690,15 +706,15 @@ abs_event( struct input_event *levt,
      devt->flags  |= DIEF_AXISABS;
      devt->axisabs = levt->value;
 
-     return 1;
+     return true;
 }
 
 /*
  * Translates a Linux input event into a DirectFB input event.
  */
-static int
-translate_event( struct input_event *levt,
-                 DFBInputEvent      *devt )
+static bool
+translate_event( const struct input_event *levt,
+                 DFBInputEvent            *devt )
 {
      devt->flags     = DIEF_TIMESTAMP;
      devt->timestamp = levt->time;
@@ -717,11 +733,11 @@ translate_event( struct input_event *levt,
                ;
      }
 
-     return 0;
+     return false;
 }
 
 static void
-set_led( LinuxInputData *data, int led, int state )
+set_led( const LinuxInputData *data, int led, int state )
 {
      struct input_event levt;
 
@@ -733,9 +749,9 @@ set_led( LinuxInputData *data, int led, int state )
 }
 
 static void
-flush_xy( LinuxInputData *data )
+flush_xy( LinuxInputData *data, bool last )
 {
-     DFBInputEvent evt;
+     DFBInputEvent evt = { .type = DIET_UNKNOWN };
 
      if (data->dx) {
           evt.type    = DIET_AXISMOTION;
@@ -744,7 +760,7 @@ flush_xy( LinuxInputData *data )
           evt.axisrel = data->dx;
 
           /* Signal immediately following event. */
-          if (data->dy)
+          if (!last || data->dy)
                evt.flags |= DIEF_FOLLOW;
 
           dfb_input_dispatch( data->device, &evt );
@@ -758,6 +774,10 @@ flush_xy( LinuxInputData *data )
           evt.axis    = DIAI_Y;
           evt.axisrel = data->dy;
 
+          /* Signal immediately following event. */
+          if (!last)
+               evt.flags |= DIEF_FOLLOW;
+
           dfb_input_dispatch( data->device, &evt );
 
           data->dy = 0;
@@ -772,17 +792,18 @@ static void*
 linux_input_EventThread( DirectThread *thread, void *driver_data )
 {
      LinuxInputData    *data = (LinuxInputData*) driver_data;
-     int                readlen, status, i;
+     int                readlen, status;
+     unsigned int       i;
      struct input_event levt[64];
      fd_set             set;
      struct touchpad_fsm_state fsm_state;
 
      /* Query min/max coordinates. */
      if (data->touchpad) {
-          touchpad_fsm_init( &fsm_state );
-
           struct input_absinfo absinfo;
 
+          touchpad_fsm_init( &fsm_state );
+
           ioctl( data->fd, EVIOCGABS(ABS_X), &absinfo );
           fsm_state.x.min = absinfo.minimum;
           fsm_state.x.max = absinfo.maximum;
@@ -793,6 +814,8 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
      }
 
      while (1) {
+          DFBInputEvent devt = { .type = DIET_UNKNOWN };
+
           FD_ZERO( &set );
           FD_SET( data->fd, &set );
 
@@ -822,15 +845,13 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
 
           /* timeout? */
           if (status == 0) {
-               DFBInputEvent devt;
-
                if (data->touchpad && touchpad_fsm( &fsm_state, NULL, &devt ) > 0)
                     dfb_input_dispatch( data->device, &devt );
 
                continue;
           }
 
-          readlen = read( data->fd, levt, sizeof(levt) ) / sizeof(levt[0]);
+          readlen = read( data->fd, levt, sizeof(levt) );
 
           if (readlen < 0 && errno != EINTR)
                break;
@@ -840,14 +861,14 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
           if (readlen <= 0)
                continue;
 
-          for (i=0; i<readlen; i++) {
-               DFBInputEvent devt;
+          for (i=0; i<readlen / sizeof(levt[0]); i++) {
+               DFBInputEvent temp = { .type = DIET_UNKNOWN };
 
                if (data->touchpad) {
-                    status = touchpad_fsm( &fsm_state, &levt[i], &devt );
+                    status = touchpad_fsm( &fsm_state, &levt[i], &temp );
                     if (status < 0) {
                          /* Not handled. Try the direct approach. */
-                         if (!translate_event( &levt[i], &devt ))
+                         if (!translate_event( &levt[i], &temp ))
                               continue;
                     }
                     else if (status == 0) {
@@ -856,11 +877,35 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                     }
                }
                else {
-                    if (!translate_event( &levt[i], &devt ))
+                    if (!translate_event( &levt[i], &temp ))
                          continue;
                }
 
-               if (devt.type == DIET_AXISMOTION && (devt.flags & DIEF_AXISREL)) {
+               /* Flush previous event with DIEF_FOLLOW? */
+               if (devt.type != DIET_UNKNOWN) {
+                    flush_xy( data, false );
+
+                    /* Signal immediately following event. */
+                    devt.flags |= DIEF_FOLLOW;
+
+                    dfb_input_dispatch( data->device, &devt );
+
+                    if (data->has_leds && (devt.locks != data->locks)) {
+                         set_led( data, LED_SCROLLL, devt.locks & DILS_SCROLL );
+                         set_led( data, LED_NUML, devt.locks & DILS_NUM );
+                         set_led( data, LED_CAPSL, devt.locks & DILS_CAPS );
+                         data->locks = devt.locks;
+                    }
+
+                    devt.type  = DIET_UNKNOWN;
+                    devt.flags = DIEF_NONE;
+               }
+
+               devt = temp;
+
+               if (D_FLAGS_IS_SET( devt.flags, DIEF_AXISREL ) && devt.type == DIET_AXISMOTION &&
+                   dfb_config->mouse_motion_compression)
+               {
                     switch (devt.axis) {
                          case DIAI_X:
                               data->dx += devt.axisrel;
@@ -875,7 +920,12 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                     }
                }
 
-               flush_xy( data );
+               /* Event is dispatched in next round of loop. */
+          }
+
+          /* Flush last event without DIEF_FOLLOW. */
+          if (devt.type != DIET_UNKNOWN) {
+               flush_xy( data, false );
 
                dfb_input_dispatch( data->device, &devt );
 
@@ -886,8 +936,8 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                     data->locks = devt.locks;
                }
           }
-
-          flush_xy( data );
+          else
+               flush_xy( data, true );
      }
 
      if (status <= 0)
@@ -911,10 +961,10 @@ get_device_info( int              fd,
      unsigned int  num_rels     = 0;
      unsigned int  num_abs      = 0;
 
-     unsigned long evbit[NBITS(EV_MAX)];
-     unsigned long keybit[NBITS(KEY_MAX)];
-     unsigned long relbit[NBITS(REL_MAX)];
-     unsigned long absbit[NBITS(ABS_MAX)];
+     unsigned long evbit[NBITS(EV_CNT)];
+     unsigned long keybit[NBITS(KEY_CNT)];
+     unsigned long relbit[NBITS(REL_CNT)];
+     unsigned long absbit[NBITS(ABS_CNT)];
 
      /* get device name */
      ioctl( fd, EVIOCGNAME(DFB_INPUT_DEVICE_DESC_NAME_LENGTH - 1), info->desc.name );
@@ -933,11 +983,11 @@ get_device_info( int              fd,
           ioctl( fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit );
 
 	  /**  count typical keyboard keys only */
-          for (i=KEY_Q; i<KEY_M; i++)
+          for (i=KEY_Q; i<=KEY_M; i++)
                if (test_bit( i, keybit ))
                     num_keys++;
 
-          for (i=KEY_OK; i<KEY_MAX; i++)
+          for (i=KEY_OK; i<KEY_CNT; i++)
                if (test_bit( i, keybit ))
                     num_ext_keys++;
 
@@ -952,7 +1002,7 @@ get_device_info( int              fd,
           /* get bits for relative axes */
           ioctl( fd, EVIOCGBIT(EV_REL, sizeof(relbit)), relbit );
 
-          for (i=0; i<REL_MAX; i++)
+          for (i=0; i<REL_CNT; i++)
                if (test_bit( i, relbit ))
                     num_rels++;
      }
@@ -1029,6 +1079,45 @@ get_device_info( int              fd,
           info->prefered_id = DIDID_ANY;
 }
 
+static bool
+check_device( const char *device )
+{
+     int  fd;
+
+     /* Check if we are able to open the device */
+     fd = open( device, O_RDWR );
+     if (fd < 0) {
+          return false;
+     }
+     else {
+          InputDeviceInfo info;
+          bool touchpad;
+
+          /* try to grab the device */
+          if (dfb_config->linux_input_grab) {
+               /* 2.4 kernels don't have EVIOCGRAB so ignore EINVAL */
+               if (ioctl( fd, EVIOCGRAB, 1 ) && errno != EINVAL) {
+                    close( fd );
+                    return false;
+               }
+          }
+
+          memset( &info, 0, sizeof(InputDeviceInfo) );
+
+          get_device_info( fd, &info, &touchpad );
+
+          if (dfb_config->linux_input_grab)
+               ioctl( fd, EVIOCGRAB, 0 );
+          close( fd );
+
+          if (!dfb_config->linux_input_ir_only ||
+              (info.desc.type & DIDTF_REMOTE))
+               return true;
+     }
+
+     return false;
+}
+
 /* exported symbols */
 
 /*
@@ -1036,49 +1125,46 @@ get_device_info( int              fd,
  * Called once during initialization of DirectFB.
  */
 static int
-driver_get_available()
+driver_get_available( void )
 {
-     int i;
+     int   i;
+     char *tsdev;
 
 #ifdef LINUX_INPUT_USE_FBDEV
      if (dfb_system_type() != CORE_FBDEV)
           return 0;
 #endif
 
+     /* Use the devices specified in the configuration. */
+     if (fusion_vector_has_elements( &dfb_config->linux_input_devices )) {
+          const char *device;
+
+          fusion_vector_foreach (device, i, dfb_config->linux_input_devices) {
+               if (num_devices >= MAX_LINUX_INPUT_DEVICES)
+                    break;
+
+               if (check_device( device ))
+                    device_names[num_devices++] = D_STRDUP( device );
+          }
+
+          return num_devices;
+     }
+
+     /* Check for tslib device being used. */
+     tsdev = getenv( "TSLIB_TSDEVICE" );
+
+     /* No devices specified. Try to guess some. */
      for (i=0; i<MAX_LINUX_INPUT_DEVICES; i++) {
-          int  fd;
           char buf[32];
 
           snprintf( buf, 32, "/dev/input/event%d", i );
 
-          /* Check if we are able to open the device */
-          fd = open( buf, O_RDWR );
-          if (fd < 0) {
-             if (errno == ENODEV)
-                  break;
-          }
-          else {
-               InputDeviceInfo info;
-               bool touchpad;
+          /* Let tslib driver handle its device. */
+          if (tsdev && !strcmp( tsdev, buf ))
+               continue;
 
-               /* try to grab the device */
-               /* 2.4 kernels don't have EVIOCGRAB so ignore EINVAL */
-               if (ioctl( fd, EVIOCGRAB, 1 ) && errno != EINVAL) {
-                    close( fd );
-                    continue;
-               }
-
-               memset( &info, 0, sizeof(InputDeviceInfo) );
-
-               get_device_info( fd, &info, &touchpad );
-
-               ioctl( fd, EVIOCGRAB, 0 );
-               close( fd );
-
-               if (!dfb_config->linux_input_ir_only ||
-                   (info.desc.type & DIDTF_REMOTE))
-                    device_nums[num_devices++] = i;
-          }
+          if (check_device( buf ))
+               device_names[num_devices++] = D_STRDUP( buf );
      }
 
      return num_devices;
@@ -1113,30 +1199,29 @@ driver_open_device( CoreInputDevice  *device,
                     void            **driver_data )
 {
      int              fd, ret;
-     char             buf[32];
-     unsigned long    ledbit[NBITS(LED_MAX)];
+     unsigned long    ledbit[NBITS(LED_CNT)];
      LinuxInputData  *data;
 #ifdef LINUX_INPUT_USE_FBDEV
      FBDev           *dfb_fbdev = dfb_system_data();
 #endif
      bool             touchpad;
 
-     snprintf( buf, 32, "/dev/input/event%d", device_nums[number] );
-
      /* open device */
-     fd = open( buf, O_RDWR );
+     fd = open( device_names[number], O_RDWR );
      if (fd < 0) {
           D_PERROR( "DirectFB/linux_input: could not open device" );
           return DFB_INIT;
      }
 
      /* grab device */
-     ret = ioctl( fd, EVIOCGRAB, 1 );
-     /* 2.4 kernels don't have EVIOCGRAB so ignore EINVAL */
-     if (ret && errno != EINVAL) {
-          D_PERROR( "DirectFB/linux_input: could not grab device" );
-          close( fd );
-          return DFB_INIT;
+     if (dfb_config->linux_input_grab) {
+          ret = ioctl( fd, EVIOCGRAB, 1 );
+          /* 2.4 kernels don't have EVIOCGRAB so ignore EINVAL */
+          if (ret && errno != EINVAL) {
+               D_PERROR( "DirectFB/linux_input: could not grab device" );
+               close( fd );
+               return DFB_INIT;
+          }
      }
 
      /* fill device info structure */
@@ -1144,6 +1229,12 @@ driver_open_device( CoreInputDevice  *device,
 
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(LinuxInputData) );
+     if (!data) {
+          if (dfb_config->linux_input_grab)
+               ioctl( fd, EVIOCGRAB, 0 );
+          close( fd );
+          return D_OOM();
+     }
 
      data->fd     = fd;
      data->device = device;
@@ -1166,7 +1257,8 @@ driver_open_device( CoreInputDevice  *device,
           ret = ioctl( fd, EVIOCGLED(sizeof(data->led_state)), data->led_state );
           if (ret < 0) {
                D_PERROR( "DirectFB/linux_input: could not get LED state" );
-               ioctl( fd, EVIOCGRAB, 0 );
+               if (dfb_config->linux_input_grab)
+                    ioctl( fd, EVIOCGRAB, 0 );
                close( fd );
                D_FREE( data );
                return DFB_INIT;
@@ -1187,6 +1279,40 @@ driver_open_device( CoreInputDevice  *device,
      return DFB_OK;
 }
 
+/*
+ * Obtain information about an axis (only absolute axis so far).
+ */
+static DFBResult
+driver_get_axis_info( CoreInputDevice              *device,
+                      void                         *driver_data,
+                      DFBInputDeviceAxisIdentifier  axis,
+                      DFBInputDeviceAxisInfo       *ret_info )
+{
+     LinuxInputData *data = (LinuxInputData*) driver_data;
+
+     if (data->touchpad)
+          return DFB_OK;
+
+     if (axis <= ABS_PRESSURE && axis < DIAI_LAST) {
+          unsigned long absbit[NBITS(ABS_CNT)];
+
+          /* check if we have an absolute axes */
+          ioctl( data->fd, EVIOCGBIT(EV_ABS, sizeof(absbit)), absbit );
+
+          if (test_bit (axis, absbit)) {
+               struct input_absinfo absinfo;
+
+               if (ioctl( data->fd, EVIOCGABS(axis), &absinfo ) == 0 &&
+                   (absinfo.minimum || absinfo.maximum)) {
+                    ret_info->flags  |= DIAIF_ABS_MIN | DIAIF_ABS_MAX;
+                    ret_info->abs_min = absinfo.minimum;
+                    ret_info->abs_max = absinfo.maximum;
+               }
+          }
+     }
+
+     return DFB_OK;
+}
 
 /*
  * Fetch one entry from the kernel keymap.
@@ -1275,7 +1401,8 @@ driver_close_device( void *driver_data )
      }
 
      /* release device */
-     ioctl( data->fd, EVIOCGRAB, 0 );
+     if (dfb_config->linux_input_grab)
+          ioctl( data->fd, EVIOCGRAB, 0 );
 
      /* close file */
      close( data->fd );
@@ -1284,19 +1411,17 @@ driver_close_device( void *driver_data )
      D_FREE( data );
 }
 
-static int
-timeout_is_set( struct timeval *timeout )
+static bool
+timeout_is_set( const struct timeval *timeout )
 {
      return timeout->tv_sec || timeout->tv_usec;
 }
 
-static int
-timeout_passed( struct timeval *timeout, struct timeval *current )
+static bool
+timeout_passed( const struct timeval *timeout, const struct timeval *current )
 {
-     if (!timeout_is_set( timeout ))
-          return 1;
-
-     return current->tv_sec > timeout->tv_sec ||
+     return !timeout_is_set( timeout ) ||
+          current->tv_sec > timeout->tv_sec ||
           (current->tv_sec == timeout->tv_sec && current->tv_usec > timeout->tv_usec);
 }
 
@@ -1308,7 +1433,7 @@ timeout_clear( struct timeval *timeout )
 }
 
 static void
-timeout_add( struct timeval *timeout, struct timeval *add )
+timeout_add( struct timeval *timeout, const struct timeval *add )
 {
      timeout->tv_sec += add->tv_sec;
      timeout->tv_usec += add->tv_usec;
@@ -1319,7 +1444,7 @@ timeout_add( struct timeval *timeout, struct timeval *add )
 }
 
 static void
-timeout_sub( struct timeval *timeout, struct timeval *sub )
+timeout_sub( struct timeval *timeout, const struct timeval *sub )
 {
      timeout->tv_sec -= sub->tv_sec;
      timeout->tv_usec -= sub->tv_usec;
@@ -1339,14 +1464,14 @@ touchpad_fsm_init( struct touchpad_fsm_state *state )
 }
 
 static int
-touchpad_normalize( struct touchpad_axis *axis, int value )
+touchpad_normalize( const struct touchpad_axis *axis, int value )
 {
      return ((value - axis->min) << 9) / (axis->max - axis->min);
 }
 
 static int
 touchpad_translate( struct touchpad_fsm_state *state,
-                    struct input_event        *levt,
+                    const struct input_event  *levt,
                     DFBInputEvent             *devt )
 {
      struct touchpad_axis *axis = NULL;
@@ -1389,19 +1514,19 @@ touchpad_translate( struct touchpad_fsm_state *state,
      return 1;
 }
 
-static int
+static bool
 touchpad_finger_landing( const struct input_event *levt )
 {
      return levt->type == EV_KEY && levt->code == BTN_TOUCH && levt->value == 1;
 }
 
-static int
+static bool
 touchpad_finger_leaving( const struct input_event *levt )
 {
      return levt->type == EV_KEY && levt->code == BTN_TOUCH && levt->value == 0;
 }
 
-static int
+static bool
 touchpad_finger_moving( const struct input_event *levt )
 {
      return levt->type == EV_ABS && (levt->code == ABS_X || levt->code == ABS_Y);
@@ -1413,7 +1538,7 @@ touchpad_finger_moving( const struct input_event *levt )
  */
 static int
 touchpad_fsm( struct touchpad_fsm_state *state,
-              struct input_event        *levt,
+              const struct input_event  *levt,
               DFBInputEvent             *devt )
 {
      struct timeval timeout = { 0, 125000 };
