@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -356,7 +359,7 @@ mouseEventThread_mousesystems( DirectThread *thread, void *driver_data )
      return NULL;
 }
 
-static MouseProtocol mouse_get_protocol()
+static MouseProtocol mouse_get_protocol( void )
 {
      MouseProtocol protocol;
 
@@ -375,7 +378,7 @@ static MouseProtocol mouse_get_protocol()
 /* exported symbols */
 
 static int
-driver_get_available()
+driver_get_available( void )
 {
      struct serial_struct serial_info;
      struct timeval       timeout;
@@ -402,7 +405,7 @@ driver_get_available()
      /* open device to read from */
      fd = open( dfb_config->mouse_source, flags );
      if (fd < 0) {
-	  D_INFO( "DirectFB/SerialMouse: could not open device '%s'!\n", dfb_config->mouse_source );
+          D_INFO( "DirectFB/SerialMouse: could not open device '%s'!\n", dfb_config->mouse_source );
           return 0;
      }
 
@@ -414,51 +417,52 @@ driver_get_available()
 
           goto success;
 
-     } else {
+     }
+     else {
+          if (ioctl( fd, TIOCGSERIAL, &serial_info ))
+               goto error;
 
-     if (ioctl( fd, TIOCGSERIAL, &serial_info ))
-          goto error;
+          /*  test if there's a mouse connected by lowering and raising RTS  */
+          if (ioctl( fd, TIOCMGET, &lines ))
+               goto error;
 
-     /*  test if there's a mouse connected by lowering and raising RTS  */
-     if (ioctl( fd, TIOCMGET, &lines ))
-          goto error;
+          lines ^= TIOCM_RTS;
+          if (ioctl( fd, TIOCMSET, &lines ))
+               goto error;
+          usleep (1000);
+          lines |= TIOCM_RTS;
+          if (ioctl( fd, TIOCMSET, &lines ))
+               goto error;
 
-     lines ^= TIOCM_RTS;
-     if (ioctl( fd, TIOCMSET, &lines ))
-          goto error;
-     usleep (1000);
-     lines |= TIOCM_RTS;
-     if (ioctl( fd, TIOCMSET, &lines ))
-          goto error;
+          /*  wait for the mouse to send 0x4D  */
+          FD_ZERO (&set);
+          FD_SET (fd, &set);
+          timeout.tv_sec  = 0;
+          timeout.tv_usec = 50000;
 
-     /*  wait for the mouse to send 0x4D  */
-     FD_ZERO (&set);
-     FD_SET (fd, &set);
-     timeout.tv_sec  = 0;
-     timeout.tv_usec = 50000;
+          while (select (fd+1, &set, NULL, NULL, &timeout) < 0 && errno == EINTR);
 
-     while (select (fd+1, &set, NULL, NULL, &timeout) < 0 && errno == EINTR);
-     if (FD_ISSET (fd, &set) && (readlen = read (fd, buf, 8) > 0)) {
-          while (readlen--) {
-               if (buf[8-readlen] == 0x4D)
-                    break;
-          }
-          if (readlen) {
+          if (FD_ISSET (fd, &set) && (readlen = read (fd, buf, 8) > 0)) {
+               int i;
+
+               for (i=0; i<readlen; i++) {
+                    if (buf[i] == 0x4D)
+                         break;
+               }
+               if (i < readlen)
                     goto success;
           }
      }
 
-    }
-
- success:
-     D_INFO("DirectFB/SerialMouse: OK\n");
-     close (fd);
-     return 1;
-
- error:
+error:
      D_INFO("DirectFB/SerialMouse: Failed\n");
      close (fd);
      return 0;
+
+success:
+     D_INFO("DirectFB/SerialMouse: OK\n");
+     close (fd);
+     return 1;
 }
 
 static void
@@ -469,8 +473,7 @@ driver_get_info( InputDriverInfo *info )
                DFB_INPUT_DRIVER_INFO_NAME_LENGTH, "Serial Mouse Driver" );
 
      snprintf( info->vendor,
-               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH,
-               "convergence integrated media GmbH" );
+               DFB_INPUT_DRIVER_INFO_VENDOR_LENGTH, "directfb.org" );
 
      info->version.major = 0;
      info->version.minor = 2;
@@ -504,6 +507,10 @@ driver_open_device( CoreInputDevice      *device,
 
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(SerialMouseData) );
+     if (!data) {
+          close( fd );
+          return D_OOM();
+     }
 
      data->fd       = fd;
      data->device   = device;
