@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
@@ -76,7 +79,7 @@ sonypiEventThread( DirectThread *thread, void *driver_data )
 {
      SonypiData    *data = (SonypiData*) driver_data;
      int            readlen;
-     __u8           buffer[16];
+     u8             buffer[16];
 
      /* loop until error occurs except EINTR */
      while ((readlen = read( data->fd, buffer, 16 )) > 0 || errno == EINTR) {
@@ -139,7 +142,7 @@ sonypiEventThread( DirectThread *thread, void *driver_data )
  * Called once during initialization of DirectFB.
  */
 static int
-driver_get_available()
+driver_get_available( void )
 {
      int fd;
 
@@ -212,6 +215,10 @@ driver_open_device( CoreInputDevice      *device,
 
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(SonypiData) );
+     if (!data) {
+          close( fd );
+          return D_OOM();
+     }
 
      data->fd     = fd;
      data->device = device;
