@@ -12,7 +12,7 @@
 
    Based on the h3600_ts.c driver:
 
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -154,7 +154,7 @@ wm97xxtsEventThread( DirectThread *thread, void *driver_data )
 
 /* exported symbols */
 
-static int driver_get_available()
+static int driver_get_available( void )
 {
      int fd;
 
@@ -213,6 +213,10 @@ driver_open_device( CoreInputDevice      *device,
 
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(WM97xxTSData) );
+     if (!data) {
+          close( fd );
+          return D_OOM();
+     }
 
      data->fd     = fd;
      data->device = device;
