@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -30,16 +31,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-
-#include <ft2build.h>
-#include FT_GLYPH_H
+#include <stdarg.h>
 
 #include <directfb.h>
 
 #include <core/fonts.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
-#include <core/surfacemanager.h>
+#include <core/surface.h>
+#include <core/surface_buffer.h>
 
 #include <gfx/convert.h>
 
@@ -48,20 +47,29 @@
 #include <direct/mem.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
+#include <direct/utf8.h>
 #include <direct/util.h>
 
 #include <misc/conf.h>
 #include <misc/util.h>
 
+#undef SIZEOF_LONG
+#include <ft2build.h>
+#include FT_GLYPH_H
+
+#ifndef FT_LOAD_TARGET_MONO
+    /* FT_LOAD_TARGET_MONO was added in FreeType-2.1.3, we have to use (less good)
+       FT_LOAD_MONOCHROME with older versions. Make it an alias for code simplicity. */
+    #define FT_LOAD_TARGET_MONO FT_LOAD_MONOCHROME
+#endif
+
 
 static DFBResult
 Probe( IDirectFBFont_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBFont      *thiz,
-           CoreDFB            *core,
-           const char         *filename,
-           DFBFontDescription *desc );
+	   ... );
 
 #include <direct/interface_implementation.h>
 
@@ -71,7 +79,7 @@ static FT_Library      library           = NULL;
 static int             library_ref_count = 0;
 static pthread_mutex_t library_mutex     = PTHREAD_MUTEX_INITIALIZER;
 
-#define KERNING_CACHE_MIN   32
+#define KERNING_CACHE_MIN    0
 #define KERNING_CACHE_MAX  127
 #define KERNING_CACHE_SIZE (KERNING_CACHE_MAX - KERNING_CACHE_MIN + 1)
 
@@ -83,11 +91,14 @@ static pthread_mutex_t library_mutex     = PTHREAD_MUTEX_INITIALIZER;
 #define KERNING_CACHE_ENTRY(a,b)   \
      (data->kerning[(a)-KERNING_CACHE_MIN][(b)-KERNING_CACHE_MIN])
 
+#define CHAR_INDEX(c)    (((c) < 256) ? data->indices[c] : FT_Get_Char_Index( data->face, c ))
 
 typedef struct {
      FT_Face      face;
      int          disable_charmap;
      int          fixed_advance;
+     bool         fixed_clip;
+     unsigned int indices[256];
 } FT2ImplData;
 
 typedef struct {
@@ -101,80 +112,207 @@ typedef struct {
      KerningCacheEntry kerning[KERNING_CACHE_SIZE][KERNING_CACHE_SIZE];
 } FT2ImplKerningData;
 
+/**********************************************************************************************************************/
+
+static DFBResult
+ft2UTF8GetCharacterIndex( CoreFont     *thiz,
+                          unsigned int  character,
+                          unsigned int *ret_index )
+{
+     FT2ImplData *data = thiz->impl_data;
+
+     D_MAGIC_ASSERT( thiz, CoreFont );
+
+     if (data->disable_charmap)
+          *ret_index = character;
+     else {
+          pthread_mutex_lock ( &library_mutex );
+
+          *ret_index = CHAR_INDEX( character );
+
+          pthread_mutex_unlock ( &library_mutex );
+     }
+
+     return DFB_OK;
+}
+
+static DFBResult
+ft2UTF8DecodeText( CoreFont       *thiz,
+                   const void     *text,
+                   int             length,
+                   unsigned int   *ret_indices,
+                   int            *ret_num )
+{
+     int pos = 0, num = 0;
+     const u8 *bytes   = text;
+     FT2ImplData *data = thiz->impl_data;
+
+     D_MAGIC_ASSERT( thiz, CoreFont );
+     D_ASSERT( text != NULL );
+     D_ASSERT( length >= 0 );
+     D_ASSERT( ret_indices != NULL );
+     D_ASSERT( ret_num != NULL );
+
+     pthread_mutex_lock ( &library_mutex );
+
+     while (pos < length) {
+          unsigned int c;
+
+          if (bytes[pos] < 128)
+               c = bytes[pos++];
+          else {
+               c = DIRECT_UTF8_GET_CHAR( &bytes[pos] );
+               pos += DIRECT_UTF8_SKIP(bytes[pos]);
+          }
+
+          if (data->disable_charmap)
+               ret_indices[num++] = c;
+          else
+               ret_indices[num++] = CHAR_INDEX( c );
+     }
+
+     pthread_mutex_unlock ( &library_mutex );
+
+     *ret_num = num;
+
+     return DFB_OK;
+}
+
+static const CoreFontEncodingFuncs ft2UTF8Funcs = {
+     .GetCharacterIndex = ft2UTF8GetCharacterIndex,
+     .DecodeText        = ft2UTF8DecodeText,
+};
+
+/**********************************************************************************************************************/
+
+static DFBResult
+ft2Latin1GetCharacterIndex( CoreFont     *thiz,
+                            unsigned int  character,
+                            unsigned int *ret_index )
+{
+     FT2ImplData *data = thiz->impl_data;
+
+     D_MAGIC_ASSERT( thiz, CoreFont );
+
+     if (data->disable_charmap)
+          *ret_index = character;
+     else
+          *ret_index = data->indices[character];
+
+     return DFB_OK;
+}
+
+static DFBResult
+ft2Latin1DecodeText( CoreFont       *thiz,
+                     const void     *text,
+                     int             length,
+                     unsigned int   *ret_indices,
+                     int            *ret_num )
+{
+     int i;
+     const u8 *bytes   = text;
+     FT2ImplData *data = thiz->impl_data;
+
+     D_MAGIC_ASSERT( thiz, CoreFont );
+     D_ASSERT( text != NULL );
+     D_ASSERT( length >= 0 );
+     D_ASSERT( ret_indices != NULL );
+     D_ASSERT( ret_num != NULL );
+
+     if (data->disable_charmap) {
+          for (i=0; i<length; i++)
+               ret_indices[i] = bytes[i];
+     }
+     else {
+          for (i=0; i<length; i++)
+               ret_indices[i] = data->indices[bytes[i]];
+     }
+
+     *ret_num = length;
+
+     return DFB_OK;
+}
+
+static const CoreFontEncodingFuncs ft2Latin1Funcs = {
+     .GetCharacterIndex = ft2Latin1GetCharacterIndex,
+     .DecodeText        = ft2Latin1DecodeText,
+};
+
+/**********************************************************************************************************************/
 
 static DFBResult
 render_glyph( CoreFont      *thiz,
-              unichar        glyph,
-              CoreGlyphData *info,
-              CoreSurface   *surface )
+              unsigned int   index,
+              CoreGlyphData *info )
 {
      FT_Error     err;
      FT_Face      face;
      FT_Int       load_flags;
-     FT_UInt      index;
-     __u8        *src;
-     void        *dst;
+     u8          *src;
      int          y;
-     int          pitch;
-     FT2ImplData *data = (FT2ImplData*) thiz->impl_data;
+     FT2ImplData *data    = thiz->impl_data;
+     CoreSurface *surface = info->surface;
+     CoreSurfaceBufferLock  lock;
 
      pthread_mutex_lock ( &library_mutex );
 
      face = data->face;
 
-     if (data->disable_charmap)
-          index = glyph;
-     else
-          index = FT_Get_Char_Index( face, glyph );
-
-     load_flags = (FT_Int) face->generic.data;
+     load_flags = (unsigned long) face->generic.data;
      load_flags |= FT_LOAD_RENDER;
 
      if ((err = FT_Load_Glyph( face, index, load_flags ))) {
-          D_HEAVYDEBUG( "DirectFB/FontFT2: "
-                         "Could not render glyph for character #%d!\n", glyph );
+          D_DEBUG( "DirectFB/FontFT2: Could not render glyph for character index #%d!\n", index );
           pthread_mutex_unlock ( &library_mutex );
           return DFB_FAILURE;
      }
 
      pthread_mutex_unlock ( &library_mutex );
 
-     err = dfb_surface_soft_lock( surface, DSLF_WRITE,
-                                  &dst, &pitch, 0 );
+     err = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAF_CPU_WRITE, &lock );
      if (err) {
-          D_ERROR( "DirectB/FontFT2: Unable to lock surface!\n" );
+          D_DERROR( err, "DirectFB/FontFT2: Unable to lock surface!\n" );
           return err;
      }
 
      info->width = face->glyph->bitmap.width;
-     if (info->width + thiz->next_x > surface->width)
-          info->width = surface->width - thiz->next_x;
+     if (info->width + info->start > surface->config.size.w)
+          info->width = surface->config.size.w - info->start;
 
      info->height = face->glyph->bitmap.rows;
-     if (info->height > surface->height)
-          info->height = surface->height;
+     if (info->height > surface->config.size.h)
+          info->height = surface->config.size.h;
 
      info->left = face->glyph->bitmap_left;
      info->top  = thiz->ascender - face->glyph->bitmap_top;
 
+     if (data->fixed_clip) {
+          while (info->left + info->width > data->fixed_advance)
+               info->left--;
+
+          if (info->left < 0)
+               info->left = 0;
+
+          if (info->width > data->fixed_advance)
+               info->width = data->fixed_advance;
+     }
+
      src = face->glyph->bitmap.buffer;
-     dst += DFB_BYTES_PER_LINE(surface->format, thiz->next_x);
+     lock.addr += DFB_BYTES_PER_LINE(surface->config.format, info->start);
 
      for (y=0; y < info->height; y++) {
-          int    i, j, n;
-          __u8  *dst8  = dst;
-          __u16 *dst16 = dst;
-          __u32 *dst32 = dst;
+          int  i, j, n;
+          u8  *dst8  = lock.addr;
+          u16 *dst16 = lock.addr;
+          u32 *dst32 = lock.addr;
 
           switch (face->glyph->bitmap.pixel_mode) {
                case ft_pixel_mode_grays:
-                    switch (surface->format) {
+                    switch (surface->config.format) {
                          case DSPF_ARGB:
                               if (thiz->surface_caps & DSCAPS_PREMULTIPLIED) {
                                    for (i=0; i<info->width; i++)
-                                        dst32[i] = ((src[i] << 24) |
-                                                    (src[i] << 16) |
-                                                    (src[i] <<  8) | src[i]);
+                                        dst32[i] = src[i] * 0x01010101;
                               }
                               else
                                    for (i=0; i<info->width; i++)
@@ -185,8 +323,14 @@ render_glyph( CoreFont      *thiz,
                                    dst32[i] = ((src[i] ^ 0xFF) << 24) | 0xFFFFFF;
                               break;
                          case DSPF_ARGB4444:
-                              for (i=0; i<info->width; i++)
-                                   dst16[i] = (src[i] << 8) | 0xFFF;
+                              if (thiz->surface_caps & DSCAPS_PREMULTIPLIED) {
+                                   for (i=0; i<info->width; i++)
+                                        dst16[i] = (src[i] >> 4) * 0x1111;
+                              }
+                              else {
+                                   for (i=0; i<info->width; i++)
+                                        dst16[i] = (src[i] << 8) | 0xFFF;
+                              }
                               break;
                          case DSPF_ARGB2554:
                               for (i=0; i<info->width; i++)
@@ -197,11 +341,15 @@ render_glyph( CoreFont      *thiz,
                                    dst16[i] = (src[i] << 8) | 0x7FFF;
                               break;
                          case DSPF_A8:
-                              direct_memcpy( dst, src, info->width );
+                              direct_memcpy( lock.addr, src, info->width );
+                              break;
+                         case DSPF_A4:
+                              for (i=0, j=0; i<info->width; i+=2, j++)
+                                   dst8[j] = (src[i] & 0xF0) | (src[i+1] >> 4);
                               break;
                          case DSPF_A1:
                               for (i=0, j=0; i < info->width; ++j) {
-                                   register __u8 p = 0;
+                                   register u8 p = 0;
 
                                    for (n=0; n<8 && i<info->width; ++i, ++n)
                                         p |= (src[i] & 0x80) >> n;
@@ -209,13 +357,24 @@ render_glyph( CoreFont      *thiz,
                                    dst8[j] = p;
                               }
                               break;
+                         case DSPF_LUT2:
+                              for (i=0, j=0; i < info->width; ++j) {
+                                   register u8 p = 0;
+
+                                   for (n=0; n<8 && i<info->width; ++i, n+=2)
+                                        p |= (src[i] & 0xC0) >> n;
+
+                                   dst8[j] = p;
+                              }
+                              break;
                          default:
+                              D_UNIMPLEMENTED();
                               break;
                     }
                     break;
 
                case ft_pixel_mode_mono:
-                    switch (surface->format) {
+                    switch (surface->config.format) {
                          case DSPF_ARGB:
                               for (i=0; i<info->width; i++)
                                    dst32[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
@@ -246,10 +405,18 @@ render_glyph( CoreFont      *thiz,
                                    dst8[i] = (src[i>>3] &
                                               (1<<(7-(i%8)))) ? 0xFF : 0x00;
                               break;
+                         case DSPF_A4:
+                              for (i=0, j=0; i<info->width; i+=2, j++)
+                                   dst8[j] = ((src[i>>3] &
+                                              (1<<(7-(i%8)))) ? 0xF0 : 0x00) |
+                                             ((src[(i+1)>>3] &
+                                              (1<<(7-((i+1)%8)))) ? 0x0F : 0x00);
+                              break;
                          case DSPF_A1:
-                              direct_memcpy( dst, src, DFB_BYTES_PER_LINE(DSPF_A1, info->width) );
+                              direct_memcpy( lock.addr, src, DFB_BYTES_PER_LINE(DSPF_A1, info->width) );
                               break;
                          default:
+                              D_UNIMPLEMENTED();
                               break;
                     }
                     break;
@@ -260,10 +427,11 @@ render_glyph( CoreFont      *thiz,
           }
 
           src += face->glyph->bitmap.pitch;
-          dst += pitch;
+
+          lock.addr += lock.pitch;
      }
 
-     dfb_surface_unlock( surface, 0 );
+     dfb_surface_unlock_buffer( surface, &lock );
 
      return DFB_OK;
 }
@@ -271,29 +439,22 @@ render_glyph( CoreFont      *thiz,
 
 static DFBResult
 get_glyph_info( CoreFont      *thiz,
-                unichar        glyph,
+                unsigned int   index,
                 CoreGlyphData *info )
 {
      FT_Error err;
      FT_Face  face;
      FT_Int   load_flags;
-     FT_UInt  index;
      FT2ImplData *data = (FT2ImplData*) thiz->impl_data;
 
      pthread_mutex_lock ( &library_mutex );
 
      face = data->face;
 
-     if (data->disable_charmap)
-          index = glyph;
-     else
-          index = FT_Get_Char_Index( face, glyph );
-
-     load_flags = (FT_Int) face->generic.data;
+     load_flags = (unsigned long) face->generic.data;
 
      if ((err = FT_Load_Glyph( face, index, load_flags ))) {
-          D_HEAVYDEBUG( "DirectB/FontFT2: "
-                         "Could not load glyph for character #%d!\n", glyph );
+          D_DEBUG( "DirectFB/FontFT2: Could not load glyph for character index #%d!\n", index );
 
           pthread_mutex_unlock ( &library_mutex );
 
@@ -301,10 +462,10 @@ get_glyph_info( CoreFont      *thiz,
      }
 
      if (face->glyph->format != ft_glyph_format_bitmap) {
-          err = FT_Render_Glyph( face->glyph, ft_render_mode_normal );
+          err = FT_Render_Glyph( face->glyph,
+                                 (load_flags & FT_LOAD_TARGET_MONO) ? ft_render_mode_mono : ft_render_mode_normal );
           if (err) {
-               D_ERROR( "DirectFB/FontFT2: Could not "
-                         "render glyph for character #%d!\n", glyph );
+               D_ERROR( "DirectFB/FontFT2: Could not render glyph for character index #%d!\n", index );
 
                pthread_mutex_unlock ( &library_mutex );
 
@@ -319,22 +480,23 @@ get_glyph_info( CoreFont      *thiz,
      info->advance = data->fixed_advance ?
                      data->fixed_advance : (face->glyph->advance.x >> 6);
 
+     if (data->fixed_clip && info->width > data->fixed_advance)
+          info->width = data->fixed_advance;
+
      return DFB_OK;
 }
 
 
 static DFBResult
-get_kerning( CoreFont *thiz,
-             unichar   prev,
-             unichar   current,
-             int      *kern_x,
-             int      *kern_y)
+get_kerning( CoreFont     *thiz,
+             unsigned int  prev,
+             unsigned int  current,
+             int          *kern_x,
+             int          *kern_y)
 {
-     FT_Vector  vector;
-     FT_UInt    prev_index;
-     FT_UInt    current_index;
+     FT_Vector vector;
 
-     FT2ImplKerningData *data = (FT2ImplKerningData*) thiz->impl_data;
+     FT2ImplKerningData *data = thiz->impl_data;
      KerningCacheEntry *cache = NULL;
 
      D_ASSUME( (kern_x != NULL) || (kern_y != NULL) );
@@ -357,13 +519,9 @@ get_kerning( CoreFont *thiz,
 
      pthread_mutex_lock ( &library_mutex );
 
-     /* Get the character indices for lookup. */
-     prev_index    = FT_Get_Char_Index( data->base.face, prev );
-     current_index = FT_Get_Char_Index( data->base.face, current );
-
      /* Lookup kerning values for the character pair. */
      FT_Get_Kerning( data->base.face,
-                     prev_index, current_index, ft_kerning_default, &vector );
+                     prev, current, ft_kerning_default, &vector );
 
      pthread_mutex_unlock ( &library_mutex );
 
@@ -382,29 +540,23 @@ init_kerning_cache( FT2ImplKerningData *data )
 {
      int a, b;
 
+     pthread_mutex_lock ( &library_mutex );
+
      for (a=KERNING_CACHE_MIN; a<=KERNING_CACHE_MAX; a++) {
           for (b=KERNING_CACHE_MIN; b<=KERNING_CACHE_MAX; b++) {
                FT_Vector          vector;
-               FT_UInt            prev;
-               FT_UInt            current;
                KerningCacheEntry *cache = &KERNING_CACHE_ENTRY( a, b );
 
-               pthread_mutex_lock ( &library_mutex );
-
-               /* Get the character indices for lookup. */
-               prev    = FT_Get_Char_Index( data->base.face, a );
-               current = FT_Get_Char_Index( data->base.face, b );
-
                /* Lookup kerning values for the character pair. */
                FT_Get_Kerning( data->base.face,
-                               prev, current, ft_kerning_default, &vector );
-
-               pthread_mutex_unlock ( &library_mutex );
+                               a, b, ft_kerning_default, &vector );
 
                cache->x = (signed char) (vector.x >> 6);
                cache->y = (signed char) (vector.y >> 6);
           }
      }
+
+     pthread_mutex_unlock ( &library_mutex );
 }
 
 static DFBResult
@@ -415,8 +567,7 @@ init_freetype( void )
      pthread_mutex_lock ( &library_mutex );
 
      if (!library) {
-          D_HEAVYDEBUG( "DirectFB/FontFT2: "
-                         "Initializing the FreeType2 library.\n" );
+          D_DEBUG( "DirectFB/FontFT2: Initializing the FreeType2 library.\n" );
           err = FT_Init_FreeType( &library );
           if (err) {
                D_ERROR( "DirectFB/FontFT2: "
@@ -440,8 +591,7 @@ release_freetype( void )
      pthread_mutex_lock( &library_mutex );
 
      if (library && --library_ref_count == 0) {
-          D_HEAVYDEBUG( "DirectFB/FontFT2: "
-                         "Releasing the FreeType2 library.\n" );
+          D_DEBUG( "DirectFB/FontFT2: Releasing the FreeType2 library.\n" );
           FT_Done_FreeType( library );
           library = NULL;
      }
@@ -473,7 +623,7 @@ IDirectFBFont_FT2_Destruct( IDirectFBFont *thiz )
 }
 
 
-static DFBResult
+static DirectResult
 IDirectFBFont_FT2_Release( IDirectFBFont *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBFont)
@@ -492,7 +642,7 @@ Probe( IDirectFBFont_ProbeContext *ctx )
      FT_Error err;
      FT_Face  face;
 
-     D_HEAVYDEBUG( "DirectFB/FontFT2: Probe font `%s'.\n", ctx->filename );
+     D_DEBUG( "DirectFB/FontFT2: Probe font `%s'.\n", ctx->filename );
 
      if (!ctx->filename)
           return DFB_UNSUPPORTED;
@@ -520,10 +670,10 @@ Probe( IDirectFBFont_ProbeContext *ctx )
 
 static DFBResult
 Construct( IDirectFBFont      *thiz,
-           CoreDFB            *core,
-           const char         *filename,
-           DFBFontDescription *desc )
+	   ... )
 {
+     int                    i;
+     DFBResult              ret;
      CoreFont              *font;
      FT_Face                face;
      FT_Error               err;
@@ -532,13 +682,25 @@ Construct( IDirectFBFont      *thiz,
      bool                   disable_charmap = false;
      bool                   disable_kerning = false;
      bool                   load_mono = false;
+     u32                    mask = 0;
 
-     D_HEAVYDEBUG( "DirectFB/FontFT2: "
-                    "Construct font from file `%s' (index %d) at pixel size %d x %d.\n",
-                    filename,
-                    (desc->flags & DFDESC_INDEX)  ? desc->index  : 0,
-                    (desc->flags & DFDESC_WIDTH)  ? desc->width  : 0,
-                    (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );
+     CoreDFB *core;
+     char *filename;
+     DFBFontDescription *desc;
+
+     va_list tag;
+     va_start(tag, thiz);
+     core = va_arg(tag, CoreDFB *);
+     filename = va_arg(tag, char *);
+     desc = va_arg(tag, DFBFontDescription *);
+     va_end( tag );
+
+     D_DEBUG( "DirectFB/FontFT2: "
+              "Construct font from file `%s' (index %d) at pixel size %d x %d.\n",
+              filename,
+              (desc->flags & DFDESC_INDEX)  ? desc->index  : 0,
+              (desc->flags & DFDESC_WIDTH)  ? desc->width  : 0,
+              (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );
 
      if (init_freetype() != DFB_OK) {
           DIRECT_DEALLOCATE_INTERFACE( thiz );
@@ -581,13 +743,8 @@ Construct( IDirectFBFont      *thiz,
                load_mono = true;
      }
 
-     if (load_mono) {
-#ifdef FT_LOAD_TARGET_MONO  /* added in FreeType-2.1.3 */
+     if (load_mono)
           load_flags |= FT_LOAD_TARGET_MONO;
-#else
-          load_flags |= FT_LOAD_MONOCHROME;
-#endif
-     }
 
      if (!disable_charmap) {
           pthread_mutex_lock ( &library_mutex );
@@ -598,14 +755,25 @@ Construct( IDirectFBFont      *thiz,
 
           /* ft_encoding_latin_1 has been introduced in freetype-2.1 */
           if (err) {
-               D_HEAVYDEBUG( "DirectFB/FontFT2: "
-                              "Couldn't select Unicode encoding, "
-                              "falling back to Latin1.\n");
+               D_DEBUG( "DirectFB/FontFT2: "
+                        "Couldn't select Unicode encoding, "
+                        "falling back to Latin1.\n");
                pthread_mutex_lock ( &library_mutex );
                err = FT_Select_Charmap( face, ft_encoding_latin_1 );
                pthread_mutex_unlock ( &library_mutex );
           }
 #endif
+          if (err) {
+               D_DEBUG( "DirectFB/FontFT2: "
+                        "Couldn't select Unicode/Latin1 encoding, "
+                        "trying Symbol.\n");
+               pthread_mutex_lock ( &library_mutex );
+               err = FT_Select_Charmap( face, ft_encoding_symbol );
+               pthread_mutex_unlock ( &library_mutex );
+
+               if (!err)
+                    mask = 0xf000;
+          }
      }
 
 #if 0
@@ -651,10 +819,17 @@ Construct( IDirectFBFont      *thiz,
           }
      }
 
-     face->generic.data = (void *) load_flags;
+     face->generic.data = (void *)(unsigned long) load_flags;
      face->generic.finalizer = NULL;
 
-     font = dfb_font_create( core );
+     ret = dfb_font_create( core, &font );
+     if (ret) {
+          pthread_mutex_lock ( &library_mutex );
+          FT_Done_Face( face );
+          pthread_mutex_unlock ( &library_mutex );
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
+          return ret;
+     }
 
      D_ASSERT( font->pixel_format == DSPF_ARGB ||
                font->pixel_format == DSPF_AiRGB ||
@@ -662,6 +837,7 @@ Construct( IDirectFBFont      *thiz,
                font->pixel_format == DSPF_ARGB2554 ||
                font->pixel_format == DSPF_ARGB1555 ||
                font->pixel_format == DSPF_A8 ||
+               font->pixel_format == DSPF_A4 ||
                font->pixel_format == DSPF_A1 );
 
      font->ascender   = face->size->metrics.ascender >> 6;
@@ -669,11 +845,10 @@ Construct( IDirectFBFont      *thiz,
      font->height     = font->ascender + ABS(font->descender) + 1;
      font->maxadvance = face->size->metrics.max_advance >> 6;
 
-     D_HEAVYDEBUG( "DirectFB/FontFT2: font->height = %d\n", font->height );
-     D_HEAVYDEBUG( "DirectFB/FontFT2: font->ascender = %d\n", font->ascender );
-     D_HEAVYDEBUG( "DirectFB/FontFT2: font->descender = %d\n",font->descender );
+     D_DEBUG( "DirectFB/FontFT2: height = %d, ascender = %d, descender = %d, maxadvance = %d\n",
+              font->height, font->ascender, font->descender, font->maxadvance );
 
-     font->GetGlyphInfo = get_glyph_info;
+     font->GetGlyphData = get_glyph_info;
      font->RenderGlyph  = render_glyph;
 
      if (FT_HAS_KERNING(face) && !disable_kerning) {
@@ -692,10 +867,19 @@ Construct( IDirectFBFont      *thiz,
      if (desc->flags & DFDESC_FIXEDADVANCE) {
           data->fixed_advance = desc->fixed_advance;
           font->maxadvance    = desc->fixed_advance;
+
+          if ((desc->flags & DFDESC_ATTRIBUTES) && (desc->attributes & DFFA_FIXEDCLIP))
+               data->fixed_clip = true;
      }
 
+     for (i=0; i<256; i++)
+          data->indices[i] = FT_Get_Char_Index( face, i | mask );
+
      font->impl_data = data;
 
+     dfb_font_register_encoding( font, "UTF8",   &ft2UTF8Funcs,   DTEID_UTF8 );
+     dfb_font_register_encoding( font, "Latin1", &ft2Latin1Funcs, DTEID_OTHER );
+
      IDirectFBFont_Construct( thiz, font );
 
      thiz->Release = IDirectFBFont_FT2_Release;
