@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -45,7 +45,7 @@
 #include <core/coretypes.h>
 
 #include <core/layers.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <misc/gfx_util.h>
 #include <direct/interface.h>
@@ -146,10 +146,10 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
 static bool ReadOK( IDirectFBDataBuffer *buffer, void *data, unsigned int len );
 
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_GIF_AddRef  ( IDirectFBImageProvider *thiz );
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_GIF_Release ( IDirectFBImageProvider *thiz );
 
 static DFBResult
@@ -240,7 +240,7 @@ IDirectFBImageProvider_GIF_Destruct( IDirectFBImageProvider *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_GIF_AddRef( IDirectFBImageProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
@@ -250,7 +250,7 @@ IDirectFBImageProvider_GIF_AddRef( IDirectFBImageProvider *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_GIF_Release( IDirectFBImageProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
@@ -267,12 +267,12 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
                                      IDirectFBSurface       *destination,
                                      const DFBRectangle     *dest_rect )
 {
+     DFBResult              ret;
      DFBRegion              clip;
      DFBRectangle           rect;
      DFBSurfacePixelFormat  format;
      IDirectFBSurface_data *dst_data;
      CoreSurface           *dst_surface;
-     int err;
 
      DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
 
@@ -297,23 +297,22 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
           rect = dst_data->area.wanted;
      }
 
-     err = destination->GetPixelFormat( destination, &format );
-     if (err)
-          return err;
+     ret = destination->GetPixelFormat( destination, &format );
+     if (ret)
+          return ret;
 
      /* actual loading and rendering */
      if (dfb_rectangle_region_intersects( &rect, &clip )) {
-          void  *dst;
-          int    pitch;
+          CoreSurfaceBufferLock lock;
 
-          err = dfb_surface_soft_lock( data->core, dst_surface, DSLF_WRITE, &dst, &pitch, 0 );
-          if (err)
-               return err;
+          ret = dfb_surface_lock_buffer( dst_surface, CSBR_BACK, CSAF_CPU_WRITE, &lock );
+          if (ret)
+               return ret;
 
           dfb_scale_linear_32( data->image, data->image_width, data->image_height,
-                               dst, pitch, &rect, dst_surface, &clip );
+                               lock.addr, lock.pitch, &rect, dst_surface, &clip );
 
-          dfb_surface_unlock( dst_surface, 0 );
+          dfb_surface_unlock_buffer( dst_surface, &lock );
 
           if (data->render_callback) {
                DIRenderCallbackResult r;
