@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -45,7 +45,7 @@
 #include <core/coretypes.h>
 
 #include <core/layers.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <misc/gfx_util.h>
 #include <misc/util.h>
@@ -54,10 +54,13 @@
 #include <direct/memcpy.h>
 #include <direct/messages.h>
 
-#include <jpeglib.h>
 #include <setjmp.h>
 #include <math.h>
 
+#undef HAVE_STDLIB_H
+#include <jpeglib.h>
+
+
 static DFBResult
 Probe( IDirectFBImageProvider_ProbeContext *ctx );
 
@@ -87,10 +90,10 @@ typedef struct {
      CoreDFB             *core;
 } IDirectFBImageProvider_JPEG_data;
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_JPEG_AddRef  ( IDirectFBImageProvider *thiz );
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_JPEG_Release ( IDirectFBImageProvider *thiz );
 
 static DFBResult
@@ -231,16 +234,47 @@ jpeglib_panic(j_common_ptr cinfo)
      longjmp(myerr->setjmp_buffer, 1);
 }
 
-
-static void
-copy_line32( u32 *dst, u8 *src, int width)
+static inline void
+copy_line32( u32 *argb, const u8 *rgb, int width )
 {
-     u32 r, g , b;
      while (width--) {
-          r = (*src++) << 16;
-          g = (*src++) << 8;
-          b = (*src++);
-          *dst++ = (0xFF000000 |r|g|b);
+          *argb++ = 0xFF000000 | (rgb[0] << 16) | (rgb[1] << 8) | rgb[2];
+
+          rgb += 3;
+     }
+}
+
+static inline void
+copy_line_nv16( u16 *yy, u16 *cbcr, const u8 *src_ycbcr, int width )
+{
+     int x;
+
+     for (x=0; x<width/2; x++) {
+#ifdef WORDS_BIGENDIAN
+          yy[x] = (src_ycbcr[0] << 8) | src_ycbcr[3];
+
+          cbcr[x] = (((src_ycbcr[1] + src_ycbcr[4]) << 7) & 0xff00) |
+                     ((src_ycbcr[2] + src_ycbcr[5]) >> 1);
+#else
+          yy[x] = (src_ycbcr[3] << 8) | src_ycbcr[0];
+
+          cbcr[x] = (((src_ycbcr[2] + src_ycbcr[5]) << 7) & 0xff00) |
+                     ((src_ycbcr[1] + src_ycbcr[4]) >> 1);
+#endif
+
+          src_ycbcr += 6;
+     }
+
+     if (width & 1) {
+          u8 *y = (u8*) yy;
+
+          y[width-1] = src_ycbcr[0];
+
+#ifdef WORDS_BIGENDIAN
+          cbcr[x] = (src_ycbcr[1] << 8) | src_ycbcr[2];
+#else
+          cbcr[x] = (src_ycbcr[2] << 8) | src_ycbcr[1];
+#endif
      }
 }
 
@@ -294,6 +328,7 @@ Construct( IDirectFBImageProvider *thiz,
 
           jpeg_destroy_decompress(&cinfo);
           buffer->Release( buffer );
+          DIRECT_DEALLOCATE_INTERFACE( thiz );
           return DFB_FAILURE;
      }
 
@@ -333,7 +368,7 @@ IDirectFBImageProvider_JPEG_Destruct( IDirectFBImageProvider *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_JPEG_AddRef( IDirectFBImageProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
@@ -343,7 +378,7 @@ IDirectFBImageProvider_JPEG_AddRef( IDirectFBImageProvider *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_JPEG_Release( IDirectFBImageProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
@@ -360,15 +395,14 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
                                       IDirectFBSurface       *destination,
                                       const DFBRectangle     *dest_rect )
 {
-     int                    err;
-     void                  *dst;
-     int                    pitch;
-     int                    direct = 0;
+     DFBResult              ret;
+     bool                   direct = false;
      DFBRegion              clip;
      DFBRectangle           rect;
      DFBSurfacePixelFormat  format;
      IDirectFBSurface_data *dst_data;
      CoreSurface           *dst_surface;
+     CoreSurfaceBufferLock  lock;
      DIRenderCallbackResult cb_result = DIRCR_OK;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_JPEG)
@@ -381,9 +415,9 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
      if (!dst_surface)
           return DFB_DESTROYED;
 
-     err = destination->GetPixelFormat( destination, &format );
-     if (err)
-          return err;
+     ret = destination->GetPixelFormat( destination, &format );
+     if (ret)
+          return ret;
 
      dfb_region_from_rectangle( &clip, &dst_data->area.current );
 
@@ -402,9 +436,9 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
           rect = dst_data->area.wanted;
      }
 
-     err = dfb_surface_soft_lock( data->core, dst_surface, DSLF_WRITE, &dst, &pitch, 0 );
-     if (err)
-          return err;
+     ret = dfb_surface_lock_buffer( dst_surface, CSBR_BACK, CSAF_CPU_WRITE, &lock );
+     if (ret)
+          return ret;
 
      /* actual loading and rendering */
      if (!data->image) {
@@ -414,6 +448,7 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
           int row_stride;         /* physical row width in output buffer */
           u32 *row_ptr;
           int y = 0;
+          int uv_offset = 0;
 
           cinfo.err = jpeg_std_error(&jerr.pub);
           jerr.pub.error_exit = jpeglib_panic;
@@ -425,8 +460,8 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
 
                if (data->image) {
                     dfb_scale_linear_32( data->image, data->width, data->height,
-                                         dst, pitch, &rect, dst_surface, &clip );
-                    dfb_surface_unlock( dst_surface, 0 );
+                                         lock.addr, lock.pitch, &rect, dst_surface, &clip );
+                    dfb_surface_unlock_buffer( dst_surface, &lock );
                     if (data->render_callback) {
                          DFBRectangle r = { 0, 0, data->width, data->height };
 
@@ -437,7 +472,7 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
                     return DFB_INCOMPLETE;
                }
                else
-                    dfb_surface_unlock( dst_surface, 0 );
+                    dfb_surface_unlock_buffer( dst_surface, &lock );
 
                return DFB_FAILURE;
           }
@@ -445,9 +480,31 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
           jpeg_create_decompress(&cinfo);
           jpeg_buffer_src(&cinfo, data->buffer, 0);
           jpeg_read_header(&cinfo, TRUE);
+          jpeg_calc_output_dimensions(&cinfo);
+
+          if (cinfo.output_width == rect.w && cinfo.output_height == rect.h)
+               direct = true;
 
-          cinfo.out_color_space = JCS_RGB;
           cinfo.output_components = 3;
+
+          switch (dst_surface->config.format) {
+               case DSPF_NV16:
+                    uv_offset = dst_surface->config.size.h * lock.pitch;
+
+                    if (direct && !rect.x && !rect.y) {
+                         D_INFO( "JPEG: Using YCbCr color space directly! (%dx%d)\n",
+                                 cinfo.output_width, cinfo.output_height );
+                         cinfo.out_color_space = JCS_YCbCr;
+                         break;
+                    }
+                    D_INFO( "JPEG: Going through RGB color space! (%dx%d -> %dx%d @%d,%d)\n",
+                            cinfo.output_width, cinfo.output_height, rect.w, rect.h, rect.x, rect.y );
+               
+               default:
+                    cinfo.out_color_space = JCS_RGB;
+                    break;
+          }
+
           jpeg_start_decompress(&cinfo);
 
           data->width = cinfo.output_width;
@@ -459,25 +516,45 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
                                               JPOOL_IMAGE, row_stride, 1);
 
           data->image = D_CALLOC( data->height, data->width * 4 );
-          if (!data->image)
+          if (!data->image) {
+               dfb_surface_unlock_buffer( dst_surface, &lock );
                return D_OOM();
+          }
           row_ptr = data->image;
 
-          direct = (rect.w == data->width && rect.h == data->height);
-
           while (cinfo.output_scanline < cinfo.output_height && cb_result == DIRCR_OK) {
                jpeg_read_scanlines(&cinfo, buffer, 1);
-               copy_line32( row_ptr, *buffer, data->width);
-               
-               if (direct) {
-                    DFBRectangle r = { rect.x, rect.y+y, rect.w, 1 };
-                    dfb_copy_buffer_32( row_ptr, dst, pitch,
-                                        &r, dst_surface, &clip );
-                    if (data->render_callback) {
-                         r = (DFBRectangle){ 0, y, data->width, 1 };
-                         cb_result = data->render_callback( &r, 
-                                             data->render_callback_context );
-                    }
+
+               switch (dst_surface->config.format) {
+                    case DSPF_NV16:
+                         if (direct) {
+                              copy_line_nv16( lock.addr, lock.addr + uv_offset, *buffer, rect.w );
+
+                              lock.addr += lock.pitch;
+
+                              if (data->render_callback) {
+                                   DFBRectangle r = { 0, y, data->width, 1 };
+
+                                   cb_result = data->render_callback( &r, 
+                                                                      data->render_callback_context );
+                              }
+                              break;
+                         }
+
+                    default:
+                         copy_line32( row_ptr, *buffer, data->width);
+
+                         if (direct) {
+                              DFBRectangle r = { rect.x, rect.y+y, rect.w, 1 };
+                              dfb_copy_buffer_32( row_ptr, lock.addr, lock.pitch,
+                                                  &r, dst_surface, &clip );
+                              if (data->render_callback) {
+                                   r = (DFBRectangle){ 0, y, data->width, 1 };
+                                   cb_result = data->render_callback( &r, 
+                                                                      data->render_callback_context );
+                              }
+                         }
+                         break;
                }
 
                row_ptr += data->width;
@@ -486,7 +563,7 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
 
           if (!direct) {
                dfb_scale_linear_32( data->image, data->width, data->height,
-                                    dst, pitch, &rect, dst_surface, &clip );
+                                    lock.addr, lock.pitch, &rect, dst_surface, &clip );
                if (data->render_callback) {
                     DFBRectangle r = { 0, 0, data->width, data->height };
                     cb_result = data->render_callback( &r, data->render_callback_context );
@@ -505,14 +582,14 @@ IDirectFBImageProvider_JPEG_RenderTo( IDirectFBImageProvider *thiz,
      }
      else {
           dfb_scale_linear_32( data->image, data->width, data->height,
-                               dst, pitch, &rect, dst_surface, &clip );
+                               lock.addr, lock.pitch, &rect, dst_surface, &clip );
           if (data->render_callback) {
                DFBRectangle r = { 0, 0, data->width, data->height };
                data->render_callback( &r, data->render_callback_context );
           }
      }
      
-     dfb_surface_unlock( dst_surface, 0 );
+     dfb_surface_unlock_buffer( dst_surface, &lock );
 
      if (cb_result != DIRCR_OK)
          return DFB_INTERRUPTED;
