@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -33,6 +34,7 @@
 #include <unistd.h>
 #include <png.h>
 #include <string.h>
+#include <stdarg.h>
 
 #include <directfb.h>
 
@@ -45,9 +47,13 @@
 
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <misc/gfx_util.h>
+#include <misc/util.h>
+
+#include <gfx/clip.h>
+#include <gfx/convert.h>
 
 #include <direct/interface.h>
 #include <direct/mem.h>
@@ -55,8 +61,6 @@
 #include <direct/messages.h>
 #include <direct/util.h>
 
-#include <misc/util.h>
-
 #include "config.h"
 
 static DFBResult
@@ -64,7 +68,7 @@ Probe( IDirectFBImageProvider_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBImageProvider *thiz,
-           IDirectFBDataBuffer    *buffer );
+           ... );
 
 #include <direct/interface_implementation.h>
 
@@ -72,6 +76,7 @@ DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, PNG )
 
 
 enum {
+     STAGE_ABORT = -2,
      STAGE_ERROR = -1,
      STAGE_START =  0,
      STAGE_INFO,
@@ -99,16 +104,21 @@ typedef struct {
      png_uint_32          color_key;
      bool                 color_keyed;
 
-     __u32               *image;
+     void                *image;
+     int                  pitch;
+     u32                  palette[256];
+     DFBColor             colors[256];
 
      DIRenderCallback     render_callback;
      void                *render_callback_context;
+
+     CoreDFB             *core;
 } IDirectFBImageProvider_PNG_data;
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_PNG_AddRef  ( IDirectFBImageProvider *thiz );
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_PNG_Release ( IDirectFBImageProvider *thiz );
 
 static DFBResult
@@ -153,6 +163,8 @@ push_data_until_stage (IDirectFBImageProvider_PNG_data *data,
                        int                              stage,
                        int                              buffer_size);
 
+/**********************************************************************************************************************/
+
 static DFBResult
 Probe( IDirectFBImageProvider_ProbeContext *ctx )
 {
@@ -164,14 +176,24 @@ Probe( IDirectFBImageProvider_ProbeContext *ctx )
 
 static DFBResult
 Construct( IDirectFBImageProvider *thiz,
-           IDirectFBDataBuffer    *buffer )
+           ... )
 {
      DFBResult ret = DFB_FAILURE;
 
+     IDirectFBDataBuffer *buffer;
+     CoreDFB             *core;
+     va_list              tag;
+
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_PNG)
 
+     va_start( tag, thiz );
+     buffer = va_arg( tag, IDirectFBDataBuffer * );
+     core = va_arg( tag, CoreDFB * );
+     va_end( tag );
+
      data->ref    = 1;
      data->buffer = buffer;
+     data->core   = core;
 
      /* Increase the data buffer reference counter. */
      buffer->AddRef( buffer );
@@ -182,6 +204,11 @@ Construct( IDirectFBImageProvider *thiz,
      if (!data->png_ptr)
           goto error;
 
+     if (setjmp( data->png_ptr->jmpbuf )) {
+          D_ERROR( "ImageProvider/PNG: Error reading header!\n" );
+          goto error;
+     }
+
      /* Create the PNG info handle. */
      data->info_ptr = png_create_info_struct( data->png_ptr );
      if (!data->info_ptr)
@@ -222,6 +249,8 @@ error:
      return ret;
 }
 
+/**********************************************************************************************************************/
+
 static void
 IDirectFBImageProvider_PNG_Destruct( IDirectFBImageProvider *thiz )
 {
@@ -240,7 +269,7 @@ IDirectFBImageProvider_PNG_Destruct( IDirectFBImageProvider *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_PNG_AddRef( IDirectFBImageProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
@@ -250,7 +279,7 @@ IDirectFBImageProvider_PNG_AddRef( IDirectFBImageProvider *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBImageProvider_PNG_Release( IDirectFBImageProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
@@ -262,18 +291,25 @@ IDirectFBImageProvider_PNG_Release( IDirectFBImageProvider *thiz )
      return DFB_OK;
 }
 
+/**********************************************************************************************************************/
+
 static DFBResult
 IDirectFBImageProvider_PNG_RenderTo( IDirectFBImageProvider *thiz,
                                      IDirectFBSurface       *destination,
                                      const DFBRectangle     *dest_rect )
 {
-     DFBResult              ret;
+     DFBResult              ret = DFB_OK;
      IDirectFBSurface_data *dst_data;
      CoreSurface           *dst_surface;
-     DFBRectangle           rect = { 0, 0, 0, 0 };
+     DFBRegion              clip;
+     DFBRectangle           rect;
+     png_infop              info;
+     int                    x, y;
 
      DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_PNG)
 
+     info = data->info_ptr;
+
      dst_data = (IDirectFBSurface_data*) destination->priv;
      if (!dst_data)
           return DFB_DEAD;
@@ -282,31 +318,178 @@ IDirectFBImageProvider_PNG_RenderTo( IDirectFBImageProvider *thiz,
      if (!dst_surface)
           return DFB_DESTROYED;
 
-     ret = destination->GetSize( destination, &rect.w, &rect.h );
-     if (ret)
-          return ret;
+     dfb_region_from_rectangle( &clip, &dst_data->area.current );
+
+     if (dest_rect) {
+          if (dest_rect->w < 1 || dest_rect->h < 1)
+               return DFB_INVARG;
+          rect = *dest_rect;
+          rect.x += dst_data->area.wanted.x;
+          rect.y += dst_data->area.wanted.y;
+     }
+     else {
+          rect = dst_data->area.wanted;
+     }
+
+     if (setjmp( data->png_ptr->jmpbuf )) {
+          D_ERROR( "ImageProvider/PNG: Error during decoding!\n" );
+
+          if (data->stage < STAGE_IMAGE)
+               return DFB_FAILURE;
+
+          data->stage = STAGE_ERROR;
+     }
 
      /* Read until image is completely decoded. */
-     ret = push_data_until_stage( data, STAGE_END, 16384 );
-     if (ret)
-          return ret;
+     if (data->stage != STAGE_ERROR) {
+          ret = push_data_until_stage( data, STAGE_END, 16384 );
+          if (ret)
+               return ret;
+     }
 
      /* actual rendering */
-     if (dest_rect == NULL || dfb_rectangle_intersect ( &rect, dest_rect )) {
-          void *dst;
-          int   pitch;
+     if (dfb_rectangle_region_intersects( &rect, &clip )) {
+          CoreSurfaceBufferLock lock;
 
-          ret = destination->Lock( destination, DSLF_WRITE, &dst, &pitch );
+          ret = dfb_surface_lock_buffer( dst_surface, CSBR_BACK, CSAF_CPU_WRITE, &lock );
           if (ret)
                return ret;
 
-          dfb_scale_linear_32( data->image, data->width, data->height,
-                               dst, pitch, &rect, dst_surface );
+          switch (data->color_type) {
+               case PNG_COLOR_TYPE_PALETTE:
+                    if (dst_surface->config.format == DSPF_LUT8 && data->info_ptr->bit_depth == 8) {
+                         /*
+                          * Special indexed PNG to LUT8 loading.
+                          */
 
-          destination->Unlock( destination );
+                         /* FIXME: Limitation for LUT8 is to load complete surface only. */
+                         dfb_clip_rectangle( &clip, &rect );
+                         if (rect.x == 0 && rect.y == 0 &&
+                             rect.w == dst_surface->config.size.w  &&
+                             rect.h == dst_surface->config.size.h &&
+                             rect.w == data->width         &&
+                             rect.h == data->height)
+                         {
+                              for (y=0; y<data->height; y++)
+                                   direct_memcpy( lock.addr + lock.pitch * y,
+                                                  data->image + data->pitch * y,
+                                                  data->width );
+
+                              break;
+                         }
+                    }
+                    /* fall through */
+
+               case PNG_COLOR_TYPE_GRAY: {
+                    /*
+                     * Convert to ARGB and use generic loading code.
+                     */
+
+                    // FIXME: allocates four additional bytes because the scaling functions
+                    //        in src/misc/gfx_util.c have an off-by-one bug which causes
+                    //        segfaults on darwin/osx (not on linux)                
+                    int size = data->width * data->height * 4 + 4;
+
+                    /* allocate image data */
+                    void *image_argb = D_MALLOC( size );
+
+                    if (!image_argb) {
+                         D_ERROR( "DirectFB/ImageProvider_PNG: Could not "
+                                  "allocate %d bytes of system memory!\n", size );
+                         ret = DFB_NOSYSTEMMEMORY;
+                    }
+                    else {
+                         if (data->color_type == PNG_COLOR_TYPE_GRAY) {
+                              int num = 1 << data->info_ptr->bit_depth;
+
+                              for (x=0; x<num; x++) {
+                                   int value = x * 255 / (num - 1);
+
+                                   data->palette[x] = 0xff000000 | (value << 16) | (value << 8) | value;
+                              }
+                         }
+
+                         switch (data->info_ptr->bit_depth) {
+                              case 8:
+                                   for (y=0; y<data->height; y++) {
+                                        u8  *S = data->image + data->pitch * y;
+                                        u32 *D = image_argb  + data->width * y * 4;
+
+                                        for (x=0; x<data->width; x++)
+                                             D[x] = data->palette[ S[x] ];
+                                   }
+                                   break;
+
+                              case 4:
+                                   for (y=0; y<data->height; y++) {
+                                        u8  *S = data->image + data->pitch * y;
+                                        u32 *D = image_argb  + data->width * y * 4;
+
+                                        for (x=0; x<data->width; x++) {
+                                             if (x & 1)
+                                                  D[x] = data->palette[ S[x>>1] & 0xf ];
+                                             else
+                                                  D[x] = data->palette[ S[x>>1] >> 4 ];
+                                        }
+                                   }
+                                   break;
+
+                              case 2:
+                                   for (y=0; y<data->height; y++) {
+                                        int  n = 6;
+                                        u8  *S = data->image + data->pitch * y;
+                                        u32 *D = image_argb  + data->width * y * 4;
+
+                                        for (x=0; x<data->width; x++) {
+                                             D[x] = data->palette[ (S[x>>2] >> n) & 3 ];
+
+                                             n = (n ? n - 2 : 6);
+                                        }
+                                   }
+                                   break;
+
+                              case 1:
+                                   for (y=0; y<data->height; y++) {
+                                        int  n = 7;
+                                        u8  *S = data->image + data->pitch * y;
+                                        u32 *D = image_argb  + data->width * y * 4;
+
+                                        for (x=0; x<data->width; x++) {
+                                             D[x] = data->palette[ (S[x>>3] >> n) & 1 ];
+
+                                             n = (n ? n - 1 : 7);
+                                        }
+                                   }
+                                   break;
+
+                              default:
+                                   D_ERROR( "ImageProvider/PNG: Unsupported indexed bit depth %d!\n",
+                                            data->info_ptr->bit_depth );
+                         }
+
+                         dfb_scale_linear_32( image_argb, data->width, data->height,
+                                              lock.addr, lock.pitch, &rect, dst_surface, &clip );
+ 
+                         D_FREE( image_argb );
+                    }
+                    break;
+               }
+               default:
+                    /*
+                     * Generic loading code.
+                     */
+                    dfb_scale_linear_32( data->image, data->width, data->height,
+                                         lock.addr, lock.pitch, &rect, dst_surface, &clip );
+                    break;
+          }
+
+          dfb_surface_unlock_buffer( dst_surface, &lock );
      }
 
-     return DFB_OK;
+     if (data->stage != STAGE_END)
+          ret = DFB_INCOMPLETE;
+
+     return ret;
 }
 
 static DFBResult
@@ -322,8 +505,6 @@ IDirectFBImageProvider_PNG_SetRenderCallback( IDirectFBImageProvider *thiz,
      return DFB_OK;
 }
 
-/* Loading routines */
-
 static DFBResult
 IDirectFBImageProvider_PNG_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                   DFBSurfaceDescription *dsc )
@@ -341,6 +522,13 @@ IDirectFBImageProvider_PNG_GetSurfaceDescription( IDirectFBImageProvider *thiz,
      else
           dsc->pixelformat = primary_format;
 
+     if (data->color_type == PNG_COLOR_TYPE_PALETTE) {
+          dsc->flags  |= DSDESC_PALETTE;
+
+          dsc->palette.entries = data->colors;  /* FIXME */
+          dsc->palette.size    = 256;
+     }
+
      return DFB_OK;
 }
 
@@ -369,29 +557,28 @@ IDirectFBImageProvider_PNG_GetImageDescription( IDirectFBImageProvider *thiz,
      return DFB_OK;
 }
 
+/**********************************************************************************************************************/
 
 #define MAXCOLORMAPSIZE 256
 
 static int SortColors (const void *a, const void *b)
 {
-     return (*((const __u8 *) a) - *((const __u8 *) b));
+     return (*((const u8 *) a) - *((const u8 *) b));
 }
 
 /*  looks for a color that is not in the colormap and ideally not
     even close to the colors used in the colormap  */
-static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
+static u32 FindColorKey( int n_colors, u8 *cmap )
 {
-     __u32 color = 0xFF000000;
-     __u8  csort[MAXCOLORMAPSIZE];
+     u32   color = 0xFF000000;
+     u8    csort[n_colors];
      int   i, j, index, d;
 
      if (n_colors < 1)
           return color;
 
-     D_ASSERT( n_colors <= MAXCOLORMAPSIZE );
-
      for (i = 0; i < 3; i++) {
-          direct_memcpy( csort, cmap[i], n_colors );
+          direct_memcpy( csort, cmap + (n_colors * i), n_colors );
           qsort( csort, n_colors, 1, SortColors );
 
           for (j = 1, index = 0, d = 0; j < n_colors; j++) {
@@ -423,9 +610,10 @@ static __u32 FindColorKey( int n_colors, __u8 cmap[3][MAXCOLORMAPSIZE] )
 
 /* Called at the start of the progressive load, once we have image info */
 static void
-png_info_callback   (png_structp png_read_ptr,
-                     png_infop   png_info_ptr)
+png_info_callback( png_structp png_read_ptr,
+                   png_infop   png_info_ptr )
 {
+     int                              i;
      IDirectFBImageProvider_PNG_data *data;
 
      data = png_get_progressive_ptr( png_read_ptr );
@@ -441,25 +629,17 @@ png_info_callback   (png_structp png_read_ptr,
                    &data->width, &data->height, &data->bpp, &data->color_type,
                    NULL, NULL, NULL );
 
-     if (data->color_type == PNG_COLOR_TYPE_PALETTE)
-          png_set_palette_to_rgb( data->png_ptr );
-
-     if (data->color_type == PNG_COLOR_TYPE_GRAY
-         || data->color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
-          png_set_gray_to_rgb( data->png_ptr );
-
      if (png_get_valid( data->png_ptr, data->info_ptr, PNG_INFO_tRNS )) {
           data->color_keyed = true;
 
           /* generate color key based on palette... */
           if (data->color_type == PNG_COLOR_TYPE_PALETTE) {
-               int        i;
-               __u32      key;
+               u32        key;
                png_colorp palette    = data->info_ptr->palette;
                png_bytep  trans      = data->info_ptr->trans;
                int        num_colors = MIN( MAXCOLORMAPSIZE,
                                             data->info_ptr->num_palette );
-               __u8       cmap[3][num_colors];
+               u8         cmap[3][num_colors];
 
                for (i=0; i<num_colors; i++) {
                     cmap[0][i] = palette[i].red;
@@ -467,12 +647,10 @@ png_info_callback   (png_structp png_read_ptr,
                     cmap[2][i] = palette[i].blue;
                }
 
-               key = FindColorKey( num_colors, cmap );
+               key = FindColorKey( num_colors, &cmap[0][0] );
 
                for (i=0; i<data->info_ptr->num_trans; i++) {
                     if (!trans[i]) {
-                         //trans[i] = 0;
-
                          palette[i].red   = (key & 0xff0000) >> 16;
                          palette[i].green = (key & 0x00ff00) >>  8;
                          palette[i].blue  = (key & 0x0000ff);
@@ -485,28 +663,66 @@ png_info_callback   (png_structp png_read_ptr,
                /* ...or based on trans rgb value */
                png_color_16p trans = &data->info_ptr->trans_values;
 
-               D_WARN("color key from non-palette source is untested");
-
                data->color_key = (((trans->red & 0xff00) << 8) |
                                   ((trans->green & 0xff00)) |
                                   ((trans->blue & 0xff00) >> 8));
           }
      }
 
-     if (data->bpp == 16)
-          png_set_strip_16( data->png_ptr );
+     switch (data->color_type) {
+          case PNG_COLOR_TYPE_PALETTE: {
+               png_colorp palette    = data->info_ptr->palette;
+               png_bytep  trans      = data->info_ptr->trans;
+               int        num_trans  = data->info_ptr->num_trans;
+               int        num_colors = MIN( MAXCOLORMAPSIZE, data->info_ptr->num_palette );
+
+               for (i=0; i<num_colors; i++) {
+                    data->colors[i].a = (i < num_trans) ? trans[i] : 0xff;
+                    data->colors[i].r = palette[i].red;
+                    data->colors[i].g = palette[i].green;
+                    data->colors[i].b = palette[i].blue;
+
+                    data->palette[i] = PIXEL_ARGB( data->colors[i].a,
+                                                   data->colors[i].r,
+                                                   data->colors[i].g,
+                                                   data->colors[i].b );
+               }
+
+               data->pitch = (data->width + 7) & ~7;
+               break;
+          }
+
+          case PNG_COLOR_TYPE_GRAY:
+               data->pitch = data->width;
+
+               if (data->bpp == 16)
+                    png_set_strip_16( data->png_ptr );
+
+               break;
+
+          case PNG_COLOR_TYPE_GRAY_ALPHA:
+               png_set_gray_to_rgb( data->png_ptr );
+               /* fall through */
+
+          default:
+               data->pitch = data->width * 4;
+
+               if (data->bpp == 16)
+                    png_set_strip_16( data->png_ptr );
 
 #ifdef WORDS_BIGENDIAN
-     if (!(data->color_type & PNG_COLOR_MASK_ALPHA))
-          png_set_filler( data->png_ptr, 0xFF, PNG_FILLER_BEFORE );
+               if (!(data->color_type & PNG_COLOR_MASK_ALPHA))
+                    png_set_filler( data->png_ptr, 0xFF, PNG_FILLER_BEFORE );
 
-     png_set_swap_alpha( data->png_ptr );
+               png_set_swap_alpha( data->png_ptr );
 #else
-     if (!(data->color_type & PNG_COLOR_MASK_ALPHA))
-          png_set_filler( data->png_ptr, 0xFF, PNG_FILLER_AFTER );
+               if (!(data->color_type & PNG_COLOR_MASK_ALPHA))
+                    png_set_filler( data->png_ptr, 0xFF, PNG_FILLER_AFTER );
 
-     png_set_bgr( data->png_ptr );
+               png_set_bgr( data->png_ptr );
 #endif
+               break;
+     }
 
      png_set_interlace_handling( data->png_ptr );
 
@@ -517,10 +733,10 @@ png_info_callback   (png_structp png_read_ptr,
 /* Called for each row; note that you will get duplicate row numbers
    for interlaced PNGs */
 static void
-png_row_callback   (png_structp png_read_ptr,
-                    png_bytep   new_row,
-                    png_uint_32 row_num,
-                    int         pass_num)
+png_row_callback( png_structp png_read_ptr,
+                  png_bytep   new_row,
+                  png_uint_32 row_num,
+                  int         pass_num )
 {
      IDirectFBImageProvider_PNG_data *data;
 
@@ -538,10 +754,10 @@ png_row_callback   (png_structp png_read_ptr,
           // FIXME: allocates four additional bytes because the scaling functions
           //        in src/misc/gfx_util.c have an off-by-one bug which causes
           //        segfaults on darwin/osx (not on linux)                
-          int size = data->width * data->height * 4 + 4 ;
+          int size = data->pitch * data->height + 4;
 
           /* allocate image data */
-          data->image = D_MALLOC( size );
+          data->image = D_CALLOC( 1, size );
           if (!data->image) {
                D_ERROR("DirectFB/ImageProvider_PNG: Could not "
                         "allocate %d bytes of system memory!\n", size);
@@ -555,15 +771,18 @@ png_row_callback   (png_structp png_read_ptr,
 
      /* write to image data */
      png_progressive_combine_row( data->png_ptr, (png_bytep) (data->image +
-                                  row_num * data->width), new_row );
+                                  row_num * data->pitch), new_row );
 
      /* increase row counter, FIXME: interlaced? */
      data->rows++;
 
      if (data->render_callback) {
+          DIRenderCallbackResult r;
           DFBRectangle rect = { 0, row_num, data->width, 1 };
 
-          data->render_callback( &rect, data->render_callback_context );
+          r = data->render_callback( &rect, data->render_callback_context );
+          if (r != DIRCR_OK)
+              data->stage = STAGE_ABORT;
      }
 }
 
@@ -614,8 +833,13 @@ push_data_until_stage (IDirectFBImageProvider_PNG_data *data,
                D_DEBUG( "ImageProvider/PNG: ...processed %d bytes.\n", len );
 
                /* are we there yet? */
-               if (data->stage < 0 || data->stage >= stage)
-                    return DFB_OK;
+               if (data->stage < 0 || data->stage >= stage) {
+                   switch (data->stage) {
+                        case STAGE_ABORT: return DFB_INTERRUPTED;
+                        case STAGE_ERROR: return DFB_FAILURE;
+                        default:          return DFB_OK;
+                   }
+               }
           }
 
           D_DEBUG( "ImageProvider/PNG: Waiting for data...\n" );
