@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -47,7 +47,7 @@
 
 #include <idirectfb.h>
 
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <display/idirectfbsurface.h>
 
@@ -662,7 +662,7 @@ IDirectFBVideoProvider_GIF_Destruct( IDirectFBVideoProvider *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBVideoProvider_GIF_AddRef( IDirectFBVideoProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA( IDirectFBVideoProvider_GIF )
@@ -672,7 +672,7 @@ IDirectFBVideoProvider_GIF_AddRef( IDirectFBVideoProvider *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBVideoProvider_GIF_Release( IDirectFBVideoProvider *thiz )
 {
      DIRECT_INTERFACE_GET_DATA( IDirectFBVideoProvider_GIF )
@@ -746,11 +746,11 @@ GIFVideo( DirectThread *self, void *arg )
      pthread_setcancelstate( PTHREAD_CANCEL_DISABLE, NULL );
      
      while (!direct_thread_is_canceled( self )) {
-          DFBResult       ret;
-          DFBRectangle    rect;
-          DFBRegion       clip;
-          void           *dst;
-          int             pitch;
+          DFBResult              ret;
+          DFBRectangle           rect;
+          DFBRegion              clip;
+          CoreSurface           *surface;
+          CoreSurfaceBufferLock  lock;
           
           pthread_mutex_lock( &data->lock );
           
@@ -780,14 +780,16 @@ GIFVideo( DirectThread *self, void *arg )
                  ? data->dst_data->area.wanted : data->dst_rect;          
           dfb_region_from_rectangle( &clip, &data->dst_data->area.current );
           
+          surface = data->dst_data->surface;
+          D_MAGIC_ASSERT( surface, CoreSurface );
+
           if (dfb_rectangle_region_intersects( &rect, &clip ) &&
-              dfb_surface_soft_lock( data->dst_data->core, data->dst_data->surface,
-                                     DSLF_WRITE, &dst, &pitch, 0 ) == DFB_OK)
+              dfb_surface_lock_buffer( surface, CSBR_BACK, CSAF_CPU_WRITE, &lock ) == DFB_OK)
           {
                dfb_scale_linear_32( data->image, data->Width, data->Height,
-                                    dst, pitch, &rect, data->dst_data->surface, &clip );
+                                    lock.addr, lock.pitch, &rect, data->dst_data->surface, &clip );
                                     
-               dfb_surface_unlock( data->dst_data->surface, 0 );
+               dfb_surface_unlock_buffer( surface, &lock );
                
                if (data->callback)
                     data->callback( data->callback_ctx );
