@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -30,10 +31,13 @@
 
 #include <sys/time.h>
 
-long long direct_clock_get_micros();
-long long direct_clock_get_millis();
+long long direct_clock_get_micros( void );
+long long direct_clock_get_millis( void );
 
 void      direct_clock_set_start( const struct timeval *start );
 
+long long direct_clock_get_abs_micros( void );
+long long direct_clock_get_abs_millis( void );
+
 #endif
 
