@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -31,10 +31,19 @@
 
 #include <direct/types.h>
 
-
 DirectResult direct_initialize( void );
 DirectResult direct_shutdown( void );
 
 
+typedef void (*DirectCleanupHandlerFunc)( void *ctx );
+
+
+DirectResult direct_cleanup_handler_add( DirectCleanupHandlerFunc   func,
+                                         void                      *ctx,
+                                         DirectCleanupHandler     **ret_handler );
+
+DirectResult direct_cleanup_handler_remove( DirectCleanupHandler   *handler );
+
+
 #endif
 
