@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -41,8 +42,8 @@ D_DEBUG_DOMAIN( Direct_Hash, "Direct/Hash", "Hash table implementation" );
 #define REMOVED  ((void *) -1)
 
 typedef struct {
-     __u32     key;
-     void     *value;
+     unsigned long  key;
+     void          *value;
 } Element;
 
 struct __D_DirectHash {
@@ -59,7 +60,7 @@ struct __D_DirectHash {
 /**************************************************************************************************/
 
 static inline int
-locate_key( const DirectHash *hash, __u32 key )
+locate_key( const DirectHash *hash, unsigned long key )
 {
      int            pos;
      const Element *element;
@@ -97,7 +98,7 @@ direct_hash_create( int          size,
      hash = D_CALLOC( 1, sizeof (DirectHash) );
      if (!hash) {
           D_WARN( "out of memory" );
-          return DFB_NOSYSTEMMEMORY;
+          return DR_NOLOCALMEMORY;
      }
 
      hash->size     = size;
@@ -106,14 +107,14 @@ direct_hash_create( int          size,
      if (!hash->elements) {
           D_WARN( "out of memory" );
           D_FREE( hash );
-          return DFB_NOSYSTEMMEMORY;
+          return DR_NOLOCALMEMORY;
      }
 
      D_MAGIC_SET( hash, DirectHash );
 
      *ret_hash = hash;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 void
@@ -128,9 +129,9 @@ direct_hash_destroy( DirectHash *hash )
 }
 
 DirectResult
-direct_hash_insert( DirectHash *hash,
-                    __u32       key,
-                    void       *value )
+direct_hash_insert( DirectHash    *hash,
+                    unsigned long  key,
+                    void          *value )
 {
      int      pos;
      Element *element;
@@ -149,7 +150,7 @@ direct_hash_insert( DirectHash *hash,
           elements = D_CALLOC( size, sizeof(Element) );
           if (!elements) {
                D_WARN( "out of memory" );
-               return DFB_NOSYSTEMMEMORY;
+               return DR_NOLOCALMEMORY;
           }
 
           for (i=0; i<hash->size; i++) {
@@ -170,6 +171,8 @@ direct_hash_insert( DirectHash *hash,
                }
           }
 
+          D_FREE( hash->elements );
+
           hash->size     = size;
           hash->elements = elements;
           hash->removed  = 0;
@@ -177,14 +180,14 @@ direct_hash_insert( DirectHash *hash,
 
      pos = key % hash->size;
 
-     D_DEBUG_AT( Direct_Hash, "Attempting to insert key 0x%08x at position %d...\n", key, pos );
+     D_DEBUG_AT( Direct_Hash, "Attempting to insert key 0x%08lx at position %d...\n", key, pos );
 
      element = &hash->elements[pos];
 
      while (element->value && element->value != REMOVED) {
           if (element->key == key) {
                D_BUG( "key already exists" );
-               return DFB_BUG;
+               return DR_BUG;
           }
 
           if (++pos == hash->size)
@@ -201,15 +204,15 @@ direct_hash_insert( DirectHash *hash,
 
      hash->count++;
 
-     D_DEBUG_AT( Direct_Hash, "...inserted at %d, new count = %d, removed = %d, size = %d, key = 0x%08x.\n",
+     D_DEBUG_AT( Direct_Hash, "...inserted at %d, new count = %d, removed = %d, size = %d, key = 0x%08lx.\n",
                  pos, hash->count, hash->removed, hash->size, key );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 void
-direct_hash_remove( DirectHash  *hash,
-                    __u32        key )
+direct_hash_remove( DirectHash    *hash,
+                    unsigned long  key )
 {
      int pos;
 
@@ -226,13 +229,13 @@ direct_hash_remove( DirectHash  *hash,
      hash->count--;
      hash->removed++;
 
-     D_DEBUG_AT( Direct_Hash, "Removed key 0x%08x at %d, new count = %d, removed = %d, size = %d.\n",
+     D_DEBUG_AT( Direct_Hash, "Removed key 0x%08lx at %d, new count = %d, removed = %d, size = %d.\n",
                  key, pos, hash->count, hash->removed, hash->size );
 }
 
 void *
-direct_hash_lookup( DirectHash *hash,
-                    __u32       key )
+direct_hash_lookup( DirectHash    *hash,
+                    unsigned long  key )
 {
      int pos;
 
