@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -37,11 +37,11 @@ static const char   *GetImplementation( void );
 static DirectResult  Allocate( void **interface );
 
 static DirectInterfaceFuncs interface_funcs = {
-     GetType:            GetType,
-     GetImplementation:  GetImplementation,
-     Allocate:           Allocate,
-     Probe:              (DirectInterfaceGenericProbeFunc) Probe,
-     Construct:          (DirectInterfaceGenericConstructFunc) Construct
+     .GetType            = GetType,
+     .GetImplementation  = GetImplementation,
+     .Allocate           = Allocate,
+     .Probe              = (DirectInterfaceGenericProbeFunc) Probe,
+     .Construct          = (DirectInterfaceGenericConstructFunc) Construct
 };
 
 #define DIRECT_INTERFACE_IMPLEMENTATION(type, impl)    \
@@ -62,15 +62,29 @@ static DirectResult                                    \
 Allocate( void **interface )                           \
 {                                                      \
      DIRECT_ALLOCATE_INTERFACE( *interface, type );    \
-     return DFB_OK;                                    \
+     return DR_OK;                                     \
 }                                                      \
                                                        \
-__attribute__((constructor)) void type##_##impl(void); \
-                                                       \
+__attribute__((constructor))                           \
 void                                                   \
-type##_##impl(void)                                    \
+type##_##impl##_ctor(void);                            \
+                                                       \
+__attribute__((constructor))                           \
+void                                                   \
+type##_##impl##_ctor(void)                             \
 {                                                      \
      DirectRegisterInterface( &interface_funcs );      \
+}                                                      \
+                                                       \
+__attribute__((destructor))                            \
+void                                                   \
+type##_##impl##_dtor(void);                            \
+                                                       \
+__attribute__((destructor))                            \
+void                                                   \
+type##_##impl##_dtor(void)                             \
+{                                                      \
+     DirectUnregisterInterface( &interface_funcs );    \
 }
 
 #endif
