@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -32,7 +33,7 @@
 
 #include <direct/build.h>
 
-void direct_print_memleaks();
+void direct_print_memleaks( void );
 
 
 void  direct_free   ( const char *file, int line,
@@ -52,7 +53,7 @@ char *direct_strdup ( const char *file, int line,
                       const char *func, const char *string );
 
 
-#if DIRECT_BUILD_DEBUG || defined(DIRECT_FORCE_DEBUG)
+#if DIRECT_BUILD_DEBUG || defined(DIRECT_ENABLE_DEBUG) || defined(DIRECT_FORCE_DEBUG)
 
 #if !DIRECT_BUILD_DEBUGS
 #warning Building with debug, but library headers suggest that debug is not supported.
