@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -40,8 +40,8 @@ extern void *(*direct_memcpy)( void *to, const void *from, size_t len );
 
 static __inline__ void *direct_memmove( void *to, const void *from, size_t len )
 {
-     if ((from < to && ((char*) from + len) < ((char*) to)) ||
-         (((char*) to + len) < ((char*) from)))
+     if ((from < to && ((const char*) from + len) < ((char*) to)) ||
+         (((char*) to + len) < ((const char*) from)))
           return direct_memcpy( to, from, len );
      else
           return memmove( to, from, len );
