@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -294,10 +294,10 @@ net_wait( DirectStream   *stream,
      fd_set s;
 
      if (stream->cache_size >= length)
-          return DFB_OK;
+          return DR_OK;
 
      if (stream->fd == -1)
-          return DFB_EOF;
+          return DR_EOF;
      
      FD_ZERO( &s );
      FD_SET( stream->fd, &s );
@@ -305,13 +305,13 @@ net_wait( DirectStream   *stream,
      switch (select( stream->fd+1, &s, NULL, NULL, tv )) {
           case 0:
                if (!tv && !stream->cache_size)
-                    return DFB_EOF;
-               return DFB_TIMEOUT;
+                    return DR_EOF;
+               return DR_TIMEOUT;
           case -1:
                return errno2result( errno );
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -325,21 +325,21 @@ net_peek( DirectStream *stream,
      int   size;
 
      if (offset < 0)
-          return DFB_UNSUPPORTED;
+          return DR_UNSUPPORTED;
 
      tmp = alloca( length + offset );
      
      size = recv( stream->fd, tmp, length+offset, MSG_PEEK );
      switch (size) {
           case 0:
-               return DFB_EOF;
+               return DR_EOF;
           case -1:
                if (errno == EAGAIN || errno == EWOULDBLOCK)
-                    return DFB_BUFFEREMPTY;
+                    return DR_BUFFEREMPTY;
                return errno2result( errno );
           default:
                if (size < offset)
-                    return DFB_BUFFEREMPTY;
+                    return DR_BUFFEREMPTY;
                size -= offset;
                break;
      }
@@ -349,7 +349,7 @@ net_peek( DirectStream *stream,
      if (read_out)
           *read_out = size;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -363,10 +363,10 @@ net_read( DirectStream *stream,
      size = recv( stream->fd, buf, length, 0 );
      switch (size) {
           case 0:
-               return DFB_EOF;
+               return DR_EOF;
           case -1:
                if (errno == EAGAIN || errno == EWOULDBLOCK)
-                    return DFB_BUFFEREMPTY;
+                    return DR_BUFFEREMPTY;
                return errno2result( errno );
      }
 
@@ -375,13 +375,13 @@ net_read( DirectStream *stream,
      if (read_out)
           *read_out = size;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
 net_connect( struct addrinfo *addr, int sock, int proto, int *ret_fd )
 {
-     DirectResult     ret = DFB_OK;
+     DirectResult     ret = DR_OK;
      int              fd  = -1;
      struct addrinfo *tmp;
 
@@ -404,12 +404,45 @@ net_connect( struct addrinfo *addr, int sock, int proto, int *ret_fd )
 
           D_DEBUG_AT( Direct_Stream, 
                       "connecting to %s...\n", tmp->ai_canonname );
- 
-          err = connect( fd, tmp->ai_addr, tmp->ai_addrlen );
+
+          if (proto == IPPROTO_UDP)
+               err = bind( fd, tmp->ai_addr, tmp->ai_addrlen );
+          else
+               err = connect( fd, tmp->ai_addr, tmp->ai_addrlen );
+
           if (err == 0 || errno == EINPROGRESS) {
                struct timeval t = { NET_TIMEOUT, 0 };
                fd_set         s;
 
+               /* Join multicast group? */
+               if (tmp->ai_addr->sa_family == AF_INET) {
+                    struct sockaddr_in *saddr = (struct sockaddr_in *) tmp->ai_addr;
+
+                    if (IN_MULTICAST( ntohl(saddr->sin_addr.s_addr) )) {
+                         struct ip_mreq req;
+
+                         D_DEBUG_AT( Direct_Stream, 
+                                     "joining multicast group (%u.%u.%u.%u)...\n",
+                                     (u8)tmp->ai_addr->sa_data[2], (u8)tmp->ai_addr->sa_data[3],
+                                     (u8)tmp->ai_addr->sa_data[4], (u8)tmp->ai_addr->sa_data[5] );
+
+                         req.imr_multiaddr.s_addr = saddr->sin_addr.s_addr;
+                         req.imr_interface.s_addr = 0;
+
+                         err = setsockopt( fd, SOL_IP, IP_ADD_MEMBERSHIP, &req, sizeof(req) );
+                         if (err < 0) {
+                              ret = errno2result( errno );
+                              D_PERROR( "Direct/Stream: Could not join multicast group (%u.%u.%u.%u)!\n",
+                                        (u8)tmp->ai_addr->sa_data[2], (u8)tmp->ai_addr->sa_data[3],
+                                        (u8)tmp->ai_addr->sa_data[4], (u8)tmp->ai_addr->sa_data[5] );
+                              close( fd );
+                              continue;
+                         }
+
+                         setsockopt( fd, SOL_SOCKET, SO_REUSEADDR, saddr, sizeof(*saddr) );
+                    }
+               }
+
                FD_ZERO( &s );
                FD_SET( fd, &s );
               
@@ -421,7 +454,7 @@ net_connect( struct addrinfo *addr, int sock, int proto, int *ret_fd )
                     fd = -1;
 
                     if (err == 0) {
-                         ret = DFB_TIMEOUT;
+                         ret = DR_TIMEOUT;
                          continue;
                     } else {
                          ret = errno2result( errno );
@@ -431,7 +464,7 @@ net_connect( struct addrinfo *addr, int sock, int proto, int *ret_fd )
 
                D_DEBUG_AT( Direct_Stream, "...connected.\n" );
                
-               ret = DFB_OK;
+               ret = DR_OK;
                break;
           }
      }
@@ -444,7 +477,7 @@ net_connect( struct addrinfo *addr, int sock, int proto, int *ret_fd )
 static DirectResult
 net_open( DirectStream *stream, const char *filename, int proto )
 {
-     DirectResult    ret  = DFB_OK;
+     DirectResult    ret  = DR_OK;
      int             sock = (proto == IPPROTO_TCP) ? SOCK_STREAM : SOCK_DGRAM;
      struct addrinfo hints;
      char            port[16];
@@ -467,7 +500,7 @@ net_open( DirectStream *stream, const char *filename, int proto )
                       &hints, &stream->remote.addr )) {
           D_ERROR( "Direct/Stream: "
                    "failed to resolve host '%s'!\n", stream->remote.host );
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
      ret = net_connect( stream->remote.addr, sock, proto, &stream->remote.sd );
@@ -529,13 +562,13 @@ http_seek( DirectStream *stream, unsigned int offset )
                if (status)
                     D_DEBUG_AT( Direct_Stream,
                                 "server returned status %d.\n", status );
-               return DFB_FAILURE;
+               return DR_FAILURE;
      }
 
      /* discard remaining headers */
      while (net_response( stream, buf, sizeof(buf) ) > 0);
 
-     return DFB_OK;
+     return DR_OK;
 }    
 
 static DirectResult
@@ -592,9 +625,13 @@ http_open( DirectStream *stream, const char *filename )
                     stream->seek = http_seek;
           }
           else if (!strncasecmp( buf, "Content-Type:", 13 )) {
+               char *mime = trim( buf+13 );
+               char *tmp  = strchr( mime, ';' );
+               if (tmp)
+                    *tmp = '\0';
                if (stream->mime)
                     D_FREE( stream->mime );
-               stream->mime = D_STRDUP( trim( buf+13 ) );
+               stream->mime = D_STRDUP( mime );
           }
           else if (!strncasecmp( buf, "Content-Length:", 15 )) {
                char *tmp = trim( buf+15 );
@@ -609,7 +646,7 @@ http_open( DirectStream *stream, const char *filename )
                     D_ERROR( "Direct/Stream: "
                              "reached maximum number of redirects (%d).\n",
                              HTTP_MAX_REDIRECTS );
-                    return DFB_LIMITEXCEEDED;
+                    return DR_LIMITEXCEEDED;
                }
                
                filename = trim( buf+9 );
@@ -620,7 +657,7 @@ http_open( DirectStream *stream, const char *filename )
                if (!strncmp( filename, "rtsp://", 7 ))
                     return rtsp_open( stream, filename+7 );
                
-               return DFB_UNSUPPORTED;
+               return DR_UNSUPPORTED;
           }
      }
 
@@ -631,10 +668,10 @@ http_open( DirectStream *stream, const char *filename )
                if (status)
                     D_DEBUG_AT( Direct_Stream,
                                 "server returned status %d.\n", status );
-               return (status == 404) ? DFB_FILENOTFOUND : DFB_FAILURE;
+               return (status == 404) ? DR_FILENOTFOUND : DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 /*****************************************************************************/
@@ -647,7 +684,7 @@ ftp_open_pasv( DirectStream *stream, char *buf, size_t size )
 
      snprintf( buf, size, "PASV" );
      if (net_command( stream, buf, size ) != 227)
-          return DFB_FAILURE;
+          return DR_FAILURE;
 
      /* parse IP and port for passive mode */
      for (i = 4; buf[i]; i++) {
@@ -674,7 +711,7 @@ ftp_open_pasv( DirectStream *stream, char *buf, size_t size )
                if (getaddrinfo( buf, buf+len+1, &hints, &addr )) {
                     D_DEBUG_AT( Direct_Stream, 
                                 "failed to resolve host '%s'.\n", buf );
-                    return DFB_FAILURE;
+                    return DR_FAILURE;
                }
                     
                ret = net_connect( addr, SOCK_STREAM, IPPROTO_TCP, &stream->fd );
@@ -685,13 +722,13 @@ ftp_open_pasv( DirectStream *stream, char *buf, size_t size )
           }
      }
 
-     return DFB_FAILURE;
+     return DR_FAILURE;
 }
 
 static DirectResult
 ftp_seek( DirectStream *stream, unsigned int offset )
 {
-     DirectResult ret = DFB_OK;
+     DirectResult ret = DR_OK;
      char         buf[512];
     
      if (stream->fd > 0) {
@@ -726,13 +763,13 @@ ftp_seek( DirectStream *stream, unsigned int offset )
 
      stream->offset = offset;
 
-     return DFB_OK;
+     return DR_OK;
 
 error:
      close( stream->fd );
      stream->fd = -1;
      
-     return DFB_FAILURE;
+     return DR_FAILURE;
 }
 
 static DirectResult
@@ -753,7 +790,7 @@ ftp_open( DirectStream *stream, const char *filename )
                break;
      }
      if (status != 220)
-          return DFB_FAILURE;
+          return DR_FAILURE;
      
      /* login */
      snprintf( buf, sizeof(buf), "USER %s", stream->remote.user ? : "anonymous" );
@@ -762,19 +799,19 @@ ftp_open( DirectStream *stream, const char *filename )
           case 331:
                break;
           default:
-               return DFB_FAILURE;
+               return DR_FAILURE;
      }
 
      if (stream->remote.pass) {
           snprintf( buf, sizeof(buf), "PASS %s", stream->remote.pass );
           if (net_command( stream, buf, sizeof(buf) ) != 230)
-               return DFB_FAILURE;
+               return DR_FAILURE;
      }
      
      /* enter binary mode */
      snprintf( buf, sizeof(buf), "TYPE I" );
      if (net_command( stream, buf, sizeof(buf) ) != 200)
-          return DFB_FAILURE;
+          return DR_FAILURE;
 
      /* get file size */
      snprintf( buf, sizeof(buf), "SIZE %s", stream->remote.path );
@@ -793,12 +830,12 @@ ftp_open( DirectStream *stream, const char *filename )
           case 150:
                break;
           default:
-               return DFB_FAILURE;
+               return DR_FAILURE;
      }
 
      stream->seek = ftp_seek;
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 /*****************************************************************************/
@@ -988,7 +1025,7 @@ sdp_parse( DirectStream *stream, int length, SDPStreamDesc **ret_streams, int *r
      *ret_streams = desc;
      *ret_num     = num;
 
-     return desc ? DFB_OK : DFB_EOF;
+     return desc ? DR_OK : DR_EOF;
 }
 
 static void
@@ -1144,7 +1181,7 @@ rmf_write_header( SDPStreamDesc *streams, int n_streams, void **ret_buf, unsigne
      dst[10] = dst[11] = dst[12] = dst[13] = 0; // num packets
      dst[14] = dst[15] = dst[16] = dst[17] = 0; // next data 
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 static int
@@ -1244,7 +1281,7 @@ rtsp_session_open( DirectStream *stream )
                ++cseq, DIRECTFB_VERSION );
      
      if (net_command( stream, buf, sizeof(buf) ) != 200)
-          return DFB_FAILURE;
+          return DR_FAILURE;
      
      while (net_response( stream, buf, sizeof(buf) ) > 0) {
           if (!strncmp( buf, "RealChallenge1:", 15 )) {
@@ -1273,7 +1310,7 @@ rtsp_session_open( DirectStream *stream )
      
      status = net_command( stream, buf, sizeof(buf) );
      if (status != 200)
-          return (status == 404) ? DFB_FILENOTFOUND : DFB_FAILURE;
+          return (status == 404) ? DR_FILENOTFOUND : DR_FAILURE;
      
      len = 0;
      while (net_response( stream, buf, sizeof(buf) ) > 0) {
@@ -1289,7 +1326,7 @@ rtsp_session_open( DirectStream *stream )
      
      if (!len) {
           D_DEBUG_AT( Direct_Stream, "Couldn't get sdp length!\n" );
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }       
          
      ret = sdp_parse( stream, len, &streams, &n_streams );
@@ -1331,7 +1368,7 @@ rtsp_session_open( DirectStream *stream )
 
           if (net_command( stream, buf, sizeof(buf) ) != 200) {
                sdp_free( streams, n_streams );
-               return DFB_FAILURE;
+               return DR_FAILURE;
           }
           
           while (net_response( stream, buf, sizeof(buf) ) > 0) {
@@ -1357,7 +1394,7 @@ rtsp_session_open( DirectStream *stream )
      
      if (net_command( stream, buf, sizeof(buf) ) != 200) {
           sdp_free( streams, n_streams );
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
      
      /* discard remaining headers */
@@ -1409,7 +1446,7 @@ rtsp_session_open( DirectStream *stream )
 
      sdp_free( streams, n_streams );
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1425,21 +1462,21 @@ rvp_read_packet( DirectStream *stream )
           do {
                size = recv( stream->fd, buf, 1, MSG_WAITALL );
                if (size < 1)
-                    return DFB_EOF;
+                    return DR_EOF;
           } while (buf[0] != '$');
      
           size = recv( stream->fd, buf, 7, MSG_WAITALL );
           if (size < 7)
-               return DFB_EOF;
+               return DR_EOF;
           
           len = (buf[0] << 16) + (buf[1] << 8) + buf[2]; 
           id = buf[3];
           if (id != 0x40 && id != 0x42) {
                if (buf[5] == 0x06) // EOS
-                    return DFB_EOF;
+                    return DR_EOF;
                size = recv( stream->fd, buf, 9, MSG_WAITALL );
                if (size < 9)
-                    return DFB_EOF;
+                    return DR_EOF;
                id = buf[5];
                len -= 9;
           }
@@ -1447,7 +1484,7 @@ rvp_read_packet( DirectStream *stream )
           
           size = recv( stream->fd, buf, 6, MSG_WAITALL );
           if (size < 6)
-               return DFB_EOF;
+               return DR_EOF;
           ts = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
                
           len -= 10;
@@ -1466,7 +1503,7 @@ rvp_read_packet( DirectStream *stream )
                while (len) {
                     size = recv( stream->fd, dst, len, MSG_WAITALL );
                     if (size < 1)
-                         return DFB_EOF;
+                         return DR_EOF;
                     dst += size;
                     len -= size;
                }
@@ -1474,7 +1511,7 @@ rvp_read_packet( DirectStream *stream )
           }
      }
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1495,12 +1532,12 @@ rtp_read_packet( DirectStream *stream )
           do {
                size = recv( stream->fd, buf, 1, MSG_WAITALL );
                if (size < 1)
-                    return DFB_EOF;
+                    return DR_EOF;
           } while (buf[0] != '$');
      
           size = recv( stream->fd, buf, 3, MSG_WAITALL );
           if (size < 3)
-               return DFB_EOF;
+               return DR_EOF;
           
           id = buf[0];
           len = (buf[1] << 8) | buf[2];
@@ -1509,7 +1546,7 @@ rtp_read_packet( DirectStream *stream )
      
           size = recv( stream->fd, buf, 12, MSG_WAITALL );
           if (size < 12)
-               return DFB_EOF;
+               return DR_EOF;
           len -= 12;
 
           if ((buf[0] & 0xc0) != (2 << 6))
@@ -1532,7 +1569,7 @@ rtp_read_packet( DirectStream *stream )
                case 32: // MPEG Video
                     size = recv( stream->fd, buf, 1, MSG_WAITALL );
                     if (size < 1)
-                         return DFB_EOF;
+                         return DR_EOF;
                     len--;
                     skip = 3;
                     if (buf[0] & (1 << 2))
@@ -1541,7 +1578,7 @@ rtp_read_packet( DirectStream *stream )
                case 34: // H263
                     size = recv( stream->fd, buf, 1, MSG_WAITALL );
                     if (size < 1)
-                         return DFB_EOF;
+                         return DR_EOF;
                     len--;
                     skip = 3;
                     if (buf[0] & (1 << 7))
@@ -1557,7 +1594,7 @@ rtp_read_packet( DirectStream *stream )
           while (skip) {
                size = recv( stream->fd, buf, MIN(skip,12), MSG_WAITALL );
                if (size < 1)
-                    return DFB_EOF;
+                    return DR_EOF;
                len -= size;
                skip -= size;
           }
@@ -1595,7 +1632,7 @@ rtp_read_packet( DirectStream *stream )
                while (len) {
                     size = recv( stream->fd, dst, len, MSG_WAITALL );
                     if (size < 1)
-                         return DFB_EOF;
+                         return DR_EOF;
                     dst += size;
                     len -= size;
                }
@@ -1603,7 +1640,7 @@ rtp_read_packet( DirectStream *stream )
           }
      }
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1617,7 +1654,7 @@ rtsp_peek( DirectStream *stream,
      unsigned int len;
 
      if (offset < 0)
-          return DFB_UNSUPPORTED;
+          return DR_UNSUPPORTED;
 
      len = length + offset;
      while (len > stream->cache_size) {
@@ -1637,7 +1674,7 @@ rtsp_peek( DirectStream *stream,
      if (read_out)
           *read_out = len;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1683,7 +1720,7 @@ rtsp_read( DirectStream *stream,
      if (read_out)
           *read_out = size;
 
-     return DFB_OK;
+     return DR_OK;
 }           
  
 static DirectResult 
@@ -1706,7 +1743,7 @@ rtsp_open( DirectStream *stream, const char *filename )
      stream->peek = rtsp_peek;
      stream->read = rtsp_read;
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 #endif /* DIRECT_BUILD_NETWORK */   
@@ -1721,7 +1758,7 @@ pipe_wait( DirectStream   *stream,
      fd_set s;
 
      if (stream->cache_size >= length)
-          return DFB_OK;
+          return DR_OK;
      
      FD_ZERO( &s );
      FD_SET( stream->fd, &s );
@@ -1729,13 +1766,13 @@ pipe_wait( DirectStream   *stream,
      switch (select( stream->fd+1, &s, NULL, NULL, tv )) {
           case 0:
                if (!tv && !stream->cache_size)
-                    return DFB_EOF;
-               return DFB_TIMEOUT;
+                    return DR_EOF;
+               return DR_TIMEOUT;
           case -1:
                return errno2result( errno );
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1749,7 +1786,7 @@ pipe_peek( DirectStream *stream,
      int          len;
 
      if (offset < 0)
-          return DFB_UNSUPPORTED;
+          return DR_UNSUPPORTED;
 
      len = length + offset;
      if (len > stream->cache_size) {
@@ -1772,7 +1809,7 @@ pipe_peek( DirectStream *stream,
           
           stream->cache_size += s;
           if (stream->cache_size <= offset)
-               return DFB_BUFFEREMPTY;
+               return DR_BUFFEREMPTY;
           
           size = stream->cache_size - offset;
      }
@@ -1782,7 +1819,7 @@ pipe_peek( DirectStream *stream,
      if (read_out)
           *read_out = size;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1817,12 +1854,12 @@ pipe_read( DirectStream *stream,
           switch (s) {
                case 0:
                     if (!size)
-                         return DFB_EOF;
+                         return DR_EOF;
                     break;
                case -1:
                     if (!size) {
                          return (errno == EAGAIN)
-                                ? DFB_BUFFEREMPTY
+                                ? DR_BUFFEREMPTY
                                 : errno2result( errno );
                     }
                     break;
@@ -1837,7 +1874,7 @@ pipe_read( DirectStream *stream,
      if (read_out)
           *read_out = size;
 
-     return DFB_OK;
+     return DR_OK;
 }
  
 /*****************************************************************************/
@@ -1849,27 +1886,27 @@ file_peek( DirectStream *stream,
            void         *buf,
            unsigned int *read_out )
 {
-     DirectResult ret = DFB_OK;
+     DirectResult ret = DR_OK;
      ssize_t      size;
      
      if (lseek( stream->fd, offset, SEEK_CUR ) < 0)
-          return DFB_FAILURE;
+          return DR_FAILURE;
 
      size = read( stream->fd, buf, length );
      switch (size) {
           case 0:
-               ret = DFB_EOF;
+               ret = DR_EOF;
                break;
           case -1:
                ret = (errno == EAGAIN)
-                     ? DFB_BUFFEREMPTY
+                     ? DR_BUFFEREMPTY
                      : errno2result( errno );
                size = 0;
                break;
      }
 
      if (lseek( stream->fd, - offset - size, SEEK_CUR ) < 0)
-          return DFB_FAILURE;
+          return DR_FAILURE;
      
      if (read_out)
           *read_out = size;
@@ -1888,10 +1925,10 @@ file_read( DirectStream *stream,
      size = read( stream->fd, buf, length );
      switch (size) {
           case 0:
-               return DFB_EOF;
+               return DR_EOF;
           case -1:
                if (errno == EAGAIN)
-                    return DFB_BUFFEREMPTY;
+                    return DR_BUFFEREMPTY;
                return errno2result( errno );
      }
 
@@ -1900,7 +1937,7 @@ file_read( DirectStream *stream,
      if (read_out)
           *read_out = size;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1910,20 +1947,20 @@ file_seek( DirectStream *stream, unsigned int offset )
 
      off = lseek( stream->fd, offset, SEEK_SET );
      if (off < 0)
-          return DFB_FAILURE;
+          return DR_FAILURE;
 
      stream->offset = off;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
-file_open( DirectStream *stream, const char *filename )
+file_open( DirectStream *stream, const char *filename, int fileno )
 {
      if (filename)
           stream->fd = open( filename, O_RDONLY | O_NONBLOCK );
      else
-          stream->fd = dup( fileno( stdin ) );
+          stream->fd = dup( fileno );
      
      if (stream->fd < 0)
           return errno2result( errno );
@@ -1949,7 +1986,7 @@ file_open( DirectStream *stream, const char *filename )
           stream->seek   = file_seek;
      }
      
-     return DFB_OK;
+     return DR_OK;
 }
 
 /*****************************************************************************/
@@ -1974,10 +2011,14 @@ direct_stream_create( const char    *filename,
      stream->fd  = -1;
 
      if (!strncmp( filename, "stdin:/", 7 )) {
-          ret = file_open( stream, NULL );
+          ret = file_open( stream, NULL, STDIN_FILENO );
      }
      else if (!strncmp( filename, "file:/", 6 )) {
-          ret = file_open( stream, filename+6 );
+          ret = file_open( stream, filename+6, -1 );
+     }
+     else if (!strncmp( filename, "fd:/", 4 )) {
+          ret = (filename[4] >= '0' && filename[4] <= '9')
+                ? file_open( stream, NULL, atoi(filename+4) ) : DR_INVARG;
      }
 #if DIRECT_BUILD_NETWORK
      else if (!strncmp( filename, "http://", 7 ) ||
@@ -1998,7 +2039,7 @@ direct_stream_create( const char    *filename,
      }
 #endif
      else {
-          ret = file_open( stream, filename );
+          ret = file_open( stream, filename, -1 );
      }
 
      if (ret) {
@@ -2009,7 +2050,7 @@ direct_stream_create( const char    *filename,
 
      *ret_stream = stream;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectStream*
@@ -2100,7 +2141,7 @@ direct_stream_wait( DirectStream   *stream,
      if (length && stream->wait)
           return stream->wait( stream, length, tv );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -2117,12 +2158,12 @@ direct_stream_peek( DirectStream *stream,
      D_MAGIC_ASSERT( stream, DirectStream );
      
      if (stream->length >= 0 && (stream->offset + offset) >= stream->length)
-          return DFB_EOF;
+          return DR_EOF;
 
      if (stream->peek)
           return stream->peek( stream, length, offset, buf, read_out );
 
-     return DFB_UNSUPPORTED;
+     return DR_UNSUPPORTED;
 }
 
 DirectResult
@@ -2138,12 +2179,12 @@ direct_stream_read( DirectStream *stream,
      D_MAGIC_ASSERT( stream, DirectStream );
 
      if (stream->length >= 0 && stream->offset >= stream->length)
-          return DFB_EOF;
+          return DR_EOF;
 
      if (stream->read)
           return stream->read( stream, length, buf, read_out );
 
-     return DFB_UNSUPPORTED;
+     return DR_UNSUPPORTED;
 }
 
 DirectResult
@@ -2155,7 +2196,7 @@ direct_stream_seek( DirectStream *stream,
      D_MAGIC_ASSERT( stream, DirectStream );
 
      if (stream->offset == offset)
-          return DFB_OK;
+          return DR_OK;
      
      if (stream->length >= 0 && offset > stream->length)
           offset = stream->length;
@@ -2163,7 +2204,7 @@ direct_stream_seek( DirectStream *stream,
      if (stream->seek)
           return stream->seek( stream, offset );
 
-     return DFB_UNSUPPORTED;
+     return DR_UNSUPPORTED;
 }
 
 static void
@@ -2223,10 +2264,8 @@ direct_stream_close( DirectStream *stream )
      }
 
      if (stream->fd >= 0) {
-          if (stream->fd == STDIN_FILENO) {
-               fcntl( stream->fd, F_SETFL, 
+          fcntl( stream->fd, F_SETFL, 
                     fcntl( stream->fd, F_GETFL ) & ~O_NONBLOCK );
-          }
           close( stream->fd );
           stream->fd = -1;
      }
