@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -33,20 +34,13 @@
 #include <direct/build.h>
 #include <direct/system.h>
 
-#if HAVE_ASM_PAGE_H
-#include <asm/page.h>
-#else
-#define PAGE_SIZE   sysconf( _SC_PAGESIZE )
-#endif
-
-
 #if DIRECT_BUILD_GETTID && defined(HAVE_LINUX_UNISTD_H)
 #include <linux/unistd.h>
 #endif
 
 __attribute__((no_instrument_function))
 pid_t
-direct_gettid()
+direct_gettid( void )
 {
      pid_t tid = -1;
 #if DIRECT_BUILD_GETTID && defined(__NR_gettid) /* present on linux >= 2.4.20 */
@@ -59,8 +53,16 @@ direct_gettid()
 }
 
 long
-direct_pagesize()
+direct_pagesize( void )
 {
-     return PAGE_SIZE;
+     return sysconf( _SC_PAGESIZE );
+}
+
+unsigned long
+direct_page_align( unsigned long value )
+{
+     unsigned long mask = sysconf( _SC_PAGESIZE ) - 1;
+
+     return (value + mask) & ~mask;
 }
 
