@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -44,36 +44,40 @@
 #include <direct/thread.h>
 #include <direct/util.h>
 
-D_DEBUG_DOMAIN( Direct_Thread, "Direct/Thread", "Thread management" );
+D_DEBUG_DOMAIN( Direct_Thread,     "Direct/Thread",      "Thread management" );
+D_DEBUG_DOMAIN( Direct_ThreadInit, "Direct/Thread/Init", "Thread initialization" );
 
 
 /* FIXME: DIRECT_THREAD_WAIT_INIT is required, but should be optional. */
 #define DIRECT_THREAD_WAIT_INIT
 
-/* FIXME: DIRECT_THREAD_WAIT_CREATE is required, but should be optional. */
-#define DIRECT_THREAD_WAIT_CREATE
-
 
 struct __D_DirectThread {
      int                   magic;
 
-     pthread_t             thread;   /* The pthread thread identifier. */
+     pthread_t             thread;      /* The pthread thread identifier. */
      pid_t                 tid;
 
      char                 *name;
 
-     DirectThreadType      type;     /* The thread's type, e.g. input thread. */
-     DirectThreadMainFunc  main;     /* The thread's main routine (or entry point). */
-     void                 *arg;      /* Custom argument passed to the main routine. */
+     DirectThreadType      type;        /* The thread's type, e.g. input thread. */
+     DirectThreadMainFunc  main;        /* The thread's main routine (or entry point). */
+     void                 *arg;         /* Custom argument passed to the main routine. */
 
-     bool                  canceled; /* Set when direct_thread_cancel() is called. */
-     bool                  joining;  /* Set when direct_thread_join() is called. */
-     bool                  joined;   /* Set when direct_thread_join() has finished. */
-     bool                  detached; /* Set when direct_thread_detach() is called. */
+     bool                  canceled;    /* Set when direct_thread_cancel() is called. */
+     bool                  joining;     /* Set when direct_thread_join() is called. */
+     bool                  joined;      /* Set when direct_thread_join() has finished. */
+     bool                  detached;    /* Set when direct_thread_detach() is called. */
+     bool                  terminated;  /* Set when direct_thread_terminate() is called. */
 
 #ifdef DIRECT_THREAD_WAIT_INIT
-     bool                  init;     /* Set to true before calling the main routine. */
+     bool                  init;        /* Set to true before calling the main routine. */
 #endif
+
+     pthread_mutex_t       lock;
+     pthread_cond_t        cond;
+
+     unsigned int          counter;
 };
 
 struct __D_DirectThreadInitHandler {
@@ -93,8 +97,6 @@ struct __D_DirectThreadInitHandler {
  */
 static void *direct_thread_main( void *arg );
 
-static const char *thread_type_name( DirectThreadType type )  D_CONST_FUNC;
-
 /******************************************************************************/
 
 static pthread_mutex_t  handler_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -157,14 +159,18 @@ direct_thread_create( DirectThreadType      thread_type,
                       void                 *arg,
                       const char           *name )
 {
-     DirectThread *thread;
+     DirectThread       *thread;
+     pthread_attr_t      attr;
+     struct sched_param  param;
+     int                 policy;
+     int                 priority;
+     size_t              stack_size;
 
      D_ASSERT( thread_main != NULL );
      D_ASSERT( name != NULL );
 
-     D_HEAVYDEBUG( "Direct/Thread: Creating '%s' (type %d)...\n", name, thread_type );
-
-     (void) thread_type_name;
+     D_DEBUG_AT( Direct_Thread, "%s( %s, %p(%p), '%s' )\n", __FUNCTION__,
+                 direct_thread_type_name(thread_type), thread_main, arg, name );
 
      /* Create the key for the TSD (thread specific data). */
      pthread_mutex_lock( &key_lock );
@@ -191,29 +197,114 @@ direct_thread_create( DirectThreadType      thread_type,
      thread->thread = (pthread_t) -1;
      thread->tid    = (pid_t) -1;
 
+     /* Initialize mutex and condition. */
+     direct_util_recursive_pthread_mutex_init( &thread->lock );
+     pthread_cond_init( &thread->cond, NULL );
+
      D_MAGIC_SET( thread, DirectThread );
 
+     /* Initialize scheduling and other parameters. */
+     pthread_attr_init( &attr );
+
+     pthread_attr_setinheritsched( &attr, PTHREAD_EXPLICIT_SCHED );
+
+     /* Select scheduler. */
+     switch (direct_config->thread_scheduler) {
+          case DCTS_FIFO:
+               policy = SCHED_FIFO;
+               break;
+
+          case DCTS_RR:
+               policy = SCHED_RR;
+               break;
+
+          default:
+               policy = SCHED_OTHER;
+               break;
+     }
+
+     if (pthread_attr_setschedpolicy( &attr, policy ))
+          D_PERROR( "Direct/Thread: Could not set scheduling policy to %s!\n", direct_thread_policy_name(policy) );
+
+     /* Read (back) value. */
+     pthread_attr_getschedpolicy( &attr, &policy );
+
+     /* Select priority. */
+     switch (thread->type) {
+          case DTT_CLEANUP:
+          case DTT_INPUT:
+          case DTT_OUTPUT:
+          case DTT_MESSAGING:
+          case DTT_CRITICAL:
+               priority = thread->type * direct_config->thread_priority_scale / 100;
+               break;
+
+          default:
+               priority = direct_config->thread_priority;
+               break;
+     }
+
+     D_DEBUG_AT( Direct_ThreadInit, "  -> %s (%d) [%d;%d]\n", direct_thread_policy_name(policy), priority,
+                 sched_get_priority_min( policy ), sched_get_priority_max( policy ) );
+
+     if (priority < sched_get_priority_min( policy ))
+          priority = sched_get_priority_min( policy );
+
+     if (priority > sched_get_priority_max( policy ))
+          priority = sched_get_priority_max( policy );
+
+     param.sched_priority = priority;
+
+     if (pthread_attr_setschedparam( &attr, &param ))
+          D_PERROR( "Direct/Thread: Could not set scheduling priority to %d!\n", priority );
+
+     /* Select stack size? */
+     if (direct_config->thread_stack_size > 0) {
+          if (pthread_attr_setstacksize( &attr, direct_config->thread_stack_size ))
+               D_PERROR( "Direct/Thread: Could not set stack size to %d!\n", direct_config->thread_stack_size );
+     }
+
+     /* Read (back) value. */
+     pthread_attr_getstacksize( &attr, &stack_size );
+
+     /* Lock the thread mutex. */
+     D_DEBUG_AT( Direct_ThreadInit, "  -> locking...\n" );
+     pthread_mutex_lock( &thread->lock );
+
      /* Create and run the thread. */
-     pthread_create( &thread->thread, NULL, direct_thread_main, thread );
+     D_DEBUG_AT( Direct_ThreadInit, "  -> creating...\n" );
+     pthread_create( &thread->thread, &attr, direct_thread_main, thread );
+
+     pthread_attr_destroy( &attr );
+
+     pthread_getschedparam( thread->thread, &policy, &param );
+
+     D_INFO( "Direct/Thread: Started '%s' (%d) [%s %s/%s %d/%d] <%zu>...\n",
+             name, thread->tid, direct_thread_type_name(thread_type),
+             direct_thread_policy_name(policy), direct_thread_scheduler_name(direct_config->thread_scheduler),
+             param.sched_priority, priority, stack_size );
 
 #ifdef DIRECT_THREAD_WAIT_INIT
-     D_HEAVYDEBUG( "Direct/Thread: Waiting for thread to run...\n" );
-
      /* Wait for completion of the thread's initialization. */
-     while (!thread->init)
-          sched_yield();
+     while (!thread->init) {
+          D_DEBUG_AT( Direct_ThreadInit, "  -> waiting...\n" );
+          pthread_cond_wait( &thread->cond, &thread->lock );
+     }
 
-     D_HEAVYDEBUG( "Direct/Thread: ...thread is running.\n" );
+     D_DEBUG_AT( Direct_ThreadInit, "  -> ...thread is running.\n" );
 #endif
 
-     D_INFO( "Direct/Thread: Running '%s' (%s, %d)...\n",
-             name, thread_type_name(thread_type), thread->tid );
+     /* Unlock the thread mutex. */
+     D_DEBUG_AT( Direct_ThreadInit, "  -> unlocking...\n" );
+     pthread_mutex_unlock( &thread->lock );
+
+     D_DEBUG_AT( Direct_ThreadInit, "  -> returning %p\n", thread );
 
      return thread;
 }
 
 DirectThread *
-direct_thread_self()
+direct_thread_self( void )
 {
      DirectThread *thread = pthread_getspecific( thread_key );
 
@@ -242,7 +333,7 @@ direct_thread_get_tid( DirectThread *thread )
 
 __attribute__((no_instrument_function))
 const char *
-direct_thread_self_name()
+direct_thread_self_name( void )
 {
      DirectThread *thread = pthread_getspecific( thread_key );
 
@@ -260,8 +351,12 @@ direct_thread_set_name( const char *name )
      char         *copy;
      DirectThread *thread = pthread_getspecific( thread_key );
 
+     D_DEBUG_AT( Direct_Thread, "%s( '%s' )\n", __FUNCTION__, name );
+
      /* Support this function for non-direct threads. */
      if (!thread) {
+          D_DEBUG_AT( Direct_Thread, "  -> attaching unknown thread %d\n", direct_gettid() );
+
           /* Create the key for the TSD (thread specific data). */
           pthread_mutex_lock( &key_lock );
 
@@ -280,8 +375,12 @@ direct_thread_set_name( const char *name )
           thread->thread = pthread_self();
           thread->tid    = direct_gettid();
 
+          D_MAGIC_SET( thread, DirectThread );
+
           pthread_setspecific( thread_key, thread );
      }
+     else
+          D_DEBUG_AT( Direct_Thread, "  -> was '%s' (%d)\n", thread->name, direct_gettid() );
 
      /* Duplicate string. */
      copy = D_STRDUP( name );
@@ -298,6 +397,89 @@ direct_thread_set_name( const char *name )
      thread->name = copy;
 }
 
+DirectResult
+direct_thread_wait( DirectThread *thread, int timeout_ms )
+{
+     unsigned int old_counter = thread->counter;
+
+     D_MAGIC_ASSERT( thread, DirectThread );
+     D_ASSERT( thread->thread != -1 );
+
+     D_ASSUME( !thread->canceled );
+
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d, %dms )\n", __FUNCTION__,
+                 thread->main, thread->name, thread->tid, timeout_ms );
+
+     while (old_counter == thread->counter && !thread->terminated)
+          pthread_cond_wait( &thread->cond, &thread->lock );
+
+     if (thread->terminated)
+          return DR_DEAD;
+
+     return DR_OK;
+}
+
+void
+direct_thread_notify( DirectThread *thread )
+{
+     D_MAGIC_ASSERT( thread, DirectThread );
+     D_ASSERT( thread->thread != -1 );
+
+     D_ASSUME( !thread->canceled );
+
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
+     pthread_mutex_lock( &thread->lock );
+
+     thread->counter++;
+
+     pthread_mutex_unlock( &thread->lock );
+
+     pthread_cond_broadcast( &thread->cond );
+}
+
+void
+direct_thread_lock( DirectThread *thread )
+{
+     D_MAGIC_ASSERT( thread, DirectThread );
+     D_ASSERT( thread->thread != -1 );
+
+     D_ASSUME( !thread->canceled );
+
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
+     pthread_mutex_lock( &thread->lock );
+}
+
+void
+direct_thread_unlock( DirectThread *thread )
+{
+     D_MAGIC_ASSERT( thread, DirectThread );
+     D_ASSERT( thread->thread != -1 );
+
+     D_ASSUME( !thread->canceled );
+
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
+     pthread_mutex_unlock( &thread->lock );
+}
+
+void
+direct_thread_terminate( DirectThread *thread )
+{
+     D_MAGIC_ASSERT( thread, DirectThread );
+     D_ASSERT( thread->thread != -1 );
+     D_ASSUME( !pthread_equal( thread->thread, pthread_self() ) );
+
+     D_ASSUME( !thread->canceled );
+
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
+     thread->terminated = true;
+
+     direct_thread_notify( thread );
+}
+
 void
 direct_thread_cancel( DirectThread *thread )
 {
@@ -307,7 +489,7 @@ direct_thread_cancel( DirectThread *thread )
 
      D_ASSUME( !thread->canceled );
 
-     D_DEBUG_AT( Direct_Thread, "Canceling %d.\n", thread->tid );
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
 
      thread->canceled = true;
 
@@ -331,7 +513,7 @@ direct_thread_detach( DirectThread *thread )
 
      D_ASSUME( !thread->canceled );
 
-     D_DEBUG_AT( Direct_Thread, "Detaching %d.\n", thread->tid );
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
 
      thread->detached = true;
 
@@ -369,19 +551,23 @@ direct_thread_join( DirectThread *thread )
      D_ASSUME( !thread->joined );
      D_ASSUME( !thread->detached );
 
-     if (thread->detached)
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
+     if (thread->detached) {
+          D_DEBUG_AT( Direct_Thread, "  -> DETACHED!\n" );
           return;
+     }
 
      if (!thread->joining && !pthread_equal( thread->thread, pthread_self() )) {
           thread->joining = true;
 
-          D_HEAVYDEBUG( "Direct/Thread: Joining %d...\n", thread->tid );
+          D_DEBUG_AT( Direct_Thread, "  -> joining...\n" );
 
           pthread_join( thread->thread, NULL );
 
           thread->joined = true;
 
-          D_HEAVYDEBUG( "Direct/Thread: ...joined %d.\n", thread->tid );
+          D_DEBUG_AT( Direct_Thread, "  -> joined.\n" );
      }
 }
 
@@ -397,21 +583,21 @@ void
 direct_thread_destroy( DirectThread *thread )
 {
      D_MAGIC_ASSERT( thread, DirectThread );
-
      D_ASSUME( !pthread_equal( thread->thread, pthread_self() ) );
-     //D_ASSUME( thread->joined );
      D_ASSUME( !thread->detached );
 
-     if (thread->detached)
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
+     if (thread->detached) {
+          D_DEBUG_AT( Direct_Thread, "  -> DETACHED!\n" );
           return;
+     }
 
      if (!thread->joined && !pthread_equal( thread->thread, pthread_self() )) {
           if (thread->canceled)
-               D_DEBUG_AT( Direct_Thread, "'%s' (%d) is canceled but not joined!\n",
-                           thread->name, thread->tid );
+               D_DEBUG_AT( Direct_Thread, "  -> cancled but not joined!\n" );
           else {
-               D_DEBUG_AT( Direct_Thread, "'%s' (%d) is still running!\n",
-                           thread->name, thread->tid );
+               D_DEBUG_AT( Direct_Thread, "  -> still running!\n" );
 
                if (thread->name)
                     D_ERROR( "Direct/Thread: Canceling '%s' (%d)!\n", thread->name, thread->tid );
@@ -435,8 +621,9 @@ direct_thread_destroy( DirectThread *thread )
 
 /******************************************************************************/
 
-static const char *
-thread_type_name( DirectThreadType type )
+#if DIRECT_BUILD_TEXT
+const char *
+direct_thread_type_name( DirectThreadType type )
 {
      switch (type) {
           case DTT_DEFAULT:
@@ -458,9 +645,44 @@ thread_type_name( DirectThreadType type )
                return "CRITICAL";
      }
 
-     return "unknown type!";
+     return "<unknown>";
 }
 
+const char *
+direct_thread_scheduler_name( DirectConfigThreadScheduler scheduler )
+{
+     switch (scheduler) {
+          case DCTS_OTHER:
+               return "OTHER";
+
+          case DCTS_FIFO:
+               return "FIFO";
+
+          case DCTS_RR:
+               return "RR";
+     }
+
+     return "<unknown>";
+}
+
+const char *
+direct_thread_policy_name( int policy )
+{
+     switch (policy) {
+          case SCHED_OTHER:
+               return "OTHER";
+
+          case SCHED_FIFO:
+               return "FIFO";
+
+          case SCHED_RR:
+               return "RR";
+     }
+
+     return "<unknown>";
+}
+#endif
+
 /******************************************************************************/
 
 static void
@@ -470,6 +692,8 @@ direct_thread_cleanup( void *arg )
 
      D_MAGIC_ASSERT( thread, DirectThread );
 
+     D_DEBUG_AT( Direct_Thread, "%s( %p, '%s' %d )\n", __FUNCTION__, thread->main, thread->name, thread->tid );
+
      if (thread->detached) {
           D_MAGIC_CLEAR( thread );
 
@@ -486,6 +710,13 @@ direct_thread_main( void *arg )
      void                    *ret;
      DirectThread            *thread = arg;
      DirectThreadInitHandler *handler;
+     pid_t                    tid;
+
+     tid = direct_gettid();
+
+     D_DEBUG_AT( Direct_ThreadInit, "%s( %p ) <- tid %d\n", __FUNCTION__, arg, tid );
+
+     D_DEBUG_AT( Direct_ThreadInit, "  -> starting...\n" );
 
      D_MAGIC_ASSERT( thread, DirectThread );
 
@@ -494,7 +725,7 @@ direct_thread_main( void *arg )
 
      pthread_setspecific( thread_key, thread );
 
-     thread->tid = direct_gettid();
+     thread->tid = tid;
 
 
      /* Call all init handlers. */
@@ -507,57 +738,38 @@ direct_thread_main( void *arg )
 
 
      /* Have all signals handled by the main thread. */
-     direct_signals_block_all();
+     if (direct_config->thread_block_signals)
+          direct_signals_block_all();
 
-     /* Adjust scheduling priority. */
-     switch (thread->type) {
-          case DTT_CLEANUP:
-          case DTT_INPUT:
-          case DTT_OUTPUT:
-          case DTT_MESSAGING:
-          case DTT_CRITICAL:
-               setpriority( PRIO_PROCESS, 0, thread->type );
-               break;
+     /* Lock the thread mutex. */
+     D_DEBUG_AT( Direct_ThreadInit, "  -> locking...\n" );
+     pthread_mutex_lock( &thread->lock );
 
-          default:
-               break;
-     }
-
-#ifdef DIRECT_THREAD_WAIT_INIT
      /* Indicate that our initialization has completed. */
      thread->init = true;
 
-     D_HEAVYDEBUG( "Direct/Thread:   (thread) Initialization done.\n" );
-
-     sched_yield();
+#ifdef DIRECT_THREAD_WAIT_INIT
+     D_DEBUG_AT( Direct_ThreadInit, "  -> signalling...\n" );
+     pthread_cond_signal( &thread->cond );
 #endif
 
-     D_MAGIC_ASSERT( thread, DirectThread );
+     /* Unlock the thread mutex. */
+     D_DEBUG_AT( Direct_ThreadInit, "  -> unlocking...\n" );
+     pthread_mutex_unlock( &thread->lock );
 
      if (thread->joining) {
-          D_HEAVYDEBUG( "Direct/Thread:   (thread) Being joined before entering main routine.\n" );
+          D_DEBUG_AT( Direct_Thread, "  -> Being joined before entering main routine!\n" );
           return NULL;
      }
 
-#ifdef DIRECT_THREAD_WAIT_CREATE
-     if (thread->thread == -1) {
-          D_HEAVYDEBUG( "Direct/Thread:   (thread) Waiting for pthread_create()...\n" );
-
-          /* Wait for completion of pthread_create(). */
-          while ((int) thread->thread == -1)
-               sched_yield();
-
-          D_HEAVYDEBUG( "Direct/Thread:   (thread) ...pthread_create() finished.\n" );
-     }
-#endif
-
      D_MAGIC_ASSERT( thread, DirectThread );
 
      /* Call main routine. */
+     D_DEBUG_AT( Direct_ThreadInit, "  -> running...\n" );
      ret = thread->main( thread, thread->arg );
 
-     D_DEBUG_AT( Direct_Thread, "Returning %p from '%s' (%s, %d)...\n",
-                 ret, thread->name, thread_type_name(thread->type), thread->tid );
+     D_DEBUG_AT( Direct_Thread, "  -> Returning %p from '%s' (%s, %d)...\n",
+                 ret, thread->name, direct_thread_type_name(thread->type), thread->tid );
 
      D_MAGIC_ASSERT( thread, DirectThread );
 
