@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -32,7 +32,7 @@
 #include <sys/types.h>
 
 #include <direct/types.h>
-
+#include <direct/conf.h>
 
 typedef enum {
      DTT_DEFAULT    =   0,
@@ -93,6 +93,25 @@ const char   *direct_thread_self_name  ( void );
  */
 void          direct_thread_set_name   ( const char   *name );
 
+/*
+ * Wait on the thread object to be notified via direct_thread_notify().
+ */
+DirectResult  direct_thread_wait       ( DirectThread *thread,
+                                         int           timeout_ms );
+
+/*
+ * Notify the thread object waking up callers of direct_thread_wait().
+ */
+void direct_thread_notify     ( DirectThread *thread );
+
+void direct_thread_lock       ( DirectThread *thread );
+void direct_thread_unlock     ( DirectThread *thread );
+
+/*
+ * Kindly ask the thread to terminate (for joining without thread cancellation).
+ */
+void direct_thread_terminate  ( DirectThread *thread );
+
 /*
  * Cancel a running thread.
  */
@@ -136,5 +155,14 @@ bool direct_thread_is_joined  ( DirectThread *thread );
  */
 void direct_thread_destroy    ( DirectThread *thread );
 
+/*
+ * Utilities for stringification.
+ */
+#if DIRECT_BUILD_TEXT
+const char *direct_thread_type_name     ( DirectThreadType type );
+const char *direct_thread_scheduler_name( DirectConfigThreadScheduler scheduler );
+const char *direct_thread_policy_name   ( int policy );
+#endif
+
 #endif
 
