@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -60,7 +60,7 @@ static DirectNode *tree_node_rotate_right( DirectNode *node );
 
 
 DirectTree *
-direct_tree_new()
+direct_tree_new( void )
 {
      return D_CALLOC( 1, sizeof (DirectTree) );
 }
