@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -89,13 +89,47 @@
 #endif
 
 
+#define D_BITn32(f)  (((f) & 0x00000001) ?  0 : \
+                      ((f) & 0x00000002) ?  1 : \
+                      ((f) & 0x00000004) ?  2 : \
+                      ((f) & 0x00000008) ?  3 : \
+                      ((f) & 0x00000010) ?  4 : \
+                      ((f) & 0x00000020) ?  5 : \
+                      ((f) & 0x00000040) ?  6 : \
+                      ((f) & 0x00000080) ?  7 : \
+                      ((f) & 0x00000100) ?  8 : \
+                      ((f) & 0x00000200) ?  9 : \
+                      ((f) & 0x00000400) ? 10 : \
+                      ((f) & 0x00000800) ? 11 : \
+                      ((f) & 0x00001000) ? 12 : \
+                      ((f) & 0x00002000) ? 13 : \
+                      ((f) & 0x00004000) ? 14 : \
+                      ((f) & 0x00008000) ? 15 : \
+                      ((f) & 0x00010000) ? 16 : \
+                      ((f) & 0x00020000) ? 17 : \
+                      ((f) & 0x00040000) ? 18 : \
+                      ((f) & 0x00080000) ? 19 : \
+                      ((f) & 0x00100000) ? 20 : \
+                      ((f) & 0x00200000) ? 21 : \
+                      ((f) & 0x00400000) ? 22 : \
+                      ((f) & 0x00800000) ? 23 : \
+                      ((f) & 0x01000000) ? 24 : \
+                      ((f) & 0x02000000) ? 25 : \
+                      ((f) & 0x04000000) ? 26 : \
+                      ((f) & 0x08000000) ? 27 : \
+                      ((f) & 0x10000000) ? 28 : \
+                      ((f) & 0x20000000) ? 29 : \
+                      ((f) & 0x40000000) ? 30 : \
+                      ((f) & 0x80000000) ? 31 : -1)
+
+
 /*
  * portable sched_yield() implementation
  */
 #ifdef _POSIX_PRIORITY_SCHEDULING
 #define direct_sched_yield()  sched_yield()
 #else
-#define direct_sched_yield()  usleep(0)
+#define direct_sched_yield()  usleep(1)
 #endif
 
 /*
