@@ -29,6 +29,8 @@
 #define __FUSION__BUILD_H__
 
 #define FUSION_BUILD_MULTI   (1)
+#define FUSION_BUILD_KERNEL  (1)
+#define FUSION_MESSAGE_SIZE  (1024)
 
 #endif
 
