@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -31,33 +31,43 @@
 
 #include <fusion/types.h>
 
-typedef int (*FusionCallHandler) (int   caller,   /* fusion id of the caller */
-                                  int   call_arg, /* optional call parameter */
-                                  void *call_ptr, /* optional call parameter */
-                                  void *ctx       /* optional handler context */
-                                  );
+typedef enum {
+     FCHR_RETURN,
+     FCHR_RETAIN
+} FusionCallHandlerResult;
+
+typedef FusionCallHandlerResult (*FusionCallHandler) (int           caller,   /* fusion id of the caller */
+                                                      int           call_arg, /* optional call parameter */
+                                                      void         *call_ptr, /* optional call parameter */
+                                                      void         *ctx,      /* optional handler context */
+                                                      unsigned int  serial,
+                                                      int          *ret_val );
 
 typedef struct {
      FusionWorldShared *shared;
      int                call_id;
-     int                fusion_id;
+     FusionID           fusion_id;
      FusionCallHandler  handler;
      void              *ctx;
 } FusionCall;
 
 
-DirectResult fusion_call_init    (FusionCall          *call,
+DirectResult fusion_call_init   ( FusionCall          *call,
                                   FusionCallHandler    handler,
                                   void                *ctx,
-                                  const FusionWorld   *world);
+                                  const FusionWorld   *world );
 
-DirectResult fusion_call_execute (FusionCall          *call,
+DirectResult fusion_call_execute( FusionCall          *call,
                                   FusionCallExecFlags  flags,
                                   int                  call_arg,
                                   void                *call_ptr,
-                                  int                 *ret_val);
+                                  int                 *ret_val );
 
-DirectResult fusion_call_destroy (FusionCall          *call);
+DirectResult fusion_call_return ( FusionCall          *call,
+                                  unsigned int         serial,
+                                  int                  val );
+
+DirectResult fusion_call_destroy( FusionCall          *call );
 
 
 #endif
