@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -44,6 +44,14 @@ typedef enum {
      FFA_FORK
 } FusionForkAction;
 
+typedef enum {
+     FFS_PREPARE,
+     FFS_PARENT,
+     FFS_CHILD
+} FusionForkState;
+
+typedef void (*FusionForkCallback) ( FusionForkAction action, FusionForkState state );
+
 /*
  * Enters a fusion world by joining or creating it.
  *
@@ -63,11 +71,25 @@ DirectResult fusion_enter( int               world_index,
 DirectResult fusion_exit( FusionWorld *world,
                           bool         emergency );
 
+DirectResult fusion_stop_dispatcher( FusionWorld *world,
+                                     bool         emergency );
+
 /*
  * Sets the fork() action of the calling Fusionee within the world.
  */
 void fusion_world_set_fork_action( FusionWorld      *world,
                                    FusionForkAction  action );
+                                   
+/*
+ * Gets the current fork() action.
+ */ 
+FusionForkAction fusion_world_get_fork_action( FusionWorld *world );
+
+/*
+ * Registers a callback called upon fork().
+ */
+void fusion_world_set_fork_callback( FusionWorld        *world,
+                                     FusionForkCallback  callback );
 
 /*
  * Return the index of the specified world.
@@ -79,6 +101,11 @@ int fusion_world_index( const FusionWorld *world );
  */
 FusionID fusion_id( const FusionWorld *world );
 
+/*
+ * Return if the world is a multi application world.
+ */
+bool fusion_is_multi( const FusionWorld *world );
+
 /*
  * Return the thread ID of the Fusion Dispatcher within the specified world.
  */
