@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -39,11 +39,16 @@
 #include <fusion/build.h>
 #include <fusion/fusion.h>
 #include <fusion/lock.h>
+#include <fusion/ref.h>
 #include <fusion/shm/shm_internal.h>
 
 #if FUSION_BUILD_MULTI
-#include <sys/ioctl.h>
-#include <linux/fusion.h>
+# if FUSION_BUILD_KERNEL
+#  include <sys/ioctl.h>
+#  include <linux/fusion.h>
+# else
+#  include <fusion/protocol.h>
+# endif
 #endif
 
 #define FUSION_MAX_WORLDS     8
@@ -54,6 +59,8 @@
 
 struct __Fusion_FusionWorldShared {
      int                  magic;
+     
+     int                  refs;     /* Increased by the master on fork(). */
 
      int                  world_index;
 
@@ -69,6 +76,18 @@ struct __Fusion_FusionWorldShared {
      FusionSHMShared      shm;
 
      FusionSHMPoolShared *main_pool;
+     
+     DirectLink          *fusionees;   /* Connected fusionees. */
+     FusionSkirmish       fusionees_lock;
+    
+     unsigned int         call_ids;    /* Generates call ids. */
+     unsigned int         lock_ids;    /* Generates locks ids. */
+     unsigned int         ref_ids;     /* Generates refs ids. */
+     unsigned int         reactor_ids; /* Generates reactors ids. */
+     unsigned int         pool_ids;    /* Generates pools ids. */
+
+     void                *pool_base;   /* SHM pool allocation base. */ 
+     void                *pool_max;    /* SHM pool max address. */
 };
 
 struct __Fusion_FusionWorld {
@@ -82,6 +101,7 @@ struct __Fusion_FusionWorld {
      FusionID             fusion_id;
 
      DirectThread        *dispatch_loop;
+     bool                 dispatch_stop;
 
      /*
       * List of reactors with at least one local reaction attached.
@@ -92,6 +112,9 @@ struct __Fusion_FusionWorld {
      FusionSHM            shm;
 
      FusionForkAction     fork_action;
+     FusionForkCallback   fork_callback;
+     
+     void                *fusionee;
 };
 
 /*******************************************
@@ -109,25 +132,54 @@ FusionWorld *_fusion_world( const FusionWorldShared *shared );
 void _fusion_reactor_free_all       ( FusionWorld   *world );
 void _fusion_reactor_process_message( FusionWorld   *world,
                                       int            reactor_id,
+                                      int            channel,
                                       const void    *msg_data );
 
+
+#if FUSION_BUILD_MULTI
 /*
  * from call.c
  */
-#if FUSION_BUILD_MULTI
 void _fusion_call_process( FusionWorld       *world,
                            int                call_id,
                            FusionCallMessage *call );
-#endif
 
+#if FUSION_BUILD_KERNEL
 /*
  * from shm.c
  */
-#if FUSION_BUILD_MULTI
 void _fusion_shmpool_process( FusionWorld          *world,
                               int                   pool_id,
                               FusionSHMPoolMessage *msg );
-#endif
+#else
+/*
+ * form fusion.c
+ */ 
+void _fusion_add_local( FusionWorld *world,
+                        FusionRef   *ref,
+                        int          add );
+
+void _fusion_check_locals( FusionWorld *world,
+                           FusionRef   *ref );
+
+void _fusion_remove_all_locals( FusionWorld     *world,
+                                const FusionRef *ref );
+                               
+DirectResult _fusion_send_message( int                  fd, 
+                                   const void          *msg, 
+                                   size_t               msg_size,
+                                   struct sockaddr_un  *addr );                                   
+DirectResult _fusion_recv_message( int                  fd, 
+                                   void                *msg,
+                                   size_t               msg_size,
+                                   struct sockaddr_un  *addr );
+
+/*
+ * from ref.c
+ */
+DirectResult _fusion_ref_change( FusionRef *ref, int add, bool global );
+                                   
+#endif /* FUSION_BUILD_KERNEL */
+#endif /* FUSION_BUILD_MULTI */
 
 #endif
-
