@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -28,6 +28,8 @@
 
 #include <config.h>
 
+#include <sys/param.h>
+
 #include <pthread.h>
 
 #include <direct/debug.h>
@@ -56,26 +58,32 @@ struct __Fusion_FusionObjectPool {
      int                     object_size;
      int                     message_size;
      FusionObjectDestructor  destructor;
+     void                   *ctx;
 
      FusionCall              call;
 };
 
-static int
-object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
+static FusionCallHandlerResult
+object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx, unsigned int serial, int *ret_val )
 {
      FusionObject     *object;
      FusionObjectPool *pool = ctx;
 
+     D_DEBUG_AT( Fusion_Object, "%s( %d, %d, %p, %p, %u, %p )\n",
+                 __FUNCTION__, caller, call_arg, call_ptr, ctx, serial, ret_val );
+
+#if FUSION_BUILD_KERNEL
      if (caller) {
           D_BUG( "Call not from Fusion/Kernel (caller %d)", caller );
-          return 0;
+          return FCHR_RETURN;
      }
+#endif
 
      D_MAGIC_ASSERT( pool, FusionObjectPool );
 
      /* Lock the pool. */
      if (fusion_skirmish_prevail( &pool->lock ))
-          return 0;
+          return FCHR_RETURN;
 
      /* Lookup the object. */
      direct_list_foreach (object, pool->objects) {
@@ -85,15 +93,15 @@ object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
           D_MAGIC_ASSERT( object, FusionObject );
 
           switch (fusion_ref_zero_trylock( &object->ref )) {
-               case DFB_OK:
+               case DR_OK:
                     break;
 
-               case DFB_DESTROYED:
+               case DR_DESTROYED:
                     D_BUG( "already destroyed %p [%ld] in '%s'", object, object->id, pool->name );
 
                     direct_list_remove( &pool->objects, &object->link );
                     fusion_skirmish_dismiss( &pool->lock );
-                    return 0;
+                    return FCHR_RETURN;
 
 
                default:
@@ -101,9 +109,9 @@ object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
                              object, object->id, pool->name );
                     /* fall through */
 
-               case DFB_BUSY:
+               case DR_BUSY:
                     fusion_skirmish_dismiss( &pool->lock );
-                    return 0;
+                    return FCHR_RETURN;
           }
 
           D_DEBUG_AT( Fusion_Object, "== %s ==\n", pool->name );
@@ -114,7 +122,7 @@ object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
                D_WARN( "won't destroy incomplete object, leaking some memory" );
                direct_list_remove( &pool->objects, &object->link );
                fusion_skirmish_dismiss( &pool->lock );
-               return 0;
+               return FCHR_RETURN;
           }
 
           /* Set "deinitializing" state. */
@@ -131,11 +139,11 @@ object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
           D_DEBUG_AT( Fusion_Object, "  -> calling destructor...\n" );
 
           /* Call the destructor. */
-          pool->destructor( object, false );
+          pool->destructor( object, false, pool->ctx );
 
           D_DEBUG_AT( Fusion_Object, "  -> destructor done.\n" );
 
-          return 0;
+          return FCHR_RETURN;
      }
 
      D_BUG( "unknown object [%d] in '%s'", call_arg, pool->name );
@@ -143,7 +151,7 @@ object_reference_watcher( int caller, int call_arg, void *call_ptr, void *ctx )
      /* Unlock the pool. */
      fusion_skirmish_dismiss( &pool->lock );
 
-     return 0;
+     return FCHR_RETURN;
 }
 
 FusionObjectPool *
@@ -151,6 +159,7 @@ fusion_object_pool_create( const char             *name,
                            int                     object_size,
                            int                     message_size,
                            FusionObjectDestructor  destructor,
+                           void                   *ctx,
                            const FusionWorld      *world )
 {
      FusionObjectPool  *pool;
@@ -182,6 +191,7 @@ fusion_object_pool_create( const char             *name,
      pool->object_size  = object_size;
      pool->message_size = message_size;
      pool->destructor   = destructor;
+     pool->ctx          = ctx;
 
      /* Destruction call from Fusion. */
      fusion_call_init( &pool->call, object_reference_watcher, pool, world );
@@ -249,13 +259,15 @@ fusion_object_pool_destroy( FusionObjectPool  *pool,
           D_DEBUG_AT( Fusion_Object, "  -> calling destructor...\n" );
 
           /* Call the destructor. */
-          pool->destructor( object, refs > 0 );
+          pool->destructor( object, refs > 0, pool->ctx );
 
           D_DEBUG_AT( Fusion_Object, "  -> destructor done.\n" );
 
           D_ASSERT( ! direct_list_contains_element_EXPENSIVE( pool->objects, (DirectLink*) object ) );
      }
 
+     pool->objects = NULL;
+
      /* Destroy the pool lock. */
      fusion_skirmish_destroy( &pool->lock );
 
@@ -267,7 +279,7 @@ fusion_object_pool_destroy( FusionObjectPool  *pool,
      SHFREE( shared->main_pool, pool->name );
      SHFREE( shared->main_pool, pool );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -278,10 +290,11 @@ fusion_object_pool_enum( FusionObjectPool     *pool,
      FusionObject *object;
 
      D_MAGIC_ASSERT( pool, FusionObjectPool );
+     D_ASSERT( callback != NULL );
 
      /* Lock the pool. */
      if (fusion_skirmish_prevail( &pool->lock ))
-          return DFB_FUSION;
+          return DR_FUSION;
 
      direct_list_foreach (object, pool->objects) {
           D_MAGIC_ASSERT( object, FusionObject );
@@ -293,7 +306,7 @@ fusion_object_pool_enum( FusionObjectPool     *pool,
      /* Unlock the pool. */
      fusion_skirmish_dismiss( &pool->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 FusionObject *
@@ -386,7 +399,7 @@ fusion_object_get( FusionObjectPool  *pool,
                    FusionObjectID     object_id,
                    FusionObject     **ret_object )
 {
-     DirectResult  ret = DFB_IDNOTFOUND;
+     DirectResult  ret = DR_IDNOTFOUND;
      FusionObject *object;
 
      D_MAGIC_ASSERT( pool, FusionObjectPool );
@@ -394,7 +407,7 @@ fusion_object_get( FusionObjectPool  *pool,
 
      /* Lock the pool. */
      if (fusion_skirmish_prevail( &pool->lock ))
-          return DFB_FUSION;
+          return DR_FUSION;
 
      direct_list_foreach (object, pool->objects) {
           D_MAGIC_ASSERT( object, FusionObject );
@@ -405,7 +418,7 @@ fusion_object_get( FusionObjectPool  *pool,
           }
      }
 
-     if (ret == DFB_OK)
+     if (ret == DR_OK)
           *ret_object = object;
 
      /* Unlock the pool. */
@@ -435,7 +448,7 @@ fusion_object_activate( FusionObject *object )
      /* Set "active" state. */
      object->state = FOS_ACTIVE;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -464,7 +477,7 @@ fusion_object_destroy( FusionObject *object )
 
           /* Lock the pool. */
           if (fusion_skirmish_prevail( &pool->lock ))
-               return DFB_FAILURE;
+               return DR_FAILURE;
 
           D_MAGIC_ASSERT( pool, FusionObjectPool );
 
@@ -492,7 +505,7 @@ fusion_object_destroy( FusionObject *object )
 
      D_MAGIC_CLEAR( object );
      SHFREE( shared->main_pool, object );
-     return DFB_OK;
+     return DR_OK;
 }
 
 /*
