@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -45,8 +45,15 @@ typedef union {
      struct {
           int                      id;
           const FusionWorldShared *shared;
+          /* builtin impl */
+          struct {
+               FusionPropertyState state;
+               pid_t               owner;
+               bool                requested;
+               bool                destroyed;
+          } builtin;
      } multi;
-
+     
      /* single app */
      struct {
           pthread_mutex_t          lock;
@@ -65,7 +72,7 @@ DirectResult fusion_property_init     (FusionProperty    *property,
  * Lease the property causing others to wait before leasing or purchasing.
  *
  * Waits as long as property is leased by another party.
- * Returns DFB_BUSY if property is/gets purchased by another party.
+ * Returns DR_BUSY if property is/gets purchased by another party.
  *
  * Succeeds if property is available,
  * puts the property into 'leased' state.
@@ -76,7 +83,7 @@ DirectResult fusion_property_lease    (FusionProperty *property);
  * Purchase the property disallowing others to lease or purchase it.
  *
  * Waits as long as property is leased by another party.
- * Returns DFB_BUSY if property is/gets purchased by another party.
+ * Returns DR_BUSY if property is/gets purchased by another party.
  *
  * Succeeds if property is available,
  * puts the property into 'purchased' state and wakes up any waiting party.
