@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -32,13 +32,10 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include <fusion/build.h>
-
-#if FUSION_BUILD_MULTI
+#include <sys/param.h>
 #include <sys/types.h>
-#include <sys/ioctl.h>
-#include <linux/fusion.h>
-#endif
+
+#include <fusion/build.h>
 
 #include <direct/debug.h>
 #include <direct/messages.h>
@@ -57,6 +54,8 @@
 D_DEBUG_DOMAIN( Fusion_Ref, "Fusion/Ref", "Fusion's Reference Counter" );
 
 
+#if FUSION_BUILD_KERNEL
+
 DirectResult
 fusion_ref_init (FusionRef         *ref,
                  const char        *name,
@@ -75,7 +74,7 @@ fusion_ref_init (FusionRef         *ref,
                continue;
 
           D_PERROR( "FUSION_REF_NEW" );
-          return DFB_FUSION;
+          return DR_FUSION;
      }
 
      D_DEBUG_AT( Fusion_Ref, "  -> new ref %p [%d]\n", ref, ref->multi.id );
@@ -91,7 +90,42 @@ fusion_ref_init (FusionRef         *ref,
      ref->multi.shared  = world->shared;
      ref->multi.creator = fusion_id( world );
 
-     return DFB_OK;
+     return DR_OK;
+}
+
+DirectResult
+fusion_ref_set_name (FusionRef  *ref,
+                     const char *name)
+{
+     FusionEntryInfo info;
+
+     D_ASSERT( ref != NULL );
+     D_ASSERT( name != NULL );
+
+     info.type = FT_REF;
+     info.id   = ref->multi.id;
+
+     direct_snputs( info.name, name, sizeof(info.name) );
+
+     while (ioctl (_fusion_fd( ref->multi.shared ), FUSION_ENTRY_SET_INFO, &info)) {
+          switch (errno) {
+               case EINTR:
+                    continue;
+               case EAGAIN:
+                    return DR_LOCKED;
+               case EINVAL:
+                    D_ERROR ("Fusion/Reference: invalid reference\n");
+                    return DR_DESTROYED;
+               default:
+                    break;
+          }
+
+          D_PERROR ("FUSION_ENTRY_SET_NAME");
+
+          return DR_FAILURE;
+     }
+
+     return DR_OK;
 }
 
 DirectResult
@@ -106,10 +140,10 @@ fusion_ref_up (FusionRef *ref, bool global)
                case EINTR:
                     continue;
                case EAGAIN:
-                    return DFB_LOCKED;
+                    return DR_LOCKED;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
@@ -119,10 +153,10 @@ fusion_ref_up (FusionRef *ref, bool global)
           else
                D_PERROR ("FUSION_REF_UP");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -138,7 +172,7 @@ fusion_ref_down (FusionRef *ref, bool global)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
@@ -148,10 +182,10 @@ fusion_ref_down (FusionRef *ref, bool global)
           else
                D_PERROR ("FUSION_REF_DOWN");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -168,19 +202,19 @@ fusion_ref_stat (FusionRef *ref, int *refs)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_STAT");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
      *refs = val;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -194,17 +228,17 @@ fusion_ref_zero_lock (FusionRef *ref)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_ZERO_LOCK");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -217,20 +251,20 @@ fusion_ref_zero_trylock (FusionRef *ref)
                case EINTR:
                     continue;
                case ETOOMANYREFS:
-                    return DFB_BUSY;
+                    return DR_BUSY;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_ZERO_TRYLOCK");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -244,17 +278,17 @@ fusion_ref_unlock (FusionRef *ref)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_UNLOCK");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -275,17 +309,17 @@ fusion_ref_watch (FusionRef *ref, FusionCall *call, int call_arg)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_WATCH");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -305,17 +339,17 @@ fusion_ref_inherit (FusionRef *ref, FusionRef *from)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_INHERIT");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -331,19 +365,261 @@ fusion_ref_destroy (FusionRef *ref)
                     continue;
                case EINVAL:
                     D_ERROR ("Fusion/Reference: invalid reference\n");
-                    return DFB_DESTROYED;
+                    return DR_DESTROYED;
                default:
                     break;
           }
 
           D_PERROR ("FUSION_REF_DESTROY");
 
-          return DFB_FAILURE;
+          return DR_FAILURE;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
+#else /* FUSION_BUILD_KERNEL */
+
+DirectResult
+fusion_ref_init (FusionRef         *ref,
+                 const char        *name,
+                 const FusionWorld *world)
+{
+     D_ASSERT( ref != NULL );
+     D_ASSERT( name != NULL );
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     D_DEBUG_AT( Fusion_Ref, "fusion_ref_init( %p, '%s' )\n", ref, name ? : "" );
+     
+     ref->multi.id = ++world->shared->ref_ids;
+     
+     ref->multi.builtin.local  = 0;
+     ref->multi.builtin.global = 0;
+     
+     fusion_skirmish_init( &ref->multi.builtin.lock, name, world );
+     
+     ref->multi.builtin.call = NULL;
+
+     /* Keep back pointer to shared world data. */
+     ref->multi.shared  = world->shared;
+     ref->multi.creator = fusion_id( world );
+
+     return DR_OK;
+}
+
+DirectResult
+fusion_ref_set_name (FusionRef  *ref,
+                     const char *name)
+{
+     return DR_OK;
+}
+
+DirectResult
+_fusion_ref_change (FusionRef *ref, int add, bool global)
+{
+     DirectResult ret;
+
+     D_ASSERT( ref != NULL );
+     D_ASSERT( add != 0 );
+     
+     ret = fusion_skirmish_prevail( &ref->multi.builtin.lock );
+     if (ret)
+          return ret;
+     
+     if (global) {
+          if (ref->multi.builtin.global+add < 0) {
+               D_BUG( "ref has no global references" );
+               fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+               return DR_BUG;
+          }
+
+          ref->multi.builtin.global += add;
+     }
+     else {          
+          if (ref->multi.builtin.local+add < 0) {
+               D_BUG( "ref has no local references" );
+               fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+               return DR_BUG;
+          }
+
+          ref->multi.builtin.local += add;
+          
+          _fusion_add_local( _fusion_world(ref->multi.shared), ref, add ); 
+     }
+
+     if (ref->multi.builtin.local+ref->multi.builtin.global == 0) {
+          fusion_skirmish_notify( &ref->multi.builtin.lock );
+
+          if (ref->multi.builtin.call) {
+               fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+               return fusion_call_execute( ref->multi.builtin.call, FCEF_ONEWAY, 
+                                           ref->multi.builtin.call_arg, NULL, NULL );
+          }
+     }
+     
+     fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+
+     return DR_OK;
+}
+
+DirectResult
+fusion_ref_up (FusionRef *ref, bool global)
+{
+     return _fusion_ref_change( ref, +1, global );
+}
+
+DirectResult
+fusion_ref_down (FusionRef *ref, bool global)
+{
+     return _fusion_ref_change( ref, -1, global );
+}
+
+DirectResult
+fusion_ref_stat (FusionRef *ref, int *refs)
+{
+     D_ASSERT( ref != NULL );
+     D_ASSERT( refs != NULL );
+          
+     *refs = ref->multi.builtin.local + ref->multi.builtin.global;
+     
+     return DR_OK;
+}
+
+DirectResult
+fusion_ref_zero_lock (FusionRef *ref)
+{
+     DirectResult ret;
+     
+     D_ASSERT( ref != NULL );
+
+     ret = fusion_skirmish_prevail( &ref->multi.builtin.lock );
+     if (ret)
+          return ret;
+     
+     if (ref->multi.builtin.call) {
+          ret = DR_ACCESSDENIED;
+     }
+     else {
+          if (ref->multi.builtin.local)
+               _fusion_check_locals( _fusion_world(ref->multi.shared), ref );
+          
+          while (ref->multi.builtin.local+ref->multi.builtin.global) {    
+               ret = fusion_skirmish_wait( &ref->multi.builtin.lock, 1000 ); /* 1 second */
+               if (ret && ret != DR_TIMEOUT);
+                    return ret;
+               
+               if (ref->multi.builtin.call) {
+                    ret = DR_ACCESSDENIED;
+                    break;
+               }
+
+               if (ref->multi.builtin.local)
+                    _fusion_check_locals( _fusion_world(ref->multi.shared), ref );
+          }
+     }
+          
+     if (ret)
+          fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+
+     return ret;
+}
+
+DirectResult
+fusion_ref_zero_trylock (FusionRef *ref)
+{
+     DirectResult ret;
+     
+     D_ASSERT( ref != NULL );
+
+     ret = fusion_skirmish_prevail( &ref->multi.builtin.lock );
+     if (ret)
+          return ret;
+
+     if (ref->multi.builtin.local)
+          _fusion_check_locals( _fusion_world(ref->multi.shared), ref );
+
+     if (ref->multi.builtin.local+ref->multi.builtin.global)
+          ret = DR_BUSY;
+     
+     if (ret)
+          fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+
+     return ret;
+}
+
+DirectResult
+fusion_ref_unlock (FusionRef *ref)
+{
+     D_ASSERT( ref != NULL );
+          
+     fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+
+     return DR_OK;
+}
+
+DirectResult
+fusion_ref_watch (FusionRef *ref, FusionCall *call, int call_arg)
+{
+     DirectResult ret;
+
+     D_ASSERT( ref != NULL );
+     D_ASSERT( call != NULL );
+
+     ret = fusion_skirmish_prevail( &ref->multi.builtin.lock );
+     if (ret)
+          return ret;
+     
+     if (ref->multi.builtin.local+ref->multi.builtin.global == 0) {
+          D_BUG( "ref has no references" );
+          ret = DR_BUG;
+     }
+     else if (ref->multi.builtin.call) {
+          ret = DR_BUSY;
+     }
+     else {
+          ref->multi.builtin.call = call;
+          ref->multi.builtin.call_arg = call_arg;
+          fusion_skirmish_notify( &ref->multi.builtin.lock );
+     }
+     
+     fusion_skirmish_dismiss( &ref->multi.builtin.lock );
+
+     return ret;
+}
+
+DirectResult
+fusion_ref_inherit (FusionRef *ref, FusionRef *from)
+{
+     D_ASSERT( ref != NULL );
+     D_ASSERT( from != NULL );
+
+     D_UNIMPLEMENTED();
+     
+     return fusion_ref_up( ref, true );
+}
+
+DirectResult
+fusion_ref_destroy (FusionRef *ref)
+{
+     FusionSkirmish *skirmish;
+     
+     D_ASSERT( ref != NULL );
+
+     D_DEBUG_AT( Fusion_Ref, "fusion_ref_destroy( %p )\n", ref );
+    
+     skirmish = &ref->multi.builtin.lock;
+     if (skirmish->multi.builtin.destroyed)
+          return DR_DESTROYED;
+
+     _fusion_remove_all_locals( _fusion_world(ref->multi.shared), ref ); 
+
+     fusion_skirmish_destroy( skirmish );
+
+     return DR_OK;
+}
+
+#endif /* FUSION_BUILD_KERNEL */
+
 #else /* FUSION_BUILD_MULTI */
 
 DirectResult
@@ -359,22 +635,31 @@ fusion_ref_init (FusionRef         *ref,
 
      ref->single.refs      = 0;
      ref->single.destroyed = false;
-     ref->single.waiting   = 0;
+     ref->single.locked    = 0;
 
-     return DFB_OK;
+     return DR_OK;
+}
+
+DirectResult
+fusion_ref_set_name (FusionRef  *ref,
+                     const char *name)
+{
+     return DR_OK;
 }
 
 DirectResult
 fusion_ref_up (FusionRef *ref, bool global)
 {
-     DirectResult ret = DFB_OK;
+     DirectResult ret = DR_OK;
 
      D_ASSERT( ref != NULL );
 
      pthread_mutex_lock (&ref->single.lock);
 
      if (ref->single.destroyed)
-          ret = DFB_DESTROYED;
+          ret = DR_DESTROYED;
+     else if (ref->single.locked)
+          ret = DR_LOCKED;
      else
           ref->single.refs++;
 
@@ -393,27 +678,32 @@ fusion_ref_down (FusionRef *ref, bool global)
      if (!ref->single.refs) {
           D_BUG( "no more references" );
           pthread_mutex_unlock (&ref->single.lock);
-          return DFB_BUG;
+          return DR_BUG;
      }
 
      if (ref->single.destroyed) {
           pthread_mutex_unlock (&ref->single.lock);
-          return DFB_DESTROYED;
+          return DR_DESTROYED;
      }
 
-     if (! --ref->single.refs && ref->single.call) {
-          FusionCall *call = ref->single.call;
+     if (! --ref->single.refs) {
+          if (ref->single.call) {
+               FusionCall *call = ref->single.call;
 
-          if (call->handler) {
-               pthread_mutex_unlock (&ref->single.lock);
-               call->handler( 0, ref->single.call_arg, NULL, call->ctx );
-               return DFB_OK;
+               if (call->handler) {
+                    int ret;
+                    pthread_mutex_unlock (&ref->single.lock);
+                    call->handler( 0, ref->single.call_arg, NULL, call->ctx, 0, &ret );
+                    return DR_OK;
+               }
           }
+          else
+               pthread_cond_broadcast (&ref->single.cond);
      }
 
      pthread_mutex_unlock (&ref->single.lock);
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -423,39 +713,36 @@ fusion_ref_stat (FusionRef *ref, int *refs)
      D_ASSERT( refs != NULL );
 
      if (ref->single.destroyed)
-          return DFB_DESTROYED;
+          return DR_DESTROYED;
 
      *refs = ref->single.refs;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
 fusion_ref_zero_lock (FusionRef *ref)
 {
-     DirectResult ret = DFB_OK;
+     DirectResult ret = DR_OK;
 
      D_ASSERT( ref != NULL );
 
      pthread_mutex_lock (&ref->single.lock);
 
-     if (ref->single.destroyed)
-          ret = DFB_DESTROYED;
-     else if (ref->single.call)
-          ret = DFB_ACCESSDENIED;
-     else while (ref->single.refs && !ret) {
-          ref->single.waiting++;
-          pthread_cond_wait (&ref->single.cond, &ref->single.lock);
-          ref->single.waiting--;
-
+     do {
           if (ref->single.destroyed)
-               ret = DFB_DESTROYED;
-          else if (ref->single.call)
-               ret = DFB_ACCESSDENIED;
-     }
+               ret = DR_DESTROYED;
+          else if (ref->single.locked)
+               ret = DR_LOCKED;
+          else if (ref->single.refs)
+               pthread_cond_wait (&ref->single.cond, &ref->single.lock);
+          else {
+               ref->single.locked = direct_gettid();
+               break;
+          }
+     } while (ret == DR_OK);
 
-     if (ret != DFB_OK)
-          pthread_mutex_unlock (&ref->single.lock);
+     pthread_mutex_unlock (&ref->single.lock);
 
      return ret;
 }
@@ -463,19 +750,22 @@ fusion_ref_zero_lock (FusionRef *ref)
 DirectResult
 fusion_ref_zero_trylock (FusionRef *ref)
 {
-     DirectResult ret = DFB_OK;
+     DirectResult ret = DR_OK;
 
      D_ASSERT( ref != NULL );
 
      pthread_mutex_lock (&ref->single.lock);
 
      if (ref->single.destroyed)
-          ret = DFB_DESTROYED;
+          ret = DR_DESTROYED;
+     else if (ref->single.locked)
+          ret = DR_LOCKED;
      else if (ref->single.refs)
-          ret = DFB_BUSY;
+          ret = DR_BUSY;
+     else
+          ref->single.locked = direct_gettid();
 
-     if (ret != DFB_OK)
-          pthread_mutex_unlock (&ref->single.lock);
+     pthread_mutex_unlock (&ref->single.lock);
 
      return ret;
 }
@@ -483,42 +773,49 @@ fusion_ref_zero_trylock (FusionRef *ref)
 DirectResult
 fusion_ref_unlock (FusionRef *ref)
 {
+     DirectResult ret = DR_OK;
+
      D_ASSERT( ref != NULL );
 
+     pthread_mutex_lock (&ref->single.lock);
+
+     if (ref->single.locked == direct_gettid()) {
+          ref->single.locked = 0;
+
+          pthread_cond_broadcast (&ref->single.cond);
+     }
+     else
+          ret = DR_ACCESSDENIED;
+
      pthread_mutex_unlock (&ref->single.lock);
 
-     return DFB_OK;
+     return ret;
 }
 
 DirectResult
 fusion_ref_watch (FusionRef *ref, FusionCall *call, int call_arg)
 {
+     DirectResult ret = DR_OK;
+
      D_ASSERT( ref != NULL );
      D_ASSERT( call != NULL );
 
      pthread_mutex_lock (&ref->single.lock);
 
-     if (ref->single.destroyed) {
-          pthread_mutex_unlock (&ref->single.lock);
-          return DFB_DESTROYED;
+     if (ref->single.destroyed)
+          ret = DR_DESTROYED;
+     else if (!ref->single.refs)
+          ret = DR_BUG;
+     else if (ref->single.call)
+          ret = DR_BUSY;
+     else {
+          ref->single.call     = call;
+          ref->single.call_arg = call_arg;
      }
 
-     if (!ref->single.refs) {
-          pthread_mutex_unlock (&ref->single.lock);
-          return DFB_BUG;
-     }
-
-     if (ref->single.call) {
-          pthread_mutex_unlock (&ref->single.lock);
-          return DFB_BUSY;
-     }
-
-     ref->single.call     = call;
-     ref->single.call_arg = call_arg;
-
      pthread_mutex_unlock (&ref->single.lock);
 
-     return DFB_OK;
+     return ret;
 }
 
 DirectResult
@@ -540,13 +837,12 @@ fusion_ref_destroy (FusionRef *ref)
 
      ref->single.destroyed = true;
 
-     if (ref->single.waiting)
-          pthread_cond_broadcast (&ref->single.cond);
+     pthread_cond_broadcast (&ref->single.cond);
 
-     pthread_mutex_unlock (&ref->single.lock);
+     pthread_mutex_destroy (&ref->single.lock);
      pthread_cond_destroy (&ref->single.cond);
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 #endif
