@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -52,11 +52,11 @@ Cambridge, MA 02139, USA.
 
 #include <unistd.h>
 
+#include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <fcntl.h>
-#include <grp.h>
 
 #include <direct/debug.h>
 #include <direct/list.h>
@@ -605,10 +605,9 @@ __shmalloc_init_heap( FusionSHM  *shm,
      int              heapsize = (space + BLOCKSIZE-1) / BLOCKSIZE;
      int              fd       = -1;
      shmalloc_heap   *heap     = NULL;
-     struct group    *pGroupInfo;
 
-     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, '%s', %p, %p )\n",
-                 __FUNCTION__, shm, filename, addr_base, ret_fd );
+     D_DEBUG_AT( Fusion_SHMHeap, "%s( %p, '%s', %p, %d, %p, %p )\n",
+                 __FUNCTION__, shm, filename, addr_base, space, ret_fd, ret_size );
 
      D_MAGIC_ASSERT( shm, FusionSHM );
      D_ASSERT( filename != NULL );
@@ -634,16 +633,10 @@ __shmalloc_init_heap( FusionSHM  *shm,
           goto error;
      }
 
-
-
-     if (fusion_config->shmfile_group) {
-          // obtain the group id #
-          pGroupInfo = getgrnam(fusion_config->shmfile_group);
-
-          // chgrp the SH_FILE dev entry
-          if (fchown(fd, -1, pGroupInfo->gr_gid) != 0) {
-               D_WARN("Fusion/SHM: changing permissions on /dev/shm/fusion.# failed... continuing on.");
-          }
+     if (fusion_config->shmfile_gid != (gid_t)-1) {
+          /* chgrp the SH_FILE dev entry */
+          if (fchown( fd, -1, fusion_config->shmfile_gid ) != 0)
+               D_WARN( "Fusion/SHM: Changing owner on %s failed... continuing on.", filename );
      }
 
      fchmod( fd, 0660 );
@@ -659,12 +652,9 @@ __shmalloc_init_heap( FusionSHM  *shm,
           goto error;
      }
 
-     close( fd );
-     fd = -1;
-
      if (heap != addr_base) {
           D_ERROR( "Fusion/SHM: mmap() returned address (%p) differs from requested (%p)\n", heap, addr_base );
-          ret = DFB_FUSION;
+          ret = DR_FUSION;
           goto error;
      }
 
@@ -680,7 +670,7 @@ __shmalloc_init_heap( FusionSHM  *shm,
      *ret_fd   = fd;
      *ret_size = size;
 
-     return DFB_OK;
+     return DR_OK;
 
 
 error:
@@ -741,12 +731,9 @@ __shmalloc_join_heap( FusionSHM  *shm,
           goto error;
      }
 
-     close( fd );
-     fd = -1;
-
      if (heap != addr_base) {
           D_ERROR( "Fusion/SHM: mmap() returned address (%p) differs from requested (%p)\n", heap, addr_base );
-          ret = DFB_FUSION;
+          ret = DR_FUSION;
           goto error;
      }
 
@@ -756,7 +743,7 @@ __shmalloc_join_heap( FusionSHM  *shm,
 
      *ret_fd = fd;
 
-     return DFB_OK;
+     return DR_OK;
 
 
 error:
@@ -797,15 +784,13 @@ __shmalloc_brk( shmalloc_heap *heap, int increment )
           int new_size = heap->size + increment;
 
           if (new_size > shared->max_size) {
-               D_WARN( "maximum shared memory file size (%dk) exceeded (%dk by %dk) in '%s'!",
-                       shared->max_size >> 10, new_size >> 10, increment >> 10, pool->filename );
+               D_WARN( "maximum shared memory size exceeded!" );
                fusion_dbg_print_memleaks( shared );
                return NULL;
           }
 
-          if (truncate( pool->filename, new_size ) < 0) {
-               D_PERROR( "Fusion/SHM: truncating shared memory file '%s' from %dkb to %dkb failed!\n",
-                         pool->filename, heap->size >> 10, new_size >> 10 );
+          if (ftruncate( pool->fd, new_size ) < 0) {
+               D_PERROR( "Fusion/SHM: ftruncating shared memory file failed!\n" );
                return NULL;
           }
 
