@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -103,7 +103,7 @@ fusion_shm_pool_create( FusionWorld          *world,
 
      if (max_size < 8192) {
           D_ERROR( "Fusion/SHMPool: Maximum size (%d) should be 8192 at least!\n", max_size );
-          return DFB_INVARG;
+          return DR_INVARG;
      }
 
      ret = fusion_skirmish_prevail( &shared->lock );
@@ -112,7 +112,7 @@ fusion_shm_pool_create( FusionWorld          *world,
 
      if (shared->num_pools == FUSION_SHM_MAX_POOLS) {
           D_ERROR( "Fusion/SHMPool: Maximum number of pools (%d) already reached!\n", FUSION_SHM_MAX_POOLS );
-          ret = DFB_LIMITEXCEEDED;
+          ret = DR_LIMITEXCEEDED;
           goto error;
      }
 
@@ -145,7 +145,7 @@ fusion_shm_pool_create( FusionWorld          *world,
 
      D_DEBUG_AT( Fusion_SHMPool, "  -> %p\n", *ret_pool );
 
-     return DFB_OK;
+     return DR_OK;
 
 
 error:
@@ -201,7 +201,7 @@ fusion_shm_pool_destroy( FusionWorld         *world,
 
      fusion_skirmish_dismiss( &shared->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -277,7 +277,7 @@ fusion_shm_pool_detach( FusionSHM           *shm,
 
      fusion_skirmish_dismiss( &pool->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -310,7 +310,7 @@ fusion_shm_pool_allocate( FusionSHMPoolShared  *pool,
      if (!data) {
           if (lock)
                fusion_skirmish_dismiss( &pool->lock );
-          return DFB_NOSHAREDMEMORY;
+          return DR_NOSHAREDMEMORY;
      }
 
      if (clear)
@@ -321,7 +321,7 @@ fusion_shm_pool_allocate( FusionSHMPoolShared  *pool,
      if (lock)
           fusion_skirmish_dismiss( &pool->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -357,7 +357,7 @@ fusion_shm_pool_reallocate( FusionSHMPoolShared  *pool,
      if (!new_data) {
           if (lock)
                fusion_skirmish_dismiss( &pool->lock );
-          return DFB_NOSHAREDMEMORY;
+          return DR_NOSHAREDMEMORY;
      }
 
      *ret_data = new_data;
@@ -365,7 +365,7 @@ fusion_shm_pool_reallocate( FusionSHMPoolShared  *pool,
      if (lock)
           fusion_skirmish_dismiss( &pool->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -396,11 +396,13 @@ fusion_shm_pool_deallocate( FusionSHMPoolShared *pool,
      if (lock)
           fusion_skirmish_dismiss( &pool->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 /**********************************************************************************************************************/
 
+#if FUSION_BUILD_KERNEL
+
 static DirectResult
 init_pool( FusionSHM           *shm,
            FusionSHMPool       *pool,
@@ -413,8 +415,8 @@ init_pool( FusionSHM           *shm,
      int                  fd;
      int                  size;
      FusionWorld         *world;
-     FusionSHMPoolNew     pool_new    = {0};
-     FusionSHMPoolAttach  pool_attach = {0};
+     FusionSHMPoolNew     pool_new    = { .pool_id = 0 };
+     FusionSHMPoolAttach  pool_attach = { .pool_id = 0 };
      FusionEntryInfo      info;
      char                 buf[FUSION_SHM_TMPFS_PATH_NAME_LEN + 32];
 
@@ -445,7 +447,7 @@ init_pool( FusionSHM           *shm,
                continue;
 
           D_PERROR( "Fusion/SHM: FUSION_SHMPOOL_NEW failed!\n" );
-          return DFB_FUSION;
+          return DR_FUSION;
      }
 
      /* Set the pool info. */
@@ -474,7 +476,7 @@ init_pool( FusionSHM           *shm,
                }
           }
 
-          return DFB_FUSION;
+          return DR_FUSION;
      }
 
 
@@ -523,7 +525,7 @@ init_pool( FusionSHM           *shm,
 
      shared->name = SHSTRDUP( shared, name );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -534,7 +536,7 @@ join_pool( FusionSHM           *shm,
      DirectResult         ret;
      int                  fd;
      FusionWorld         *world;
-     FusionSHMPoolAttach  pool_attach = {0};
+     FusionSHMPoolAttach  pool_attach = { .pool_id = 0 };
      char                 buf[FUSION_SHM_TMPFS_PATH_NAME_LEN + 32];
 
      D_DEBUG_AT( Fusion_SHMPool, "%s( %p, %p, %p )\n", __FUNCTION__, shm, pool, shared );
@@ -546,7 +548,7 @@ join_pool( FusionSHM           *shm,
 #if !DIRECT_BUILD_DEBUGS
      if (shared->debug) {
           D_ERROR( "Fusion/SHM: Can't join debug enabled pool with pure-release library!\n" );
-          return DFB_UNSUPPORTED;
+          return DR_UNSUPPORTED;
      }
 #endif
 
@@ -564,7 +566,7 @@ join_pool( FusionSHM           *shm,
                continue;
 
           D_PERROR( "Fusion/SHM: FUSION_SHMPOOL_ATTACH failed!\n" );
-          return DFB_FUSION;
+          return DR_FUSION;
      }
 
 
@@ -597,7 +599,7 @@ join_pool( FusionSHM           *shm,
 
      D_MAGIC_SET( pool, FusionSHMPool );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static void
@@ -687,8 +689,221 @@ shutdown_pool( FusionSHM           *shm,
      D_MAGIC_CLEAR( shared );
 }
 
+#else /* FUSION_BUILD_KERNEL */
+
+static DirectResult
+init_pool( FusionSHM           *shm,
+           FusionSHMPool       *pool,
+           FusionSHMPoolShared *shared,
+           const char          *name,
+           unsigned int         max_size,
+           bool                 debug )
+{
+     DirectResult   ret;
+     int            fd;
+     int            size;
+     long           page_size;
+     int            pool_id;
+     unsigned int   pool_max_size;
+     void          *pool_addr_base = NULL;
+     FusionWorld   *world;
+     char           buf[FUSION_SHM_TMPFS_PATH_NAME_LEN + 32];
+
+     D_DEBUG_AT( Fusion_SHMPool, "%s( %p, %p, %p, '%s', %d, %sdebug )\n",
+                 __FUNCTION__, shm, pool, shared, name, max_size, debug ? "" : "non-" );
+
+     D_MAGIC_ASSERT( shm, FusionSHM );
+     D_MAGIC_ASSERT( shm->shared, FusionSHMShared );
+     D_ASSERT( name != NULL );
+     D_ASSERT( max_size > sizeof(shmalloc_heap) );
+
+     world = shm->world;
+
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     D_ASSERT( pool != NULL );
+     D_ASSERT( shared != NULL );
+
+     page_size = direct_pagesize();
+
+     pool_id = ++world->shared->pool_ids;
+
+     pool_max_size = max_size + BLOCKALIGN(sizeof(shmalloc_heap)) +
+                                BLOCKALIGN( (max_size + BLOCKSIZE-1) / BLOCKSIZE * sizeof(shmalloc_info) );
+
+     pool_addr_base = world->shared->pool_base;
+     world->shared->pool_base += ((pool_max_size + page_size - 1) & ~(page_size - 1)) + page_size;
+     /* Exceeded limit? */
+     if (world->shared->pool_base > world->shared->pool_max)
+          return DR_NOSHAREDMEMORY;
+
+     /* Generate filename. */
+     snprintf( buf, sizeof(buf), "%s/fusion.%d.%d", shm->shared->tmpfs,
+               fusion_world_index( world ), pool_id );
+
+     /* Initialize the heap. */
+     ret = __shmalloc_init_heap( shm, buf, pool_addr_base, max_size, &fd, &size );
+     if (ret)
+          return ret;
+
+     /* Initialize local data. */
+     pool->attached = true;
+     pool->shm      = shm;
+     pool->shared   = shared;
+     pool->pool_id  = pool_id;
+     pool->fd       = fd;
+     pool->filename = D_STRDUP( buf );
+
+     /* Initialize shared data. */
+     shared->active     = true;
+     shared->debug      = debug;
+     shared->shm        = shm->shared;
+     shared->max_size   = pool_max_size;
+     shared->pool_id    = pool_id;
+     shared->addr_base  = pool_addr_base;
+     shared->heap       = pool_addr_base;
+     shared->heap->pool = shared;
+
+     fusion_skirmish_init( &shared->lock, name, world );
+
+
+     D_MAGIC_SET( pool, FusionSHMPool );
+     D_MAGIC_SET( shared, FusionSHMPoolShared );
+
+
+     shared->name = SHSTRDUP( shared, name );
+
+     return DR_OK;
+}
+
+static DirectResult
+join_pool( FusionSHM           *shm,
+           FusionSHMPool       *pool,
+           FusionSHMPoolShared *shared )
+{
+     DirectResult  ret;
+     int           fd;
+     FusionWorld  *world;
+     char          buf[FUSION_SHM_TMPFS_PATH_NAME_LEN + 32];
+
+     D_DEBUG_AT( Fusion_SHMPool, "%s( %p, %p, %p )\n", __FUNCTION__, shm, pool, shared );
+
+     D_MAGIC_ASSERT( shm, FusionSHM );
+     D_MAGIC_ASSERT( shm->shared, FusionSHMShared );
+     D_MAGIC_ASSERT( shared, FusionSHMPoolShared );
+
+#if !DIRECT_BUILD_DEBUGS
+     if (shared->debug) {
+          D_ERROR( "Fusion/SHM: Can't join debug enabled pool with pure-release library!\n" );
+          return DR_UNSUPPORTED;
+     }
+#endif
+
+     world = shm->world;
+
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     /* Generate filename. */
+     snprintf( buf, sizeof(buf), "%s/fusion.%d.%d", shm->shared->tmpfs,
+               fusion_world_index( shm->world ), shared->pool_id );
+
+     /* Join the heap. */
+     ret = __shmalloc_join_heap( shm, buf, shared->addr_base, shared->max_size, &fd );
+     if (ret)
+          return ret;
+
+     /* Initialize local data. */
+     pool->attached = true;
+     pool->shm      = shm;
+     pool->shared   = shared;
+     pool->pool_id  = shared->pool_id;
+     pool->fd       = fd;
+     pool->filename = D_STRDUP( buf );
+
+
+     D_MAGIC_SET( pool, FusionSHMPool );
+
+     return DR_OK;
+}
+
+static void
+leave_pool( FusionSHM           *shm,
+            FusionSHMPool       *pool,
+            FusionSHMPoolShared *shared )
+{
+     FusionWorld *world;
+
+     D_DEBUG_AT( Fusion_SHMPool, "%s( %p, %p, %p )\n", __FUNCTION__, shm, pool, shared );
+
+     D_MAGIC_ASSERT( shm, FusionSHM );
+     D_MAGIC_ASSERT( pool, FusionSHMPool );
+     D_MAGIC_ASSERT( shared, FusionSHMPoolShared );
+
+     world = shm->world;
+
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     if (munmap( shared->addr_base, shared->max_size ))
+          D_PERROR( "Fusion/SHM: Could not munmap shared memory file '%s'!\n", pool->filename );
+
+     if (pool->fd != -1 && close( pool->fd ))
+          D_PERROR( "Fusion/SHM: Could not close shared memory file '%s'!\n", pool->filename );
+
+     pool->attached = false;
+
+     D_FREE( pool->filename );
+
+     D_MAGIC_CLEAR( pool );
+}
+
+static void
+shutdown_pool( FusionSHM           *shm,
+               FusionSHMPool       *pool,
+               FusionSHMPoolShared *shared )
+{
+     FusionWorld *world;
+
+     D_DEBUG_AT( Fusion_SHMPool, "%s( %p, %p, %p )\n", __FUNCTION__, shm, pool, shared );
+
+     D_MAGIC_ASSERT( shm, FusionSHM );
+     D_MAGIC_ASSERT( pool, FusionSHMPool );
+     D_MAGIC_ASSERT( shared, FusionSHMPoolShared );
+
+     world = shm->world;
+
+     D_MAGIC_ASSERT( world, FusionWorld );
+
+     SHFREE( shared, shared->name );
+
+     fusion_dbg_print_memleaks( shared );
+
+     if (munmap( shared->addr_base, shared->max_size ))
+          D_PERROR( "Fusion/SHM: Could not munmap shared memory file '%s'!\n", pool->filename );
+
+     if (pool->fd != -1 && close( pool->fd ))
+          D_PERROR( "Fusion/SHM: Could not close shared memory file '%s'!\n", pool->filename );
+
+     if (unlink( pool->filename ))
+          D_PERROR( "Fusion/SHM: Could not unlink shared memory file '%s'!\n", pool->filename );
+
+     shared->active = false;
+
+     pool->attached = false;
+
+     D_FREE( pool->filename );
+
+     D_MAGIC_CLEAR( pool );
+
+     fusion_skirmish_destroy( &shared->lock );
+
+     D_MAGIC_CLEAR( shared );
+}
+
+#endif /* FUSION_BUILD_KERNEL */
+
 /**********************************************************************************************************************/
 
+#if FUSION_BUILD_KERNEL
 void
 _fusion_shmpool_process( FusionWorld          *world,
                          int                   pool_id,
@@ -736,4 +951,4 @@ _fusion_shmpool_process( FusionWorld          *world,
 
      fusion_skirmish_dismiss( &shared->lock );
 }
-
+#endif /* FUSION_BUILD_KERNEL */
