@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -31,6 +31,7 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/vfs.h>
@@ -72,8 +73,8 @@ find_tmpfs( char *name, int len )
           mount_point = strsep( &pointer, " " );
           mount_fs = strsep( &pointer, " " );
 
-          if (mount_fs && mount_point && (!strcmp( mount_fs, "tmpfs" ) ||
-                                          !strcmp( mount_fs, "shmfs" )))
+          if (mount_fs && mount_point && !access( mount_point, W_OK ) && 
+             (!strcmp( mount_fs, "tmpfs" ) || !strcmp( mount_fs, "shmfs" ) || !strcmp( mount_fs, "ramfs" )))
           {
                struct statfs stat;
                int           bytes;
@@ -185,7 +186,7 @@ fusion_shm_init( FusionWorld *world )
           fusion_skirmish_dismiss( &shared->lock );
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -245,7 +246,7 @@ fusion_shm_deinit( FusionWorld *world )
 
      D_MAGIC_CLEAR( shm );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -269,7 +270,6 @@ fusion_shm_attach_unattached( FusionWorld *world )
      if (ret)
           return ret;
 
-
      for (i=0; i<FUSION_SHM_MAX_POOLS; i++) {
           if (!shared->pools[i].active)
                continue;
@@ -285,9 +285,53 @@ fusion_shm_attach_unattached( FusionWorld *world )
                D_MAGIC_ASSERT( &shm->pools[i], FusionSHMPool );
      }
 
+     fusion_skirmish_dismiss( &shared->lock );
+
+     return DR_OK;
+}
+
+DirectResult
+fusion_shm_enum_pools( FusionWorld           *world,
+                       FusionSHMPoolCallback  callback,
+                       void                  *ctx )
+{
+     int              i;
+     DirectResult     ret;
+     FusionSHM       *shm;
+     FusionSHMShared *shared;
+
+     D_MAGIC_ASSERT( world, FusionWorld );
+     D_MAGIC_ASSERT( world->shared, FusionWorldShared );
+     D_ASSERT( callback != NULL );
+
+     shm    = &world->shm;
+     shared = &world->shared->shm;
+
+     D_MAGIC_ASSERT( shm, FusionSHM );
+     D_MAGIC_ASSERT( shared, FusionSHMShared );
+
+     ret = fusion_skirmish_prevail( &shared->lock );
+     if (ret)
+          return ret;
+
+     for (i=0; i<FUSION_SHM_MAX_POOLS; i++) {
+          if (!shared->pools[i].active)
+               continue;
+
+          if (!shm->pools[i].attached) {
+               D_BUG( "not attached to pool" );
+               continue;
+          }
+
+          D_MAGIC_ASSERT( &shm->pools[i], FusionSHMPool );
+          D_MAGIC_ASSERT( &shared->pools[i], FusionSHMPoolShared );
+
+          if (callback( &shm->pools[i], ctx ) == DENUM_CANCEL)
+               break;
+     }
 
      fusion_skirmish_dismiss( &shared->lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
