@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -31,6 +31,8 @@
 
 #include <fusion/types.h>
 
+typedef DirectEnumerationResult (*FusionSHMPoolCallback)( FusionSHMPool *pool,
+                                                          void          *ctx );
 
 DirectResult fusion_shm_init  ( FusionWorld *world );
 
@@ -38,5 +40,9 @@ DirectResult fusion_shm_deinit( FusionWorld *world );
 
 DirectResult fusion_shm_attach_unattached( FusionWorld *world );
 
+DirectResult fusion_shm_enum_pools( FusionWorld           *world,
+                                    FusionSHMPoolCallback  callback,
+                                    void                  *ctx );
+
 #endif
 
