@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -39,6 +40,7 @@
 #include <fusion/build.h>
 #include <fusion/shmalloc.h>
 
+#include <fusion/fusion_internal.h>
 #include <fusion/shm/shm_internal.h>
 
 
@@ -48,11 +50,14 @@
 
 #if DIRECT_BUILD_DEBUGS  /* Build with debug support? */
 
+D_DEBUG_DOMAIN( Fusion_SHM, "Fusion/SHM", "Fusion Shared Memory" );
+
 void
 fusion_dbg_print_memleaks( FusionSHMPoolShared *pool )
 {
      DirectResult  ret;
      SHMemDesc    *desc;
+     unsigned int  total = 0;
 
      D_MAGIC_ASSERT( pool, FusionSHMPoolShared );
 
@@ -63,19 +68,26 @@ fusion_dbg_print_memleaks( FusionSHMPoolShared *pool )
      }
 
      if (pool->allocs) {
-          D_DEBUG( "Shared memory allocations remaining (%d): \n",
-                   direct_list_count_elements_EXPENSIVE( pool->allocs ) );
+          direct_log_printf( NULL, "\nShared memory allocations remaining (%d) in '%s': \n",
+                             direct_list_count_elements_EXPENSIVE( pool->allocs ), pool->name );
 
-          direct_list_foreach (desc, pool->allocs)
-               D_DEBUG( "%7d bytes at %p allocated in %s (%s: %u)\n",
-                        desc->bytes, desc->mem, desc->func, desc->file, desc->line );
+          direct_list_foreach (desc, pool->allocs) {
+               direct_log_printf( NULL, " %9zu bytes at %p [%8lu] in %-30s [%3lx] (%s: %u)\n",
+                                  desc->bytes, desc->mem, (ulong)desc->mem - (ulong)pool->heap,
+                                  desc->func, desc->fid, desc->file, desc->line );
+
+               total += desc->bytes;
+          }
+
+          direct_log_printf( NULL, "   -------\n  %7dk total\n", total >> 10 );
+          direct_log_printf( NULL, "\nShared memory file size: %dk\n", pool->heap->size >> 10 );
      }
 
      fusion_skirmish_dismiss( &pool->lock );
 }
 
 static SHMemDesc *
-fill_shmem_desc( SHMemDesc *desc, int bytes, const char *func, const char *file, int line )
+fill_shmem_desc( SHMemDesc *desc, int bytes, const char *func, const char *file, int line, FusionID fusion_id )
 {
      D_ASSERT( desc != NULL );
 
@@ -86,6 +98,7 @@ fill_shmem_desc( SHMemDesc *desc, int bytes, const char *func, const char *file,
      snprintf( desc->file, SHMEMDESC_FILE_NAME_LENGTH, file );
 
      desc->line = line;
+     desc->fid  = fusion_id;
 
      return desc;
 }
@@ -109,8 +122,6 @@ fusion_dbg_shmalloc( FusionSHMPoolShared *pool,
      if (!pool->debug)
           return fusion_shmalloc( pool, __size );
 
-     D_HEAVYDEBUG("Fusion/SHM: allocating %7d bytes in %s (%s: %u)\n", __size, func, file, line);
-
      /* Lock the pool. */
      ret = fusion_skirmish_prevail( &pool->lock );
      if (ret) {
@@ -121,13 +132,17 @@ fusion_dbg_shmalloc( FusionSHMPoolShared *pool,
      /* Allocate memory from the pool. */
      ret = fusion_shm_pool_allocate( pool, __size + sizeof(SHMemDesc), false, false, &data );
      if (ret) {
-          D_DERROR( ret, "Fusion/SHM: Could not allocate %d bytes from pool!\n", __size + sizeof(SHMemDesc) );
+          D_DERROR( ret, "Fusion/SHM: Could not allocate %zu bytes from pool!\n", __size + sizeof(SHMemDesc) );
           fusion_skirmish_dismiss( &pool->lock );
           return NULL;
      }
 
      /* Fill description. */
-     desc = fill_shmem_desc( data, __size, func, file, line );
+     desc = fill_shmem_desc( data, __size, func, file, line, _fusion_id(pool->shm->world) );
+
+     D_DEBUG_AT( Fusion_SHM, "allocating %9zu bytes at %p [%8lu] in %-30s [%3lx] (%s: %u)\n",
+                 desc->bytes, desc->mem, (ulong)desc->mem - (ulong)pool->heap,
+                 desc->func, desc->fid, desc->file, desc->line );
 
      /* Add description to list. */
      direct_list_append( &pool->allocs, &desc->link );
@@ -158,8 +173,6 @@ fusion_dbg_shcalloc( FusionSHMPoolShared *pool,
      if (!pool->debug)
           return fusion_shcalloc( pool, __nmemb, __size );
 
-     D_HEAVYDEBUG("Fusion/SHM: allocating %7d bytes in %s (%s: %u)\n", __size, func, file, line);
-
      /* Lock the pool. */
      ret = fusion_skirmish_prevail( &pool->lock );
      if (ret) {
@@ -170,13 +183,17 @@ fusion_dbg_shcalloc( FusionSHMPoolShared *pool,
      /* Allocate memory from the pool. */
      ret = fusion_shm_pool_allocate( pool, __nmemb * __size + sizeof(SHMemDesc), true, false, &data );
      if (ret) {
-          D_DERROR( ret, "Fusion/SHM: Could not allocate %d bytes from pool!\n", __nmemb * __size + sizeof(SHMemDesc) );
+          D_DERROR( ret, "Fusion/SHM: Could not allocate %zu bytes from pool!\n", __nmemb * __size + sizeof(SHMemDesc) );
           fusion_skirmish_dismiss( &pool->lock );
           return NULL;
      }
 
      /* Fill description. */
-     desc = fill_shmem_desc( data, __nmemb * __size, func, file, line );
+     desc = fill_shmem_desc( data, __nmemb * __size, func, file, line, _fusion_id(pool->shm->world) );
+
+     D_DEBUG_AT( Fusion_SHM, "allocating %9zu bytes at %p [%8lu] in %-30s [%3lx] (%s: %u)\n",
+                 desc->bytes, desc->mem, (ulong)desc->mem - (ulong)pool->heap,
+                 desc->func, desc->fid, desc->file, desc->line );
 
      /* Add description to list. */
      direct_list_append( &pool->allocs, &desc->link );
@@ -242,15 +259,19 @@ fusion_dbg_shrealloc( FusionSHMPoolShared *pool,
      /* Reallocate the memory block. */
      ret = fusion_shm_pool_reallocate( pool, __ptr - sizeof(SHMemDesc), __size + sizeof(SHMemDesc), false, &data );
      if (ret) {
-          D_DERROR( ret, "Fusion/SHM: Could not reallocate from %d to %d bytes!\n",
+          D_DERROR( ret, "Fusion/SHM: Could not reallocate from %zu to %zu bytes!\n",
                     desc->bytes + sizeof(SHMemDesc), __size + sizeof(SHMemDesc) );
           fusion_skirmish_dismiss( &pool->lock );
           return NULL;
      }
 
      /* Fill description. */
-     desc = fill_shmem_desc( data, __size, func, file, line );
-     
+     desc = fill_shmem_desc( data, __size, func, file, line, _fusion_id(pool->shm->world) );
+
+     D_DEBUG_AT( Fusion_SHM, "reallocating %9zu bytes at %p [%8lu] in %-30s [%3lx] (%s: %u) '%s'\n",
+                 desc->bytes, desc->mem, (ulong)desc->mem - (ulong)pool->heap,
+                 desc->func, desc->fid, desc->file, desc->line, what );
+
      /* Add description to list. */
      direct_list_append( &pool->allocs, &desc->link );
 
@@ -299,6 +320,10 @@ fusion_dbg_shfree( FusionSHMPoolShared *pool,
           return; /* shouldn't happen due to the break */
      }
 
+     D_DEBUG_AT( Fusion_SHM, "freeing %9zu bytes at %p [%8lu] in %-30s [%3lx] (%s: %u) '%s'\n",
+                 desc->bytes, desc->mem, (ulong)desc->mem - (ulong)pool->heap,
+                 desc->func, desc->fid, desc->file, desc->line, what );
+
      /* Remove the description. */
      direct_list_remove( &pool->allocs, &desc->link );
 
@@ -331,8 +356,6 @@ fusion_dbg_shstrdup( FusionSHMPoolShared *pool,
 
      length = strlen( string ) + 1;
 
-     D_HEAVYDEBUG("Fusion/SHM: allocating %7d bytes in %s (%s: %u)\n", length, func, file, line);
-
      /* Lock the pool. */
      ret = fusion_skirmish_prevail( &pool->lock );
      if (ret) {
@@ -343,13 +366,19 @@ fusion_dbg_shstrdup( FusionSHMPoolShared *pool,
      /* Allocate memory from the pool. */
      ret = fusion_shm_pool_allocate( pool, length + sizeof(SHMemDesc), false, false, &data );
      if (ret) {
-          D_DERROR( ret, "Fusion/SHM: Could not allocate %d bytes from pool!\n", length + sizeof(SHMemDesc) );
+          D_DERROR( ret, "Fusion/SHM: Could not allocate %zu bytes from pool!\n", length + sizeof(SHMemDesc) );
           fusion_skirmish_dismiss( &pool->lock );
           return NULL;
      }
 
      /* Fill description. */
-     desc = fill_shmem_desc( data, length, func, file, line );
+     desc = fill_shmem_desc( data, length, func, file, line, _fusion_id(pool->shm->world) );
+
+     D_DEBUG_AT( Fusion_SHM, "allocating %9zu bytes at %p [%8lu] in %-30s [%3lx] (%s: %u) <- \"%s\"\n",
+                 desc->bytes, desc->mem, (ulong)desc->mem - (ulong)pool->heap,
+                 desc->func, desc->fid, desc->file, desc->line, string );
+
+     D_DEBUG_AT( Fusion_SHM, "  -> allocs: %p\n", pool->allocs );
 
      /* Add description to list. */
      direct_list_append( &pool->allocs, &desc->link );
