@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -181,7 +181,7 @@ voodoo_manager_create( int             fd,
      manager = D_CALLOC( 1, sizeof(VoodooManager) );
      if (!manager) {
           D_WARN( "out of memory" );
-          return DFB_NOSYSTEMMEMORY;
+          return DR_NOLOCALMEMORY;
      }
 
      D_DEBUG( "Voodoo/Manager: Creating manager at %p.\n", manager );
@@ -248,7 +248,7 @@ voodoo_manager_create( int             fd,
      /* Return the new manager. */
      *ret_manager = manager;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -258,7 +258,7 @@ voodoo_manager_quit( VoodooManager *manager )
      D_ASSUME( !manager->quit );
 
      if (manager->quit)
-          return DFB_OK;
+          return DR_OK;
 
      D_DEBUG( "Voodoo/Manager: Quitting manager at %p!\n", manager );
 
@@ -278,14 +278,14 @@ voodoo_manager_quit( VoodooManager *manager )
      pthread_cond_broadcast( &manager->output.wait );
      pthread_mutex_unlock( &manager->output.lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static bool
-instance_iterator( DirectHash *hash,
-                   u32         key,
-                   void       *value,
-                   void       *ctx )
+instance_iterator( DirectHash    *hash,
+                   unsigned long  key,
+                   void          *value,
+                   void          *ctx )
 {
      bool            super    = (unsigned long) ctx;
      VoodooInstance *instance = value;
@@ -300,7 +300,7 @@ instance_iterator( DirectHash *hash,
      }
 
 
-     D_DEBUG( "Voodoo/Manager: Releasing dispatcher interface %p %s(instance %u)...\n",
+     D_DEBUG( "Voodoo/Manager: Releasing dispatcher interface %p %s(instance %lu)...\n",
               instance->proxy, instance->super ? "[super] " : "", key );
 
      D_ASSERT( instance->proxy != NULL );
@@ -309,7 +309,7 @@ instance_iterator( DirectHash *hash,
      instance->proxy->Release( instance->proxy );
 
 
-     D_DEBUG( "Voodoo/Manager: Releasing real interface %p %s(instance %u)...\n",
+     D_DEBUG( "Voodoo/Manager: Releasing real interface %p %s(instance %lu)...\n",
               instance->real, instance->super ? "[super] " : "", key );
 
      D_ASSERT( instance->real != NULL );
@@ -366,7 +366,7 @@ voodoo_manager_destroy( VoodooManager *manager )
      /* Deallocate manager structure. */
      D_FREE( manager );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 bool
@@ -418,7 +418,7 @@ voodoo_manager_super( VoodooManager    *manager,
      /* Append the name of the super interface to create. */
      direct_memcpy( msg + 1, name, len );
 
-     D_DEBUG( "Voodoo/Manager: Sending SUPER message %zu for '%s' (%d bytes).\n", serial, name, size );
+     D_DEBUG( "Voodoo/Manager: Sending SUPER message %llu for '%s' (%d bytes).\n", (unsigned long long)serial, name, size );
 
      /* Unlock the output buffer. */
      manager_unlock_output( manager, true );
@@ -432,9 +432,9 @@ voodoo_manager_super( VoodooManager    *manager,
           return ret;
      }
 
-     D_DEBUG( "Voodoo/Manager: Got response %zu (%s) with instance %u for request %zu "
-              "(%d bytes).\n", response->header.serial, DirectResultString( ret ),
-              response->instance, response->request, response->header.size );
+     D_DEBUG( "Voodoo/Manager: Got response %llu (%s) with instance %u for request %llu "
+              "(%d bytes).\n", (unsigned long long)response->header.serial, DirectResultString( ret ),
+              response->instance, (unsigned long long)response->request, response->header.size );
 
      ret = response->result;
      if (ret) {
@@ -452,7 +452,7 @@ voodoo_manager_super( VoodooManager    *manager,
      /* Unlock the response buffer. */
      manager_unlock_response( manager, response );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static inline int
@@ -523,7 +523,7 @@ calc_blocks( VoodooMessageBlockType type, va_list args )
                     D_BREAK( "unknown message block type" );
           }
 
-          size += len + 8; /* for type and length */
+          size += 8 + VOODOO_MSG_ALIGN(len);
 
           type = va_arg( args, VoodooMessageBlockType );
      }
@@ -534,9 +534,11 @@ calc_blocks( VoodooMessageBlockType type, va_list args )
 static inline void
 write_blocks( void *dst, VoodooMessageBlockType type, va_list args )
 {
+     u32 *d32 = dst;
+
      while (type != VMBT_NONE) {
-          u32 arg = 0;
-          int   len = 0;
+          u32   arg = 0;
+          u32   len = 0;
           void *ptr = NULL;
 
           switch (type) {
@@ -565,24 +567,27 @@ write_blocks( void *dst, VoodooMessageBlockType type, va_list args )
           }
 
           /* Write block type and length. */
-          *(u32*) (dst + 0) = type;
-          *(s32*) (dst + 4) = len;
+          d32[0] = type;
+          d32[1] = len;
 
           /* Write block content. */
           if (ptr)
-               direct_memcpy( dst + 8, ptr, len );
-          else if (len)
-               *(u32*) (dst + 8) = arg;
+               direct_memcpy( &d32[2], ptr, len );
+          else if (len) {
+               D_ASSERT( len == 4 );
+
+               d32[2] = arg;
+          }
 
           /* Advance message data pointer. */
-          dst += len + 8;
+          d32 += 2 + (VOODOO_MSG_ALIGN(len) >> 2);
 
           /* Fetch next message block type. */
           type = va_arg( args, VoodooMessageBlockType );
      }
 
      /* Write terminator. */
-     *(u32*) dst = VMBT_NONE;
+     d32[0] = VMBT_NONE;
 }
 
 DirectResult
@@ -641,8 +646,8 @@ voodoo_manager_request( VoodooManager           *manager,
      va_end( args );
 
 
-     D_DEBUG( "Voodoo/Manager: Sending REQUEST message %zu to %u::%u %s(%d bytes).\n",
-              serial, instance, method, (flags & VREQ_RESPOND) ? "[RESPONDING] " : "", size );
+     D_DEBUG( "Voodoo/Manager: Sending REQUEST message %llu to %u::%u %s(%d bytes).\n",
+              (unsigned long long)serial, instance, method, (flags & VREQ_RESPOND) ? "[RESPONDING] " : "", size );
 
      /* Unlock the output buffer. */
      manager_unlock_output( manager, !(flags & VREQ_QUEUE) );
@@ -659,14 +664,14 @@ voodoo_manager_request( VoodooManager           *manager,
                return ret;
           }
 
-          D_DEBUG( "Voodoo/Manager: Got response %zu (%s) with instance %u for request %zu "
-                   "(%d bytes).\n", response->header.serial, DirectResultString( response->result ),
-                   response->instance, response->request, response->header.size );
+          D_DEBUG( "Voodoo/Manager: Got response %llu (%s) with instance %u for request %llu "
+                   "(%d bytes).\n", (unsigned long long)response->header.serial, DirectResultString( response->result ),
+                   response->instance, (unsigned long long)response->request, response->header.size );
 
           *ret_response = response;
      }
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -697,7 +702,7 @@ voodoo_manager_respond( VoodooManager          *manager,
      D_MAGIC_ASSERT( manager, VoodooManager );
 
      D_DEBUG( "Voodoo/Response: "
-              "Request %zu, result %d, instance %u...\n", request, result, instance );
+              "Request %llu, result %d, instance %u...\n", (unsigned long long)request, result, instance );
 
      /* Calculate the total message size. */
      va_start( args, block_type );
@@ -733,13 +738,13 @@ voodoo_manager_respond( VoodooManager          *manager,
 
 
      D_DEBUG( "Voodoo/Manager: "
-              "Sending RESPONSE message %zu (%s) with instance %u for request %zu (%d bytes).\n",
-              serial, DirectResultString( result ), instance, request, size );
+              "Sending RESPONSE message %llu (%s) with instance %u for request %llu (%d bytes).\n",
+              (unsigned long long)serial, DirectResultString( result ), instance, (unsigned long long)request, size );
 
      /* Unlock the output buffer. */
      manager_unlock_output( manager, true );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -763,7 +768,7 @@ voodoo_manager_register_local( VoodooManager    *manager,
      instance = D_CALLOC( 1, sizeof(VoodooInstance) );
      if (!instance) {
           D_WARN( "out of memory" );
-          return DFB_NOSYSTEMMEMORY;
+          return DR_NOLOCALMEMORY;
      }
 
      instance->super    = super;
@@ -790,7 +795,7 @@ voodoo_manager_register_local( VoodooManager    *manager,
 
      *ret_instance = instance_id;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -812,7 +817,7 @@ voodoo_manager_lookup_local( VoodooManager     *manager,
      pthread_mutex_unlock( &manager->instances.lock );
 
      if (!instance)
-          return DFB_NOSUCHINSTANCE;
+          return DR_NOSUCHINSTANCE;
 
      if (ret_dispatcher)
           *ret_dispatcher = instance->proxy;
@@ -820,7 +825,7 @@ voodoo_manager_lookup_local( VoodooManager     *manager,
      if (ret_real)
           *ret_real = instance->real;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 DirectResult
@@ -839,7 +844,7 @@ voodoo_manager_register_remote( VoodooManager    *manager,
      instance = D_CALLOC( 1, sizeof(VoodooInstance) );
      if (!instance) {
           D_WARN( "out of memory" );
-          return DFB_NOSYSTEMMEMORY;
+          return DR_NOLOCALMEMORY;
      }
 
      instance->super = super;
@@ -860,7 +865,7 @@ voodoo_manager_register_remote( VoodooManager    *manager,
      D_DEBUG( "Voodoo/Manager: "
               "Added remote instance %u, requestor %p.\n", instance_id, requestor );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 
@@ -882,11 +887,11 @@ voodoo_manager_lookup_remote( VoodooManager     *manager,
      pthread_mutex_unlock( &manager->instances.lock );
 
      if (!instance)
-          return DFB_NOSUCHINSTANCE;
+          return DR_NOSUCHINSTANCE;
 
      *ret_requestor = instance->proxy;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 /**************************************************************************************************/
@@ -925,8 +930,8 @@ handle_super( VoodooManager      *manager,
 
      name = (const char *) (super + 1);
 
-     D_DEBUG( "Voodoo/Dispatch: Handling SUPER message %zu for '%s' (%d bytes).\n",
-              super->header.serial, name, super->header.size );
+     D_DEBUG( "Voodoo/Dispatch: Handling SUPER message %llu for '%s' (%d bytes).\n",
+              (unsigned long long)super->header.serial, name, super->header.size );
 
      if (manager->server) {
           VoodooInstanceID instance;
@@ -940,13 +945,13 @@ handle_super( VoodooManager      *manager,
           }
 
           voodoo_manager_respond( manager, super->header.serial,
-                                  DFB_OK, instance,
+                                  DR_OK, instance,
                                   VMBT_NONE );
      }
      else {
           D_WARN( "can't handle this as a client" );
           voodoo_manager_respond( manager, super->header.serial,
-                                  DFB_UNSUPPORTED, VOODOO_INSTANCE_NONE,
+                                  DR_UNSUPPORTED, VOODOO_INSTANCE_NONE,
                                   VMBT_NONE );
      }
 }
@@ -990,8 +995,8 @@ handle_request( VoodooManager        *manager,
      D_ASSERT( request->header.size >= sizeof(VoodooRequestMessage) );
      D_ASSERT( request->header.type == VMSG_REQUEST );
 
-     D_DEBUG( "Voodoo/Dispatch: Handling REQUEST message %zu to %u::%u %s%s(%d bytes).\n",
-              request->header.serial, request->instance, request->method,
+     D_DEBUG( "Voodoo/Dispatch: Handling REQUEST message %llu to %u::%u %s%s(%d bytes).\n",
+              (unsigned long long)request->header.serial, request->instance, request->method,
               (request->flags & VREQ_RESPOND) ? "[RESPONDING] " : "",
               (request->flags & VREQ_ASYNC) ? "[ASYNC] " : "",
               request->header.size );
@@ -1007,7 +1012,7 @@ handle_request( VoodooManager        *manager,
 
           if (request->flags & VREQ_RESPOND)
                voodoo_manager_respond( manager, request->header.serial,
-                                       DFB_NOSUCHINSTANCE, VOODOO_INSTANCE_NONE,
+                                       DR_NOSUCHINSTANCE, VOODOO_INSTANCE_NONE,
                                        VMBT_NONE );
 
           return;
@@ -1055,9 +1060,9 @@ handle_response( VoodooManager         *manager,
      D_ASSERT( response->header.type == VMSG_RESPONSE );
      D_ASSERT( response->request < manager->msg_serial );
 
-     D_DEBUG( "Voodoo/Dispatch: Handling RESPONSE message %zu (%s) with instance %u for request "
-              "%zu (%d bytes).\n", response->header.serial, DirectResultString( response->result ),
-              response->instance, response->request, response->header.size );
+     D_DEBUG( "Voodoo/Dispatch: Handling RESPONSE message %llu (%s) with instance %u for request "
+              "%llu (%d bytes).\n", (unsigned long long)response->header.serial, DirectResultString( response->result ),
+              response->instance, (unsigned long long)response->request, response->header.size );
 
      pthread_mutex_lock( &manager->response.lock );
 
@@ -1341,7 +1346,7 @@ manager_lock_output( VoodooManager  *manager,
 
      if (length > MAX_MSG_SIZE) {
           D_WARN( "%d exceeds maximum message size of %d", length, MAX_MSG_SIZE );
-          return DFB_LIMITEXCEEDED;
+          return DR_LIMITEXCEEDED;
      }
 
      aligned = VOODOO_MSG_ALIGN( length );
@@ -1355,7 +1360,7 @@ manager_lock_output( VoodooManager  *manager,
 
           if (manager->quit) {
                pthread_mutex_lock( &manager->output.lock );
-               return DFB_DESTROYED;
+               return DR_DESTROYED;
           }
      }
 
@@ -1363,7 +1368,7 @@ manager_lock_output( VoodooManager  *manager,
 
      manager->output.end += aligned;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1377,7 +1382,7 @@ manager_unlock_output( VoodooManager *manager,
 
      pthread_mutex_unlock( &manager->output.lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 /**************************************************************************************************/
@@ -1387,12 +1392,12 @@ manager_lock_response( VoodooManager          *manager,
                        VoodooMessageSerial     request,
                        VoodooResponseMessage **ret_response )
 {
-     VoodooResponseMessage *response;
+     VoodooResponseMessage *response = NULL;
 
      D_MAGIC_ASSERT( manager, VoodooManager );
      D_ASSERT( ret_response != NULL );
 
-     D_DEBUG( "Voodoo/Manager: Locking response to request %zu...\n", request );
+     D_DEBUG( "Voodoo/Manager: Locking response to request %llu...\n", (unsigned long long)request );
 
      pthread_mutex_lock( &manager->response.lock );
 
@@ -1402,24 +1407,24 @@ manager_lock_response( VoodooManager          *manager,
                break;
 
           if (response)
-               D_DEBUG( "Voodoo/Manager: ...current response is for request %zu...\n", response->request );
+               D_DEBUG( "Voodoo/Manager: ...current response is for request %llu...\n", (unsigned long long)response->request );
 
-          D_DEBUG( "Voodoo/Manager: ...(still) waiting for response to request %zu...\n", request );
+          D_DEBUG( "Voodoo/Manager: ...(still) waiting for response to request %llu...\n", (unsigned long long)request );
 
           pthread_cond_wait( &manager->response.wait, &manager->response.lock );
      }
 
      if (manager->quit) {
           pthread_mutex_unlock( &manager->response.lock );
-          return DFB_DESTROYED;
+          return DR_DESTROYED;
      }
 
-     D_DEBUG( "Voodoo/Manager: ...locked response %zu to request %zu (%d bytes).\n",
-              response->header.serial, request, response->header.size );
+     D_DEBUG( "Voodoo/Manager: ...locked response %llu to request %llu (%d bytes).\n",
+              (unsigned long long)response->header.serial, (unsigned long long)request, response->header.size );
 
      *ret_response = response;
 
-     return DFB_OK;
+     return DR_OK;
 }
 
 static DirectResult
@@ -1430,8 +1435,8 @@ manager_unlock_response( VoodooManager         *manager,
      D_ASSERT( response != NULL );
      D_ASSERT( response == manager->response.current );
 
-     D_DEBUG( "Voodoo/Manager: Unlocking response %zu to request %zu (%d bytes)...\n",
-              response->header.serial, response->request, response->header.size );
+     D_DEBUG( "Voodoo/Manager: Unlocking response %llu to request %llu (%d bytes)...\n",
+              (unsigned long long)response->header.serial, (unsigned long long)response->request, response->header.size );
 
      manager->response.current = NULL;
 
@@ -1439,6 +1444,6 @@ manager_unlock_response( VoodooManager         *manager,
 
      pthread_mutex_unlock( &manager->response.lock );
 
-     return DFB_OK;
+     return DR_OK;
 }
 
