@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -51,9 +51,10 @@
 #include <core/layer_context.h>
 #include <core/layer_region.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
 #include <core/windows.h>
+#include <core/windows_internal.h>
 
 #include <direct/build.h>
 #include <direct/debug.h>
@@ -74,14 +75,68 @@
 
 D_DEBUG_DOMAIN( DirectFB_Core, "DirectFB/Core", "DirectFB Core" );
 
-extern CorePart dfb_core_clipboard;
-extern CorePart dfb_core_colorhash;
-extern CorePart dfb_core_gfxcard;
-extern CorePart dfb_core_input;
-extern CorePart dfb_core_layers;
-extern CorePart dfb_core_screens;
-extern CorePart dfb_core_system;
-extern CorePart dfb_core_wm;
+/******************************************************************************/
+
+extern CorePart dfb_clipboard_core;
+extern CorePart dfb_colorhash_core;
+extern CorePart dfb_graphics_core;
+extern CorePart dfb_input_core;
+extern CorePart dfb_layer_core;
+extern CorePart dfb_screen_core;
+extern CorePart dfb_surface_core;
+extern CorePart dfb_system_core;
+extern CorePart dfb_wm_core;
+
+static CorePart *core_parts[] = {
+     &dfb_clipboard_core,
+     &dfb_colorhash_core,
+     &dfb_surface_core,
+     &dfb_system_core,
+     &dfb_input_core,
+     &dfb_graphics_core,
+     &dfb_screen_core,
+     &dfb_layer_core,
+     &dfb_wm_core
+};
+
+void *
+dfb_core_get_part( CoreDFB        *core,
+                   DFBCorePartID   part_id )
+{
+     switch (part_id) {
+          case DFCP_CLIPBOARD:
+               return dfb_clipboard_core.data_local;
+
+          case DFCP_COLORHASH:
+               return dfb_colorhash_core.data_local;
+
+          case DFCP_GRAPHICS:
+               return dfb_graphics_core.data_local;
+
+          case DFCP_INPUT:
+               return dfb_input_core.data_local;
+
+          case DFCP_LAYER:
+               return dfb_layer_core.data_local;
+
+          case DFCP_SCREEN:
+               return dfb_screen_core.data_local;
+
+          case DFCP_SURFACE:
+               return dfb_surface_core.data_local;
+
+          case DFCP_SYSTEM:
+               return dfb_system_core.data_local;
+
+          case DFCP_WM:
+               return dfb_wm_core.data_local;
+
+          default:
+               D_BUG( "unknown core part" );
+     }
+
+     return NULL;
+}
 
 /******************************************************************************/
 
@@ -97,6 +152,8 @@ struct _CoreCleanup {
                                       emergency shutdown (from signal hadler) */
 };
 
+/******************************************************************************/
+
 struct __DFB_CoreDFBShared {
      int                  magic;
 
@@ -133,6 +190,8 @@ struct __DFB_CoreDFB {
      DirectThreadInitHandler *init_handler;
 
      DirectSignalHandler     *signal_handler;
+     
+     DirectCleanupHandler    *cleanup_handler;
 };
 
 /******************************************************************************/
@@ -140,7 +199,7 @@ struct __DFB_CoreDFB {
 /*
  * ckecks if stack is clean, otherwise prints warning, then calls core_deinit()
  */
-static void dfb_core_deinit_check();
+static void dfb_core_deinit_check( void *ctx );
 
 static void dfb_core_thread_init_handler( DirectThread *thread, void *arg );
 
@@ -165,19 +224,6 @@ static int dfb_core_arena_leave     ( FusionArena *arena,
 
 /******************************************************************************/
 
-static CorePart *core_parts[] = {
-     &dfb_core_clipboard,
-     &dfb_core_colorhash,
-     &dfb_core_system,
-     &dfb_core_input,
-     &dfb_core_gfxcard,
-     &dfb_core_screens,
-     &dfb_core_layers,
-     &dfb_core_wm
-};
-
-#define NUM_CORE_PARTS ((int)(sizeof(core_parts)/sizeof(CorePart*)))
-
 #if defined(DFB_DYNAMIC_LINKING) && defined(SOPATH)
 /*
  * the library handle for dlopen'ing ourselves
@@ -283,11 +329,11 @@ dfb_core_create( CoreDFB **ret_core )
 #endif
 
      if (dfb_config->sync) {
-          D_INFO( "DirectFB/Core: doing sync()...\n" );
+          D_INFO( "DirectFB/Core: calling sync()...\n" );
           sync();
      }
 
-     direct_signal_handler_add( -1, dfb_core_signal_handler, core, &core->signal_handler );
+     direct_signal_handler_add( DIRECT_SIGNAL_ANY, dfb_core_signal_handler, core, &core->signal_handler );
 
      if (fusion_arena_enter( core->world, "DirectFB/Core",
                              dfb_core_arena_initialize, dfb_core_arena_join,
@@ -304,19 +350,14 @@ dfb_core_create( CoreDFB **ret_core )
           direct_signals_block_all();
 
      if (dfb_config->deinit_check)
-          atexit( dfb_core_deinit_check );
+          direct_cleanup_handler_add( dfb_core_deinit_check, NULL, &core->cleanup_handler );
 
 
      fusion_skirmish_prevail( &shared->lock );
 
      if (!core->master) {
-          while (!shared->active) {
-               fusion_skirmish_dismiss( &shared->lock );
-               
-               usleep( 50000 );
-               
-               fusion_skirmish_prevail( &shared->lock );
-          }
+          while (!shared->active)
+               fusion_skirmish_wait( &shared->lock, 0 );
      }
 
      fusion_skirmish_dismiss( &shared->lock );
@@ -374,6 +415,9 @@ dfb_core_destroy( CoreDFB *core, bool emergency )
      }
 
      direct_signal_handler_remove( core->signal_handler );
+     
+     if (core->cleanup_handler)
+          direct_cleanup_handler_remove( core->cleanup_handler );
 
      if (core->master) {
           if (emergency) {
@@ -599,6 +643,8 @@ dfb_core_activate( CoreDFB *core )
 
      shared->active = true;
 
+     fusion_skirmish_notify( &shared->lock );
+
      fusion_skirmish_dismiss( &shared->lock );
 }
 
@@ -684,32 +730,32 @@ dfb_core_suspend( CoreDFB *core )
      if (core->suspended)
           return DFB_BUSY;
 
-     ret = dfb_core_input.Suspend( core );
+     ret = dfb_input_core.Suspend( dfb_input_core.data_local );
      if (ret)
           goto error_input;
 
-     ret = dfb_core_layers.Suspend( core );
+     ret = dfb_layer_core.Suspend( dfb_layer_core.data_local );
      if (ret)
           goto error_layers;
 
-     ret = dfb_core_screens.Suspend( core );
+     ret = dfb_screen_core.Suspend( dfb_screen_core.data_local );
      if (ret)
           goto error_screens;
 
-     ret = dfb_core_gfxcard.Suspend( core );
+     ret = dfb_graphics_core.Suspend( dfb_graphics_core.data_local );
      if (ret)
-          goto error_gfxcard;
+          goto error_graphics;
 
      core->suspended = true;
 
      return DFB_OK;
 
-error_gfxcard:
-     dfb_core_screens.Resume( core );
+error_graphics:
+     dfb_screen_core.Resume( dfb_screen_core.data_local );
 error_screens:
-     dfb_core_layers.Resume( core );
+     dfb_layer_core.Resume( dfb_layer_core.data_local );
 error_layers:
-     dfb_core_input.Resume( core );
+     dfb_input_core.Resume( dfb_input_core.data_local );
 error_input:
      return ret;
 }
@@ -732,19 +778,19 @@ dfb_core_resume( CoreDFB *core )
      if (!core->suspended)
           return DFB_BUSY;
 
-     ret = dfb_core_gfxcard.Resume( core );
+     ret = dfb_graphics_core.Resume( dfb_graphics_core.data_local );
      if (ret)
-          goto error_gfxcard;
+          goto error_graphics;
 
-     ret = dfb_core_screens.Resume( core );
+     ret = dfb_screen_core.Resume( dfb_screen_core.data_local );
      if (ret)
           goto error_screens;
 
-     ret = dfb_core_layers.Resume( core );
+     ret = dfb_layer_core.Resume( dfb_layer_core.data_local );
      if (ret)
           goto error_layers;
 
-     ret = dfb_core_input.Resume( core );
+     ret = dfb_input_core.Resume( dfb_input_core.data_local );
      if (ret)
           goto error_input;
 
@@ -753,12 +799,12 @@ dfb_core_resume( CoreDFB *core )
      return DFB_OK;
 
 error_input:
-     dfb_core_layers.Suspend( core );
+     dfb_layer_core.Suspend( dfb_layer_core.data_local );
 error_layers:
-     dfb_core_screens.Suspend( core );
+     dfb_screen_core.Suspend( dfb_screen_core.data_local );
 error_screens:
-     dfb_core_gfxcard.Suspend( core );
-error_gfxcard:
+     dfb_graphics_core.Suspend( dfb_graphics_core.data_local );
+error_graphics:
      return ret;
 }
 
@@ -807,16 +853,12 @@ dfb_core_cleanup_remove( CoreDFB     *core,
 /******************************************************************************/
 
 static void
-dfb_core_deinit_check()
+dfb_core_deinit_check( void *ctx )
 {
      if (core_dfb && core_dfb->refs) {
           D_WARN( "Application exited without deinitialization of DirectFB!" );
           dfb_core_destroy( core_dfb, true );
      }
-
-     direct_print_memleaks();
-
-     direct_print_interface_leaks();
 }
 
 static void
@@ -869,30 +911,41 @@ dfb_core_shutdown( CoreDFB *core, bool emergency )
 
      D_MAGIC_ASSERT( shared, CoreDFBShared );
 
+     /* Suspend input core to stop all input threads before shutting down. */
+     if (dfb_input_core.initialized)
+          dfb_input_core.Suspend( dfb_input_core.data_local );
+
+     fusion_stop_dispatcher( core->world, emergency );
+
+     /* Destroy window objects. */
+     fusion_object_pool_destroy( shared->window_pool, core->world );
+
+     /* Close window stacks. */
+     if (dfb_wm_core.initialized)
+          dfb_wm_close_all_stacks( dfb_wm_core.data_local );
+
      /* Destroy layer context and region objects. */
      fusion_object_pool_destroy( shared->layer_region_pool, core->world );
      fusion_object_pool_destroy( shared->layer_context_pool, core->world );
 
      /* Shutdown WM core. */
-     dfb_core_part_shutdown( core, &dfb_core_wm, emergency );
-
-     /* Destroy window objects. */
-     fusion_object_pool_destroy( shared->window_pool, core->world );
+     dfb_core_part_shutdown( core, &dfb_wm_core, emergency );
 
      /* Shutdown layer core. */
-     dfb_core_part_shutdown( core, &dfb_core_layers, emergency );
-     dfb_core_part_shutdown( core, &dfb_core_screens, emergency );
+     dfb_core_part_shutdown( core, &dfb_layer_core, emergency );
+     dfb_core_part_shutdown( core, &dfb_screen_core, emergency );
 
      /* Destroy surface and palette objects. */
      fusion_object_pool_destroy( shared->surface_pool, core->world );
      fusion_object_pool_destroy( shared->palette_pool, core->world );
 
      /* Destroy remaining core parts. */
-     dfb_core_part_shutdown( core, &dfb_core_gfxcard, emergency );
-     dfb_core_part_shutdown( core, &dfb_core_input, emergency );
-     dfb_core_part_shutdown( core, &dfb_core_system, emergency );
-     dfb_core_part_shutdown( core, &dfb_core_colorhash, emergency );
-     dfb_core_part_shutdown( core, &dfb_core_clipboard, emergency );
+     dfb_core_part_shutdown( core, &dfb_graphics_core, emergency );
+     dfb_core_part_shutdown( core, &dfb_surface_core, emergency );
+     dfb_core_part_shutdown( core, &dfb_input_core, emergency );
+     dfb_core_part_shutdown( core, &dfb_system_core, emergency );
+     dfb_core_part_shutdown( core, &dfb_colorhash_core, emergency );
+     dfb_core_part_shutdown( core, &dfb_clipboard_core, emergency );
 
      /* Destroy shared memory pool for surface data. */
      fusion_shm_pool_destroy( core->world, shared->shmpool_data );
@@ -913,7 +966,7 @@ dfb_core_initialize( CoreDFB *core )
 
      D_MAGIC_ASSERT( shared, CoreDFBShared );
 
-     ret = fusion_shm_pool_create( core->world, "DirectFB Surface Pool", 0x8000000,
+     ret = fusion_shm_pool_create( core->world, "DirectFB Data Pool", 0x1000000,
                                    fusion_config->debugshm, &shared->shmpool_data );
      if (ret)
           return ret;
@@ -924,7 +977,7 @@ dfb_core_initialize( CoreDFB *core )
      shared->surface_pool       = dfb_surface_pool_create( core->world );
      shared->window_pool        = dfb_window_pool_create( core->world );
 
-     for (i=0; i<NUM_CORE_PARTS; i++) {
+     for (i=0; i<D_ARRAY_SIZE(core_parts); i++) {
           DFBResult ret;
 
           if ((ret = dfb_core_part_initialize( core, core_parts[i] ))) {
@@ -943,7 +996,7 @@ dfb_core_leave( CoreDFB *core, bool emergency )
 
      D_MAGIC_ASSERT( core, CoreDFB );
 
-     for (i=NUM_CORE_PARTS-1; i>=0; i--)
+     for (i=D_ARRAY_SIZE(core_parts)-1; i>=0; i--)
           dfb_core_part_leave( core, core_parts[i], emergency );
 
      return DFB_OK;
@@ -956,7 +1009,7 @@ dfb_core_join( CoreDFB *core )
 
      D_MAGIC_ASSERT( core, CoreDFB );
 
-     for (i=0; i<NUM_CORE_PARTS; i++) {
+     for (i=0; i<D_ARRAY_SIZE(core_parts); i++) {
           DFBResult ret;
 
           if ((ret = dfb_core_part_join( core, core_parts[i] ))) {
