@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -41,7 +41,7 @@
 
 #include <core/fonts.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <direct/debug.h>
 #include <direct/hash.h>
@@ -61,10 +61,10 @@ D_DEBUG_DOMAIN( Core_FontSurfaces, "Core/Font/Surf", "DirectFB Core Font Surface
 
 /**********************************************************************************************************************/
 
-static bool free_glyphs( DirectHash *hash,
-                         u32         key,
-                         void       *value,
-                         void       *ctx );
+static bool free_glyphs( DirectHash    *hash,
+                         unsigned long  key,
+                         void          *value,
+                         void          *ctx );
 
 /**********************************************************************************************************************/
 
@@ -97,12 +97,13 @@ dfb_font_create( CoreDFB *core, CoreFont **ret_font )
      /* the proposed pixel_format, may be changed by the font provider */
      font->pixel_format = dfb_config->font_format ? : DSPF_A8;
 
-     if (font->pixel_format == DSPF_ARGB)
+     if ((font->pixel_format == DSPF_ARGB || font->pixel_format == DSPF_ARGB4444) && dfb_config->font_premult)
           font->surface_caps = DSCAPS_PREMULTIPLIED;
 
      /* the state used to blit the glyphs, may be changed by the font provider */
      dfb_state_init( &font->state, core );
-     font->state.blittingflags = DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE;
+
+     font->blittingflags = DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE;
 
      D_MAGIC_SET( font, CoreFont );
 
@@ -249,7 +250,8 @@ dfb_font_get_glyph_data( CoreFont        *font,
 
      D_MAGIC_SET( data, CoreGlyphData );
 
-     align = DFB_PIXELFORMAT_ALIGNMENT( font->pixel_format );
+     align = (8 / (DFB_BYTES_PER_PIXEL( font->pixel_format ) ? : 1)) *
+                  (DFB_PIXELFORMAT_ALIGNMENT( font->pixel_format ) + 1) - 1;
 
      ret = font->GetGlyphData( font, index, data );
      if (ret) {
@@ -317,8 +319,8 @@ dfb_font_get_glyph_data( CoreFont        *font,
                     D_MAGIC_ASSERT( row, CoreFontCacheRow );
 
                     D_DEBUG_AT( Core_FontSurfaces, "  -> using trailing space of row %d - %dx%d %s\n",
-                                font->active_row, row->surface->width, row->surface->height,
-                                dfb_pixelformat_name(row->surface->format) );
+                                font->active_row, row->surface->config.size.w, row->surface->config.size.h,
+                                dfb_pixelformat_name(row->surface->config.format) );
                }
                else {
                     CoreGlyphData *d, *n;
@@ -346,8 +348,8 @@ dfb_font_get_glyph_data( CoreFont        *font,
                     D_MAGIC_ASSERT( row, CoreFontCacheRow );
 
                     D_DEBUG_AT( Core_FontSurfaces, "  -> reusing row %d - %dx%d %s\n",
-                                font->active_row, row->surface->width, row->surface->height,
-                                dfb_pixelformat_name(row->surface->format) );
+                                font->active_row, row->surface->config.size.w, row->surface->config.size.h,
+                                dfb_pixelformat_name(row->surface->config.format) );
 
                     /* Kick out all glyphs. */
                     direct_list_foreach_safe (d, n, row->glyphs) {
@@ -377,11 +379,13 @@ dfb_font_get_glyph_data( CoreFont        *font,
                }
 
                /* Create a new font surface. */
-               ret = dfb_surface_create( font->core,
-                                         font->row_width,
-                                         MAX( font->height + 1, 8 ),
-                                         font->pixel_format,
-                                         CSP_VIDEOLOW, font->surface_caps, NULL, &row->surface );
+               ret = dfb_surface_create_simple( font->core,
+                                                font->row_width,
+                                                MAX( font->height + 1, 8 ),
+                                                font->pixel_format,
+                                                font->surface_caps, CSTF_FONT,
+                                                0 /* FIXME: no shared fonts, no font id */,
+                                                NULL, &row->surface );
                if (ret) {
                     D_DERROR( ret, "Core/Font: Could not create font surface!\n" );
                     D_FREE( row );
@@ -389,8 +393,8 @@ dfb_font_get_glyph_data( CoreFont        *font,
                }
 
                D_DEBUG_AT( Core_FontSurfaces, "  -> new row %d - %dx%d %s\n", font->num_rows,
-                           row->surface->width, row->surface->height,
-                           dfb_pixelformat_name(row->surface->format) );
+                           row->surface->config.size.w, row->surface->config.size.h,
+                           dfb_pixelformat_name(row->surface->config.format) );
 
                D_MAGIC_SET( row, CoreFontCacheRow );
 
@@ -628,10 +632,10 @@ dfb_font_decode_character( CoreFont          *font,
 /**********************************************************************************************************************/
 
 static bool
-free_glyphs( DirectHash *hash,
-             u32         key,
-             void       *value,
-             void       *ctx )
+free_glyphs( DirectHash    *hash,
+             unsigned long  key,
+             void          *value,
+             void          *ctx )
 {
      CoreGlyphData *data = value;
 
