@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -38,12 +38,14 @@
 #include <directfb.h>
 #include <core/coretypes.h>
 
+
 typedef enum {
      CCF_CLIPPING   = 0x00000001,
      CCF_NOTRIEMU   = 0x00000002,
      CCF_READSYSMEM = 0x00000004,
      /* CCF_WRITESYSMEM ?! */
-     CCF_AUXMEMORY  = 0x00000010
+     CCF_AUXMEMORY  = 0x00000010,
+     CCF_RENDEROPTS = 0x00000020
 } CardCapabilitiesFlags;
 
 struct __DFB_CoreGraphicsSerial {
@@ -57,6 +59,7 @@ typedef struct {
      DFBAccelerationMask     accel;
      DFBSurfaceBlittingFlags blitting;
      DFBSurfaceDrawingFlags  drawing;
+     DFBAccelerationMask     clip;
 } CardCapabilities;
 
 typedef struct {
@@ -182,12 +185,12 @@ typedef struct _GraphicsDeviceFuncs {
       * Called before a software access to a video surface buffer.
       */
      void (*SurfaceEnter)( void *driver_data, void *device_data,
-                           SurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
+                           CoreSurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
 
      /*
       * Called after a software access to a video surface buffer.
       */
-     void (*SurfaceLeave)( void *driver_data, void *device_data, SurfaceBuffer *buffer );
+     void (*SurfaceLeave)( void *driver_data, void *device_data, CoreSurfaceBuffer *buffer );
 
      /*
       * Return the serial of the last (queued) operation.
@@ -253,28 +256,39 @@ typedef struct _GraphicsDeviceFuncs {
      bool (*TextureTriangles)( void *driver_data, void *device_data,
                                DFBVertex *vertices, int num,
                                DFBTriangleFormation formation );
+
+     /*
+      * Signal beginning of a sequence of operations using this state.
+      * Any number of states can be 'drawing'.
+      */
+     void (*StartDrawing)( void *driver_data, void *device_data, CardState *state );
+
+     /*
+      * Signal end of sequence, i.e. destination surface is consistent again.
+      */
+     void (*StopDrawing)( void *driver_data, void *device_data, CardState *state );
 } GraphicsDeviceFuncs;
 
 typedef struct {
-     int       (*Probe)          (GraphicsDevice      *device);
-     void      (*GetDriverInfo)  (GraphicsDevice      *device,
+     int       (*Probe)          (CoreGraphicsDevice  *device);
+     void      (*GetDriverInfo)  (CoreGraphicsDevice  *device,
                                   GraphicsDriverInfo  *driver_info);
 
-     DFBResult (*InitDriver)     (GraphicsDevice      *device,
+     DFBResult (*InitDriver)     (CoreGraphicsDevice  *device,
                                   GraphicsDeviceFuncs *funcs,
                                   void                *driver_data,
                                   void                *device_data,
                                   CoreDFB             *core);
 
-     DFBResult (*InitDevice)     (GraphicsDevice      *device,
+     DFBResult (*InitDevice)     (CoreGraphicsDevice  *device,
                                   GraphicsDeviceInfo  *device_info,
                                   void                *driver_data,
                                   void                *device_data);
 
-     void      (*CloseDevice)    (GraphicsDevice      *device,
+     void      (*CloseDevice)    (CoreGraphicsDevice  *device,
                                   void                *driver_data,
                                   void                *device_data);
-     void      (*CloseDriver)    (GraphicsDevice      *device,
+     void      (*CloseDriver)    (CoreGraphicsDevice  *device,
                                   void                *driver_data);
 } GraphicsDriverFuncs;
 
@@ -292,44 +306,86 @@ void dfb_gfxcard_unlock( void );
 void dfb_gfxcard_holdup( void );
 
 bool dfb_gfxcard_state_check( CardState *state, DFBAccelerationMask accel );
-//bool dfb_gfxcard_state_acquire( CardState *state, DFBAccelerationMask accel );
-//void dfb_gfxcard_state_release( CardState *state );
+
+/*
+ * Signal beginning of a sequence of operations using this state.
+ * Any number of states can be 'drawing'.
+ */
+void dfb_gfxcard_start_drawing( CoreGraphicsDevice *device,
+                                CardState          *state );
+
+/*
+ * Signal end of sequence, i.e. destination surface is consistent again.
+ */
+void dfb_gfxcard_stop_drawing ( CoreGraphicsDevice *device,
+                                CardState          *state );
 
 /*
  * drawing functions, lock source and destination surfaces,
  * handle clipping and drawing method (hardware/software)
  */
-void dfb_gfxcard_fillrectangles( const DFBRectangle *rects, int num, CardState *state );
+void dfb_gfxcard_fillrectangles         ( const DFBRectangle   *rects,
+                                          int                   num,
+                                          CardState            *state );
 
-void dfb_gfxcard_drawrectangle( DFBRectangle *rect, CardState *state );
+void dfb_gfxcard_drawrectangle          ( DFBRectangle         *rect,
+                                          CardState            *state );
 
-void dfb_gfxcard_drawlines( DFBRegion *lines, int num_lines, CardState *state );
+void dfb_gfxcard_drawlines              ( DFBRegion            *lines,
+                                          int                   num_lines,
+                                          CardState            *state );
 
-void dfb_gfxcard_fillspans( int y, DFBSpan *spans, int num_spans, CardState *state );
+void dfb_gfxcard_fillspans              ( int                   y,
+                                          DFBSpan              *spans,
+                                          int                   num_spans,
+                                          CardState            *state );
 
-void dfb_gfxcard_filltriangle( DFBTriangle *tri, CardState *state );
+void dfb_gfxcard_filltriangles          ( const DFBTriangle    *tris,
+                                          int                   num,
+                                          CardState            *state );
 
-void dfb_gfxcard_blit( DFBRectangle *rect, int dx, int dy, CardState *state );
+void dfb_gfxcard_blit                   ( DFBRectangle         *rect,
+                                          int                   dx,
+                                          int                   dy,
+                                          CardState            *state );
 
-void dfb_gfxcard_batchblit( DFBRectangle *rects, DFBPoint *points,
-                            int num, CardState *state );
+void dfb_gfxcard_batchblit              ( DFBRectangle         *rects,
+                                          DFBPoint             *points,
+                                          int                   num,
+                                          CardState            *state );
 
-void dfb_gfxcard_tileblit( DFBRectangle *rect, int dx1, int dy1, int dx2, int dy2,
-                           CardState *state );
+void dfb_gfxcard_tileblit               ( DFBRectangle         *rect,
+                                          int                   dx1,
+                                          int                   dy1,
+                                          int                   dx2,
+                                          int                   dy2,
+                                          CardState            *state );
 
-void dfb_gfxcard_stretchblit( DFBRectangle *srect, DFBRectangle *drect,
-                              CardState *state );
+void dfb_gfxcard_stretchblit            ( DFBRectangle         *srect,
+                                          DFBRectangle         *drect,
+                                          CardState            *state );
 
-void dfb_gfxcard_texture_triangles( DFBVertex *vertices, int num,
-                                    DFBTriangleFormation formation,
-                                    CardState *state );
+void dfb_gfxcard_texture_triangles      ( DFBVertex            *vertices,
+                                          int                   num,
+                                          DFBTriangleFormation  formation,
+                                          CardState            *state );
 
-void dfb_gfxcard_drawstring( const u8 *text, int bytes,
-                             DFBTextEncodingID encoding, int x, int y,
-                             CoreFont *font, CardState *state );
-void dfb_gfxcard_drawstring_check_state( CoreFont *font, CardState *state );
-void dfb_gfxcard_drawglyph( unsigned int index, int x, int y,
-                            CoreFont *font, CardState *state );
+void dfb_gfxcard_drawstring             ( const u8             *text,
+                                          int                   bytes,
+                                          DFBTextEncodingID     encoding,
+                                          int                   x,
+                                          int                   y,
+                                          CoreFont             *font,
+                                          CardState            *state );
+
+void dfb_gfxcard_drawglyph              ( unsigned int          index,
+                                          int                   x,
+                                          int                   y,
+                                          CoreFont             *font,
+                                          CardState            *state );
+
+void dfb_gfxcard_drawstring_check_state ( CoreFont             *font,
+                                          CardState            *state );
 
 DFBResult dfb_gfxcard_sync( void );
 void dfb_gfxcard_invalidate_state( void );
@@ -337,25 +393,27 @@ DFBResult dfb_gfxcard_wait_serial( const CoreGraphicsSerial *serial );
 void dfb_gfxcard_flush_texture_cache( void );
 void dfb_gfxcard_flush_read_cache( void );
 void dfb_gfxcard_after_set_var( void );
-void dfb_gfxcard_surface_enter( SurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
-void dfb_gfxcard_surface_leave( SurfaceBuffer *buffer );
+void dfb_gfxcard_surface_enter( CoreSurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
+void dfb_gfxcard_surface_leave( CoreSurfaceBuffer *buffer );
 
 DFBResult dfb_gfxcard_adjust_heap_offset( int offset );
 
-SurfaceManager *dfb_gfxcard_surface_manager   ( void );
 void            dfb_gfxcard_get_capabilities  ( CardCapabilities   *ret_caps );
 void            dfb_gfxcard_get_device_info   ( GraphicsDeviceInfo *ret_info );
 void            dfb_gfxcard_get_driver_info   ( GraphicsDriverInfo *ret_info );
 
-int             dfb_gfxcard_reserve_memory    ( GraphicsDevice      *device,
+int             dfb_gfxcard_reserve_memory    ( CoreGraphicsDevice  *device,
                                                 unsigned int         size );
-int             dfb_gfxcard_reserve_auxmemory ( GraphicsDevice      *device,
+int             dfb_gfxcard_reserve_auxmemory ( CoreGraphicsDevice  *device,
                                                 unsigned int         size );
 
 unsigned int    dfb_gfxcard_memory_length     ( void );
 unsigned int    dfb_gfxcard_auxmemory_length  ( void );
 
 void           *dfb_gfxcard_get_device_data   ( void );
+void           *dfb_gfxcard_get_driver_data   ( void );
+
+CoreGraphicsDevice *dfb_gfxcard_get_primary   ( void );
 
 /*
  * Graphics drivers call this function to get access to MMIO regions.
@@ -366,9 +424,9 @@ void           *dfb_gfxcard_get_device_data   ( void );
  *
  * Returns the virtual address or NULL if mapping failed.
  */
-volatile void *dfb_gfxcard_map_mmio( GraphicsDevice *device,
-                                     unsigned int    offset,
-                                     int             length );
+volatile void *dfb_gfxcard_map_mmio( CoreGraphicsDevice *device,
+                                     unsigned int        offset,
+                                     int                 length );
 
 /*
  * Graphics drivers call this function to unmap MMIO regions.
@@ -376,21 +434,29 @@ volatile void *dfb_gfxcard_map_mmio( GraphicsDevice *device,
  * addr:   Virtual address returned by gfxcard_map_mmio
  * length: Length of mapped region (-1 uses default length)
  */
-void dfb_gfxcard_unmap_mmio( GraphicsDevice *device,
-                             volatile void  *addr,
-                             int             length );
+void dfb_gfxcard_unmap_mmio( CoreGraphicsDevice *device,
+                             volatile void      *addr,
+                             int                 length );
 
-int dfb_gfxcard_get_accelerator( GraphicsDevice *device );
+int dfb_gfxcard_get_accelerator( CoreGraphicsDevice *device );
 
-unsigned long  dfb_gfxcard_memory_physical( GraphicsDevice *device,
-                                            unsigned int    offset );
-void          *dfb_gfxcard_memory_virtual ( GraphicsDevice *device,
-                                            unsigned int    offset );
+void dfb_gfxcard_get_limits( CoreGraphicsDevice *device,
+                             CardLimitations    *ret_limits );
 
-unsigned long  dfb_gfxcard_auxmemory_physical( GraphicsDevice *device,
-                                               unsigned int    offset );
-void          *dfb_gfxcard_auxmemory_virtual ( GraphicsDevice *device,
-                                               unsigned int    offset );
+void dfb_gfxcard_calc_buffer_size( CoreGraphicsDevice *device,
+                                   CoreSurfaceBuffer  *buffer,
+                                   int                *ret_pitch,
+                                   int                *ret_length );
+
+unsigned long  dfb_gfxcard_memory_physical   ( CoreGraphicsDevice *device,
+                                               unsigned int        offset );
+void          *dfb_gfxcard_memory_virtual    ( CoreGraphicsDevice *device,
+                                               unsigned int        offset );
+
+unsigned long  dfb_gfxcard_auxmemory_physical( CoreGraphicsDevice *device,
+                                               unsigned int        offset );
+void          *dfb_gfxcard_auxmemory_virtual ( CoreGraphicsDevice *device,
+                                               unsigned int        offset );
 
 
 /* Hook for registering additional screen(s) and layer(s) in app or lib initializing DirectFB. */
