@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -36,7 +36,11 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <directfb.h>
+
+#include <direct/debug.h>
 #include <direct/list.h>
+#include <direct/memcpy.h>
 #include <direct/messages.h>
 
 
@@ -44,8 +48,6 @@
 #include <fusion/reactor.h>
 #include <fusion/arena.h>
 
-#include <directfb.h>
-
 #include <core/core.h>
 #include <core/coredefs.h>
 #include <core/coretypes.h>
@@ -53,7 +55,8 @@
 #include <core/core_parts.h>
 
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
+#include <core/surface_buffer.h>
 #include <core/system.h>
 #include <core/layer_context.h>
 #include <core/layer_control.h>
@@ -77,11 +80,11 @@
 #include <gfx/convert.h>
 
 
-D_DEBUG_DOMAIN( Core_Input, "Core/Input", "DirectFB Input Core" );
+D_DEBUG_DOMAIN( Core_Input,    "Core/Input",     "DirectFB Input Core" );
+D_DEBUG_DOMAIN( Core_InputEvt, "Core/Input/Evt", "DirectFB Input Core Events & Dispatch" );
 
 
-DEFINE_MODULE_DIRECTORY( dfb_input_modules, "inputdrivers",
-                         DFB_INPUT_DRIVER_ABI_VERSION );
+DEFINE_MODULE_DIRECTORY( dfb_input_modules, "inputdrivers", DFB_INPUT_DRIVER_ABI_VERSION );
 
 /**********************************************************************************************************************/
 
@@ -134,6 +137,9 @@ typedef struct {
      FusionSkirmish               lock;
 
      FusionCall                   call;          /* driver call via master */
+
+     unsigned int                 axis_num;
+     DFBInputDeviceAxisInfo      *axis_info;
 } InputDeviceShared;
 
 struct __DFB_CoreInputDevice {
@@ -149,19 +155,28 @@ struct __DFB_CoreInputDevice {
      CoreDFB            *core;
 };
 
-typedef struct {
-     int                num;
-     InputDeviceShared *devices[MAX_INPUTDEVICES];
-} CoreInput;
-
 /**********************************************************************************************************************/
 
-static DirectLink *drivers;
-static DirectLink *devices;
+typedef struct {
+     int                 magic;
 
-static CoreInput  *core_input;
+     int                 num;
+     InputDeviceShared  *devices[MAX_INPUTDEVICES];
+} DFBInputCoreShared;
 
-DFB_CORE_PART( input, 0, sizeof(CoreInput) )
+struct __DFB_DFBInputCore {
+     int                 magic;
+
+     CoreDFB            *core;
+
+     DFBInputCoreShared *shared;
+
+     DirectLink         *drivers;
+     DirectLink         *devices;
+};
+
+
+DFB_CORE_PART( input_core, InputCore );
 
 /**********************************************************************************************************************/
 
@@ -342,34 +357,57 @@ dfb_input_set_global( ReactionFunc func,
      return DFB_OK;
 }
 
-/** public **/
+/**********************************************************************************************************************/
+
+static DFBInputCore       *core_local; /* FIXME */
+static DFBInputCoreShared *core_input; /* FIXME */
+
 
 static DFBResult
-dfb_input_initialize( CoreDFB *core, void *data_local, void *data_shared )
+dfb_input_core_initialize( CoreDFB            *core,
+                           DFBInputCore       *data,
+                           DFBInputCoreShared *shared )
 {
-     D_DEBUG_AT( Core_Input, "%s( %p, %p, %p )\n", __FUNCTION__, core, data_local, data_shared );
+     D_DEBUG_AT( Core_Input, "dfb_input_core_initialize( %p, %p, %p )\n", core, data, shared );
 
-     D_ASSERT( core_input == NULL );
+     D_ASSERT( data != NULL );
+     D_ASSERT( shared != NULL );
+
+     core_local = data;   /* FIXME */
+     core_input = shared; /* FIXME */
+
+     data->core   = core;
+     data->shared = shared;
 
-     core_input = data_shared;
 
      direct_modules_explore_directory( &dfb_input_modules );
 
      init_devices( core );
 
+
+     D_MAGIC_SET( data, DFBInputCore );
+     D_MAGIC_SET( shared, DFBInputCoreShared );
+
      return DFB_OK;
 }
 
 static DFBResult
-dfb_input_join( CoreDFB *core, void *data_local, void *data_shared )
+dfb_input_core_join( CoreDFB            *core,
+                     DFBInputCore       *data,
+                     DFBInputCoreShared *shared )
 {
      int i;
 
-     D_DEBUG_AT( Core_Input, "%s( %p, %p, %p )\n", __FUNCTION__, core, data_local, data_shared );
+     D_DEBUG_AT( Core_Input, "dfb_input_core_join( %p, %p, %p )\n", core, data, shared );
 
-     D_ASSERT( core_input == NULL );
+     D_ASSERT( data != NULL );
+     D_MAGIC_ASSERT( shared, DFBInputCoreShared );
 
-     core_input = data_shared;
+     core_local = data;   /* FIXME */
+     core_input = shared; /* FIXME */
+
+     data->core   = core;
+     data->shared = shared;
 
      for (i=0; i<core_input->num; i++) {
           CoreInputDevice *device;
@@ -382,108 +420,161 @@ dfb_input_join( CoreDFB *core, void *data_local, void *data_shared )
           device->shared = core_input->devices[i];
 
           /* add it to the list */
-          direct_list_append( &devices, &device->link );
+          direct_list_append( &data->devices, &device->link );
 
           D_MAGIC_SET( device, CoreInputDevice );
      }
 
+
+     D_MAGIC_SET( data, DFBInputCore );
+
      return DFB_OK;
 }
 
 static DFBResult
-dfb_input_shutdown( CoreDFB *core, bool emergency )
+dfb_input_core_shutdown( DFBInputCore *data,
+                         bool          emergency )
 {
+     DFBInputCoreShared  *shared;
      DirectLink          *n;
      CoreInputDevice     *device;
-     FusionSHMPoolShared *pool = dfb_core_shmpool( core );
+     FusionSHMPoolShared *pool = dfb_core_shmpool( data->core );
 
-     D_DEBUG_AT( Core_Input, "%s( %p, %semergency )\n", __FUNCTION__, core, emergency ? "" : "no " );
+     D_DEBUG_AT( Core_Input, "dfb_input_core_shutdown( %p, %semergency )\n", data, emergency ? "" : "no " );
 
-     D_ASSERT( core_input != NULL );
+     D_MAGIC_ASSERT( data, DFBInputCore );
+     D_MAGIC_ASSERT( data->shared, DFBInputCoreShared );
 
-     direct_list_foreach_safe (device, n, devices) {
-          InputDriver       *driver = device->driver;
-          InputDeviceShared *shared = device->shared;
+     shared = data->shared;
+
+
+     direct_list_foreach_safe (device, n, data->devices) {
+          InputDriver       *driver;
+          InputDeviceShared *devshared;
 
           D_MAGIC_ASSERT( device, CoreInputDevice );
 
-          fusion_call_destroy( &shared->call );
-          fusion_skirmish_destroy( &shared->lock );
+          driver = device->driver;
+          D_ASSERT( driver != NULL );
 
-          driver->funcs->CloseDevice( device->driver_data );
+          devshared = device->shared;
+          D_ASSERT( devshared != NULL );
+
+          fusion_call_destroy( &devshared->call );
+          fusion_skirmish_destroy( &devshared->lock );
+
+          if (device->driver_data != NULL) {
+               void *driver_data;
+
+               D_ASSERT( driver->funcs != NULL );
+               D_ASSERT( driver->funcs->CloseDevice != NULL );
+
+               D_DEBUG_AT( Core_Input, "  -> closing '%s' (%d) %d.%d (%s)\n",
+                           devshared->device_info.desc.name, devshared->num + 1,
+                           driver->info.version.major,
+                           driver->info.version.minor, driver->info.vendor );
+
+               driver_data = device->driver_data;
+               device->driver_data = NULL;
+               driver->funcs->CloseDevice( driver_data );
+          }
 
           if (!--driver->nr_devices) {
                direct_module_unref( driver->module );
                D_FREE( driver );
           }
 
-          fusion_reactor_free( shared->reactor );
+          fusion_reactor_free( devshared->reactor );
 
-          if (shared->keymap.entries)
-               SHFREE( pool, shared->keymap.entries );
+          if (devshared->keymap.entries)
+               SHFREE( pool, devshared->keymap.entries );
+  
+          if (devshared->axis_info)
+               SHFREE( pool, devshared->axis_info );
 
-          SHFREE( pool, shared );
+          SHFREE( pool, devshared );
 
           D_MAGIC_CLEAR( device );
 
           D_FREE( device );
      }
 
-     devices = NULL;
-     drivers = NULL;
-
-     core_input = NULL;
+     D_MAGIC_CLEAR( data );
+     D_MAGIC_CLEAR( shared );
 
      return DFB_OK;
 }
 
 static DFBResult
-dfb_input_leave( CoreDFB *core, bool emergency )
+dfb_input_core_leave( DFBInputCore *data,
+                      bool          emergency )
 {
-     DirectLink      *n;
-     CoreInputDevice *device;
+     DFBInputCoreShared *shared;
+     DirectLink         *n;
+     CoreInputDevice    *device;
 
-     D_DEBUG_AT( Core_Input, "%s( %p, %semergency )\n", __FUNCTION__, core, emergency ? "" : "no " );
+     D_DEBUG_AT( Core_Input, "dfb_input_core_leave( %p, %semergency )\n", data, emergency ? "" : "no " );
 
-     D_ASSERT( core_input != NULL );
+     D_MAGIC_ASSERT( data, DFBInputCore );
+     D_MAGIC_ASSERT( data->shared, DFBInputCoreShared );
 
-     direct_list_foreach_safe (device, n, devices) {
+     shared = data->shared;
+
+
+     direct_list_foreach_safe (device, n, data->devices) {
           D_MAGIC_ASSERT( device, CoreInputDevice );
 
           D_FREE( device );
      }
 
-     devices    = NULL;
-     core_input = NULL;
+
+     D_MAGIC_CLEAR( data );
 
      return DFB_OK;
 }
 
 static DFBResult
-dfb_input_suspend( CoreDFB *core )
+dfb_input_core_suspend( DFBInputCore *data )
 {
-     CoreInputDevice *device;
+     DFBInputCoreShared *shared;
+     CoreInputDevice    *device;
 
-     D_DEBUG_AT( Core_Input, "%s( %p )\n", __FUNCTION__, core );
+     D_DEBUG_AT( Core_Input, "dfb_input_core_suspend( %p )\n", data );
 
-     D_ASSERT( core_input != NULL );
+     D_MAGIC_ASSERT( data, DFBInputCore );
+     D_MAGIC_ASSERT( data->shared, DFBInputCoreShared );
+
+     shared = data->shared;
 
      D_DEBUG_AT( Core_Input, "  -> suspending...\n" );
 
-     direct_list_foreach (device, devices) {
-          InputDriver       *driver = device->driver;
-          InputDeviceShared *shared = device->shared;
-
-          (void) shared;
-
+     direct_list_foreach (device, data->devices) {
+          InputDriver       *driver;
+          InputDeviceShared *devshared;
+          
           D_MAGIC_ASSERT( device, CoreInputDevice );
+          
+          driver = device->driver;
+          D_ASSERT( driver != NULL );
+          
+          devshared = device->shared;
+          D_ASSERT( devshared != NULL );
 
-          D_DEBUG_AT( Core_Input, "  -> closing '%s' (%d) %d.%d (%s)\n",
-                      shared->device_info.desc.name, shared->num + 1,
-                      driver->info.version.major,
-                      driver->info.version.minor, driver->info.vendor );
+          if (device->driver_data != NULL) {
+               void *driver_data;
 
-          driver->funcs->CloseDevice( device->driver_data );
+               D_ASSERT( driver->funcs != NULL );
+               D_ASSERT( driver->funcs->CloseDevice != NULL );
+
+               D_DEBUG_AT( Core_Input, "  -> closing '%s' (%d) %d.%d (%s)\n",
+                           devshared->device_info.desc.name, devshared->num + 1,
+                           driver->info.version.major,
+                           driver->info.version.minor, driver->info.vendor );
+
+               driver_data = device->driver_data;
+               device->driver_data = NULL;
+               driver->funcs->CloseDevice( driver_data );
+          }
 
           flush_keys( device );
      }
@@ -494,18 +585,22 @@ dfb_input_suspend( CoreDFB *core )
 }
 
 static DFBResult
-dfb_input_resume( CoreDFB *core )
+dfb_input_core_resume( DFBInputCore *data )
 {
-     DFBResult        ret;
-     CoreInputDevice *device;
+     DFBInputCoreShared *shared;
+     DFBResult           ret;
+     CoreInputDevice    *device;
 
-     D_DEBUG_AT( Core_Input, "%s( %p )\n", __FUNCTION__, core );
+     D_DEBUG_AT( Core_Input, "dfb_input_core_resume( %p )\n", data );
 
-     D_ASSERT( core_input != NULL );
+     D_MAGIC_ASSERT( data, DFBInputCore );
+     D_MAGIC_ASSERT( data->shared, DFBInputCoreShared );
+
+     shared = data->shared;
 
      D_DEBUG_AT( Core_Input, "  -> resuming...\n" );
 
-     direct_list_foreach (device, devices) {
+     direct_list_foreach (device, data->devices) {
           D_MAGIC_ASSERT( device, CoreInputDevice );
 
           D_DEBUG_AT( Core_Input, "  -> reopening '%s' (%d) %d.%d (%s)\n",
@@ -514,12 +609,15 @@ dfb_input_resume( CoreDFB *core )
                       device->driver->info.version.minor,
                       device->driver->info.vendor );
 
+          D_ASSERT( device->driver_data == NULL );
+
           ret = device->driver->funcs->OpenDevice( device, device->shared->num,
                                                    &device->shared->device_info,
                                                    &device->driver_data );
           if (ret) {
                D_DERROR( ret, "DirectFB/Input: Failed reopening device "
                          "during resume (%s)!\n", device->shared->device_info.desc.name );
+               device->driver_data = NULL;
           }
      }
 
@@ -537,10 +635,19 @@ dfb_input_enumerate_devices( InputDeviceCallback         callback,
 
      D_ASSERT( core_input != NULL );
 
-     direct_list_foreach (device, devices) {
-          D_MAGIC_ASSERT( device, CoreInputDevice );
+     direct_list_foreach (device, core_local->devices) {
+          DFBInputDeviceCapabilities dev_caps;
 
-          if ((device->shared->device_info.desc.caps & caps) && callback( device, ctx ) == DFENUM_CANCEL)
+          D_MAGIC_ASSERT( device, CoreInputDevice );
+          D_ASSERT( device->shared != NULL );
+
+          dev_caps = device->shared->device_info.desc.caps;
+
+          /* Always match if unclassified */
+          if (!dev_caps)
+               dev_caps = DICAPS_ALL;
+
+          if ((dev_caps & caps) && callback( device, ctx ) == DFENUM_CANCEL)
                break;
      }
 }
@@ -609,6 +716,35 @@ dfb_input_detach_global( CoreInputDevice *device,
      return fusion_reactor_detach_global( device->shared->reactor, reaction );
 }
 
+const char *
+dfb_input_event_type_name( DFBInputEventType type )
+{
+     switch (type) {
+          case DIET_UNKNOWN:
+               return "UNKNOWN";
+
+          case DIET_KEYPRESS:
+               return "KEYPRESS";
+
+          case DIET_KEYRELEASE:
+               return "KEYRELEASE";
+
+          case DIET_BUTTONPRESS:
+               return "BUTTONPRESS";
+
+          case DIET_BUTTONRELEASE:
+               return "BUTTONRELEASE";
+
+          case DIET_AXISMOTION:
+               return "AXISMOTION";
+
+          default:
+               break;
+     }
+
+     return "<invalid>";
+}
+
 void
 dfb_input_dispatch( CoreInputDevice *device, DFBInputEvent *event )
 {
@@ -622,13 +758,54 @@ dfb_input_dispatch( CoreInputDevice *device, DFBInputEvent *event )
 
      D_ASSUME( device->shared != NULL );
 
-     if (!device->shared)
+     /*
+      * 0. Sanity checks & debugging...
+      */
+     if (!device->shared) {
+          D_DEBUG_AT( Core_Input, "  -> No shared data!\n" );
           return;
+     }
 
      D_ASSUME( device->shared->reactor != NULL );
 
-     if (!device->shared->reactor)
+     if (!device->shared->reactor) {
+          D_DEBUG_AT( Core_Input, "  -> No reactor!\n" );
           return;
+     }
+
+     D_DEBUG_AT( Core_InputEvt, "  -> (%02x) %s%s%s\n", event->type,
+                 dfb_input_event_type_name( event->type ),
+                 (event->flags & DIEF_FOLLOW) ? " [FOLLOW]" : "",
+                 (event->flags & DIEF_REPEAT) ? " [REPEAT]" : "" );
+
+#if D_DEBUG_ENABLED
+     if (event->flags & DIEF_TIMESTAMP)
+          D_DEBUG_AT( Core_InputEvt, "  -> TIMESTAMP  %lu.%06lu\n", event->timestamp.tv_sec, event->timestamp.tv_usec );
+     if (event->flags & DIEF_AXISABS)
+          D_DEBUG_AT( Core_InputEvt, "  -> AXISABS    %d at %d\n",  event->axis, event->axisabs );
+     if (event->flags & DIEF_AXISREL)
+          D_DEBUG_AT( Core_InputEvt, "  -> AXISREL    %d by %d\n",  event->axis, event->axisrel );
+     if (event->flags & DIEF_KEYCODE)
+          D_DEBUG_AT( Core_InputEvt, "  -> KEYCODE    %d\n",        event->key_code );
+     if (event->flags & DIEF_KEYID)
+          D_DEBUG_AT( Core_InputEvt, "  -> KEYID      0x%04x\n",    event->key_id );
+     if (event->flags & DIEF_KEYSYMBOL)
+          D_DEBUG_AT( Core_InputEvt, "  -> KEYSYMBOL  0x%04x\n",    event->key_symbol );
+     if (event->flags & DIEF_MODIFIERS)
+          D_DEBUG_AT( Core_InputEvt, "  -> MODIFIERS  0x%04x\n",    event->modifiers );
+     if (event->flags & DIEF_LOCKS)
+          D_DEBUG_AT( Core_InputEvt, "  -> LOCKS      0x%04x\n",    event->locks );
+     if (event->flags & DIEF_BUTTONS)
+          D_DEBUG_AT( Core_InputEvt, "  -> BUTTONS    0x%04x\n",    event->buttons );
+     if (event->flags & DIEF_GLOBAL)
+          D_DEBUG_AT( Core_InputEvt, "  -> GLOBAL\n" );
+#endif
+
+     /*
+      * 1. Fixup event...
+      */
+     event->clazz     = DFEC_INPUT;
+     event->device_id = device->shared->id;
 
      if (!(event->flags & DIEF_TIMESTAMP)) {
           gettimeofday( &event->timestamp, NULL );
@@ -638,11 +815,15 @@ dfb_input_dispatch( CoreInputDevice *device, DFBInputEvent *event )
      switch (event->type) {
           case DIET_BUTTONPRESS:
           case DIET_BUTTONRELEASE:
+               D_DEBUG_AT( Core_InputEvt, "  -> BUTTON     0x%04x\n", event->button );
+
                if (dfb_config->lefty) {
                     if (event->button == DIBI_LEFT)
                          event->button = DIBI_RIGHT;
                     else if (event->button == DIBI_RIGHT)
                          event->button = DIBI_LEFT;
+
+                    D_DEBUG_AT( Core_InputEvt, "  -> lefty!  => 0x%04x <=\n", event->button );
                }
                /* fallthru */
 
@@ -666,19 +847,38 @@ dfb_input_dispatch( CoreInputDevice *device, DFBInputEvent *event )
                }
 
                fixup_key_event( device, event );
-
-               D_DEBUG_AT( Core_Input, "  -> key code: %x, id: %x, symbol: %x\n",
-                           event->key_code, event->key_id, event->key_symbol );
                break;
 
           default:
                ;
      }
 
-     event->clazz     = DFEC_INPUT;
-     event->device_id = device->shared->id;
+#if D_DEBUG_ENABLED
+     if (event->flags & DIEF_TIMESTAMP)
+          D_DEBUG_AT( Core_InputEvt, "  => TIMESTAMP  %lu.%06lu\n", event->timestamp.tv_sec, event->timestamp.tv_usec );
+     if (event->flags & DIEF_AXISABS)
+          D_DEBUG_AT( Core_InputEvt, "  => AXISABS    %d at %d\n",  event->axis, event->axisabs );
+     if (event->flags & DIEF_AXISREL)
+          D_DEBUG_AT( Core_InputEvt, "  => AXISREL    %d by %d\n",  event->axis, event->axisrel );
+     if (event->flags & DIEF_KEYCODE)
+          D_DEBUG_AT( Core_InputEvt, "  => KEYCODE    %d\n",        event->key_code );
+     if (event->flags & DIEF_KEYID)
+          D_DEBUG_AT( Core_InputEvt, "  => KEYID      0x%04x\n",    event->key_id );
+     if (event->flags & DIEF_KEYSYMBOL)
+          D_DEBUG_AT( Core_InputEvt, "  => KEYSYMBOL  0x%04x\n",    event->key_symbol );
+     if (event->flags & DIEF_MODIFIERS)
+          D_DEBUG_AT( Core_InputEvt, "  => MODIFIERS  0x%04x\n",    event->modifiers );
+     if (event->flags & DIEF_LOCKS)
+          D_DEBUG_AT( Core_InputEvt, "  => LOCKS      0x%04x\n",    event->locks );
+     if (event->flags & DIEF_BUTTONS)
+          D_DEBUG_AT( Core_InputEvt, "  => BUTTONS    0x%04x\n",    event->buttons );
+     if (event->flags & DIEF_GLOBAL)
+          D_DEBUG_AT( Core_InputEvt, "  => GLOBAL\n" );
+#endif
 
-     if (!core_input_filter( device, event ))
+     if (core_input_filter( device, event ))
+          D_DEBUG_AT( Core_InputEvt, "  ****>> FILTERED\n" );
+     else
           fusion_reactor_dispatch( device->shared->reactor, event, true, dfb_input_globals );
 }
 
@@ -701,7 +901,7 @@ dfb_input_device_at( DFBInputDeviceID id )
 
      D_ASSERT( core_input != NULL );
 
-     direct_list_foreach (device, devices) {
+     direct_list_foreach (device, core_local->devices) {
           D_MAGIC_ASSERT( device, CoreInputDevice );
 
           if (device->shared->id == id)
@@ -785,7 +985,7 @@ input_add_device( CoreInputDevice *device )
           return;
      }
 
-     direct_list_append( &devices, &device->link );
+     direct_list_append( &core_local->devices, &device->link );
 
      core_input->devices[ core_input->num++ ] = device->shared;
 }
@@ -834,7 +1034,7 @@ make_id( DFBInputDeviceID prefered )
 
      D_ASSERT( core_input != NULL );
 
-     direct_list_foreach (device, devices) {
+     direct_list_foreach (device, core_local->devices) {
           D_MAGIC_ASSERT( device, CoreInputDevice );
 
           if (device->shared->id == prefered)
@@ -876,11 +1076,13 @@ reload_keymap( CoreInputDevice *device )
      return DFB_OK;
 }
 
-static int
-input_device_call_handler( int   caller,   /* fusion id of the caller */
-                           int   call_arg, /* optional call parameter */
-                           void *call_ptr, /* optional call parameter */
-                           void *ctx )     /* optional handler context */
+static FusionCallHandlerResult
+input_device_call_handler( int           caller,   /* fusion id of the caller */
+                           int           call_arg, /* optional call parameter */
+                           void         *call_ptr, /* optional call parameter */
+                           void         *ctx,      /* optional handler context */
+                           unsigned int  serial,
+                           int          *ret_val )
 {
      CoreInputDeviceCommand  command = call_arg;
      CoreInputDevice        *device  = ctx;
@@ -891,30 +1093,74 @@ input_device_call_handler( int   caller,   /* fusion id of the caller */
 
      switch (command) {
           case CIDC_RELOAD_KEYMAP:
-               return reload_keymap( device );
+               *ret_val = reload_keymap( device );
+               break;
 
           default:
                D_BUG( "unknown Core Input Device Command '%d'", command );
+               *ret_val = DFB_BUG;
      }
 
-     return DFB_BUG;
+     return FCHR_RETURN;
+}
+
+static DFBResult
+init_axes( CoreInputDevice *device )
+{
+     int                     i, num;
+     DFBResult               ret;
+     InputDeviceShared      *shared;
+     const InputDriverFuncs *funcs;
+
+     D_DEBUG_AT( Core_Input, "%s( %p )\n", __FUNCTION__, device );
+
+     D_MAGIC_ASSERT( device, CoreInputDevice );
+     D_ASSERT( device->driver != NULL );
+
+     funcs = device->driver->funcs;
+     D_ASSERT( funcs != NULL );
+
+     shared = device->shared;
+     D_ASSERT( shared != NULL );
+
+     if (shared->device_info.desc.max_axis < 0)
+          return DFB_OK;
+
+     num = shared->device_info.desc.max_axis + 1;
+
+     shared->axis_info = SHCALLOC( dfb_core_shmpool(device->core), num, sizeof(DFBInputDeviceAxisInfo) );
+     if (!shared->axis_info)
+          return D_OOSHM();
+
+     shared->axis_num = num;
+
+     if (funcs->GetAxisInfo) {
+          for (i=0; i<num; i++) {
+               ret = funcs->GetAxisInfo( device, device->driver_data, i, &shared->axis_info[i] );
+               if (ret)
+                    D_DERROR( ret, "Core/Input: GetAxisInfo() failed for '%s' [%d] on axis %d!\n",
+                              shared->device_info.desc.name, shared->id, i );
+          }
+     }
+
+     return DFB_OK;
 }
 
 static void
 init_devices( CoreDFB *core )
 {
-     DirectLink          *link;
+     DirectLink          *next;
+     DirectModuleEntry   *module;
      FusionSHMPoolShared *pool = dfb_core_shmpool( core );
 
      D_DEBUG_AT( Core_Input, "%s( %p )\n", __FUNCTION__, core );
 
      D_ASSERT( core_input != NULL );
 
-     direct_list_foreach( link, dfb_input_modules.entries ) {
+     direct_list_foreach_safe (module, next, dfb_input_modules.entries) {
           int                     n;
           InputDriver            *driver;
           const InputDriverFuncs *funcs;
-          DirectModuleEntry      *module = (DirectModuleEntry*) link;
 
           funcs = direct_module_ref( module );
           if (!funcs)
@@ -945,7 +1191,7 @@ init_devices( CoreDFB *core )
           driver->module = module;
           driver->funcs  = funcs;
 
-          direct_list_prepend( &drivers, &driver->link );
+          direct_list_prepend( &core_local->drivers, &driver->link );
 
 
           for (n=0; n<driver->nr_devices; n++) {
@@ -975,6 +1221,10 @@ init_devices( CoreDFB *core )
                     continue;
                }
 
+               D_DEBUG_AT( Core_Input, "  -> opened '%s' (%d) %d.%d (%s)\n",
+                           device_info.desc.name, n + 1, driver->info.version.major,
+                           driver->info.version.minor, driver->info.vendor );
+
                if (driver->nr_devices > 1)
                     snprintf( buf, sizeof(buf), "%s (%d)", device_info.desc.name, n+1 );
                else
@@ -1016,6 +1266,8 @@ init_devices( CoreDFB *core )
                         device_info.desc.max_keycode >= 0)
                     allocate_device_keymap( core, device );
 
+               init_axes( device );
+
                /* add it to the list */
                input_add_device( device );
           }
@@ -1471,6 +1723,27 @@ fixup_mouse_event( CoreInputDevice *device, DFBInputEvent *event )
 
           event->buttons = shared->buttons;
      }
+
+     switch (event->type) {
+          case DIET_AXISMOTION:
+               if ((event->flags & DIEF_AXISABS) && event->axis >= 0 && event->axis < shared->axis_num) {
+                    if (!(event->flags & DIEF_MIN) && (shared->axis_info[event->axis].flags & DIAIF_ABS_MIN)) {
+                         event->min = shared->axis_info[event->axis].abs_min;
+
+                         event->flags |= DIEF_MIN;
+                    }
+                         
+                    if (!(event->flags & DIEF_MAX) && (shared->axis_info[event->axis].flags & DIAIF_ABS_MAX)) {
+                         event->max = shared->axis_info[event->axis].abs_max;
+
+                         event->flags |= DIEF_MAX;
+                    }
+               }
+               break;
+
+          default:
+               break;
+     }
 }
 
 static DFBInputDeviceKeyIdentifier
@@ -1844,8 +2117,7 @@ dump_primary_layer_surface( CoreDFB *core )
                /* Get the surface of the region. */
                if (dfb_layer_region_get_surface( region, &surface ) == DFB_OK) {
                     /* Dump the surface contents. */
-                    dfb_surface_dump( core, surface,
-                                      dfb_config->screenshot_dir, "dfb" );
+                    dfb_surface_dump_buffer( surface, CSBR_FRONT, dfb_config->screenshot_dir, "dfb" );
 
                     /* Release the surface. */
                     dfb_surface_unref( surface );
