@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -39,6 +39,7 @@
 #include <core/coretypes.h>
 
 
+
 DECLARE_MODULE_DIRECTORY( dfb_input_modules );
 
 
@@ -77,15 +78,20 @@ typedef struct {
 
 typedef struct {
      int       (*GetAvailable)   (void);
-     void      (*GetDriverInfo)  (InputDriverInfo            *driver_info);
-     DFBResult (*OpenDevice)     (CoreInputDevice            *device,
-                                  unsigned int                number,
-                                  InputDeviceInfo            *device_info,
-                                  void                      **driver_data);
-     DFBResult (*GetKeymapEntry) (CoreInputDevice            *device,
-                                  void                       *driver_data,
-                                  DFBInputDeviceKeymapEntry  *entry);
-     void      (*CloseDevice)    (void                       *driver_data);
+     void      (*GetDriverInfo)  (InputDriverInfo              *driver_info);
+     DFBResult (*OpenDevice)     (CoreInputDevice              *device,
+                                  unsigned int                  number,
+                                  InputDeviceInfo              *device_info,
+                                  void                        **driver_data);
+     DFBResult (*GetKeymapEntry) (CoreInputDevice              *device,
+                                  void                         *driver_data,
+                                  DFBInputDeviceKeymapEntry    *entry);
+     void      (*CloseDevice)    (void                         *driver_data);
+
+     DFBResult (*GetAxisInfo)    (CoreInputDevice              *device,
+                                  void                         *driver_data,
+                                  DFBInputDeviceAxisIdentifier  axis,
+                                  DFBInputDeviceAxisInfo       *ret_info);
 } InputDriverFuncs;
 
 
