@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -55,7 +55,6 @@
 #include <core/state.h>
 #include <core/palette.h>
 #include <core/system.h>
-#include <core/surfacemanager.h>
 #include <core/windows.h>
 
 #include <gfx/convert.h>
@@ -253,6 +252,10 @@ dfb_layer_get_active_context( CoreLayer         *layer,
           return DFB_FUSION;
      }
 
+     D_DEBUG_AT( Core_Layers, "  => %p [%4dx%4d %8s]\n", context,
+                 context->config.width, context->config.height,
+                 dfb_pixelformat_name( context->config.pixelformat ) );
+
      /* Return the context. */
      *ret_context = context;
 
@@ -527,7 +530,7 @@ dfb_layer_get_current_output_field( CoreLayer *layer, int *field )
 DFBResult
 dfb_layer_get_level( CoreLayer *layer, int *ret_level )
 {
-     DisplayLayerFuncs *funcs;
+     const DisplayLayerFuncs *funcs;
 
      D_ASSERT( layer != NULL );
      D_ASSERT( layer->funcs != NULL );
@@ -545,7 +548,7 @@ dfb_layer_get_level( CoreLayer *layer, int *ret_level )
 DFBResult
 dfb_layer_set_level( CoreLayer *layer, int level )
 {
-     DisplayLayerFuncs *funcs;
+     const DisplayLayerFuncs *funcs;
 
      D_ASSERT( layer != NULL );
      D_ASSERT( layer->funcs != NULL );
