@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <directfb.h>
 
 #include <core/coredefs.h>
@@ -42,7 +45,7 @@
 #include <core/layer_control.h>
 #include <core/layer_region.h>
 #include <core/layers_internal.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <gfx/util.h>
 
@@ -50,6 +53,10 @@
 D_DEBUG_DOMAIN( Core_Layers, "Core/Layers", "DirectFB Display Layer Core" );
 
 
+static DFBResult region_buffer_lock( CoreLayerRegion       *region,
+                                     CoreSurface           *surface,
+                                     CoreSurfaceBufferRole  role );
+
 static DFBResult set_region      ( CoreLayerRegion            *region,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags  flags,
@@ -62,15 +69,13 @@ static DFBResult unrealize_region( CoreLayerRegion            *region );
 /******************************************************************************/
 
 static void
-region_destructor( FusionObject *object, bool zombie )
+region_destructor( FusionObject *object, bool zombie, void *ctx )
 {
      CoreLayerRegion  *region  = (CoreLayerRegion*) object;
      CoreLayerContext *context = region->context;
      CoreLayer        *layer   = dfb_layer_at( context->layer_id );
      CoreLayerShared  *shared  = layer->shared;
 
-     (void) shared;
-
      D_DEBUG_AT( Core_Layers, "destroying region %p (%s, %dx%d, "
                  "%s, %s, %s, %s%s)\n", region, shared->description.name,
                  region->config.width, region->config.height,
@@ -106,7 +111,7 @@ region_destructor( FusionObject *object, bool zombie )
 
      /* Free driver's region data. */
      if (region->region_data)
-          SHFREE( region->region_data );
+          SHFREE( shared->shmpool, region->region_data );
 
      /* Deinitialize the lock. */
      fusion_skirmish_destroy( &region->lock );
@@ -118,12 +123,12 @@ region_destructor( FusionObject *object, bool zombie )
 /******************************************************************************/
 
 FusionObjectPool *
-dfb_layer_region_pool_create()
+dfb_layer_region_pool_create( const FusionWorld *world )
 {
      return fusion_object_pool_create( "Layer Region Pool",
                                        sizeof(CoreLayerRegion),
                                        sizeof(CoreLayerRegionNotification),
-                                       region_destructor );
+                                       region_destructor, NULL, world );
 }
 
 /******************************************************************************/
@@ -152,7 +157,7 @@ dfb_layer_region_create( CoreLayerContext  *context,
      }
 
      /* Initialize the lock. */
-     if (fusion_skirmish_init( &region->lock, "Layer Region" )) {
+     if (fusion_skirmish_init( &region->lock, "Layer Region", dfb_core_world(layer->core) )) {
           dfb_layer_context_unlink( &region->context );
           fusion_object_destroy( &region->object );
           return DFB_FUSION;
@@ -161,6 +166,8 @@ dfb_layer_region_create( CoreLayerContext  *context,
      /* Change global reaction lock. */
      fusion_object_set_lock( &region->object, &region->lock );
 
+     region->state = CLRSF_FROZEN;
+
      /* Activate the object. */
      fusion_object_activate( &region->object );
 
@@ -227,8 +234,8 @@ dfb_layer_region_deactivate( CoreLayerRegion *region )
           return DFB_OK;
      }
 
-     /* Unrealize the region if it's enabled. */
-     if (D_FLAGS_IS_SET( region->state, CLRSF_ENABLED )) {
+     /* Unrealize the region? */
+     if (D_FLAGS_IS_SET( region->state, CLRSF_REALIZED )) {
           ret = unrealize_region( region );
           if (ret)
                return ret;
@@ -298,8 +305,8 @@ dfb_layer_region_disable( CoreLayerRegion *region )
           return DFB_OK;
      }
 
-     /* Unrealize the region if it's active. */
-     if (D_FLAGS_IS_SET( region->state, CLRSF_ACTIVE )) {
+     /* Unrealize the region? */
+     if (D_FLAGS_IS_SET( region->state, CLRSF_REALIZED )) {
           ret = unrealize_region( region );
           if (ret)
                return ret;
@@ -408,12 +415,11 @@ dfb_layer_region_flip_update( CoreLayerRegion     *region,
                               const DFBRegion     *update,
                               DFBSurfaceFlipFlags  flags )
 {
-     DFBResult          ret = DFB_OK;
-     CoreLayer         *layer;
-     CoreLayerContext  *context;
-     CoreSurface       *surface;
-     SurfaceBuffer     *buffer;
-     DisplayLayerFuncs *funcs;
+     DFBResult                ret = DFB_OK;
+     CoreLayer               *layer;
+     CoreLayerContext        *context;
+     CoreSurface             *surface;
+     const DisplayLayerFuncs *funcs;
 
      if (update)
           D_DEBUG_AT( Core_Layers,
@@ -442,23 +448,43 @@ dfb_layer_region_flip_update( CoreLayerRegion     *region,
 
      context = region->context;
      surface = region->surface;
-     buffer  = surface->back_buffer;
      layer   = dfb_layer_at( context->layer_id );
 
      D_ASSERT( layer->funcs != NULL );
 
      funcs = layer->funcs;
 
+     /* Unfreeze region? */
+     if (D_FLAGS_IS_SET( region->state, CLRSF_FROZEN )) {
+          D_FLAGS_CLEAR( region->state, CLRSF_FROZEN );
+
+          if (D_FLAGS_IS_SET( region->state, CLRSF_REALIZED )) {
+               ret = set_region( region, &region->config, CLRCF_ALL, surface );
+               if (ret)
+                    D_DERROR( ret, "Core/LayerRegion: set_region() in dfb_layer_region_flip_update() failed!\n" );
+          }
+          else if (D_FLAGS_ARE_SET( region->state, CLRSF_ENABLED | CLRSF_ACTIVE )) {
+               ret = realize_region( region );
+               if (ret)
+                    D_DERROR( ret, "Core/LayerRegion: realize_region() in dfb_layer_region_flip_update() failed!\n" );
+          }
+
+          if (ret) {
+               dfb_layer_region_unlock( region );
+               return ret;
+          }
+     }
+
      /* Depending on the buffer mode... */
      switch (region->config.buffermode) {
           case DLBM_TRIPLE:
           case DLBM_BACKVIDEO:
                /* Check if simply swapping the buffers is possible... */
-               if (!(flags & DSFLIP_BLIT) &&
+               if (!(flags & DSFLIP_BLIT) && !context->rotation &&
                    (!update || (update->x1 == 0 &&
                                 update->y1 == 0 &&
-                                update->x2 == surface->width - 1 &&
-                                update->y2 == surface->height - 1)))
+                                update->x2 == surface->config.size.w - 1 &&
+                                update->y2 == surface->config.size.h - 1)))
                {
                     D_DEBUG_AT( Core_Layers, "  -> Going to swap buffers...\n" );
 
@@ -466,12 +492,10 @@ dfb_layer_region_flip_update( CoreLayerRegion     *region,
                     if (D_FLAGS_IS_SET( region->state, CLRSF_REALIZED )) {
                          D_ASSUME( funcs->FlipRegion != NULL );
 
-                         D_DEBUG_AT( Core_Layers, "  -> Waiting for pending writes...\n" );
-
-                         if (buffer->video.access & VAF_HARDWARE_WRITE) {
-                              dfb_gfxcard_wait_serial( &buffer->video.serial );
-
-                              buffer->video.access &= ~VAF_HARDWARE_WRITE;
+                         ret = region_buffer_lock( region, surface, CSBR_BACK );
+                         if (ret) {
+                              dfb_layer_region_unlock( region );
+                              return ret;
                          }
 
                          D_DEBUG_AT( Core_Layers, "  -> Flipping region using driver...\n" );
@@ -481,15 +505,18 @@ dfb_layer_region_flip_update( CoreLayerRegion     *region,
                                                        layer->driver_data,
                                                        layer->layer_data,
                                                        region->region_data,
-                                                       surface, flags );
+                                                       surface, flags, &region->surface_lock );
+
+                         dfb_surface_unlock( surface );
                     }
                     else {
                          D_DEBUG_AT( Core_Layers, "  -> Flipping region not using driver...\n" );
 
                          /* Just do the hardware independent work. */
-                         dfb_surface_flip_buffers( surface, false );
+                         dfb_surface_lock( surface );
+                         dfb_surface_flip( surface, false );
+                         dfb_surface_unlock( surface );
                     }
-
                     break;
                }
 
@@ -504,18 +531,13 @@ dfb_layer_region_flip_update( CoreLayerRegion     *region,
                     dfb_layer_wait_vsync( layer );
                }
 
-               D_DEBUG_AT( Core_Layers, "  -> Waiting for pending writes...\n" );
-
-               if (buffer->video.access & VAF_HARDWARE_WRITE) {
-                    dfb_gfxcard_wait_serial( &buffer->video.serial );
-
-                    buffer->video.access &= ~VAF_HARDWARE_WRITE;
-               }
-
                D_DEBUG_AT( Core_Layers, "  -> Copying content from back to front buffer...\n" );
 
                /* ...or copy updated contents from back to front buffer. */
-               dfb_back_to_front_copy( surface, update );
+               if (context->rotation == 180)
+                    dfb_back_to_front_copy_180( surface, update );
+               else
+                    dfb_back_to_front_copy( surface, update );
 
                if ((flags & DSFLIP_WAITFORSYNC) == DSFLIP_WAIT) {
                     D_DEBUG_AT( Core_Layers, "  -> Waiting for VSync...\n" );
@@ -528,13 +550,31 @@ dfb_layer_region_flip_update( CoreLayerRegion     *region,
           case DLBM_FRONTONLY:
                /* Tell the driver about the update if the region is realized. */
                if (funcs->UpdateRegion && D_FLAGS_IS_SET( region->state, CLRSF_REALIZED )) {
+                    if (region->surface) {
+                         CoreSurfaceAllocation *allocation;
+
+                         allocation = region->surface_lock.allocation;
+                         D_ASSERT( allocation != NULL );
+
+                         /* If hardware has written or is writing... */
+                         if (allocation->accessed & CSAF_GPU_WRITE) {
+                              D_DEBUG_AT( Core_Layers, "  -> Waiting for pending writes...\n" );
+
+                              /* ...wait for the operation to finish. */
+                              if (!(flags & DSFLIP_PIPELINE))
+                                   dfb_gfxcard_sync(); /* TODO: wait for serial instead */
+
+                              allocation->accessed &= ~CSAF_GPU_WRITE;
+                         }
+                    }
+
                     D_DEBUG_AT( Core_Layers, "  -> Notifying driver about updated content...\n" );
 
                     ret = funcs->UpdateRegion( layer,
                                                layer->driver_data,
                                                layer->layer_data,
                                                region->region_data,
-                                               surface, update );
+                                               surface, update, &region->surface_lock );
                }
                break;
 
@@ -556,10 +596,10 @@ dfb_layer_region_set_configuration( CoreLayerRegion            *region,
                                     CoreLayerRegionConfig      *config,
                                     CoreLayerRegionConfigFlags  flags )
 {
-     DFBResult              ret;
-     CoreLayer             *layer;
-     DisplayLayerFuncs     *funcs;
-     CoreLayerRegionConfig  new_config;
+     DFBResult                ret;
+     CoreLayer               *layer;
+     const DisplayLayerFuncs *funcs;
+     CoreLayerRegionConfig    new_config;
 
      D_ASSERT( region != NULL );
      D_ASSERT( region->context != NULL );
@@ -636,6 +676,12 @@ dfb_layer_region_set_configuration( CoreLayerRegion            *region,
 
           if (flags & CLRCF_PARITY)
                new_config.parity = config->parity;
+
+          if (flags & CLRCF_CLIPS) {
+               new_config.clips     = config->clips;
+               new_config.num_clips = config->num_clips;
+               new_config.positive  = config->positive;
+          }
      }
 
      /* Check if the new configuration is supported. */
@@ -646,6 +692,10 @@ dfb_layer_region_set_configuration( CoreLayerRegion            *region,
           return ret;
      }
 
+     /* Check if the region should be frozen, thus requiring to apply changes explicitly. */
+     if (flags & CLRCF_FREEZE)
+          region->state |= CLRSF_FROZEN;
+
      /* Propagate new configuration to the driver if the region is realized. */
      if (D_FLAGS_IS_SET( region->state, CLRSF_REALIZED )) {
           ret = set_region( region, &new_config, flags, region->surface );
@@ -717,7 +767,7 @@ _dfb_layer_region_surface_listener( const void *msg_data, void *ctx )
      CoreSurface                   *surface;
      CoreLayer                     *layer;
      CoreLayerShared               *shared;
-     DisplayLayerFuncs             *funcs;
+     const DisplayLayerFuncs       *funcs;
      const CoreSurfaceNotification *notification = msg_data;
      CoreLayerRegion               *region       = ctx;
 
@@ -757,13 +807,16 @@ _dfb_layer_region_surface_listener( const void *msg_data, void *ctx )
      if (dfb_layer_region_lock( region ))
           return RS_OK;
 
-     if (D_FLAGS_ARE_SET( region->state, CLRSF_REALIZED | CLRSF_CONFIGURED )) {
+     if (D_FLAGS_ARE_SET( region->state, CLRSF_REALIZED | CLRSF_CONFIGURED ) &&
+         !D_FLAGS_IS_SET( region->state, CLRSF_FROZEN ))
+     {
           if (D_FLAGS_IS_SET( flags, CSNF_PALETTE_CHANGE | CSNF_PALETTE_UPDATE )) {
                if (surface->palette)
                     funcs->SetRegion( layer,
                                       layer->driver_data, layer->layer_data,
                                       region->region_data, &region->config,
-                                      CLRCF_PALETTE, surface, surface->palette );
+                                      CLRCF_PALETTE, surface, surface->palette,
+                                      &region->surface_lock );
           }
 
           if ((flags & CSNF_FIELD) && funcs->SetInputField)
@@ -780,7 +833,8 @@ _dfb_layer_region_surface_listener( const void *msg_data, void *ctx )
                funcs->SetRegion( layer,
                                  layer->driver_data, layer->layer_data,
                                  region->region_data, &region->config,
-                                 CLRCF_ALPHA_RAMP, surface, surface->palette );
+                                 CLRCF_ALPHA_RAMP, surface, surface->palette,
+                                 &region->surface_lock );
           }
      }
 
@@ -791,14 +845,75 @@ _dfb_layer_region_surface_listener( const void *msg_data, void *ctx )
 
 /******************************************************************************/
 
+static DFBResult
+region_buffer_lock( CoreLayerRegion       *region,
+                    CoreSurface           *surface,
+                    CoreSurfaceBufferRole  role )
+{
+     DFBResult               ret;
+     CoreSurfaceAccessFlags  flags;
+     CoreSurfaceBuffer      *buffer;
+     CoreSurfaceAllocation  *allocation;
+
+     /* First unlock any previously locked buffer. */
+     if (region->surface_lock.buffer) {
+          D_MAGIC_ASSERT( region->surface_lock.buffer, CoreSurfaceBuffer );
+
+          dfb_surface_unlock_buffer( region->surface_lock.buffer->surface, &region->surface_lock );
+     }
+
+     /* Determine flags to use. */
+     if (surface->config.caps & DSCAPS_SYSTEMONLY)
+          flags = CSAF_CPU_READ;
+     else
+          flags = CSAF_GPU_READ;
+         
+     if (dfb_surface_lock( surface ))
+          return DFB_FUSION;
+
+     buffer = dfb_surface_get_buffer( surface, role );
+     D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );
+
+     /* Lock the surface buffer. */
+     ret = dfb_surface_buffer_lock( buffer, flags, &region->surface_lock );
+     if (ret) {
+          D_DERROR( ret, "Core/LayerRegion: Could not lock region surface for SetRegion()!\n" );
+          dfb_surface_unlock( surface );
+          return ret;
+     }
+
+     allocation = region->surface_lock.allocation;
+     D_ASSERT( allocation != NULL );
+
+     /* If hardware has written or is writing... */
+     if (allocation->accessed & CSAF_GPU_WRITE) {
+          D_DEBUG_AT( Core_Layers, "  -> Waiting for pending writes...\n" );
+
+          /* ...wait for the operation to finish. */
+          dfb_gfxcard_sync(); /* TODO: wait for serial instead */
+
+          allocation->accessed &= ~CSAF_GPU_WRITE;
+     }
+
+     return DFB_OK;
+}
+
 static DFBResult
 set_region( CoreLayerRegion            *region,
             CoreLayerRegionConfig      *config,
             CoreLayerRegionConfigFlags  flags,
             CoreSurface                *surface )
 {
-     CoreLayer         *layer;
-     DisplayLayerFuncs *funcs;
+     DFBResult                ret;
+     CoreLayer               *layer;
+     CoreLayerShared         *shared;
+     const DisplayLayerFuncs *funcs;
+
+     D_DEBUG_AT( Core_Layers, "%s( %p, %p, 0x%08x, %p )\n", __FUNCTION__, region, config, flags, surface );
+
+     DFB_CORE_LAYER_REGION_CONFIG_DEBUG_AT( Core_Layers, config );
+
+     D_DEBUG_AT( Core_Layers, "  -> state    0x%08x\n", region->state );
 
      D_ASSERT( region != NULL );
      D_ASSERT( region->context != NULL );
@@ -810,26 +925,59 @@ set_region( CoreLayerRegion            *region,
      layer = dfb_layer_at( region->context->layer_id );
 
      D_ASSERT( layer != NULL );
+     D_ASSERT( layer->shared != NULL );
      D_ASSERT( layer->funcs != NULL );
      D_ASSERT( layer->funcs->SetRegion != NULL );
 
-     funcs = layer->funcs;
+     if (region->state & CLRSF_FROZEN) {
+          D_DEBUG_AT( Core_Layers, "  -> FROZEN!\n" );
+          return DFB_OK;
+     }
+
+     shared = layer->shared;
+     funcs  = layer->funcs;
+
+     if (surface) {
+          if (flags & (CLRCF_SURFACE | CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT)) {
+               ret = region_buffer_lock( region, surface, CSBR_FRONT );
+               if (ret)
+                    return ret;
+
+               dfb_surface_unlock( surface );
+          }
+
+          D_ASSERT( region->surface_lock.buffer != NULL );
+     }
+     else if (region->surface_lock.buffer) {
+          D_MAGIC_ASSERT( region->surface_lock.buffer, CoreSurfaceBuffer );
+
+          dfb_surface_unlock_buffer( region->surface_lock.buffer->surface, &region->surface_lock );
+     }
+
+     D_DEBUG_AT( Core_Layers, "  => setting region of '%s'\n", shared->description.name );
 
      /* Setup hardware. */
      return funcs->SetRegion( layer, layer->driver_data, layer->layer_data,
                               region->region_data, config, flags,
-                              surface, surface ? surface->palette : NULL );
+                              surface, surface ? surface->palette : NULL, &region->surface_lock );
 }
 
 static DFBResult
 realize_region( CoreLayerRegion *region )
 {
-     DFBResult          ret;
-     CoreLayer         *layer;
-     CoreLayerShared   *shared;
-     DisplayLayerFuncs *funcs;
+     DFBResult                ret;
+     CoreLayer               *layer;
+     CoreLayerShared         *shared;
+     const DisplayLayerFuncs *funcs;
+
+     D_DEBUG_AT( Core_Layers, "%s( %p )\n", __FUNCTION__, region );
 
      D_ASSERT( region != NULL );
+
+     DFB_CORE_LAYER_REGION_CONFIG_DEBUG_AT( Core_Layers, &region->config );
+
+     D_DEBUG_AT( Core_Layers, "  -> state    0x%08x\n", region->state );
+
      D_ASSERT( region->context != NULL );
      D_ASSERT( D_FLAGS_IS_SET( region->state, CLRSF_CONFIGURED ) );
      D_ASSERT( ! D_FLAGS_IS_SET( region->state, CLRSF_REALIZED ) );
@@ -845,19 +993,23 @@ realize_region( CoreLayerRegion *region )
 
      D_ASSERT( ! fusion_vector_contains( &shared->added_regions, region ) );
 
+     if (region->state & CLRSF_FROZEN) {
+          D_DEBUG_AT( Core_Layers, "  -> FROZEN!\n" );
+          return DFB_OK;
+     }
+
      /* Allocate the driver's region data. */
      if (funcs->RegionDataSize) {
           int size = funcs->RegionDataSize();
 
           if (size > 0) {
-               region->region_data = SHCALLOC( 1, size );
+               region->region_data = SHCALLOC( shared->shmpool, 1, size );
                if (!region->region_data)
                     return D_OOSHM();
           }
      }
 
-     D_DEBUG_AT( Core_Layers, "Adding region (%d, %d - %dx%d) to '%s'.\n",
-                 DFB_RECTANGLE_VALS( &region->config.dest ), shared->description.name );
+     D_DEBUG_AT( Core_Layers, "  => adding region to '%s'\n", shared->description.name );
 
      /* Add the region to the driver. */
      if (funcs->AddRegion) {
@@ -868,7 +1020,7 @@ realize_region( CoreLayerRegion *region )
                D_DERROR( ret, "Core/Layers: Could not add region!\n" );
 
                if (region->region_data) {
-                    SHFREE( region->region_data );
+                    SHFREE( shared->shmpool, region->region_data );
                     region->region_data = NULL;
                }
 
@@ -895,13 +1047,20 @@ realize_region( CoreLayerRegion *region )
 static DFBResult
 unrealize_region( CoreLayerRegion *region )
 {
-     DFBResult          ret;
-     int                index;
-     CoreLayer         *layer;
-     CoreLayerShared   *shared;
-     DisplayLayerFuncs *funcs;
+     DFBResult                ret;
+     int                      index;
+     CoreLayer               *layer;
+     CoreLayerShared         *shared;
+     const DisplayLayerFuncs *funcs;
+
+     D_DEBUG_AT( Core_Layers, "%s( %p )\n", __FUNCTION__, region );
 
      D_ASSERT( region != NULL );
+
+     DFB_CORE_LAYER_REGION_CONFIG_DEBUG_AT( Core_Layers, &region->config );
+
+     D_DEBUG_AT( Core_Layers, "  -> state    0x%08x\n", region->state );
+
      D_ASSERT( region->context != NULL );
      D_ASSERT( D_FLAGS_IS_SET( region->state, CLRSF_REALIZED ) );
 
@@ -918,8 +1077,7 @@ unrealize_region( CoreLayerRegion *region )
 
      index = fusion_vector_index_of( &shared->added_regions, region );
 
-     D_DEBUG_AT( Core_Layers, "Removing region (%d, %d - %dx%d) from '%s'.\n",
-                 DFB_RECTANGLE_VALS( &region->config.dest ), shared->description.name );
+     D_DEBUG_AT( Core_Layers, "  => removing region from '%s'\n", shared->description.name );
 
      /* Remove the region from hardware and driver. */
      if (funcs->RemoveRegion) {
@@ -936,12 +1094,18 @@ unrealize_region( CoreLayerRegion *region )
 
      /* Deallocate the driver's region data. */
      if (region->region_data) {
-          SHFREE( region->region_data );
+          SHFREE( shared->shmpool, region->region_data );
           region->region_data = NULL;
      }
 
      /* Update the region's state. */
      D_FLAGS_CLEAR( region->state, CLRSF_REALIZED );
+     D_FLAGS_SET( region->state, CLRSF_FROZEN );
+
+     if (region->surface && region->surface_lock.buffer) {
+          dfb_surface_unlock_buffer( region->surface, &region->surface_lock );
+          dfb_surface_destroy_buffers( region->surface );
+     }
 
      return DFB_OK;
 }
