@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -33,6 +33,10 @@
 
 #include <core/coretypes.h>
 
+#include <core/gfxcard.h>
+#include <core/surface_buffer.h>
+
+
 struct __DFB_CoreLayerRegionConfig {
      int                        width;            /* width of the source in pixels */
      int                        height;           /* height of the source in pixels */
@@ -49,8 +53,8 @@ struct __DFB_CoreLayerRegionConfig {
 
      u8                         opacity;          /* global region alpha */
 
-     DFBColor                   src_key;          /* source color key */
-     DFBColor                   dst_key;          /* destination color key */
+     DFBColorKey                src_key;          /* source color key */
+     DFBColorKey                dst_key;          /* destination color key */
 
      int                        parity;           /* field parity (for interlaced) */
 
@@ -61,6 +65,28 @@ struct __DFB_CoreLayerRegionConfig {
      DFBBoolean                 positive;         /* show or cut out regions */
 };
 
+#if D_DEBUG_ENABLED
+#define DFB_CORE_LAYER_REGION_CONFIG_DEBUG_AT( domain, config )                                                    \
+     do {                                                                                                          \
+          const CoreLayerRegionConfig *_config = config;                                                           \
+                                                                                                                   \
+          D_DEBUG_AT( domain, "  -> size       %dx%d\n", _config->width, _config->height );                        \
+          D_DEBUG_AT( domain, "  -> format     %s\n", dfb_pixelformat_name( _config->format ) );                   \
+          D_DEBUG_AT( domain, "  -> surf caps  0x%08x\n", _config->surface_caps );                                 \
+          D_DEBUG_AT( domain, "  -> buffermode %d\n", _config->buffermode );                                       \
+          D_DEBUG_AT( domain, "  -> options    0x%08x\n", _config->options );                                      \
+          D_DEBUG_AT( domain, "  -> source     %d,%d-%dx%d\n", DFB_RECTANGLE_VALS(&_config->source) );             \
+          D_DEBUG_AT( domain, "  -> dest       %d,%d-%dx%d\n", DFB_RECTANGLE_VALS(&_config->dest) );               \
+          D_DEBUG_AT( domain, "  -> opacity    %d\n", _config->opacity );                                          \
+          D_DEBUG_AT( domain, "  -> src_key    %02x%02x%02x (index %d)\n", DFB_COLORKEY_VALS(&_config->src_key) ); \
+          D_DEBUG_AT( domain, "  -> dst_key    %02x%02x%02x (index %d)\n", DFB_COLORKEY_VALS(&_config->dst_key) ); \
+     } while (0)
+#else
+#define DFB_CORE_LAYER_REGION_CONFIG_DEBUG_AT( domain, config )                                                    \
+     do {                                                                                                          \
+     } while (0)
+#endif
+
 typedef enum {
      CLRCF_NONE         = 0x00000000,
 
@@ -88,7 +114,9 @@ typedef enum {
      CLRCF_SURFACE      = 0x10000000,
      CLRCF_PALETTE      = 0x20000000,
 
-     CLRCF_ALL          = 0x3013377F
+     CLRCF_FREEZE       = 0x80000000,
+
+     CLRCF_ALL          = 0xB013377F
 } CoreLayerRegionConfigFlags;
 
 typedef struct {
@@ -97,12 +125,12 @@ typedef struct {
      /*
       * Return size of layer data (shared memory).
       */
-     int       (*LayerDataSize) ();
+     int       (*LayerDataSize) ( void );
 
      /*
       * Return size of region data (shared memory).
       */
-     int       (*RegionDataSize)();
+     int       (*RegionDataSize)( void );
 
      /*
       * Called once by the master to initialize layer data and reset hardware.
@@ -195,7 +223,8 @@ typedef struct {
                                  CoreLayerRegionConfig      *config,
                                  CoreLayerRegionConfigFlags  updated,
                                  CoreSurface                *surface,
-                                 CorePalette                *palette );
+                                 CorePalette                *palette,
+                                 CoreSurfaceBufferLock      *lock );
 
      /*
       * Remove a region from the layer.
@@ -213,7 +242,8 @@ typedef struct {
                                  void                       *layer_data,
                                  void                       *region_data,
                                  CoreSurface                *surface,
-                                 DFBSurfaceFlipFlags         flags );
+                                 DFBSurfaceFlipFlags         flags,
+                                 CoreSurfaceBufferLock      *lock );
 
      /*
       * Indicate updates to the front buffer content.
@@ -223,7 +253,8 @@ typedef struct {
                                  void                       *layer_data,
                                  void                       *region_data,
                                  CoreSurface                *surface,
-                                 const DFBRegion            *update );
+                                 const DFBRegion            *update,
+                                 CoreSurfaceBufferLock      *lock );
 
      /*
       * Control hardware deinterlacing.
@@ -273,9 +304,9 @@ typedef struct {
  * containing driver functions. The supplied driver data
  * will be passed to these functions.
  */
-CoreLayer *dfb_layers_register( CoreScreen        *screen,
-                                void              *driver_data,
-                                DisplayLayerFuncs *funcs );
+CoreLayer *dfb_layers_register( CoreScreen              *screen,
+                                void                    *driver_data,
+                                const DisplayLayerFuncs *funcs );
 
 /*
  * Replace functions of the primary layer implementation by passing
@@ -284,7 +315,7 @@ CoreLayer *dfb_layers_register( CoreScreen        *screen,
  * The original function table is written to 'primary_funcs' before to allow
  * drivers to use existing functionality from the original implementation.
  */
-CoreLayer *dfb_layers_hook_primary( GraphicsDevice     *device,
+CoreLayer *dfb_layers_hook_primary( CoreGraphicsDevice *device,
                                     void               *driver_data,
                                     DisplayLayerFuncs  *funcs,
                                     DisplayLayerFuncs  *primary_funcs,
@@ -294,7 +325,7 @@ CoreLayer *dfb_layers_hook_primary( GraphicsDevice     *device,
  * Replace functions of the primary layer implementation completely by passing
  * an alternative driver function table.
  */
-CoreLayer *dfb_layers_replace_primary( GraphicsDevice     *device,
+CoreLayer *dfb_layers_replace_primary( CoreGraphicsDevice *device,
                                        void               *driver_data,
                                        DisplayLayerFuncs  *funcs );
 
@@ -305,7 +336,7 @@ void dfb_layers_enumerate( DisplayLayerCallback  callback,
                            void                 *ctx );
 
 
-int        dfb_layer_num();
+int        dfb_layer_num( void );
 
 CoreLayer *dfb_layer_at( DFBDisplayLayerID id );
 
@@ -323,6 +354,6 @@ DFBDisplayLayerID dfb_layer_id( const CoreLayer *layer );
 
 DFBDisplayLayerID dfb_layer_id_translated( const CoreLayer *layer );
 
-DFBSurfacePixelFormat dfb_primary_layer_pixelformat();
+DFBSurfacePixelFormat dfb_primary_layer_pixelformat( void );
 
 #endif
