@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -77,20 +77,20 @@ typedef struct {
 } CoreLayerShared;
 
 struct __DFB_CoreLayer {
-     CoreLayerShared   *shared;
+     CoreLayerShared         *shared;
 
-     CoreDFB           *core;
+     CoreDFB                 *core;
 
-     GraphicsDevice    *device;
+     CoreGraphicsDevice      *device;
 
-     CoreScreen        *screen;
+     CoreScreen              *screen;
 
-     void              *driver_data;
-     void              *layer_data;   /* copy of shared->layer_data */
+     void                    *driver_data;
+     void                    *layer_data;   /* copy of shared->layer_data */
 
-     DisplayLayerFuncs *funcs;
+     const DisplayLayerFuncs *funcs;
 
-     CardState          state;
+     CardState                state;
 };
 
 typedef enum {
@@ -110,6 +110,7 @@ struct __DFB_CoreLayerContext {
      bool                        active;     /* Is this the active context? */
 
      DFBDisplayLayerConfig       config;     /* Current layer configuration. */
+     int                         rotation;
 
      FusionVector                regions;    /* All regions created within
                                                 this context. */
@@ -145,7 +146,9 @@ typedef enum {
      CLRSF_ACTIVE     = 0x00000004,
      CLRSF_REALIZED   = 0x00000008,
 
-     CLRSF_ALL        = 0x0000000F
+     CLRSF_FROZEN     = 0x00000010,
+
+     CLRSF_ALL        = 0x0000001F
 } CoreLayerRegionStateFlags;
 
 struct __DFB_CoreLayerRegion {
@@ -160,6 +163,7 @@ struct __DFB_CoreLayerRegion {
      CoreLayerRegionConfig       config;
 
      CoreSurface                *surface;
+     CoreSurfaceBufferLock       surface_lock;
      GlobalReaction              surface_reaction;
 
      void                       *region_data;
