@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -36,11 +36,13 @@
 #include <core/coretypes.h>
 
 #include <core/core.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/gfxcard.h>
 #include <core/palette.h>
 #include <core/colorhash.h>
 
+#include <gfx/convert.h>
+
 #include <misc/util.h>
 
 D_DEBUG_DOMAIN( Core_Palette, "Core/Palette", "DirectFB Palette Core" );
@@ -57,26 +59,34 @@ static const ReactionFunc dfb_palette_globals[] = {
 
 /**********************************************************************************************************************/
 
-static void palette_destructor( FusionObject *object, bool zombie )
+static void palette_destructor( FusionObject *object, bool zombie, void *ctx )
 {
      CorePaletteNotification  notification;
      CorePalette             *palette = (CorePalette*) object;
 
+     D_MAGIC_ASSERT( palette, CorePalette );
+
      D_DEBUG_AT( Core_Palette, "destroying %p (%d)%s\n", palette,
                  palette->num_entries, zombie ? " (ZOMBIE)" : "");
 
+     D_ASSERT( palette->entries != NULL );
+     D_ASSERT( palette->entries_yuv != NULL );
+
      notification.flags   = CPNF_DESTROY;
      notification.palette = palette;
 
      dfb_palette_dispatch( palette, &notification, dfb_palette_globals );
 
      if (palette->hash_attached) {
-          dfb_colorhash_invalidate( palette );
-          dfb_colorhash_detach( palette );
+          dfb_colorhash_invalidate( NULL, palette );
+          dfb_colorhash_detach( NULL, palette );
      }
 
+     SHFREE( palette->shmpool, palette->entries_yuv );
      SHFREE( palette->shmpool, palette->entries );
 
+     D_MAGIC_CLEAR( palette );
+
      fusion_object_destroy( object );
 }
 
@@ -88,7 +98,7 @@ dfb_palette_pool_create( const FusionWorld *world )
      pool = fusion_object_pool_create( "Palette Pool",
                                        sizeof(CorePalette),
                                        sizeof(CorePaletteNotification),
-                                       palette_destructor, world );
+                                       palette_destructor, NULL, world );
 
      return pool;
 }
@@ -118,6 +128,13 @@ dfb_palette_create( CoreDFB       *core,
                fusion_object_destroy( &palette->object );
                return D_OOSHM();
           }
+
+          palette->entries_yuv = SHCALLOC( palette->shmpool, size, sizeof(DFBColorYUV) );
+          if (!palette->entries_yuv) {
+               SHFREE( palette->shmpool, palette->entries );
+               fusion_object_destroy( &palette->object );
+               return D_OOSHM();
+          }
      }
 
      palette->num_entries = size;
@@ -125,6 +142,8 @@ dfb_palette_create( CoreDFB       *core,
      /* reset cache */
      palette->search_cache.index = -1;
 
+     D_MAGIC_SET( palette, CorePalette );
+
      /* activate object */
      fusion_object_activate( &palette->object );
 
@@ -143,7 +162,7 @@ dfb_palette_generate_rgb332_map( CorePalette *palette )
 
      D_DEBUG_AT( Core_Palette, "%s( %p )\n", __FUNCTION__, palette );
 
-     D_ASSERT( palette != NULL );
+     D_MAGIC_ASSERT( palette, CorePalette );
 
      if (!palette->num_entries)
           return;
@@ -153,6 +172,11 @@ dfb_palette_generate_rgb332_map( CorePalette *palette )
           palette->entries[i].r = lookup3to8[ (i & 0xE0) >> 5 ];
           palette->entries[i].g = lookup3to8[ (i & 0x1C) >> 2 ];
           palette->entries[i].b = lookup2to8[ (i & 0x03) ];
+
+          palette->entries_yuv[i].a = palette->entries[i].a;
+
+          RGB_TO_YCBCR( palette->entries[i].r, palette->entries[i].g, palette->entries[i].b,
+                        palette->entries_yuv[i].y, palette->entries_yuv[i].u, palette->entries_yuv[i].v );
      }
 
      dfb_palette_update( palette, 0, palette->num_entries - 1 );
@@ -165,7 +189,7 @@ dfb_palette_generate_rgb121_map( CorePalette *palette )
 
      D_DEBUG_AT( Core_Palette, "%s( %p )\n", __FUNCTION__, palette );
 
-     D_ASSERT( palette != NULL );
+     D_MAGIC_ASSERT( palette, CorePalette );
 
      if (!palette->num_entries)
           return;
@@ -175,6 +199,11 @@ dfb_palette_generate_rgb121_map( CorePalette *palette )
           palette->entries[i].r = (i & 0x8) ? 0xff : 0x00;
           palette->entries[i].g = lookup2to8[ (i & 0x6) >> 1 ];
           palette->entries[i].b = (i & 0x1) ? 0xff : 0x00;
+
+          palette->entries_yuv[i].a = palette->entries[i].a;
+
+          RGB_TO_YCBCR( palette->entries[i].r, palette->entries[i].g, palette->entries[i].b,
+                        palette->entries_yuv[i].y, palette->entries_yuv[i].u, palette->entries_yuv[i].v );
      }
 
      dfb_palette_update( palette, 0, palette->num_entries - 1 );
@@ -189,7 +218,7 @@ dfb_palette_search( CorePalette *palette,
 {
      unsigned int index;
 
-     D_ASSERT( palette != NULL );
+     D_MAGIC_ASSERT( palette, CorePalette );
 
      /* check local cache first */
      if (palette->search_cache.index != -1 &&
@@ -201,11 +230,11 @@ dfb_palette_search( CorePalette *palette,
 
      /* check the global color hash table, returns the closest match */
      if (!palette->hash_attached) {
-          dfb_colorhash_attach( palette );
+          dfb_colorhash_attach( NULL, palette );
           palette->hash_attached = true;
      }
 
-     index = dfb_colorhash_lookup( palette, r, g, b, a );
+     index = dfb_colorhash_lookup( NULL, palette, r, g, b, a );
 
      /* write into local cache */
      palette->search_cache.index = index;
@@ -224,7 +253,7 @@ dfb_palette_update( CorePalette *palette, int first, int last )
 
      D_DEBUG_AT( Core_Palette, "%s( %p, %d, %d )\n", __FUNCTION__, palette, first, last );
 
-     D_ASSERT( palette != NULL );
+     D_MAGIC_ASSERT( palette, CorePalette );
      D_ASSERT( first >= 0 );
      D_ASSERT( first < palette->num_entries );
      D_ASSERT( last >= 0 );
@@ -243,7 +272,7 @@ dfb_palette_update( CorePalette *palette, int first, int last )
 
      /* invalidate entries in colorhash */
      if (palette->hash_attached)
-          dfb_colorhash_invalidate( palette );
+          dfb_colorhash_invalidate( NULL, palette );
 
      /* post message about palette update */
      dfb_palette_dispatch( palette, &notification, dfb_palette_globals );
