@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -35,9 +35,11 @@
 
 struct _CorePalette {
      FusionObject  object;
+     int           magic;
 
      unsigned int  num_entries;
      DFBColor     *entries;
+     DFBColorYUV  *entries_yuv;
 
      struct {
           int      index;
