@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -32,6 +32,7 @@
 
 #include <direct/debug.h>
 
+#include <core/layers.h>
 #include <core/screen.h>
 #include <core/screens_internal.h>
 
@@ -457,9 +458,83 @@ dfb_screen_get_screen_size( CoreScreen *screen,
      D_ASSERT( ret_width != NULL );
      D_ASSERT( ret_height != NULL );
 
-     /* Test the mixer configuration. */
      return screen->funcs->GetScreenSize( screen,
                                           screen->driver_data, screen->screen_data,
                                           ret_width, ret_height );
 }
 
+DFBResult
+dfb_screen_get_layer_dimension( CoreScreen *screen,
+                                CoreLayer  *layer,
+                                int        *ret_width,
+                                int        *ret_height )
+{
+     int               i;
+     DFBResult         ret = DFB_UNSUPPORTED;
+     CoreScreenShared *shared;
+     ScreenFuncs      *funcs;
+
+     D_ASSERT( screen != NULL );
+     D_ASSERT( screen->shared != NULL );
+     D_ASSERT( screen->funcs != NULL );
+     D_ASSERT( layer != NULL );
+     D_ASSERT( ret_width != NULL );
+     D_ASSERT( ret_height != NULL );
+
+     shared = screen->shared;
+     funcs  = screen->funcs;
+
+     if (funcs->GetMixerState) {
+          for (i=0; i<shared->description.mixers; i++) {
+               const DFBScreenMixerConfig *config = &shared->mixers[i].configuration;
+
+               if ((config->flags & DSMCONF_LAYERS) &&
+                   DFB_DISPLAYLAYER_IDS_HAVE( config->layers, dfb_layer_id(layer) ))
+               {
+                    CoreMixerState state;
+
+                    ret = funcs->GetMixerState( screen, screen->driver_data, screen->screen_data, i, &state );
+                    if (ret == DFB_OK) {
+                         if (state.flags & CMSF_DIMENSION) {
+                              *ret_width  = state.dimension.w;
+                              *ret_height = state.dimension.h;
+
+                              return DFB_OK;
+                         }
+
+                         ret = DFB_UNSUPPORTED;
+                    }
+               }
+          }
+
+          for (i=0; i<shared->description.mixers; i++) {
+               const DFBScreenMixerDescription *desc = &shared->mixers[i].description;
+
+               if ((desc->caps & DSMCAPS_SUB_LAYERS) &&
+                   DFB_DISPLAYLAYER_IDS_HAVE( desc->sub_layers, dfb_layer_id(layer) ))
+               {
+                    CoreMixerState state;
+
+                    ret = funcs->GetMixerState( screen, screen->driver_data, screen->screen_data, i, &state );
+                    if (ret == DFB_OK) {
+                         if (state.flags & CMSF_DIMENSION) {
+                              *ret_width  = state.dimension.w;
+                              *ret_height = state.dimension.h;
+
+                              return DFB_OK;
+                         }
+
+                         ret = DFB_UNSUPPORTED;
+                    }
+               }
+          }
+     }
+
+     if (funcs->GetScreenSize)
+          ret = funcs->GetScreenSize( screen,
+                                      screen->driver_data, screen->screen_data,
+                                      ret_width, ret_height );
+
+     return ret;
+}
+
