@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -33,9 +33,24 @@
 
 #include <core/coretypes.h>
 
+
 typedef DFBEnumerationResult (*CoreScreenCallback) (CoreScreen *screen,
                                                     void       *ctx);
 
+typedef enum {
+     CMSF_NONE      = 0x00000000,  /* none of these */
+
+     CMSF_DIMENSION = 0x00000001,  /* dimension is set */
+
+     CMSF_ALL       = 0x00000001,  /* all of these */
+} CoreMixerStateFlags;
+
+typedef struct {
+     CoreMixerStateFlags flags;
+
+     DFBDimension        dimension;
+} CoreMixerState;
+
 typedef struct {
    /** Driver Control **/
 
@@ -49,7 +64,7 @@ typedef struct {
       * Driver has to fill the screen description.
       */
      DFBResult (*InitScreen)    ( CoreScreen                   *screen,
-                                  GraphicsDevice               *device,
+                                  CoreGraphicsDevice           *device,
                                   void                         *driver_data,
                                   void                         *screen_data,
                                   DFBScreenDescription         *description );
@@ -187,6 +202,15 @@ typedef struct {
                                     void                        *screen_data,
                                     int                         *ret_width,
                                     int                         *ret_height );
+
+
+   /** States **/
+
+     DFBResult (*GetMixerState)   ( CoreScreen                  *screen,
+                                    void                        *driver_data,
+                                    void                        *screen_data,
+                                    int                          mixer,
+                                    CoreMixerState              *ret_state );
 } ScreenFuncs;
 
 
@@ -194,7 +218,7 @@ typedef struct {
  * Add a screen to a graphics device by pointing to a table containing
  * driver functions. The driver data will be passed to these functions.
  */
-CoreScreen *dfb_screens_register( GraphicsDevice     *device,
+CoreScreen *dfb_screens_register( CoreGraphicsDevice *device,
                                   void               *driver_data,
                                   ScreenFuncs        *funcs );
 
@@ -205,18 +229,18 @@ CoreScreen *dfb_screens_register( GraphicsDevice     *device,
  * The original function table is written to 'primary_funcs' before to allow
  * drivers to use existing functionality from the original implementation.
  */
-CoreScreen *dfb_screens_hook_primary( GraphicsDevice  *device,
-                                      void            *driver_data,
-                                      ScreenFuncs     *funcs,
-                                      ScreenFuncs     *primary_funcs,
-                                      void           **primary_driver_data );
+CoreScreen *dfb_screens_hook_primary( CoreGraphicsDevice  *device,
+                                      void                *driver_data,
+                                      ScreenFuncs         *funcs,
+                                      ScreenFuncs         *primary_funcs,
+                                      void               **primary_driver_data );
 
 /*
  * Replace the default implementation for the primary screen.
  */
-CoreScreen *dfb_screens_register_primary( GraphicsDevice *device,
-                                          void           *driver_data,
-                                          ScreenFuncs    *funcs );
+CoreScreen *dfb_screens_register_primary( CoreGraphicsDevice *device,
+                                          void               *driver_data,
+                                          ScreenFuncs        *funcs );
 
 /*
  * Enumerate all registered screens by invoking the callback for each screen.
