@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -44,7 +44,8 @@ typedef enum {
      CORE_OSX,
      CORE_SDL,
      CORE_VNC,
-     CORE_DEVMEM
+     CORE_DEVMEM,
+     CORE_TI_CMEM
 } CoreSystemType;
 
 typedef enum {
