@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -36,32 +36,60 @@
 
 #include <fusion/object.h>
 
+typedef enum {
+     CWMGT_KEYBOARD,
+     CWMGT_POINTER,
+     CWMGT_KEY,
+     CWMGT_UNSELECTED_KEYS,
+} CoreWMGrabTarget;
 
 typedef enum {
-     CWCF_NONE      = 0x00000000,
+     CWCF_NONE          = 0x00000000,
 
-     CWCF_POSITION  = 0x00000001,
-     CWCF_SIZE      = 0x00000002,
-     CWCF_OPACITY   = 0x00000004,
-     CWCF_STACKING  = 0x00000008,
+     CWCF_POSITION      = 0x00000001,
+     CWCF_SIZE          = 0x00000002,
+     CWCF_OPACITY       = 0x00000004,
+     CWCF_STACKING      = 0x00000008,
 
-     CWCF_OPTIONS   = 0x00000010,
-     CWCF_EVENTS    = 0x00000020,
+     CWCF_OPTIONS       = 0x00000010,
+     CWCF_EVENTS        = 0x00000020,
+     CWCF_ASSOCIATION   = 0x00000040,
 
-     CWCF_COLOR_KEY = 0x00000100,
-     CWCF_OPAQUE    = 0x00000200,
+     CWCF_COLOR_KEY     = 0x00000100,
+     CWCF_OPAQUE        = 0x00000200,
+     CWCF_COLOR         = 0x00000400,
 
-     CWCF_ALL       = 0x0000033F
+     CWCF_KEY_SELECTION = 0x00001000,
+
+     CWCF_SRC_GEOMETRY  = 0x00010000,
+     CWCF_DST_GEOMETRY  = 0x00020000,
+
+     CWCF_APPLICATION_ID= 0x00080000,
+
+     CWCF_ALL           = 0x000B177F
 } CoreWindowConfigFlags;
 
 struct __DFB_CoreWindowConfig {
-     DFBRectangle            bounds;         /* position and size */
-     int                     opacity;        /* global alpha factor */
-     DFBWindowStackingClass  stacking;       /* level boundaries */
-     DFBWindowOptions        options;        /* flags for appearance/behaviour */
-     DFBWindowEventType      events;         /* mask of enabled events */
-     u32                     color_key;      /* transparent pixel */
-     DFBRegion               opaque;         /* region of the window forced to be opaque */
+     DFBRectangle             bounds;         /* position and size */
+     int                      opacity;        /* global alpha factor */
+     DFBWindowStackingClass   stacking;       /* level boundaries */
+     DFBWindowOptions         options;        /* flags for appearance/behaviour */
+     DFBWindowEventType       events;         /* mask of enabled events */
+     DFBColor                 color;          /* color for DWCAPS_COLOR */
+     u32                      color_key;      /* transparent pixel */
+     DFBRegion                opaque;         /* region of the window forced to be opaque */
+
+     DFBWindowKeySelection    key_selection;  /* how to filter keys in focus */
+     DFBInputDeviceKeySymbol *keys;           /* list of keys for DWKS_LIST */
+     unsigned int             num_keys;       /* number of entries in key array */
+
+     DFBWindowGeometry        src_geometry;   /* advanced source geometry */
+     DFBWindowGeometry        dst_geometry;   /* advanced destination geometry */
+
+     DFBWindowID              association;
+
+     unsigned long            application_id; /* this can be changed at runtime by the application.
+                                               * it's here so appman can get a callback on change. */
 };
 
 
@@ -86,15 +114,9 @@ FUSION_OBJECT_METHODS( CoreWindow, dfb_window )
  * creates a window on a given stack
  */
 DFBResult
-dfb_window_create( CoreWindowStack        *stack,
-                   int                     x,
-                   int                     y,
-                   int                     width,
-                   int                     height,
-                   DFBWindowCapabilities   caps,
-                   DFBSurfaceCapabilities  surface_caps,
-                   DFBSurfacePixelFormat   pixelformat,
-                   CoreWindow            **window );
+dfb_window_create( CoreWindowStack             *stack,
+                   const DFBWindowDescription  *desc,
+                   CoreWindow                 **ret_window );
 
 /*
  * deinitializes a window and removes it from the window stack
@@ -129,6 +151,22 @@ dfb_window_resize( CoreWindow   *window,
                    int           width,
                    int           height );
 
+/*
+ * binds a window to this window
+ */
+DFBResult
+dfb_window_bind( CoreWindow *window,
+                 CoreWindow *source,
+                 int         x,
+                 int         y );
+
+/*
+ * unbinds a window from this window
+ */
+DFBResult
+dfb_window_unbind( CoreWindow *window,
+                   CoreWindow *source );
+
 /*
  * changes stacking class
  */
@@ -174,6 +212,13 @@ DFBResult
 dfb_window_putbelow( CoreWindow *window,
                      CoreWindow *upper );
 
+/*
+ * sets the source color key
+ */
+DFBResult
+dfb_window_set_color( CoreWindow *window,
+                      DFBColor    color );
+
 /*
  * sets the source color key
  */
@@ -181,6 +226,14 @@ DFBResult
 dfb_window_set_colorkey( CoreWindow *window,
                          u32         color_key );
 
+/*
+ * change window configuration
+ */
+DFBResult
+dfb_window_set_config( CoreWindow             *window,
+                       const CoreWindowConfig *config,
+                       CoreWindowConfigFlags   flags );
+
 /*
  * sets the global alpha factor
  */
@@ -225,10 +278,14 @@ dfb_window_repaint( CoreWindow          *window,
 DFBResult
 dfb_window_request_focus( CoreWindow *window );
 
-DFBResult dfb_window_grab_keyboard  ( CoreWindow                 *window );
-DFBResult dfb_window_ungrab_keyboard( CoreWindow                 *window );
-DFBResult dfb_window_grab_pointer   ( CoreWindow                 *window );
-DFBResult dfb_window_ungrab_pointer ( CoreWindow                 *window );
+DFBResult dfb_window_set_key_selection( CoreWindow                    *window,
+                                        DFBWindowKeySelection          selection,
+                                        const DFBInputDeviceKeySymbol *keys,
+                                        unsigned int                   num_keys );
+
+DFBResult dfb_window_change_grab    ( CoreWindow                 *window,
+                                      CoreWMGrabTarget            target,
+                                      bool                        grab );
 DFBResult dfb_window_grab_key       ( CoreWindow                 *window,
                                       DFBInputDeviceKeySymbol     symbol,
                                       DFBInputDeviceModifierMask  modifiers );
