@@ -100,7 +100,7 @@ struct __DFB_CoreWindow {
 
      CoreWindow             *boundto;        /* window to which this window is bound */
 
-     DFBWindowID             parent_id;      /* window id of parent window */
+     unsigned long           resource_id;
 };
 
 typedef enum {
