@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -42,7 +42,7 @@
 #include <core/input.h>
 #include <core/layer_context.h>
 #include <core/layers_internal.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/windows_internal.h>
 #include <core/windowstack.h>
 #include <core/wm.h>
@@ -97,6 +97,7 @@ static DFBEnumerationResult stack_attach_devices( CoreInputDevice *device,
 CoreWindowStack*
 dfb_windowstack_create( CoreLayerContext *context )
 {
+     DFBResult          ret;
      CoreWindowStack   *stack;
      CoreSurfacePolicy  policy = CSP_SYSTEMONLY;
 
@@ -106,8 +107,10 @@ dfb_windowstack_create( CoreLayerContext *context )
 
      /* Allocate window stack data (completely shared) */
      stack = (CoreWindowStack*) SHCALLOC( context->shmpool, 1, sizeof(CoreWindowStack) );
-     if (!stack)
+     if (!stack) {
+          D_OOSHM();
           return NULL;
+     }
 
      stack->shmpool = context->shmpool;
 
@@ -138,9 +141,18 @@ dfb_windowstack_create( CoreLayerContext *context )
      stack->cursor.policy = policy;
 
      /* Set default background mode. */
-     stack->bg.mode = DLBM_COLOR;
+     stack->bg.mode        = DLBM_DONTCARE;
+     stack->bg.color_index = -1;
 
-     dfb_wm_init_stack( stack );
+     D_MAGIC_SET( stack, CoreWindowStack );
+
+     /* Initialize window manager */
+     ret = dfb_wm_init_stack( stack );
+     if (ret) {
+          D_MAGIC_CLEAR( stack );
+          SHFREE( context->shmpool, stack );
+          return NULL;
+     }
 
      /* Attach to all input devices */
      dfb_input_enumerate_devices( stack_attach_devices, stack, DICAPS_ALL );
@@ -151,13 +163,13 @@ dfb_windowstack_create( CoreLayerContext *context )
 }
 
 void
-dfb_windowstack_destroy( CoreWindowStack *stack )
+dfb_windowstack_detach_devices( CoreWindowStack *stack )
 {
      DirectLink *l;
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p )\n", __FUNCTION__, stack );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Detach all input devices. */
      l = stack->devices;
@@ -172,12 +184,22 @@ dfb_windowstack_destroy( CoreWindowStack *stack )
 
           l = next;
      }
+}
+
+void
+dfb_windowstack_destroy( CoreWindowStack *stack )
+{
+     D_DEBUG_AT( Core_WindowStack, "%s( %p )\n", __FUNCTION__, stack );
+
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Unlink cursor surface. */
      if (stack->cursor.surface)
           dfb_surface_unlink( &stack->cursor.surface );
 
-     dfb_wm_close_stack( stack, true );
+     /* Shutdown window manager? */
+     if (stack->flags & CWSF_INITIALIZED)
+          dfb_wm_close_stack( stack );
 
      /* detach listener from background surface and unlink it */
      if (stack->bg.image) {
@@ -187,6 +209,14 @@ dfb_windowstack_destroy( CoreWindowStack *stack )
           dfb_surface_unlink( &stack->bg.image );
      }
 
+     /* Deallocate shared stack data. */
+     if (stack->stack_data) {
+          SHFREE( stack->shmpool, stack->stack_data );
+          stack->stack_data = NULL;
+     }
+
+     D_MAGIC_CLEAR( stack );
+
      /* Free stack data. */
      SHFREE( stack->shmpool, stack );
 }
@@ -198,7 +228,7 @@ dfb_windowstack_resize( CoreWindowStack *stack,
 {
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %dx%d )\n", __FUNCTION__, stack, width, height );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -228,7 +258,7 @@ dfb_windowstack_resize( CoreWindowStack *stack,
 DirectResult
 dfb_windowstack_lock( CoreWindowStack *stack )
 {
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->context != NULL );
 
      return dfb_layer_context_lock( stack->context );
@@ -240,7 +270,7 @@ dfb_windowstack_lock( CoreWindowStack *stack )
 DirectResult
 dfb_windowstack_unlock( CoreWindowStack *stack )
 {
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->context != NULL );
 
      return dfb_layer_context_unlock( stack->context );
@@ -254,7 +284,7 @@ dfb_windowstack_repaint_all( CoreWindowStack *stack )
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p )\n", __FUNCTION__, stack );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -285,7 +315,7 @@ dfb_windowstack_set_background_mode ( CoreWindowStack               *stack,
 {
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %d )\n", __FUNCTION__, stack, mode );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -319,9 +349,12 @@ dfb_windowstack_set_background_image( CoreWindowStack *stack,
 {
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %p )\n", __FUNCTION__, stack, image );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( image != NULL );
 
+     if (!(image->type & CSTF_SHARED))
+          return DFB_INVARG;
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -357,14 +390,15 @@ dfb_windowstack_set_background_image( CoreWindowStack *stack,
 
 DFBResult
 dfb_windowstack_set_background_color( CoreWindowStack *stack,
-                                      DFBColor        *color )
+                                      const DFBColor  *color )
 {
-     D_ASSERT( stack != NULL );
      D_ASSERT( color != NULL );
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p, 0x%08x )\n", __FUNCTION__, stack,
                  PIXEL_ARGB( color->a, color->r, color->g, color->b ) );
 
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -385,6 +419,34 @@ dfb_windowstack_set_background_color( CoreWindowStack *stack,
      return DFB_OK;
 }
 
+DFBResult
+dfb_windowstack_set_background_color_index( CoreWindowStack *stack,
+                                            int              index )
+{
+     D_DEBUG_AT( Core_WindowStack, "%s( %p, %d )\n", __FUNCTION__, stack, index );
+
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
+
+     /* Lock the window stack. */
+     if (dfb_windowstack_lock( stack ))
+          return DFB_FUSION;
+
+     /* do nothing if color didn't change */
+     if (stack->bg.color_index != index) {
+          /* set new color index */
+          stack->bg.color_index = index;
+
+          /* force an update of the window stack */
+          if (stack->bg.mode == DLBM_COLOR)
+               dfb_windowstack_repaint_all( stack );
+     }
+
+     /* Unlock the window stack. */
+     dfb_windowstack_unlock( stack );
+
+     return DFB_OK;
+}
+
 /**********************************************************************************************************************/
 
 /*
@@ -398,7 +460,7 @@ dfb_windowstack_cursor_enable( CoreDFB *core, CoreWindowStack *stack, bool enabl
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %s )\n", __FUNCTION__, stack, enable ? "enable" : "disable" );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -436,7 +498,7 @@ dfb_windowstack_cursor_set_opacity( CoreWindowStack *stack, u8 opacity )
 {
      D_DEBUG_AT( Core_WindowStack, "%s( %p, 0x%02x )\n", __FUNCTION__, stack, opacity );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -468,9 +530,10 @@ dfb_windowstack_cursor_set_shape( CoreWindowStack *stack,
      CoreCursorUpdateFlags  flags = CCUF_SHAPE;
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %p, hot %d, %d ) <- size %dx%d\n",
-                 __FUNCTION__, stack, shape, hot_x, hot_y, shape->width, shape->height );
+                 __FUNCTION__, stack, shape, hot_x, hot_y,
+                 shape->config.size.w, shape->config.size.h );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( shape != NULL );
 
      if (dfb_config->no_cursor)
@@ -485,7 +548,7 @@ dfb_windowstack_cursor_set_shape( CoreWindowStack *stack,
           D_ASSUME( !stack->cursor.enabled );
 
           /* Create the a surface for the shape. */
-          ret = create_cursor_surface( stack, shape->width, shape->height );
+          ret = create_cursor_surface( stack, shape->config.size.w, shape->config.size.h );
           if (ret) {
                dfb_windowstack_unlock( stack );
                return ret;
@@ -493,11 +556,11 @@ dfb_windowstack_cursor_set_shape( CoreWindowStack *stack,
 
           cursor = stack->cursor.surface;
      }
-     else if (stack->cursor.size.w != shape->width || stack->cursor.size.h != shape->height) {
-          dfb_surface_reformat( NULL, cursor, shape->width, shape->height, DSPF_ARGB );
+     else if (stack->cursor.size.w != shape->config.size.w || stack->cursor.size.h != shape->config.size.h) {
+          dfb_surface_reformat( cursor, shape->config.size.w, shape->config.size.h, DSPF_ARGB );
 
-          stack->cursor.size.w = shape->width;
-          stack->cursor.size.h = shape->height;
+          stack->cursor.size.w = shape->config.size.w;
+          stack->cursor.size.h = shape->config.size.h;
 
           /* Notify about new size. */
           flags |= CCUF_SIZE;
@@ -514,7 +577,7 @@ dfb_windowstack_cursor_set_shape( CoreWindowStack *stack,
      /* Copy the content of the new shape. */
      dfb_gfx_copy( shape, cursor, NULL );
 
-     cursor->caps = ((cursor->caps & ~DSCAPS_PREMULTIPLIED) | (shape->caps & DSCAPS_PREMULTIPLIED));
+     cursor->config.caps = ((cursor->config.caps & ~DSCAPS_PREMULTIPLIED) | (shape->config.caps & DSCAPS_PREMULTIPLIED));
 
      /* Notify the WM. */
      if (stack->cursor.enabled)
@@ -531,7 +594,7 @@ dfb_windowstack_cursor_warp( CoreWindowStack *stack, int x, int y )
 {
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %d, %d )\n", __FUNCTION__, stack, x, y );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -571,7 +634,7 @@ dfb_windowstack_cursor_set_acceleration( CoreWindowStack *stack,
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %d, %d, %d )\n",
                  __FUNCTION__, stack, numerator, denominator, threshold );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -592,7 +655,7 @@ dfb_windowstack_get_cursor_position( CoreWindowStack *stack, int *ret_x, int *re
 {
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %p, %p )\n", __FUNCTION__, stack, ret_x, ret_y );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSUME( ret_x != NULL || ret_y != NULL );
 
      /* Lock the window stack. */
@@ -623,7 +686,7 @@ _dfb_windowstack_inputdevice_listener( const void *msg_data,
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %p )\n", __FUNCTION__, msg_data, ctx );
 
      D_ASSERT( msg_data != NULL );
-     D_ASSERT( ctx != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
@@ -652,7 +715,7 @@ _dfb_windowstack_background_image_listener( const void *msg_data,
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %p )\n", __FUNCTION__, msg_data, ctx );
 
      D_ASSERT( notification != NULL );
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      if (notification->flags & CSNF_DESTROY) {
           if (stack->bg.image == notification->surface) {
@@ -686,7 +749,7 @@ stack_attach_devices( CoreInputDevice *device,
      StackDevice     *dev;
      CoreWindowStack *stack = (CoreWindowStack*) ctx;
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      dev = SHCALLOC( stack->shmpool, 1, sizeof(StackDevice) );
      if (!dev) {
@@ -711,15 +774,15 @@ stack_attach_devices( CoreInputDevice *device,
 static DFBResult
 load_default_cursor( CoreDFB *core, CoreWindowStack *stack )
 {
-     DFBResult ret;
-     int       i;
-     int       pitch;
-     void     *data;
-     FILE     *f;
+     DFBResult              ret;
+     int                    i;
+     FILE                  *f;
+     CoreSurfaceBufferLock  lock;
+     void                  *data;
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p )\n", __FUNCTION__, stack );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
 
      if (!stack->cursor.surface) {
           ret = create_cursor_surface( stack, 40, 40 );
@@ -733,15 +796,17 @@ load_default_cursor( CoreDFB *core, CoreWindowStack *stack )
           stack->cursor.size.h = 40;
      }
 
-     /* lock the surface of the window */
-     ret = dfb_surface_soft_lock( core, stack->cursor.surface, DSLF_WRITE, &data, &pitch, false );
+     /* lock the cursor surface */
+     ret = dfb_surface_lock_buffer( stack->cursor.surface, CSBR_BACK, CSAF_CPU_WRITE, &lock );
      if (ret) {
-          D_ERROR( "Core/WindowStack: cannot lock the surface for cursor window data!\n" );
+          D_ERROR( "Core/WindowStack: cannot lock the cursor surface!\n" );
           return ret;
      }
 
+     data = lock.addr;
+
      /* initialize as empty cursor */
-     memset( data, 0, 40 * pitch );
+     memset( data, 0, 40 * lock.pitch );
 
      /* open the file containing the cursors image data */
      f = fopen( CURSORFILE, "rb" );
@@ -759,7 +824,7 @@ load_default_cursor( CoreDFB *core, CoreWindowStack *stack )
 
      /* read from file directly into the cursor window surface */
      for (i=0; i<40; i++) {
-          if (fread( data, MIN (40*4, pitch), 1, f ) != 1) {
+          if (fread( data, MIN (40*4, lock.pitch), 1, f ) != 1) {
                ret = errno2result( errno );
 
                D_ERROR( "Core/WindowStack: unexpected end or read error of cursor data!\n" );
@@ -768,7 +833,7 @@ load_default_cursor( CoreDFB *core, CoreWindowStack *stack )
           }
 #ifdef WORDS_BIGENDIAN
           {
-               int i = MIN (40, pitch/4);
+               int i = MIN (40, lock.pitch/4);
                u32 *tmp_data = data;
 
                while (i--) {
@@ -780,14 +845,14 @@ load_default_cursor( CoreDFB *core, CoreWindowStack *stack )
                }
           }
 #endif
-          data += pitch;
+          data += lock.pitch;
      }
 
 finish:
      if (f)
           fclose( f );
 
-     dfb_surface_unlock( stack->cursor.surface, false );
+     dfb_surface_unlock_buffer( stack->cursor.surface, &lock );
 
      return ret;
 }
@@ -797,14 +862,15 @@ create_cursor_surface( CoreWindowStack *stack,
                        int              width,
                        int              height )
 {
-     DFBResult          ret;
-     CoreSurface       *surface;
-     CoreLayer         *layer;
-     CoreLayerContext  *context;
+     DFBResult               ret;
+     CoreSurface            *surface;
+     CoreLayer              *layer;
+     CoreLayerContext       *context;
+     DFBSurfaceCapabilities  surface_caps = DSCAPS_NONE;
 
      D_DEBUG_AT( Core_WindowStack, "%s( %p, %dx%d )\n", __FUNCTION__, stack, width, height );
 
-     D_ASSERT( stack != NULL );
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->cursor.surface == NULL );
 
      context = stack->context;
@@ -815,8 +881,8 @@ create_cursor_surface( CoreWindowStack *stack,
 
      D_ASSERT( layer != NULL );
 
-     stack->cursor.x   = stack->width  / 2;
-     stack->cursor.y   = stack->height / 2;
+     stack->cursor.x       = stack->width  / 2;
+     stack->cursor.y       = stack->height / 2;
      stack->cursor.hot.x   = 0;
      stack->cursor.hot.y   = 0;
      stack->cursor.size.w  = width;
@@ -826,9 +892,13 @@ create_cursor_surface( CoreWindowStack *stack,
      if (context->config.buffermode == DLBM_WINDOWS)
           D_WARN( "cursor not yet visible with DLBM_WINDOWS" );
 
+     dfb_surface_caps_apply_policy( stack->cursor.policy, &surface_caps );
+
      /* Create the cursor surface. */
-     ret = dfb_surface_create( layer->core, width, height, DSPF_ARGB,
-                               stack->cursor.policy, DSCAPS_NONE, NULL, &surface );
+     ret = dfb_surface_create_simple( layer->core, width, height, DSPF_ARGB,
+                                      surface_caps, CSTF_SHARED | CSTF_CURSOR,
+                                      0, /* FIXME: no shared cursor objects, no cursor id */
+                                      NULL, &surface );
      if (ret) {
           D_ERROR( "Core/WindowStack: Failed creating a surface for software cursor!\n" );
           return ret;
