@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -39,6 +39,8 @@
  */
 CoreWindowStack *dfb_windowstack_create ( CoreLayerContext *context );
 
+void             dfb_windowstack_detach_devices( CoreWindowStack  *stack );
+
 void             dfb_windowstack_destroy( CoreWindowStack  *stack );
 
 void             dfb_windowstack_resize ( CoreWindowStack  *stack,
@@ -61,10 +63,13 @@ DFBResult dfb_windowstack_set_background_mode ( CoreWindowStack               *s
                                                 DFBDisplayLayerBackgroundMode  mode );
 
 DFBResult dfb_windowstack_set_background_image( CoreWindowStack               *stack,
-                                                CoreSurface                   *image);
+                                                CoreSurface                   *image );
 
 DFBResult dfb_windowstack_set_background_color( CoreWindowStack               *stack,
-                                                DFBColor                      *color);
+                                                const DFBColor                *color );
+
+DFBResult dfb_windowstack_set_background_color_index( CoreWindowStack         *stack,
+                                                      int                      index );
 
 
 /*
