@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -43,7 +43,7 @@ DECLARE_MODULE_DIRECTORY( dfb_core_wm_modules );
 /*
  * Increase this number when changes result in binary incompatibility!
  */
-#define DFB_CORE_WM_ABI_VERSION           8
+#define DFB_CORE_WM_ABI_VERSION           9
 
 #define DFB_CORE_WM_INFO_NAME_LENGTH     60
 #define DFB_CORE_WM_INFO_VENDOR_LENGTH   80
@@ -72,12 +72,6 @@ typedef struct {
      unsigned int   window_data_size;
 } CoreWMInfo;
 
-typedef enum {
-     CWMGT_KEYBOARD,
-     CWMGT_POINTER,
-     CWMGT_KEY
-} CoreWMGrabTarget;
-
 typedef struct {
      CoreWMGrabTarget            target;
 
@@ -132,6 +126,9 @@ typedef struct {
      DFBResult (*Resume)            ( void                   *wm_data,
                                       void                   *shared_data );
 
+     DFBResult (*PostInit)          ( void                   *wm_data,
+                                      void                   *shared_data );
+
 
    /** Stack **/
 
@@ -182,8 +179,6 @@ typedef struct {
                                       CoreWMWindowCallback    callback,
                                       void                   *callback_ctx );
 
-     DFBResult (*StartDesktop)      ( CoreWindowStack        *stack );
-
 
    /** Window **/
     DFBResult (*SetWindowProperty)( CoreWindowStack  *stack,
@@ -284,12 +279,12 @@ typedef struct {
 
 void dfb_wm_get_info( CoreWMInfo *info );
 
+DFBResult dfb_wm_post_init          ( CoreDFB                *core );
 
 DFBResult dfb_wm_init_stack         ( CoreWindowStack        *stack );
                                     
-DFBResult dfb_wm_close_stack        ( CoreWindowStack        *stack,
-                                      bool                    final );
-                                    
+DFBResult dfb_wm_close_stack        ( CoreWindowStack        *stack );
+
 DFBResult dfb_wm_set_active         ( CoreWindowStack        *stack,
                                       bool                    active );
                                     
@@ -315,8 +310,6 @@ DFBResult dfb_wm_enum_windows       ( CoreWindowStack        *stack,
                                       CoreWMWindowCallback    callback,
                                       void                   *callback_ctx );
                                     
-DFBResult dfb_wm_start_desktop      ( CoreWindowStack        *stack);
-
 DFBResult dfb_wm_get_insets         ( CoreWindowStack        *stack,
                                       CoreWindow             *window,
                                       DFBInsets              *insets );
