@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -40,9 +40,10 @@
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/gfxcard.h>
 #include <core/layers.h>
+#include <core/layers_internal.h>
 #include <core/layer_context.h>
 #include <core/layer_control.h>
 #include <core/layer_region.h>
@@ -59,10 +60,10 @@
 #include <direct/mem.h>
 #include <direct/messages.h>
 
-#include "idirectfbdisplaylayer.h"
-#include "idirectfbscreen.h"
-#include "idirectfbsurface.h"
-#include "idirectfbsurface_layer.h"
+#include <display/idirectfbdisplaylayer.h>
+#include <display/idirectfbscreen.h>
+#include <display/idirectfbsurface.h>
+#include <display/idirectfbsurface_layer.h>
 
 
 D_DEBUG_DOMAIN( Layer, "IDirectFBDisplayLayer", "Display Layer Interface" );
@@ -105,7 +106,7 @@ IDirectFBDisplayLayer_Destruct( IDirectFBDisplayLayer *thiz )
      D_DEBUG_AT( Layer, "  -> done.\n" );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBDisplayLayer_AddRef( IDirectFBDisplayLayer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
@@ -115,7 +116,7 @@ IDirectFBDisplayLayer_AddRef( IDirectFBDisplayLayer *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBDisplayLayer_Release( IDirectFBDisplayLayer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
@@ -179,9 +180,14 @@ IDirectFBDisplayLayer_GetSurface( IDirectFBDisplayLayer  *thiz,
 
      DIRECT_ALLOCATE_INTERFACE( surface, IDirectFBSurface );
 
-     ret = IDirectFBSurface_Layer_Construct( surface, NULL, NULL,
+     ret = IDirectFBSurface_Layer_Construct( surface, NULL, NULL, NULL,
                                              region, DSCAPS_NONE, data->core );
 
+     if (region->config.buffermode == DLBM_FRONTONLY && data->level == DLSCL_EXCLUSIVE) {
+          surface->Clear( surface, 0, 0, 0, 0 );
+          dfb_layer_region_flip_update( region, NULL, DSFLIP_NONE );
+     }
+
      *interface = ret ? NULL : surface;
 
      dfb_layer_region_unref( region );
@@ -428,7 +434,7 @@ IDirectFBDisplayLayer_SetSrcColorKey( IDirectFBDisplayLayer *thiz,
      if (data->level == DLSCL_SHARED)
           return DFB_ACCESSDENIED;
 
-     return dfb_layer_context_set_src_colorkey( data->context, r, g, b );
+     return dfb_layer_context_set_src_colorkey( data->context, r, g, b, -1 );
 }
 
 static DFBResult
@@ -442,7 +448,7 @@ IDirectFBDisplayLayer_SetDstColorKey( IDirectFBDisplayLayer *thiz,
      if (data->level == DLSCL_SHARED)
           return DFB_ACCESSDENIED;
 
-     return dfb_layer_context_set_dst_colorkey( data->context, r, g, b );
+     return dfb_layer_context_set_dst_colorkey( data->context, r, g, b, -1 );
 }
 
 static DFBResult
@@ -603,43 +609,61 @@ IDirectFBDisplayLayer_CreateWindow( IDirectFBDisplayLayer       *thiz,
                                     const DFBWindowDescription  *desc,
                                     IDirectFBWindow            **window )
 {
-     CoreWindow            *w;
-     DFBResult              ret;
-     unsigned int           width        = 128;
-     unsigned int           height       = 128;
-     int                    posx         = 0;
-     int                    posy         = 0;
-     DFBWindowCapabilities  caps         = 0;
-     DFBSurfaceCapabilities surface_caps = DSCAPS_NONE;
-     DFBSurfacePixelFormat  format       = DSPF_UNKNOWN;
+     CoreWindow           *w;
+     DFBResult             ret;
+     DFBWindowDescription  wd;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
 
+     memset( &wd, 0, sizeof(wd) );
+
+     wd.flags = DWDESC_WIDTH | DWDESC_HEIGHT | DWDESC_POSX | DWDESC_POSY |
+                DWDESC_PIXELFORMAT | DWDESC_SURFACE_CAPS | DWDESC_CAPS;
+
+     wd.width  = (desc->flags & DWDESC_WIDTH)  ? desc->width  : 480;
+     wd.height = (desc->flags & DWDESC_HEIGHT) ? desc->height : 300;
+     wd.posx   = (desc->flags & DWDESC_POSX)   ? desc->posx   : 100;
+     wd.posy   = (desc->flags & DWDESC_POSY)   ? desc->posy   : 100;
+
+     D_DEBUG_AT( Layer, "CreateWindow() <- %d,%d - %dx%d )\n", wd.posx, wd.posy, wd.width, wd.height );
 
-     if (desc->flags & DWDESC_WIDTH)
-          width = desc->width;
-     if (desc->flags & DWDESC_HEIGHT)
-          height = desc->height;
-     if (desc->flags & DWDESC_PIXELFORMAT)
-          format = desc->pixelformat;
-     if (desc->flags & DWDESC_POSX)
-          posx = desc->posx;
-     if (desc->flags & DWDESC_POSY)
-          posy = desc->posy;
      if (desc->flags & DWDESC_CAPS)
-          caps = desc->caps;
+          wd.caps = desc->caps;
+
+     if (desc->flags & DWDESC_PIXELFORMAT)
+          wd.pixelformat = desc->pixelformat;
+
      if (desc->flags & DWDESC_SURFACE_CAPS)
-          surface_caps = desc->surface_caps;
+          wd.surface_caps = desc->surface_caps;
 
-     if ((caps & ~DWCAPS_ALL) || !window)
+     if (desc->flags & DWDESC_PARENT) {
+          wd.flags     |= DWDESC_PARENT;
+          wd.parent_id  = desc->parent_id;
+     }
+
+     if (desc->flags & DWDESC_OPTIONS) {
+          wd.flags   |= DWDESC_OPTIONS;
+          wd.options  = desc->options;
+     }
+
+     if (desc->flags & DWDESC_STACKING) {
+          wd.flags    |= DWDESC_STACKING;
+          wd.stacking  = desc->stacking;
+     }
+
+     if (desc->flags & DWDESC_RESOURCE_ID) {
+          wd.flags       |= DWDESC_RESOURCE_ID;
+          wd.resource_id  = desc->resource_id;
+     }
+
+
+     if ((wd.caps & ~DWCAPS_ALL) || !window)
           return DFB_INVARG;
 
-     if (width < 1 || width > 4096 || height < 1 || height > 4096)
+     if (wd.width < 1 || wd.width > 4096 || wd.height < 1 || wd.height > 4096)
           return DFB_INVARG;
 
-     ret = dfb_layer_context_create_window( data->core, data->context,
-                                            posx, posy, width, height, caps,
-                                            surface_caps, format, &w );
+     ret = dfb_layer_context_create_window( data->core, data->context, &wd, &w );
      if (ret)
           return ret;
 
@@ -745,8 +769,8 @@ IDirectFBDisplayLayer_SetCursorShape( IDirectFBDisplayLayer *thiz,
 
      if (hot_x < 0  ||
          hot_y < 0  ||
-         hot_x >= shape_data->surface->width  ||
-         hot_y >= shape_data->surface->height)
+         hot_x >= shape_data->surface->config.size.w  ||
+         hot_y >= shape_data->surface->config.size.h)
           return DFB_INVARG;
 
      return dfb_windowstack_cursor_set_shape( data->stack,
@@ -850,6 +874,18 @@ IDirectFBDisplayLayer_SwitchContext( IDirectFBDisplayLayer *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBDisplayLayer_SetRotation( IDirectFBDisplayLayer *thiz,
+                                   int                    rotation )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer)
+
+     if (data->level == DLSCL_SHARED)
+          return DFB_ACCESSDENIED;
+
+     return dfb_layer_context_set_rotation( data->context, rotation );
+}
+
 DFBResult
 IDirectFBDisplayLayer_Construct( IDirectFBDisplayLayer *thiz,
                                  CoreLayer             *layer,
@@ -923,6 +959,7 @@ IDirectFBDisplayLayer_Construct( IDirectFBDisplayLayer *thiz,
      thiz->SetScreenPosition     = IDirectFBDisplayLayer_SetScreenPosition;
      thiz->SetScreenRectangle    = IDirectFBDisplayLayer_SetScreenRectangle;
      thiz->SwitchContext         = IDirectFBDisplayLayer_SwitchContext;
+     thiz->SetRotation           = IDirectFBDisplayLayer_SetRotation;
 
      return DFB_OK;
 }
