@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -89,7 +89,7 @@ IDirectFBScreen_Destruct( IDirectFBScreen *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBScreen_AddRef( IDirectFBScreen *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBScreen)
@@ -99,7 +99,7 @@ IDirectFBScreen_AddRef( IDirectFBScreen *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBScreen_Release( IDirectFBScreen *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBScreen)
@@ -604,9 +604,51 @@ PatchEncoderConfig( DFBScreenEncoderConfig       *patched,
      if (patch->flags & ~patched->flags)
           return DFB_UNSUPPORTED;
 
-     if (patch->flags & DSECONF_TV_STANDARD)
-          patched->tv_standard = patch->tv_standard;
+     if (patch->flags & DSECONF_RESOLUTION)
+          patched->resolution = patch->resolution;
 
+     if (patch->flags & DSECONF_FREQUENCY)
+          patched->frequency = patch->frequency;
+
+     /**
+      * Need to be backwards compatible with these so that
+      * they specify resolution and frequency as well.
+      * If you have set a TV_STANDARD
+      * (and set the flag) it will override the resolution and
+      * frequency chosen above.*/
+     if (patch->flags & DSECONF_TV_STANDARD) {
+          patched->tv_standard = patch->tv_standard;
+          switch (patched->tv_standard) {
+               case DSETV_PAL:
+               case DSETV_PAL_BG:
+               case DSETV_PAL_I:
+               case DSETV_PAL_N:
+               case DSETV_PAL_NC:
+                    patched->resolution = DSOR_720_576;
+                    patched->frequency = DSEF_50HZ;
+                    break;
+
+               case DSETV_PAL_60:
+               case DSETV_PAL_M:
+                    patched->resolution = DSOR_720_480;
+                    patched->frequency = DSEF_60HZ;
+                    break;
+
+               case DSETV_SECAM:
+                    patched->resolution = DSOR_720_576;
+                    patched->frequency = DSEF_50HZ;
+                    break;
+
+               case DSETV_NTSC:
+               case DSETV_NTSC_M_JPN:
+                    patched->resolution = DSOR_720_480;
+                    patched->frequency = DSEF_60HZ;
+                    break;
+
+               default:
+                    break;
+          }
+     }
      if (patch->flags & DSECONF_TEST_PICTURE)
           patched->test_picture = patch->test_picture;
 
@@ -639,6 +681,9 @@ PatchOutputConfig( DFBScreenOutputConfig       *patched,
      if (patch->flags & ~patched->flags)
           return DFB_UNSUPPORTED;
 
+     if (patch->flags & DSOCONF_RESOLUTION)
+          patched->resolution = patch->resolution;
+
      if (patch->flags & DSOCONF_ENCODER)
           patched->encoder = patch->encoder;
 
