@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -34,28 +34,29 @@
 
 #include <string.h>
 
-#include "directfb.h"
+#include <directfb.h>
 
-#include "core/core.h"
-#include "core/coretypes.h"
+#include <core/core.h>
+#include <core/coretypes.h>
 
-#include "core/fonts.h"
-#include "core/gfxcard.h"
-#include "core/layer_region.h"
-#include "core/state.h"
-#include "core/surfaces.h"
-#include "core/windows.h"
-#include "core/wm.h"
-#include "core/windows_internal.h" /* FIXME */
+#include <core/fonts.h>
+#include <core/gfxcard.h>
+#include <core/layer_context.h>
+#include <core/layer_region.h>
+#include <core/state.h>
+#include <core/surface.h>
+#include <core/windows.h>
+#include <core/windows_internal.h> /* FIXME */
+#include <core/wm.h>
 
-#include "idirectfbsurface.h"
-#include "idirectfbsurface_window.h"
+#include <display/idirectfbsurface.h>
+#include <display/idirectfbsurface_window.h>
 
 #include <direct/interface.h>
 #include <direct/mem.h>
-#include "misc/util.h"
+#include <misc/util.h>
 
-#include "gfx/util.h"
+#include <gfx/util.h>
 
 
 D_DEBUG_DOMAIN( Surface, "IDirectFBSurfaceW", "IDirectFBSurface_Window Interface" );
@@ -103,7 +104,7 @@ IDirectFBSurface_Window_Destruct( IDirectFBSurface *thiz )
      IDirectFBSurface_Destruct( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBSurface_Window_Release( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Window)
@@ -121,6 +122,7 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
                               const DFBRegion     *region,
                               DFBSurfaceFlipFlags  flags )
 {
+     DFBResult ret;
      DFBRegion reg;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Window)
@@ -138,6 +140,21 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
           return DFB_INVAREA;
 
 
+     IDirectFBSurface_StopAll( &data->base );
+
+     if (data->base.parent) {
+          IDirectFBSurface_data *parent_data;
+
+          DIRECT_INTERFACE_GET_DATA_FROM( data->base.parent, parent_data, IDirectFBSurface );
+
+          /* Signal end of sequence of operations. */
+          dfb_state_lock( &parent_data->state );
+          dfb_state_stop_drawing( &parent_data->state );
+          dfb_state_unlock( &parent_data->state );
+     }
+
+
+
      dfb_region_from_rectangle( &reg, &data->base.area.current );
 
      if (region) {
@@ -149,7 +166,7 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
                return DFB_INVAREA;
      }
 
-     D_DEBUG_AT( Surface, "  -> %d, %d - %dx%d\n", DFB_RECTANGLE_VALS_FROM_REGION( &reg ) );
+     D_DEBUG_AT( Surface, "  -> FLIP %4d,%4d-%4dx%4d\n", DFB_RECTANGLE_VALS_FROM_REGION( &reg ) );
 
 
      if (flags & DSFLIP_PIPELINE) {
@@ -165,11 +182,19 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
           dfb_layer_region_flip_update( data->window->region, &reg, flags );
      }
      else {
-          if (data->base.surface->caps & DSCAPS_FLIPPING) {
+          if (data->base.surface->config.caps & DSCAPS_FLIPPING) {
                if (!(flags & DSFLIP_BLIT) && reg.x1 == 0 && reg.y1 == 0 &&
-                   reg.x2 == data->base.surface->width  - 1 &&
-                   reg.y2 == data->base.surface->height - 1)
-                    dfb_surface_flip_buffers( data->base.surface, false );
+                   reg.x2 == data->base.surface->config.size.w  - 1 &&
+                   reg.y2 == data->base.surface->config.size.h - 1)
+               {
+                    ret = dfb_surface_lock( data->base.surface );
+                    if (ret)
+                         return ret;
+
+                    dfb_surface_flip( data->base.surface, false );
+
+                    dfb_surface_unlock( data->base.surface );
+               }
                else
                     dfb_back_to_front_copy( data->base.surface, &reg );
           }
@@ -229,13 +254,13 @@ IDirectFBSurface_Window_GetSubSurface( IDirectFBSurface    *thiz,
           dfb_rectangle_intersect( &granted, &data->base.area.granted );
           
           /* Construct */
-          ret = IDirectFBSurface_Window_Construct( *surface, &wanted, &granted,
+          ret = IDirectFBSurface_Window_Construct( *surface, thiz, &wanted, &granted,
                                                    data->window, data->base.caps |
                                                    DSCAPS_SUBSURFACE, data->base.core );
      }
      else {
           /* Construct */
-          ret = IDirectFBSurface_Window_Construct( *surface, NULL, NULL,
+          ret = IDirectFBSurface_Window_Construct( *surface, thiz, NULL, NULL,
                                                    data->window, data->base.caps |
                                                    DSCAPS_SUBSURFACE, data->base.core );
      }
@@ -245,22 +270,31 @@ IDirectFBSurface_Window_GetSubSurface( IDirectFBSurface    *thiz,
 
 DFBResult
 IDirectFBSurface_Window_Construct( IDirectFBSurface       *thiz,
+                                   IDirectFBSurface       *parent,
                                    DFBRectangle           *wanted,
                                    DFBRectangle           *granted,
                                    CoreWindow             *window,
                                    DFBSurfaceCapabilities  caps,
                                    CoreDFB                *core )
 {
-     DFBResult ret;
-     DFBInsets insets;
+     DFBResult        ret;
+     DFBInsets        insets;
+     CoreWindowStack *stack;
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBSurface_Window)
 
      D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     stack = window->stack;
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
+
+     dfb_layer_context_lock( stack->context );
+
+     dfb_wm_get_insets( stack, window, &insets );
      
-     dfb_wm_get_insets( window->stack, window, &insets );
-     
-     ret = IDirectFBSurface_Construct( thiz, wanted, granted, &insets,
+     dfb_layer_context_unlock( stack->context );
+
+     ret = IDirectFBSurface_Construct( thiz, parent, wanted, granted, &insets,
                                        window->surface, caps, core );
      if (ret)
           return ret;
@@ -278,8 +312,12 @@ IDirectFBSurface_Window_Construct( IDirectFBSurface       *thiz,
       * requested a (primary) surface that doesn't need to be flipped.
       * Window surfaces even need to be flipped when they are single buffered.
       */
-     if (!(caps & DSCAPS_FLIPPING) && !(caps & DSCAPS_SUBSURFACE))
-          pthread_create( &data->flip_thread, NULL, Flipping_Thread, thiz );
+     if (!(caps & DSCAPS_FLIPPING) && !(caps & DSCAPS_SUBSURFACE)) {
+          if (dfb_config->autoflip_window)
+               pthread_create( &data->flip_thread, NULL, Flipping_Thread, thiz );
+          else
+               D_WARN( "Non-flipping window surface and no 'autoflip-window' option used" );
+     }
 
      thiz->Release = IDirectFBSurface_Window_Release;
      thiz->Flip = IDirectFBSurface_Window_Flip;
