@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -37,9 +38,11 @@
  * reallocates private data and overloads functions of the interface
  */
 DFBResult IDirectFBSurface_Window_Construct( IDirectFBSurface       *thiz,
+                                             IDirectFBSurface       *parent,
                                              DFBRectangle           *req_rect,
                                              DFBRectangle           *clip_rect,
                                              CoreWindow             *window,
-                                             DFBSurfaceCapabilities  caps );
+                                             DFBSurfaceCapabilities  caps,
+                                             CoreDFB                *core );
 
 #endif
