@@ -2,7 +2,7 @@
 #define __YUV_TABLES_H__
 
 
-const __u16 y_for_rgb[256] = {
+const u16 y_for_rgb[256] = {
         0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
         0,    0,    0,    0,    0,    0,    0,    1,    2,    3,
         4,    5,    6,    8,    9,   10,   11,   12,   13,   15,
@@ -32,7 +32,7 @@ const __u16 y_for_rgb[256] = {
 };
 
 
-const __s16 cr_for_r[256] = {
+const s16 cr_for_r[256] = {
      -178, -178, -178, -178, -178, -178, -178, -178, -178, -178,
      -178, -178, -178, -178, -178, -178, -178, -177, -175, -173,
      -172, -170, -169, -167, -165, -164, -162, -161, -159, -158,
@@ -62,7 +62,7 @@ const __s16 cr_for_r[256] = {
 };
 
 
-const __s16 cr_for_g[256] = {
+const s16 cr_for_g[256] = {
        91,   91,   91,   91,   91,   91,   91,   91,   91,   91,
        91,   91,   91,   91,   91,   91,   91,   90,   89,   88,
        87,   86,   86,   85,   84,   83,   82,   82,   81,   80,
@@ -92,7 +92,7 @@ const __s16 cr_for_g[256] = {
 };
 
 
-const __s16 cb_for_g[256] = {
+const s16 cb_for_g[256] = {
        43,   43,   43,   43,   43,   43,   43,   43,   43,   43,
        43,   43,   43,   43,   43,   43,   43,   43,   43,   42,
        42,   41,   41,   41,   40,   40,   39,   39,   39,   38,
@@ -122,7 +122,7 @@ const __s16 cb_for_g[256] = {
 };
 
 
-const __s16 cb_for_b[256] = {
+const s16 cb_for_b[256] = {
      -226, -226, -226, -226, -226, -226, -226, -226, -226, -226,
      -226, -226, -226, -226, -226, -226, -226, -223, -221, -219,
      -217, -215, -213, -211, -209, -207, -205, -203, -201, -199,
@@ -152,7 +152,7 @@ const __s16 cb_for_b[256] = {
 };
 
 
-const __u16 y_from_ey[256] = {
+const u16 y_from_ey[256] = {
        16,   16,   17,   18,   19,   20,   21,   22,   22,   23,
        24,   25,   26,   27,   28,   28,   29,   30,   31,   32,
        33,   34,   34,   35,   36,   37,   38,   39,   40,   40,
@@ -182,7 +182,7 @@ const __u16 y_from_ey[256] = {
 };
 
 
-static const __u16 _cb_from_bey[512] = {
+const u16 cb_from_bey[512] = {
        16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
        16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
        16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
@@ -238,7 +238,7 @@ static const __u16 _cb_from_bey[512] = {
 };
 
 
-static const __u16 _cr_from_rey[512] = {
+const u16 cr_from_rey[512] = {
        16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
        16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
        16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
@@ -293,7 +293,4 @@ static const __u16 _cr_from_rey[512] = {
       240,  240
 };
 
-const __u16 *cb_from_bey = &_cb_from_bey[255];
-const __u16 *cr_from_rey = &_cr_from_rey[255];
-
 #endif /* __YUV_TABLES_H__ */
