@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -38,7 +38,6 @@
 
 #include <core/state.h>
 #include <core/gfxcard.h>
-#include <core/surfacemanager.h>
 
 #include <gfx/util.h>
 
@@ -51,24 +50,28 @@ static CardState copy_state;
 static bool      btf_state_inited;
 static CardState btf_state;
 
+#if FIXME_SC_3
 static bool      cd_state_inited;
 static CardState cd_state;
+#endif
 
 static pthread_mutex_t copy_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t btf_lock  = PTHREAD_MUTEX_INITIALIZER;
+#if FIXME_SC_3
 static pthread_mutex_t cd_lock   = PTHREAD_MUTEX_INITIALIZER;
+#endif
 
 
 void
-dfb_gfx_copy( CoreSurface *source, CoreSurface *destination, DFBRectangle *rect )
+dfb_gfx_copy( CoreSurface *source, CoreSurface *destination, const DFBRectangle *rect )
 {
      dfb_gfx_copy_to( source, destination, rect, rect ? rect->x : 0, rect ? rect->y : 0, false );
 }
 
 void
-dfb_gfx_copy_to( CoreSurface *source, CoreSurface *destination, DFBRectangle *rect, int x, int y, bool from_back )
+dfb_gfx_copy_to( CoreSurface *source, CoreSurface *destination, const DFBRectangle *rect, int x, int y, bool from_back )
 {
-     SurfaceBuffer *tmp;
+     DFBRectangle sourcerect = { 0, 0, source->config.size.w, source->config.size.h };
 
      pthread_mutex_lock( &copy_lock );
 
@@ -79,46 +82,31 @@ dfb_gfx_copy_to( CoreSurface *source, CoreSurface *destination, DFBRectangle *re
 
      copy_state.modified   |= SMF_CLIP | SMF_SOURCE | SMF_DESTINATION;
 
-     copy_state.clip.x2     = destination->width - 1;
-     copy_state.clip.y2     = destination->height - 1;
+     copy_state.clip.x2     = destination->config.size.w - 1;
+     copy_state.clip.y2     = destination->config.size.h - 1;
      copy_state.source      = source;
      copy_state.destination = destination;
-
-     if (from_back) {
-          dfb_surfacemanager_lock( source->manager );
-
-          tmp = source->front_buffer;
-          source->front_buffer = source->back_buffer;
-          source->back_buffer = tmp;
-     }
+     copy_state.from        = from_back ? CSBR_BACK : CSBR_FRONT;
 
      if (rect) {
-          /* Wrokaround for window managers passing negative rectangles */
-          DFBRectangle sourcerect = { 0, 0, source->width, source->height };
-          if (dfb_rectangle_intersect( rect, &sourcerect ))
-               dfb_gfxcard_blit( rect, x, y, &copy_state );
+          if (dfb_rectangle_intersect( &sourcerect, rect ))
+               dfb_gfxcard_blit( &sourcerect,
+                                 x + sourcerect.x - rect->x,
+                                 y + sourcerect.y - rect->y, &copy_state );
      }
-     else {
-          DFBRectangle sourcerect = { 0, 0, source->width, source->height };
+     else
           dfb_gfxcard_blit( &sourcerect, x, y, &copy_state );
-     }
 
-     if (from_back) {
-          tmp = source->front_buffer;
-          source->front_buffer = source->back_buffer;
-          source->back_buffer = tmp;
-
-          dfb_surfacemanager_unlock( source->manager );
-     }
+     /* Signal end of sequence. */
+     dfb_state_stop_drawing( &copy_state );
 
      pthread_mutex_unlock( &copy_lock );
 }
 
-void
-dfb_back_to_front_copy( CoreSurface *surface, const DFBRegion *region )
+static void
+back_to_front_copy( CoreSurface *surface, const DFBRegion *region, DFBSurfaceBlittingFlags flags )
 {
-     SurfaceBuffer *tmp;
-     DFBRectangle   rect;
+     DFBRectangle rect;
 
      if (region) {
           rect.x = region->x1;
@@ -129,46 +117,56 @@ dfb_back_to_front_copy( CoreSurface *surface, const DFBRegion *region )
      else {
           rect.x = 0;
           rect.y = 0;
-          rect.w = surface->width;
-          rect.h = surface->height;
+          rect.w = surface->config.size.w;
+          rect.h = surface->config.size.h;
      }
 
      pthread_mutex_lock( &btf_lock );
 
      if (!btf_state_inited) {
           dfb_state_init( &btf_state, NULL );
+
+          btf_state.from = CSBR_BACK;
+          btf_state.to   = CSBR_FRONT;
+
           btf_state_inited = true;
      }
 
-     btf_state.modified   |= SMF_CLIP | SMF_SOURCE | SMF_DESTINATION;
+     btf_state.modified     |= SMF_CLIP | SMF_SOURCE | SMF_DESTINATION;
 
-     btf_state.clip.x2     = surface->width - 1;
-     btf_state.clip.y2     = surface->height - 1;
-     btf_state.source      = surface;
-     btf_state.destination = surface;
+     btf_state.clip.x2       = surface->config.size.w - 1;
+     btf_state.clip.y2       = surface->config.size.h - 1;
+     btf_state.source        = surface;
+     btf_state.destination   = surface;
 
-     dfb_surfacemanager_lock( surface->manager );
-
-     tmp = surface->front_buffer;
-     surface->front_buffer = surface->back_buffer;
-     surface->back_buffer = tmp;
+     dfb_state_set_blitting_flags( &btf_state, flags );
 
      dfb_gfxcard_blit( &rect, rect.x, rect.y, &btf_state );
 
-     tmp = surface->front_buffer;
-     surface->front_buffer = surface->back_buffer;
-     surface->back_buffer = tmp;
-
-     dfb_surfacemanager_unlock( surface->manager );
+     /* Signal end of sequence. */
+     dfb_state_stop_drawing( &btf_state );
 
      pthread_mutex_unlock( &btf_lock );
 }
 
+void
+dfb_back_to_front_copy( CoreSurface *surface, const DFBRegion *region )
+{
+     back_to_front_copy( surface, region, DSBLIT_NOFX );
+}
+
+void
+dfb_back_to_front_copy_180( CoreSurface *surface, const DFBRegion *region )
+{
+     back_to_front_copy( surface, region, DSBLIT_ROTATE180 );
+}
+
 void
 dfb_clear_depth( CoreSurface *surface, const DFBRegion *region )
 {
+#if FIXME_SC_3
      SurfaceBuffer *tmp;
-     DFBRectangle   rect = { 0, 0, surface->width - 1, surface->height - 1 };
+     DFBRectangle   rect = { 0, 0, surface->config.size.w - 1, surface->config.size.h - 1 };
 
      if (region && !dfb_rectangle_intersect_by_region( &rect, region ))
           return;
@@ -187,8 +185,8 @@ dfb_clear_depth( CoreSurface *surface, const DFBRegion *region )
 
      cd_state.modified   |= SMF_CLIP | SMF_DESTINATION;
 
-     cd_state.clip.x2     = surface->width - 1;
-     cd_state.clip.y2     = surface->height - 1;
+     cd_state.clip.x2     = surface->config.size.w - 1;
+     cd_state.clip.y2     = surface->config.size.h - 1;
      cd_state.destination = surface;
 
      dfb_surfacemanager_lock( surface->manager );
@@ -202,7 +200,11 @@ dfb_clear_depth( CoreSurface *surface, const DFBRegion *region )
 
      dfb_surfacemanager_unlock( surface->manager );
 
+     /* Signal end of sequence. */
+     dfb_state_stop_drawing( &cd_state );
+
      pthread_mutex_unlock( &cd_lock );
+#endif
 }
 
 
