@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -29,11 +29,12 @@
 #ifndef __GFX__UTIL_H__
 #define __GFX__UTIL_H__
 
-#include <core/surfaces.h>
+#include <core/surface.h>
 
-void dfb_gfx_copy( CoreSurface *source, CoreSurface *destination, DFBRectangle *rect );
-void dfb_gfx_copy_to( CoreSurface *source, CoreSurface *destination, DFBRectangle *rect, int x, int y, bool from_back );
+void dfb_gfx_copy( CoreSurface *source, CoreSurface *destination, const DFBRectangle *rect );
+void dfb_gfx_copy_to( CoreSurface *source, CoreSurface *destination, const DFBRectangle *rect, int x, int y, bool from_back );
 void dfb_back_to_front_copy( CoreSurface *surface, const DFBRegion *region );
+void dfb_back_to_front_copy_180( CoreSurface *surface, const DFBRegion *region );
 void dfb_clear_depth( CoreSurface *surface, const DFBRegion *region );
 
 void dfb_sort_triangle( DFBTriangle *tri );
