@@ -1,12 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002       convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de> and
-              Sven Neumann <sven@convergence.de>.
+              Andreas Hundt <andi@fischlustig.de>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -40,6 +42,7 @@
 
 #include <directfb.h>
 
+#include <direct/debug.h>
 #include <direct/interface.h>
 #include <direct/list.h>
 #include <direct/mem.h>
@@ -55,11 +58,16 @@
 
 #include <core/input.h>
 #include <core/windows.h>
+#include <core/windows_internal.h>
 
 #include <misc/util.h>
 
 #include "idirectfbinputbuffer.h"
 
+
+D_DEBUG_DOMAIN( IDFBEvBuf, "IDFBEventBuffer", "IDirectFBEventBuffer Interface" );
+
+
 typedef struct {
      DirectLink   link;
      DFBEvent     evt;
@@ -70,6 +78,8 @@ typedef struct {
 
      CoreInputDevice *device;       /* pointer to input core device struct */
      Reaction         reaction;
+     
+     DFBInputDeviceDescription desc;
 } AttachedDevice;
 
 typedef struct {
@@ -135,6 +145,8 @@ IDirectFBEventBuffer_Destruct( IDirectFBEventBuffer *thiz )
      EventBufferItem           *item;
      DirectLink                *n;
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      pthread_mutex_lock( &data->events_mutex );
 
      if (data->pipe) {
@@ -160,9 +172,11 @@ IDirectFBEventBuffer_Destruct( IDirectFBEventBuffer *thiz )
      }
 
      direct_list_foreach_safe (window, n, data->windows) {
-          if (window->window)
+          if (window->window) {
                dfb_window_detach( window->window, &window->reaction );
-
+               dfb_window_unref( window->window );
+          }
+               
           D_FREE( window );
      }
 
@@ -175,21 +189,25 @@ IDirectFBEventBuffer_Destruct( IDirectFBEventBuffer *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBEventBuffer_AddRef( IDirectFBEventBuffer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      data->ref++;
 
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBEventBuffer_Release( IDirectFBEventBuffer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (--data->ref == 0)
           IDirectFBEventBuffer_Destruct( thiz );
 
@@ -204,6 +222,8 @@ IDirectFBEventBuffer_Reset( IDirectFBEventBuffer *thiz )
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -226,6 +246,8 @@ IDirectFBEventBuffer_WaitForEvent( IDirectFBEventBuffer *thiz )
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -254,6 +276,8 @@ IDirectFBEventBuffer_WaitForEventWithTimeout( IDirectFBEventBuffer *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %u, %u )\n", __FUNCTION__, thiz, seconds, milli_seconds );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -295,6 +319,8 @@ IDirectFBEventBuffer_WakeUp( IDirectFBEventBuffer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -311,6 +337,8 @@ IDirectFBEventBuffer_GetEvent( IDirectFBEventBuffer *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p )\n", __FUNCTION__, thiz, event );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -336,6 +364,10 @@ IDirectFBEventBuffer_GetEvent( IDirectFBEventBuffer *thiz,
                event->user = item->evt.user;
                break;
 
+          case DFEC_VIDEOPROVIDER:
+               event->videoprovider = item->evt.videoprovider;
+               break;
+
           case DFEC_UNIVERSAL:
                direct_memcpy( event, &item->evt, item->evt.universal.size );
                break;
@@ -353,6 +385,8 @@ IDirectFBEventBuffer_GetEvent( IDirectFBEventBuffer *thiz,
 
      pthread_mutex_unlock( &data->events_mutex );
 
+     D_DEBUG_AT( IDFBEvBuf, "  -> class %d, type/size %d, data/id %p\n", event->clazz, event->user.type, event->user.data );
+
      return DFB_OK;
 }
 
@@ -364,6 +398,8 @@ IDirectFBEventBuffer_PeekEvent( IDirectFBEventBuffer *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p )\n", __FUNCTION__, thiz, event );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -389,6 +425,10 @@ IDirectFBEventBuffer_PeekEvent( IDirectFBEventBuffer *thiz,
                event->user = item->evt.user;
                break;
 
+          case DFEC_VIDEOPROVIDER:
+               event->videoprovider = item->evt.videoprovider;
+               break;
+
           case DFEC_UNIVERSAL:
                direct_memcpy( event, &item->evt, item->evt.universal.size );
                break;
@@ -399,6 +439,8 @@ IDirectFBEventBuffer_PeekEvent( IDirectFBEventBuffer *thiz,
 
      pthread_mutex_unlock( &data->events_mutex );
 
+     D_DEBUG_AT( IDFBEvBuf, "  -> class %d, type/size %d, data/id %p\n", event->clazz, event->user.type, event->user.data );
+
      return DFB_OK;
 }
 
@@ -407,6 +449,8 @@ IDirectFBEventBuffer_HasEvent( IDirectFBEventBuffer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p ) <- events: %p, pipe: %d\n", __FUNCTION__, thiz, data->events, data->pipe );
+
      if (data->pipe)
           return DFB_UNSUPPORTED;
 
@@ -422,10 +466,14 @@ IDirectFBEventBuffer_PostEvent( IDirectFBEventBuffer *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p [class %d, type/size %d, data/id %p] )\n", __FUNCTION__,
+                 thiz, event, event->clazz, event->user.type, event->user.data );
+
      switch (event->clazz) {
           case DFEC_INPUT:
           case DFEC_WINDOW:
           case DFEC_USER:
+          case DFEC_VIDEOPROVIDER:
                size = sizeof(EventBufferItem);
                break;
 
@@ -433,6 +481,10 @@ IDirectFBEventBuffer_PostEvent( IDirectFBEventBuffer *thiz,
                size = event->universal.size;
                if (size < sizeof(DFBUniversalEvent))
                     return DFB_INVARG;
+               /* We must not exceed the union to avoid crashes in generic code (reading DFBEvents)
+                * and to support pipe mode where each written block has to have a fixed size. */
+               if (size > sizeof(DFBEvent))
+                    return DFB_INVARG;
                size += sizeof(DirectLink);
                break;
 
@@ -457,6 +509,10 @@ IDirectFBEventBuffer_PostEvent( IDirectFBEventBuffer *thiz,
                item->evt.user = event->user;
                break;
 
+          case DFEC_VIDEOPROVIDER:
+               item->evt.videoprovider = event->videoprovider;
+               break;
+
           case DFEC_UNIVERSAL:
                direct_memcpy( &item->evt, event, event->universal.size );
                break;
@@ -476,6 +532,8 @@ IDirectFBEventBuffer_CreateFileDescriptor( IDirectFBEventBuffer *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p )\n", __FUNCTION__, thiz );
+
      /* Check arguments. */
      if (!ret_fd)
           return DFB_INVARG;
@@ -491,10 +549,13 @@ IDirectFBEventBuffer_CreateFileDescriptor( IDirectFBEventBuffer *thiz,
 
      /* Create the file descriptor(s). */
      if (socketpair( PF_LOCAL, SOCK_STREAM, 0, data->pipe_fds )) {
+          D_PERROR( "%s(): socketpair( PF_LOCAL, SOCK_STREAM, 0, fds ) failed!\n", __FUNCTION__ );
           pthread_mutex_unlock( &data->events_mutex );
           return errno2result( errno );
      }
 
+     D_DEBUG_AT( IDFBEvBuf, "  -> entering pipe mode\n" );
+
      /* Enter pipe mode. */
      data->pipe = true;
 
@@ -512,6 +573,8 @@ IDirectFBEventBuffer_CreateFileDescriptor( IDirectFBEventBuffer *thiz,
      /* Return the file descriptor for reading. */
      *ret_fd = data->pipe_fds[0];
 
+     D_DEBUG_AT( IDFBEvBuf, "  -> fd %d/%d\n", data->pipe_fds[0], data->pipe_fds[1] );
+
      return DFB_OK;
 }
 
@@ -521,6 +584,8 @@ IDirectFBEventBuffer_EnableStatistics( IDirectFBEventBuffer *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %sable )\n", __FUNCTION__, thiz, enable ? "en" : "dis" );
+
      /* Lock the event queue. */
      pthread_mutex_lock( &data->events_mutex );
 
@@ -557,6 +622,8 @@ IDirectFBEventBuffer_GetStatistics( IDirectFBEventBuffer *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p )\n", __FUNCTION__, thiz, ret_stats );
+
      if (!ret_stats)
           return DFB_INVARG;
 
@@ -585,6 +652,8 @@ IDirectFBEventBuffer_Construct( IDirectFBEventBuffer      *thiz,
 {
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBEventBuffer)
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, filter %p, ctx %p )\n", __FUNCTION__, thiz, filter, filter_ctx );
+
      data->ref        = 1;
      data->filter     = filter;
      data->filter_ctx = filter_ctx;
@@ -606,6 +675,8 @@ IDirectFBEventBuffer_Construct( IDirectFBEventBuffer      *thiz,
      thiz->EnableStatistics        = IDirectFBEventBuffer_EnableStatistics;
      thiz->GetStatistics           = IDirectFBEventBuffer_GetStatistics;
 
+     D_DEBUG_AT( IDFBEvBuf, "  -> %p [%p]\n", thiz, thiz->priv );
+
      return DFB_OK;
 }
 
@@ -614,12 +685,21 @@ IDirectFBEventBuffer_Construct( IDirectFBEventBuffer      *thiz,
 DFBResult IDirectFBEventBuffer_AttachInputDevice( IDirectFBEventBuffer *thiz,
                                                   CoreInputDevice      *device )
 {
-     AttachedDevice *attached;
+     AttachedDevice            *attached;
+     DFBInputDeviceDescription  desc;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_ASSERT( device != NULL );
+
+     dfb_input_device_description( device, &desc );
+
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p [%02u - %s] )\n", __FUNCTION__, thiz, device,
+                 dfb_input_device_id(device), desc.name );
+
      attached = D_CALLOC( 1, sizeof(AttachedDevice) );
      attached->device = device;
+     attached->desc   = desc;
 
      direct_list_prepend( &data->devices, &attached->link );
 
@@ -629,16 +709,52 @@ DFBResult IDirectFBEventBuffer_AttachInputDevice( IDirectFBEventBuffer *thiz,
      return DFB_OK;
 }
 
+DFBResult IDirectFBEventBuffer_DetachInputDevice( IDirectFBEventBuffer *thiz,
+                                                  CoreInputDevice      *device )
+{
+     AttachedDevice *attached;
+     DirectLink     *link;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
+
+     D_ASSERT( device != NULL );
+
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p [%02u] )\n", __FUNCTION__, thiz, device,
+                 dfb_input_device_id(device) );
+     
+     direct_list_foreach_safe (attached, link, data->devices) {
+          if (attached->device == device) {
+               direct_list_remove( &data->devices, &attached->link );
+               
+               dfb_input_detach( attached->device, &attached->reaction );
+               
+               D_FREE( attached );
+               
+               return DFB_OK;
+          }
+     }
+
+     return DFB_ITEMNOTFOUND;
+}
+
 DFBResult IDirectFBEventBuffer_AttachWindow( IDirectFBEventBuffer *thiz,
                                              CoreWindow           *window )
 {
      AttachedWindow *attached;
-
+     
      DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
 
+     D_ASSERT( window != NULL );
+
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p [%02u - %d,%d-%dx%d] )\n", __FUNCTION__, thiz,
+                 window, window->id, window->config.bounds.x, window->config.bounds.y,
+                 window->config.bounds.w, window->config.bounds.h );
+
      attached = D_CALLOC( 1, sizeof(AttachedWindow) );
      attached->window = window;
 
+     dfb_window_ref( window );
+
      direct_list_prepend( &data->windows, &attached->link );
 
      dfb_window_attach( window, IDirectFBEventBuffer_WindowReact,
@@ -647,6 +763,36 @@ DFBResult IDirectFBEventBuffer_AttachWindow( IDirectFBEventBuffer *thiz,
      return DFB_OK;
 }
 
+DFBResult IDirectFBEventBuffer_DetachWindow( IDirectFBEventBuffer *thiz,
+                                             CoreWindow           *window )
+{
+     AttachedWindow *attached;
+     DirectLink     *link;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBEventBuffer)
+
+     D_ASSERT( window != NULL );
+     
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p [%02u - %d,%d-%dx%d] )\n", __FUNCTION__, thiz,
+                 window, window->id, window->config.bounds.x, window->config.bounds.y,
+                 window->config.bounds.w, window->config.bounds.h );
+
+     direct_list_foreach_safe (attached, link, data->windows) {
+          if (!attached->window || attached->window == window) {
+               direct_list_remove( &data->windows, &attached->link );
+
+               if (attached->window) {
+                    dfb_window_detach( attached->window, &attached->reaction );
+                    dfb_window_unref( attached->window );
+               }
+               
+               D_FREE( attached );
+          }
+     }
+
+     return DFB_OK;
+}
+
 /* file internals */
 
 static void IDirectFBEventBuffer_AddItem( IDirectFBEventBuffer_data *data,
@@ -676,6 +822,8 @@ static ReactionResult IDirectFBEventBuffer_InputReact( const void *msg_data,
      IDirectFBEventBuffer_data *data = ctx;
      EventBufferItem           *item;
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p ) <- type %06x\n", __FUNCTION__, evt, data, evt->type );
+
      item = D_CALLOC( 1, sizeof(EventBufferItem) );
 
      item->evt.input = *evt;
@@ -693,6 +841,8 @@ static ReactionResult IDirectFBEventBuffer_WindowReact( const void *msg_data,
      IDirectFBEventBuffer_data *data = ctx;
      EventBufferItem           *item;
 
+     D_DEBUG_AT( IDFBEvBuf, "%s( %p, %p ) <- type %06x\n", __FUNCTION__, evt, data, evt->type );
+
      item = D_CALLOC( 1, sizeof(EventBufferItem) );
 
      item->evt.window = *evt;
@@ -710,7 +860,7 @@ static ReactionResult IDirectFBEventBuffer_WindowReact( const void *msg_data,
                if (dfb_window_id( window->window ) == evt->window_id) {
                     /* FIXME: free memory later, because reactor writes to it
                        after we return RS_REMOVE */
-                    //D_FREE( window );
+                    dfb_window_unref( window->window );
                     window->window = NULL;
                }
           }
@@ -745,13 +895,13 @@ IDirectFBEventBuffer_Feed( DirectThread *thread, void *arg )
 
                pthread_mutex_unlock( &data->events_mutex );
 
-               D_HEAVYDEBUG( "DirectFB/EventBuffer: Going to write %d bytes to file descriptor %d...\n",
-                             sizeof(DFBEvent), data->pipe_fds[1] );
+               D_DEBUG_AT( IDFBEvBuf, "Going to write %zu bytes to file descriptor %d...\n",
+                           sizeof(DFBEvent), data->pipe_fds[1] );
 
                ret = write( data->pipe_fds[1], &item->evt, sizeof(DFBEvent) );
 
-               D_HEAVYDEBUG( "DirectFB/EventBuffer: ...wrote %d bytes to file descriptor %d.\n",
-                             ret, data->pipe_fds[1] );
+               D_DEBUG_AT( IDFBEvBuf, "...wrote %d bytes to file descriptor %d.\n",
+                           ret, data->pipe_fds[1] );
 
                D_FREE( item );
 
@@ -877,6 +1027,75 @@ CollectEventStatistics( DFBEventBufferStats *stats,
                stats->DFEC_USER += incdec;
                break;
 
+          case DFEC_VIDEOPROVIDER:
+               stats->DFEC_VIDEOPROVIDER +=incdec;
+
+               switch (event->videoprovider.type) {
+                    case DVPET_STARTED:
+                         stats->DVPET_STARTED += incdec;
+                         break;
+
+                    case DVPET_STOPPED:
+                         stats->DVPET_STOPPED += incdec;
+                         break;
+
+                    case DVPET_SPEEDCHANGE:
+                         stats->DVPET_SPEEDCHANGE += incdec;
+                         break;
+
+                    case DVPET_STREAMCHANGE:
+                         stats->DVPET_STREAMCHANGE += incdec;
+                         break;
+
+                    case DVPET_FATALERROR:
+                         stats->DVPET_FATALERROR += incdec;
+                         break;
+                    
+                    case DVPET_FINISHED:
+                         stats->DVPET_FINISHED += incdec;
+                         break;
+                         
+                    case DVPET_SURFACECHANGE:
+                         stats->DVPET_SURFACECHANGE += incdec;
+                         break;
+
+                    case DVPET_FRAMEDECODED:
+                         stats->DVPET_FRAMEDECODED += incdec;
+                         break;
+
+                    case DVPET_FRAMEDISPLAYED:
+                         stats->DVPET_FRAMEDISPLAYED += incdec;
+                         break;
+
+                    case DVPET_DATAEXHAUSTED:
+                         stats->DVPET_DATAEXHAUSTED += incdec;
+                         break;
+
+                    case DVPET_VIDEOACTION:
+                         stats->DVPET_VIDEOACTION += incdec;
+                         break;
+
+                    case DVPET_DATALOW:
+                         stats->DVPET_DATALOW += incdec;
+                         break;
+
+                   case DVPET_DATAHIGH:
+                         stats->DVPET_DATAHIGH += incdec;
+                         break;
+
+                   case DVPET_BUFFERTIMELOW:
+                         stats->DVPET_BUFFERTIMELOW += incdec;
+                         break;
+
+                   case DVPET_BUFFERTIMEHIGH:
+                         stats->DVPET_BUFFERTIMEHIGH += incdec;
+                         break;
+
+                    default:
+                         D_BUG( "unknown video provider event type 0x%08x\n", event->videoprovider.type );
+               }
+               break;
+
           case DFEC_UNIVERSAL:
                stats->DFEC_UNIVERSAL += incdec;
                break;
