@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -39,8 +39,32 @@ typedef struct {
      char        *filename;   /* Only set if databuffer is created from file. */
 
      CoreDFB     *core;
+
+     bool         is_memory;
 } IDirectFBDataBuffer_data;
 
+/*
+ * private data struct of IDirectFBDataBuffer_File
+ */
+typedef struct {
+     IDirectFBDataBuffer_data base;
+
+     DirectStream    *stream;
+     pthread_mutex_t  mutex;
+} IDirectFBDataBuffer_File_data;
+
+/*
+ * private data struct of IDirectFBDataBuffer_Memory
+ */
+typedef struct {
+     IDirectFBDataBuffer_data  base;
+
+     const void               *buffer;
+     unsigned int              length;
+
+     unsigned int              pos;
+} IDirectFBDataBuffer_Memory_data;
+
 /*
  * base constructor
  *
