@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -40,13 +40,13 @@
 #include <media/idirectfbdatabuffer.h>
 
 
-static DFBResult
+static DirectResult
 IDirectFBVideoProvider_AddRef( IDirectFBVideoProvider *thiz )
 {
      return DFB_UNIMPLEMENTED;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBVideoProvider_Release( IDirectFBVideoProvider *thiz )
 {
      return DFB_UNIMPLEMENTED;
@@ -88,6 +88,37 @@ IDirectFBVideoProvider_GetStreamDescription( IDirectFBVideoProvider *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBVideoProvider_GetBufferOccupancy( IDirectFBVideoProvider *thiz,
+                                           DFBBufferOccupancy     *ret_occ )
+{
+     if (!ret_occ)
+          return DFB_INVARG;
+          
+     memset( ret_occ, 0, sizeof(DFBBufferOccupancy) );
+     
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_GetBufferThresholds( IDirectFBVideoProvider *thiz,
+                                            DFBBufferThresholds    *ret_thresh )
+{
+     if (!ret_thresh)
+          return DFB_INVARG;
+          
+     memset( ret_thresh, 0, sizeof(DFBBufferThresholds) );
+     
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_SetBufferThresholds( IDirectFBVideoProvider *thiz,
+                                            DFBBufferThresholds    thresh )
+{
+    return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBVideoProvider_PlayTo( IDirectFBVideoProvider *thiz,
                                IDirectFBSurface       *destination,
@@ -203,6 +234,69 @@ IDirectFBVideoProvider_GetVolume( IDirectFBVideoProvider *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBVideoProvider_SetStreamAttributes(IDirectFBVideoProvider   *thiz,
+          				    DFBStreamAttributes       attr)
+{
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_SetAudioOutputs(IDirectFBVideoProvider         *thiz,
+                                       DFBVideoProviderAudioUnits*    audioUnits)
+{
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_GetAudioOutputs(IDirectFBVideoProvider         *thiz,
+                                       DFBVideoProviderAudioUnits*    audioUnits)
+{
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_SetAudioDelay( IDirectFBVideoProvider *thiz,
+                                 long                  delay )
+{
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_CreateEventBuffer( IDirectFBVideoProvider  *thiz,
+                                          IDirectFBEventBuffer    **buffer )
+{
+    return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_AttachEventBuffer( IDirectFBVideoProvider  *thiz,
+                                          IDirectFBEventBuffer    *buffer )
+{
+    return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_EnableEvents(IDirectFBVideoProvider         *thiz,
+                                    DFBVideoProviderEventType      mask )
+{
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_DisableEvents(IDirectFBVideoProvider         *thiz,
+                                     DFBVideoProviderEventType      mask )
+{
+    return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBVideoProvider_DetachEventBuffer( IDirectFBVideoProvider  *thiz,
+                                          IDirectFBEventBuffer    *buffer )
+{
+    return DFB_UNIMPLEMENTED;
+}
+
 static void
 IDirectFBVideoProvider_Construct( IDirectFBVideoProvider *thiz )
 {
@@ -211,6 +305,9 @@ IDirectFBVideoProvider_Construct( IDirectFBVideoProvider *thiz )
      thiz->GetCapabilities       = IDirectFBVideoProvider_GetCapabilities;
      thiz->GetSurfaceDescription = IDirectFBVideoProvider_GetSurfaceDescription;
      thiz->GetStreamDescription  = IDirectFBVideoProvider_GetStreamDescription;
+     thiz->GetBufferOccupancy    = IDirectFBVideoProvider_GetBufferOccupancy;
+     thiz->SetBufferThresholds   = IDirectFBVideoProvider_SetBufferThresholds;
+     thiz->GetBufferThresholds   = IDirectFBVideoProvider_GetBufferThresholds;
      thiz->PlayTo                = IDirectFBVideoProvider_PlayTo;
      thiz->Stop                  = IDirectFBVideoProvider_Stop;
      thiz->GetStatus             = IDirectFBVideoProvider_GetStatus;
@@ -225,6 +322,15 @@ IDirectFBVideoProvider_Construct( IDirectFBVideoProvider *thiz )
      thiz->GetSpeed              = IDirectFBVideoProvider_GetSpeed;
      thiz->SetVolume             = IDirectFBVideoProvider_SetVolume;
      thiz->GetVolume             = IDirectFBVideoProvider_GetVolume;
+     thiz->SetStreamAttributes   = IDirectFBVideoProvider_SetStreamAttributes;
+     thiz->SetAudioOutputs       = IDirectFBVideoProvider_SetAudioOutputs;
+     thiz->GetAudioOutputs       = IDirectFBVideoProvider_GetAudioOutputs;
+     thiz->CreateEventBuffer     = IDirectFBVideoProvider_CreateEventBuffer;
+     thiz->AttachEventBuffer     = IDirectFBVideoProvider_AttachEventBuffer;
+     thiz->EnableEvents          = IDirectFBVideoProvider_EnableEvents;
+     thiz->DisableEvents         = IDirectFBVideoProvider_DisableEvents;
+     thiz->DetachEventBuffer     = IDirectFBVideoProvider_DetachEventBuffer;
+     thiz->SetAudioDelay         = IDirectFBVideoProvider_SetAudioDelay;
 }
 
 
