@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -43,7 +43,7 @@
 #include <core/coretypes.h>
 
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <direct/memcpy.h>
 #include <direct/mem.h>
@@ -82,7 +82,7 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
      u8          *d1,*d2;
      int          i, j;
 
-     if (dst_surface->caps & DSCAPS_PREMULTIPLIED) {
+     if (dst_surface->config.caps & DSCAPS_PREMULTIPLIED) {
           for (i = 0; i < len; i++) {
                u32 s = src[i];
                u32 a = (s >> 24) + 1;
@@ -93,7 +93,7 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
           }      
      }
 
-     switch (dst_surface->format) {
+     switch (dst_surface->config.format) {
           case DSPF_A1:
                for (i = 0; i < len; i++) {
                     if (i & 7)
@@ -378,9 +378,14 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
                     d[i+0] = y0;
                     d[i+1] = y1;
                                   
-                    if (dst_surface->format == DSPF_NV16 || dy & 1) {
+                    if (dst_surface->config.format == DSPF_NV16 || dy & 1) {
+#ifdef WORDS_BIGENDIAN
+                         ((u16*)d1)[i>>1] =    ((v0 + v1) >> 1)     |
+                                              (((u0 + u1) >> 1) << 8);
+#else
                          ((u16*)d1)[i>>1] =    ((u0 + u1) >> 1)     |
                                               (((v0 + v1) >> 1) << 8);
+#endif
                     }
                }
                if (len & 1) {
@@ -393,8 +398,12 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
                                   (src[i]      ) & 0xff, y, u, v );
                                   
                     d[i] = y;
-                    if (dst_surface->format == DSPF_NV16 || dy & 1)
+                    if (dst_surface->config.format == DSPF_NV16 || dy & 1)
+#ifdef WORDS_BIGENDIAN
+                         ((u16*)d1)[i>>1] = v | (u << 8);
+#else
                          ((u16*)d1)[i>>1] = u | (v << 8);
+#endif
                }
                break;
                
@@ -415,8 +424,13 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
                     d[i+1] = y1;
                                   
                     if (dy & 1) {
+#ifdef WORDS_BIGENDIAN
+                         ((u16*)d1)[i>>1] =    ((u0 + u1) >> 1)     |
+                                              (((v0 + v1) >> 1) << 8);
+#else
                          ((u16*)d1)[i>>1] =    ((v0 + v1) >> 1)     |
                                               (((u0 + u1) >> 1) << 8);
+#endif
                     }
                }
                if (len & 1) {
@@ -430,7 +444,11 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
                                   
                     d[i] = y;
                     if (dy & 1)
+#ifdef WORDS_BIGENDIAN
+                         ((u16*)d1)[i>>1] = u | (v << 8);
+#else
                          ((u16*)d1)[i>>1] = v | (u << 8);
+#endif
                }  
                break;
 
@@ -439,13 +457,18 @@ static void write_argb_span (u32 *src, u8 *dst[], int len,
                     ((u16*)d)[i] = ARGB_TO_RGB555( src[i] );
                break;
 
+          case DSPF_BGR555:
+               for (i = 0; i < len; i++)
+                    ((u16*)d)[i] = ARGB_TO_BGR555( src[i] );
+               break;
+
           case DSPF_RGB444:
                for (i = 0; i < len; i++)
                     ((u16*)d)[i] = ARGB_TO_RGB444( src[i] );
                break;
 
           default:
-               D_ONCE( "unimplemented destination format (0x%08x)", dst_surface->format );
+               D_ONCE( "unimplemented destination format (0x%08x)", dst_surface->config.format );
                break;
      }
 }
@@ -490,26 +513,26 @@ void dfb_copy_buffer_32( u32 *src,
           return;
      x = drect->x;
      
-     switch (dst_surface->format) {
+     switch (dst_surface->config.format) {
           case DSPF_YV12:
           case DSPF_I420:               
-               if (dst_surface->format == DSPF_I420) {
-                    dst1 = dst  + dpitch   * dst_surface->height;
-                    dst2 = dst1 + dpitch/2 * dst_surface->height/2;
+               if (dst_surface->config.format == DSPF_I420) {
+                    dst1 = dst  + dpitch   * dst_surface->config.size.h;
+                    dst2 = dst1 + dpitch/2 * dst_surface->config.size.h/2;
                } else {
-                    dst2 = dst  + dpitch   * dst_surface->height;
-                    dst1 = dst2 + dpitch/2 * dst_surface->height/2;
+                    dst2 = dst  + dpitch   * dst_surface->config.size.h;
+                    dst1 = dst2 + dpitch/2 * dst_surface->config.size.h/2;
                }
                
                for (y = drect->y; y < drect->y + drect->h; y++) {
                     u8 *d[3];
                     
-                    d[0] = LINE_PTR( dst, dst_surface->caps, y,
-                                     dst_surface->height, dpitch ) + x;
-                    d[1] = LINE_PTR( dst1, dst_surface->caps, y/2,
-                                     dst_surface->height/2, dpitch/2 ) + x/2;
-                    d[2] = LINE_PTR( dst2, dst_surface->caps, y/2,
-                                     dst_surface->height/2, dpitch/2 ) + x/2;
+                    d[0] = LINE_PTR( dst, dst_surface->config.caps, y,
+                                     dst_surface->config.size.h, dpitch ) + x;
+                    d[1] = LINE_PTR( dst1, dst_surface->config.caps, y/2,
+                                     dst_surface->config.size.h/2, dpitch/2 ) + x/2;
+                    d[2] = LINE_PTR( dst2, dst_surface->config.caps, y/2,
+                                     dst_surface->config.size.h/2, dpitch/2 ) + x/2;
                                      
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
@@ -519,15 +542,15 @@ void dfb_copy_buffer_32( u32 *src,
                
           case DSPF_NV12:
           case DSPF_NV21:
-               dst1 = dst + dpitch * dst_surface->height;
+               dst1 = dst + dpitch * dst_surface->config.size.h;
                
                for (y = drect->y; y < drect->y + drect->h; y++) {
                     u8 *d[2];
                     
-                    d[0] = LINE_PTR( dst, dst_surface->caps, y,
-                                     dst_surface->height, dpitch ) + x;
-                    d[1] = LINE_PTR( dst1, dst_surface->caps, y/2,
-                                     dst_surface->height/2, dpitch ) + (x&~1);
+                    d[0] = LINE_PTR( dst, dst_surface->config.caps, y,
+                                     dst_surface->config.size.h, dpitch ) + x;
+                    d[1] = LINE_PTR( dst1, dst_surface->config.caps, y/2,
+                                     dst_surface->config.size.h/2, dpitch ) + (x&~1);
  
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
@@ -536,15 +559,15 @@ void dfb_copy_buffer_32( u32 *src,
                break;
           
           case DSPF_NV16:
-               dst1 = dst + dpitch * dst_surface->height;
+               dst1 = dst + dpitch * dst_surface->config.size.h;
                
                for (y = drect->y; y < drect->y + drect->h; y++) {
                     u8 *d[2];
                     
-                    d[0] = LINE_PTR( dst, dst_surface->caps, y,
-                                     dst_surface->height, dpitch ) + x;
-                    d[1] = LINE_PTR( dst1, dst_surface->caps, y,
-                                     dst_surface->height, dpitch ) + (x&~1);
+                    d[0] = LINE_PTR( dst, dst_surface->config.caps, y,
+                                     dst_surface->config.size.h, dpitch ) + x;
+                    d[1] = LINE_PTR( dst1, dst_surface->config.caps, y,
+                                     dst_surface->config.size.h, dpitch ) + (x&~1);
  
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
@@ -556,9 +579,9 @@ void dfb_copy_buffer_32( u32 *src,
                for (y = drect->y; y < drect->y + drect->h; y++) {
                     u8 *d[1];
                     
-                    d[0] = LINE_PTR( dst, dst_surface->caps,
-                                     y, dst_surface->height, dpitch ) +
-                           DFB_BYTES_PER_LINE( dst_surface->format, x );
+                    d[0] = LINE_PTR( dst, dst_surface->config.caps,
+                                     y, dst_surface->config.size.h, dpitch ) +
+                           DFB_BYTES_PER_LINE( dst_surface->config.format, x );
 
                     write_argb_span( src, d, drect->w, x, y, dst_surface );
                     
@@ -568,7 +591,7 @@ void dfb_copy_buffer_32( u32 *src,
      }
 }
 
-static int bilinear_make_fast_weights( PixopsFilter *filter, float x_scale, float y_scale )
+static int bilinear_make_fast_weights( PixopsFilter *filter, const float x_scale, const float y_scale )
 {
      int i_offset, j_offset;
      float *x_weights, *y_weights;
@@ -576,7 +599,7 @@ static int bilinear_make_fast_weights( PixopsFilter *filter, float x_scale, floa
 
      if (x_scale > 1.0) {      /* Bilinear */
           n_x = 2;
-          filter->x_offset = 0.5 * (1/x_scale - 1);
+          filter->x_offset = 0.5 * (1.0 / x_scale - 1);
      }
      else {                    /* Tile */
           n_x = D_ICEIL (1.0 + 1.0 / x_scale);
@@ -585,7 +608,7 @@ static int bilinear_make_fast_weights( PixopsFilter *filter, float x_scale, floa
 
      if (y_scale > 1.0) {      /* Bilinear */
           n_y = 2;
-          filter->y_offset = 0.5 * (1/y_scale - 1);
+          filter->y_offset = 0.5 * (1.0 / y_scale - 1);
      }
      else {                    /* Tile */
           n_y = D_ICEIL (1.0 + 1.0/y_scale);
@@ -636,13 +659,13 @@ static int bilinear_make_fast_weights( PixopsFilter *filter, float x_scale, floa
                     for (i = 0; i < n_x; i++) {
                          if (i < x) {
                               if (i + 1 > x)
-                                   x_weights[i] = MIN( i+ 1, x+ 1/x_scale ) -x;
+                                   x_weights[i] = MIN( i + 1, x + 1.0 / x_scale ) -x;
                               else
                                    x_weights[i] = 0;
                          }
                          else {
                               if (x + 1/x_scale > i)
-                                   x_weights[i] = MIN( i+ 1, x+ 1/x_scale ) -i;
+                                   x_weights[i] = MIN( i + 1, x + 1.0 / x_scale ) -i;
                               else
                                    x_weights[i] = 0;
                          }
@@ -658,13 +681,13 @@ static int bilinear_make_fast_weights( PixopsFilter *filter, float x_scale, floa
                     for (i = 0; i < n_y; i++) {
                          if (i < y) {
                               if (i + 1 > y)
-                                   y_weights[i] = MIN( i+ 1, y+ 1/y_scale ) -y;
+                                   y_weights[i] = MIN( i + 1, y + 1.0 / y_scale ) -y;
                               else
                                    y_weights[i] = 0;
                          }
                          else {
                               if (y + 1/y_scale > i)
-                                   y_weights[i] = MIN( i+ 1, y+ 1/y_scale ) -i;
+                                   y_weights[i] = MIN( i + 1, y + 1.0 / y_scale ) -i;
                               else
                                    y_weights[i] = 0;
                          }
@@ -809,19 +832,19 @@ void dfb_scale_linear_32( u32 *src, int sw, int sh,
      scaled_x_offset = D_IFLOOR( filter.x_offset * (1 << SCALE_SHIFT) );
      sy = D_IFLOOR( filter.y_offset * (1 << SCALE_SHIFT) );
      
-     switch (dst_surface->format) {
+     switch (dst_surface->config.format) {
           case DSPF_I420:
-               dst1 = dst  + dpitch   * dst_surface->height;
-               dst2 = dst1 + dpitch/2 * dst_surface->height/2;
+               dst1 = dst  + dpitch   * dst_surface->config.size.h;
+               dst2 = dst1 + dpitch/2 * dst_surface->config.size.h/2;
                break;
           case DSPF_YV12:
-               dst2 = dst  + dpitch   * dst_surface->height;
-               dst1 = dst2 + dpitch/2 * dst_surface->height/2;
+               dst2 = dst  + dpitch   * dst_surface->config.size.h;
+               dst1 = dst2 + dpitch/2 * dst_surface->config.size.h/2;
                break;
           case DSPF_NV12:
           case DSPF_NV21:
           case DSPF_NV16:
-               dst1 = dst + dpitch * dst_surface->height;
+               dst1 = dst + dpitch * dst_surface->config.size.h;
                break;
           default:
                break;
@@ -856,7 +879,7 @@ void dfb_scale_linear_32( u32 *src, int sw, int sh,
 
                y_start++;
           }
-         
+
           sx = scaled_x_offset;
           x_start = sx >> SCALE_SHIFT;
 
@@ -887,26 +910,26 @@ void dfb_scale_linear_32( u32 *src, int sw, int sh,
 
           sy += y_step;
           
-          d[0] = LINE_PTR( dst, dst_surface->caps,
-                           i, dst_surface->height, dpitch ) +
-                 DFB_BYTES_PER_LINE( dst_surface->format, drect->x );
+          d[0] = LINE_PTR( dst, dst_surface->config.caps,
+                           i, dst_surface->config.size.h, dpitch ) +
+                 DFB_BYTES_PER_LINE( dst_surface->config.format, drect->x );
                  
-          switch (dst_surface->format) {
+          switch (dst_surface->config.format) {
                case DSPF_I420:
                case DSPF_YV12:
-                    d[1] = LINE_PTR( dst1, dst_surface->caps, i/2,
-                                     dst_surface->height/2, dpitch/2 ) + drect->x/2;
-                    d[2] = LINE_PTR( dst2, dst_surface->caps, i/2,
-                                     dst_surface->height/2, dpitch/2 ) + drect->x/2;
+                    d[1] = LINE_PTR( dst1, dst_surface->config.caps, i/2,
+                                     dst_surface->config.size.h/2, dpitch/2 ) + drect->x/2;
+                    d[2] = LINE_PTR( dst2, dst_surface->config.caps, i/2,
+                                     dst_surface->config.size.h/2, dpitch/2 ) + drect->x/2;
                     break;
                case DSPF_NV12:
                case DSPF_NV21:
-                    d[1] = LINE_PTR( dst1, dst_surface->caps, i/2,
-                                     dst_surface->height/2, dpitch ) + (drect->x&~1);
+                    d[1] = LINE_PTR( dst1, dst_surface->config.caps, i/2,
+                                     dst_surface->config.size.h/2, dpitch ) + (drect->x&~1);
                     break;
                case DSPF_NV16:
-                    d[1] = LINE_PTR( dst1, dst_surface->caps, i,
-                                     dst_surface->height, dpitch ) + (drect->x&~1);
+                    d[1] = LINE_PTR( dst1, dst_surface->config.caps, i,
+                                     dst_surface->config.size.h, dpitch ) + (drect->x&~1);
                     break;
                default:
                     break;
