@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -38,12 +38,21 @@
 #include <sys/time.h>
 #include <time.h>
 
+#include <direct/debug.h>
 #include <direct/messages.h>
 #include <direct/util.h>
 
 #include <misc/util.h>
 
 
+D_DEBUG_DOMAIN( DFB_Updates, "DirectFB/Updates", "DirectFB Updates" );
+
+/**********************************************************************************************************************/
+
+const DirectFBPixelFormatNames( dfb_pixelformat_names );
+
+/**********************************************************************************************************************/
+
 bool
 dfb_region_intersect( DFBRegion *region,
                       int x1, int y1, int x2, int y2 )
@@ -331,7 +340,12 @@ dfb_updates_add( DFBUpdates      *updates,
      D_ASSERT( updates->num_regions >= 0 );
      D_ASSERT( updates->num_regions <= updates->max_regions );
 
+     D_DEBUG_AT( DFB_Updates, "%s( %p, %4d,%4d-%4dx%4d )\n", __FUNCTION__, updates,
+                 DFB_RECTANGLE_VALS_FROM_REGION(region) );
+
      if (updates->num_regions == 0) {
+          D_DEBUG_AT( DFB_Updates, "  -> added as first\n" );
+
           updates->regions[0]  = updates->bounding = *region;
           updates->num_regions = 1;
 
@@ -339,16 +353,18 @@ dfb_updates_add( DFBUpdates      *updates,
      }
 
      for (i=0; i<updates->num_regions; i++) {
-          DFBRegion inter = updates->regions[i];
+          if (dfb_region_region_extends( &updates->regions[i], region ) ||
+              dfb_region_region_intersects( &updates->regions[i], region ))
+          {
+               D_DEBUG_AT( DFB_Updates, "  -> combined with [%d] %4d,%4d-%4dx%4d\n", i,
+                           DFB_RECTANGLE_VALS_FROM_REGION(&updates->regions[i]) );
 
-          if (dfb_region_region_intersect( &inter, region )) {
-               DFBRegion uni = updates->regions[i];
+               dfb_region_region_union( &updates->regions[i], region );
 
-               dfb_region_region_union( &uni, region );
+               dfb_region_region_union( &updates->bounding, region );
 
-               updates->regions[i] = uni;
-
-               dfb_region_region_union( &updates->bounding, &uni );
+               D_DEBUG_AT( DFB_Updates, "  -> resulting in  [%d] %4d,%4d-%4dx%4d\n", i,
+                           DFB_RECTANGLE_VALS_FROM_REGION(&updates->regions[i]) );
 
                return;
           }
@@ -359,11 +375,17 @@ dfb_updates_add( DFBUpdates      *updates,
 
           updates->regions[0]  = updates->bounding;
           updates->num_regions = 1;
+
+          D_DEBUG_AT( DFB_Updates, "  -> collapsing to [0] %4d,%4d-%4dx%4d\n",
+                      DFB_RECTANGLE_VALS_FROM_REGION(&updates->regions[0]) );
      }
      else {
           updates->regions[updates->num_regions++] = *region;
 
           dfb_region_region_union( &updates->bounding, region );
+
+          D_DEBUG_AT( DFB_Updates, "  -> added as      [%d] %4d,%4d-%4dx%4d\n", updates->num_regions - 1,
+                      DFB_RECTANGLE_VALS_FROM_REGION(&updates->regions[updates->num_regions - 1]) );
      }
 }
 
@@ -406,3 +428,44 @@ dfb_updates_stat( DFBUpdates *updates,
                           (updates->bounding.y2 - updates->bounding.y1 + 1);
 }
 
+void
+dfb_updates_get_rectangles( DFBUpdates   *updates,
+                            DFBRectangle *ret_rects,
+                            int          *ret_num )
+{
+     D_MAGIC_ASSERT( updates, DFBUpdates );
+     D_ASSERT( updates->regions != NULL );
+     D_ASSERT( updates->num_regions >= 0 );
+     D_ASSERT( updates->num_regions <= updates->max_regions );
+
+     switch (updates->num_regions) {
+          case 0:
+               *ret_num = 0;
+               break;
+
+          default: {
+               int n, d, total, bounding;
+
+               dfb_updates_stat( updates, &total, &bounding );
+
+               n = updates->max_regions - updates->num_regions + 1;
+               d = n + 1;
+
+               /* Try to optimize updates. Use individual regions only if not too much overhead. */
+               if (total < bounding * n / d) {
+                    *ret_num = updates->num_regions;
+
+                    for (n=0; n<updates->num_regions; n++)
+                         ret_rects[n] = DFB_RECTANGLE_INIT_FROM_REGION( &updates->regions[n] );
+
+                    break;
+               }
+          }
+          /* fall through */
+
+          case 1:
+               *ret_num   = 1;
+               *ret_rects = DFB_RECTANGLE_INIT_FROM_REGION( &updates->bounding );
+               break;
+     }
+}
