@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -50,8 +51,9 @@
 #include <core/layers.h>
 #include <core/palette.h>
 #include <core/state.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/windows.h>
+#include <core/wm.h>
 #include <core/windowstack.h>
 #include <core/windows_internal.h> /* FIXME */
 
@@ -99,6 +101,8 @@ typedef struct {
 
      bool               detached;
      bool               destroyed;
+
+     CoreDFB           *core;
 } IDirectFBWindow_data;
 
 
@@ -115,11 +119,10 @@ IDirectFBWindow_Destruct( IDirectFBWindow *thiz )
           dfb_window_detach( data->window, &data->reaction );
      }
 
-     if (!data->destroyed) {
-          D_DEBUG_AT( IDirectFB_Window, "  -> unrefing...\n" );
+     /* this will destroy the fusion object and (eventually) the window */
+     D_DEBUG_AT( IDirectFB_Window, "  -> unrefing...\n" );
 
-          dfb_window_unref( data->window );
-     }
+     dfb_window_unref( data->window );
 
      D_DEBUG_AT( IDirectFB_Window, "  -> releasing surface...\n" );
 
@@ -136,21 +139,25 @@ IDirectFBWindow_Destruct( IDirectFBWindow *thiz )
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
-static DFBResult
+static DirectResult
 IDirectFBWindow_AddRef( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      data->ref++;
 
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBWindow_Release( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (--data->ref == 0)
           IDirectFBWindow_Destruct( thiz );
 
@@ -165,6 +172,8 @@ IDirectFBWindow_CreateEventBuffer( IDirectFBWindow       *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -187,6 +196,8 @@ IDirectFBWindow_AttachEventBuffer( IDirectFBWindow       *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -197,12 +208,25 @@ IDirectFBWindow_AttachEventBuffer( IDirectFBWindow       *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBWindow_DetachEventBuffer( IDirectFBWindow       *thiz,
+                                   IDirectFBEventBuffer  *buffer )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     return IDirectFBEventBuffer_DetachWindow( buffer, data->window );
+}
+
 static DFBResult
 IDirectFBWindow_EnableEvents( IDirectFBWindow       *thiz,
                               DFBWindowEventType     mask )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -218,6 +242,8 @@ IDirectFBWindow_DisableEvents( IDirectFBWindow       *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -233,6 +259,8 @@ IDirectFBWindow_GetID( IDirectFBWindow *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -249,19 +277,27 @@ IDirectFBWindow_GetPosition( IDirectFBWindow *thiz,
                              int             *x,
                              int             *y )
 {
+     DFBInsets insets;
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
      if (!x && !y)
           return DFB_INVARG;
 
+     dfb_windowstack_lock( data->window->stack );
+     dfb_wm_get_insets( data->window->stack, data->window, &insets );
+     dfb_windowstack_unlock( data->window->stack );
+
      if (x)
-          *x = data->window->config.bounds.x;
+          *x = data->window->config.bounds.x-insets.l;
 
      if (y)
-          *y = data->window->config.bounds.y;
+          *y = data->window->config.bounds.y-insets.t;
 
      return DFB_OK;
 }
@@ -271,19 +307,26 @@ IDirectFBWindow_GetSize( IDirectFBWindow *thiz,
                          int             *width,
                          int             *height )
 {
+     DFBInsets insets;
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
      if (!width && !height)
           return DFB_INVARG;
 
+     dfb_windowstack_lock( data->window->stack );
+     dfb_wm_get_insets( data->window->stack, data->window, &insets );
+     dfb_windowstack_unlock( data->window->stack );
+
      if (width)
-          *width = data->window->config.bounds.w;
+          *width = data->window->config.bounds.w-insets.l-insets.r;
 
      if (height)
-          *height = data->window->config.bounds.h;
+          *height = data->window->config.bounds.h-insets.t-insets.b;
 
      return DFB_OK;
 }
@@ -294,13 +337,15 @@ IDirectFBWindow_GetSurface( IDirectFBWindow   *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
      if (!surface)
           return DFB_INVARG;
 
-     if (data->window->caps & DWCAPS_INPUTONLY)
+     if (data->window->caps & (DWCAPS_INPUTONLY | DWCAPS_COLOR))
           return DFB_UNSUPPORTED;
 
      if (!data->surface) {
@@ -308,9 +353,9 @@ IDirectFBWindow_GetSurface( IDirectFBWindow   *thiz,
 
           DIRECT_ALLOCATE_INTERFACE( *surface, IDirectFBSurface );
 
-          ret = IDirectFBSurface_Window_Construct( *surface,
+          ret = IDirectFBSurface_Window_Construct( *surface, NULL,
                                                    NULL, NULL, data->window,
-                                                   DSCAPS_DOUBLE );
+                                                   DSCAPS_DOUBLE, data->core );
           if (ret)
                return ret;
 
@@ -324,12 +369,91 @@ IDirectFBWindow_GetSurface( IDirectFBWindow   *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBWindow_SetProperty( IDirectFBWindow   *thiz,
+                             const char        *key,
+                             void              *value,
+                             void            **old_value )
+{
+     DFBResult ret;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     /* Check arguments */
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     if (!key)
+          return DFB_INVARG;
+
+     dfb_windowstack_lock( data->window->stack );
+     ret = dfb_wm_set_window_property( data->window->stack, data->window, key, value, old_value );
+     dfb_windowstack_unlock( data->window->stack );
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBWindow_GetProperty( IDirectFBWindow  *thiz,
+                             const char       *key,
+                             void            **ret_value )
+{
+     DFBResult ret;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     if (!key)
+          return DFB_INVARG;
+
+     if (!ret_value)
+          return DFB_INVARG;
+
+     dfb_windowstack_lock( data->window->stack );
+     ret = dfb_wm_get_window_property( data->window->stack, data->window, key, ret_value );
+     dfb_windowstack_unlock( data->window->stack );
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBWindow_RemoveProperty( IDirectFBWindow  *thiz,
+                                const char       *key,
+                                void            **ret_value )
+{
+     DFBResult ret;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     if (!key)
+          return DFB_INVARG;
+
+     dfb_windowstack_lock( data->window->stack );
+     ret = dfb_wm_remove_window_property( data->window->stack, data->window, key, ret_value );
+     dfb_windowstack_unlock( data->window->stack );
+
+     return ret;
+}
+
 static DFBResult
 IDirectFBWindow_SetOptions( IDirectFBWindow  *thiz,
                             DFBWindowOptions  options )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      /* Check arguments */
      if (data->destroyed)
           return DFB_DESTROYED;
@@ -350,6 +474,8 @@ IDirectFBWindow_GetOptions( IDirectFBWindow  *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -362,16 +488,39 @@ IDirectFBWindow_GetOptions( IDirectFBWindow  *thiz,
 }
 
 static DFBResult
-IDirectFBWindow_SetColorKey( IDirectFBWindow *thiz,
-                             __u8             r,
-                             __u8             g,
-                             __u8             b )
+IDirectFBWindow_SetColor( IDirectFBWindow *thiz,
+                          u8               r,
+                          u8               g,
+                          u8               b,
+                          u8               a )
 {
-     __u32        key;
+     DFBColor color = { a: a, r: r, g: g, b: b };
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     dfb_window_set_color( data->window, color );
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBWindow_SetColorKey( IDirectFBWindow *thiz,
+                             u8               r,
+                             u8               g,
+                             u8               b )
+{
+     u32          key;
      CoreSurface *surface;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -380,10 +529,10 @@ IDirectFBWindow_SetColorKey( IDirectFBWindow *thiz,
 
      surface = data->window->surface;
 
-     if (DFB_PIXELFORMAT_IS_INDEXED( surface->format ))
+     if (DFB_PIXELFORMAT_IS_INDEXED( surface->config.format ))
           key = dfb_palette_search( surface->palette, r, g, b, 0x80 );
      else
-          key = dfb_color_to_pixel( surface->format, r, g, b );
+          key = dfb_color_to_pixel( surface->config.format, r, g, b );
 
      return dfb_window_set_colorkey( data->window, key );
 }
@@ -392,10 +541,12 @@ static DFBResult
 IDirectFBWindow_SetColorKeyIndex( IDirectFBWindow *thiz,
                                   unsigned int     index )
 {
-     __u32 key = index;
+     u32 key = index;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -416,6 +567,8 @@ IDirectFBWindow_SetOpaqueRegion( IDirectFBWindow *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -429,10 +582,12 @@ IDirectFBWindow_SetOpaqueRegion( IDirectFBWindow *thiz,
 
 static DFBResult
 IDirectFBWindow_SetOpacity( IDirectFBWindow *thiz,
-                            __u8             opacity )
+                            u8               opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -441,10 +596,12 @@ IDirectFBWindow_SetOpacity( IDirectFBWindow *thiz,
 
 static DFBResult
 IDirectFBWindow_GetOpacity( IDirectFBWindow *thiz,
-                            __u8            *opacity )
+                            u8              *opacity )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -466,6 +623,8 @@ IDirectFBWindow_SetCursorShape( IDirectFBWindow  *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -509,6 +668,8 @@ IDirectFBWindow_RequestFocus( IDirectFBWindow *thiz )
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -528,10 +689,12 @@ IDirectFBWindow_GrabKeyboard( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     return dfb_window_grab_keyboard( data->window );
+     return dfb_window_change_grab( data->window, CWMGT_KEYBOARD, true );
 }
 
 static DFBResult
@@ -539,10 +702,12 @@ IDirectFBWindow_UngrabKeyboard( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     return dfb_window_ungrab_keyboard( data->window );
+     return dfb_window_change_grab( data->window, CWMGT_KEYBOARD, false );
 }
 
 static DFBResult
@@ -550,10 +715,12 @@ IDirectFBWindow_GrabPointer( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     return dfb_window_grab_pointer( data->window );
+     return dfb_window_change_grab( data->window, CWMGT_POINTER, true );
 }
 
 static DFBResult
@@ -561,10 +728,12 @@ IDirectFBWindow_UngrabPointer( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     return dfb_window_ungrab_pointer( data->window );
+     return dfb_window_change_grab( data->window, CWMGT_POINTER, false );
 }
 
 static DFBResult
@@ -574,6 +743,8 @@ IDirectFBWindow_GrabKey( IDirectFBWindow            *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -587,6 +758,8 @@ IDirectFBWindow_UngrabKey( IDirectFBWindow            *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -598,6 +771,8 @@ IDirectFBWindow_Move( IDirectFBWindow *thiz, int dx, int dy )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -610,12 +785,26 @@ IDirectFBWindow_Move( IDirectFBWindow *thiz, int dx, int dy )
 static DFBResult
 IDirectFBWindow_MoveTo( IDirectFBWindow *thiz, int x, int y )
 {
+     DFBResult ret;
+     DFBInsets insets;
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
-     return dfb_window_move( data->window, x, y, false );
+     dfb_windowstack_lock( data->window->stack );
+
+     dfb_wm_get_insets( data->window->stack, data->window, &insets );
+     x += insets.l;
+     y += insets.t;
+
+     ret = dfb_window_move( data->window, x, y, false );
+
+     dfb_windowstack_unlock( data->window->stack );
+
+     return ret;
 }
 
 static DFBResult
@@ -623,15 +812,30 @@ IDirectFBWindow_Resize( IDirectFBWindow *thiz,
                         int              width,
                         int              height )
 {
+     DFBResult ret;
+     DFBInsets insets;
+     
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
      if (width < 1 || width > 4096 || height < 1 || height > 4096)
           return DFB_INVARG;
+     
+     dfb_windowstack_lock( data->window->stack );
 
-     return dfb_window_resize( data->window, width, height );
+     dfb_wm_get_insets( data->window->stack, data->window, &insets );
+     width  += insets.l+insets.r;
+     height += insets.t+insets.b;
+
+     ret = dfb_window_resize( data->window, width, height );
+
+     dfb_windowstack_unlock( data->window->stack );
+
+     return ret;
 }
 
 static DFBResult
@@ -639,6 +843,8 @@ IDirectFBWindow_Raise( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -651,6 +857,8 @@ IDirectFBWindow_SetStackingClass( IDirectFBWindow        *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -671,6 +879,8 @@ IDirectFBWindow_Lower( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -682,6 +892,8 @@ IDirectFBWindow_RaiseToTop( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -693,6 +905,8 @@ IDirectFBWindow_LowerToBottom( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -707,6 +921,8 @@ IDirectFBWindow_PutAtop( IDirectFBWindow *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -731,6 +947,8 @@ IDirectFBWindow_PutBelow( IDirectFBWindow *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -754,6 +972,8 @@ IDirectFBWindow_Close( IDirectFBWindow *thiz )
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -769,6 +989,8 @@ IDirectFBWindow_Destroy( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
 
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
      if (data->destroyed)
           return DFB_DESTROYED;
 
@@ -779,10 +1001,286 @@ IDirectFBWindow_Destroy( IDirectFBWindow *thiz )
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBWindow_SetBounds( IDirectFBWindow *thiz,
+                           int              x,
+                           int              y,
+                           int              width,
+                           int              height )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     D_DEBUG_AT( IDirectFB_Window, "IDirectFBWindow_SetBounds( %d, %d - %dx%d )\n", x, y, width, height );
+
+     return dfb_window_set_bounds( data->window, x, y, width, height );
+}
+
+static DFBResult
+IDirectFBWindow_ResizeSurface( IDirectFBWindow *thiz,
+                               int              width,
+                               int              height )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     if (width < 1 || width > 4096 || height < 1 || height > 4096)
+          return DFB_INVARG;
+
+     return dfb_surface_reformat( data->window->surface, width, height, data->window->surface->config.format );
+}
+
+static DFBResult
+IDirectFBWindow_SetKeySelection( IDirectFBWindow               *thiz,
+                                 DFBWindowKeySelection          selection,
+                                 const DFBInputDeviceKeySymbol *keys,
+                                 unsigned int                   num_keys )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     /* What a lovely switch */
+     switch (selection) {
+         case DWKS_ALL:
+         case DWKS_NONE:
+             break;
+         case DWKS_LIST:
+             if (!keys || num_keys == 0)
+         default:
+                 return DFB_INVARG;
+     }
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     return dfb_window_set_key_selection( data->window, selection, keys, num_keys );
+}
+
+static DFBResult
+IDirectFBWindow_GrabUnselectedKeys( IDirectFBWindow *thiz )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     return dfb_window_change_grab( data->window, CWMGT_UNSELECTED_KEYS, true );
+}
+
+static DFBResult
+IDirectFBWindow_UngrabUnselectedKeys( IDirectFBWindow *thiz )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     return dfb_window_change_grab( data->window, CWMGT_UNSELECTED_KEYS, false );
+}
+
+static DFBResult
+IDirectFBWindow_Bind( IDirectFBWindow *thiz,
+                      IDirectFBWindow *source,
+                      int              x,
+                      int              y )
+{
+     IDirectFBWindow_data *source_data;
+ 
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     DIRECT_INTERFACE_GET_DATA_FROM(source, source_data, IDirectFBWindow);
+
+     if (source_data->destroyed)
+          return DFB_DESTROYED;
+
+     return dfb_window_bind( data->window, source_data->window, x, y );
+}
+
+static DFBResult
+IDirectFBWindow_Unbind( IDirectFBWindow *thiz,
+                        IDirectFBWindow *source )
+{
+     IDirectFBWindow_data *source_data;
+ 
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     DIRECT_INTERFACE_GET_DATA_FROM(source, source_data, IDirectFBWindow);
+
+     if (source_data->destroyed)
+          return DFB_DESTROYED;
+
+     return dfb_window_unbind( data->window, source_data->window );
+}
+
+static DFBResult
+CheckGeometry( const DFBWindowGeometry *geometry )
+{
+     if (!geometry)
+          return DFB_INVARG;
+
+     switch (geometry->mode) {
+          case DWGM_DEFAULT:
+          case DWGM_FOLLOW:
+               break;
+
+          case DWGM_RECTANGLE:
+               if (geometry->rectangle.x < 0 ||
+                   geometry->rectangle.y < 0 ||
+                   geometry->rectangle.w < 1 ||
+                   geometry->rectangle.h < 1)
+                    return DFB_INVARG;
+               break;
+
+          case DWGM_LOCATION:
+               if (geometry->location.x < 0.0f ||
+                   geometry->location.y < 0.0f ||
+                   geometry->location.w > 1.0f ||
+                   geometry->location.h > 1.0f ||
+                   geometry->location.w <= 0.0f ||
+                   geometry->location.h <= 0.0f ||
+                   geometry->location.x + geometry->location.w > 1.0f ||
+                   geometry->location.y + geometry->location.h > 1.0f)
+                    return DFB_INVARG;
+               break;
+
+          default:
+               return DFB_INVARG;
+     }
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBWindow_SetSrcGeometry( IDirectFBWindow         *thiz,
+                                const DFBWindowGeometry *geometry )
+{
+     DFBResult        ret;
+     CoreWindowConfig config;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     ret = CheckGeometry( geometry );
+     if (ret)
+          return ret;
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     config.src_geometry = *geometry;
+
+     return dfb_window_set_config( data->window, &config, CWCF_SRC_GEOMETRY );
+}
+
+static DFBResult
+IDirectFBWindow_SetDstGeometry( IDirectFBWindow         *thiz,
+                                const DFBWindowGeometry *geometry )
+{
+     DFBResult        ret;
+     CoreWindowConfig config;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     ret = CheckGeometry( geometry );
+     if (ret)
+          return ret;
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     config.dst_geometry = *geometry;
+
+     return dfb_window_set_config( data->window, &config, CWCF_DST_GEOMETRY );
+}
+
+static DFBResult
+IDirectFBWindow_SetAssociation( IDirectFBWindow *thiz,
+                                DFBWindowID      window_id )
+{
+     CoreWindowConfig config;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     config.association = window_id;
+
+     return dfb_window_set_config( data->window, &config, CWCF_ASSOCIATION );
+}
+
+static DFBResult
+IDirectFBWindow_SetApplicationID( IDirectFBWindow *thiz,
+                                  unsigned long    application_id )
+                                  
+{
+     CoreWindowConfig config;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     config.application_id = application_id;
+
+     return dfb_window_set_config( data->window, &config, CWCF_APPLICATION_ID );
+}
+
+static DFBResult
+IDirectFBWindow_GetApplicationID( IDirectFBWindow *thiz,
+                                  unsigned long   *ret_application_id )
+                                  
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     if (!ret_application_id)
+          return DFB_INVARG;
+
+     *ret_application_id = data->window->config.application_id;
+
+     return DFB_OK;
+}
+
 DFBResult
 IDirectFBWindow_Construct( IDirectFBWindow *thiz,
                            CoreWindow      *window,
-                           CoreLayer       *layer )
+                           CoreLayer       *layer,
+                           CoreDFB         *core )
 {
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBWindow)
 
@@ -792,6 +1290,7 @@ IDirectFBWindow_Construct( IDirectFBWindow *thiz,
      data->ref    = 1;
      data->window = window;
      data->layer  = layer;
+     data->core   = core;
 
      dfb_window_attach( window, IDirectFBWindow_React, data, &data->reaction );
 
@@ -799,14 +1298,19 @@ IDirectFBWindow_Construct( IDirectFBWindow *thiz,
      thiz->Release = IDirectFBWindow_Release;
      thiz->CreateEventBuffer = IDirectFBWindow_CreateEventBuffer;
      thiz->AttachEventBuffer = IDirectFBWindow_AttachEventBuffer;
+     thiz->DetachEventBuffer = IDirectFBWindow_DetachEventBuffer;
      thiz->EnableEvents = IDirectFBWindow_EnableEvents;
      thiz->DisableEvents = IDirectFBWindow_DisableEvents;
      thiz->GetID = IDirectFBWindow_GetID;
      thiz->GetPosition = IDirectFBWindow_GetPosition;
      thiz->GetSize = IDirectFBWindow_GetSize;
      thiz->GetSurface = IDirectFBWindow_GetSurface;
+     thiz->SetProperty = IDirectFBWindow_SetProperty;
+     thiz->GetProperty = IDirectFBWindow_GetProperty;
+     thiz->RemoveProperty = IDirectFBWindow_RemoveProperty;
      thiz->SetOptions = IDirectFBWindow_SetOptions;
      thiz->GetOptions = IDirectFBWindow_GetOptions;
+     thiz->SetColor = IDirectFBWindow_SetColor;
      thiz->SetColorKey = IDirectFBWindow_SetColorKey;
      thiz->SetColorKeyIndex = IDirectFBWindow_SetColorKeyIndex;
      thiz->SetOpaqueRegion = IDirectFBWindow_SetOpaqueRegion;
@@ -832,6 +1336,18 @@ IDirectFBWindow_Construct( IDirectFBWindow *thiz,
      thiz->PutBelow = IDirectFBWindow_PutBelow;
      thiz->Close = IDirectFBWindow_Close;
      thiz->Destroy = IDirectFBWindow_Destroy;
+     thiz->SetBounds = IDirectFBWindow_SetBounds;
+     thiz->ResizeSurface = IDirectFBWindow_ResizeSurface;
+     thiz->Bind = IDirectFBWindow_Bind;
+     thiz->Unbind = IDirectFBWindow_Unbind;
+     thiz->SetKeySelection = IDirectFBWindow_SetKeySelection;
+     thiz->GrabUnselectedKeys = IDirectFBWindow_GrabUnselectedKeys;
+     thiz->UngrabUnselectedKeys = IDirectFBWindow_UngrabUnselectedKeys;
+     thiz->SetSrcGeometry = IDirectFBWindow_SetSrcGeometry;
+     thiz->SetDstGeometry = IDirectFBWindow_SetDstGeometry;
+     thiz->SetAssociation = IDirectFBWindow_SetAssociation;
+     thiz->SetApplicationID = IDirectFBWindow_SetApplicationID;
+     thiz->GetApplicationID = IDirectFBWindow_GetApplicationID;
 
      return DFB_OK;
 }
@@ -846,24 +1362,15 @@ IDirectFBWindow_React( const void *msg_data,
      const DFBWindowEvent *evt  = msg_data;
      IDirectFBWindow_data *data = ctx;
 
-     D_DEBUG_AT( IDirectFB_Window, "IDirectFBWindow_React()\n");
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
 
      switch (evt->type) {
           case DWET_DESTROYED:
                D_DEBUG_AT( IDirectFB_Window, "  -> window destroyed\n" );
-
-               if (!data->destroyed) {
-                    data->destroyed = true;
-
-                    D_DEBUG_AT( IDirectFB_Window, "  -> unrefing...\n" );
-
-                    dfb_window_unref( data->window );
-
-                    D_DEBUG_AT( IDirectFB_Window, "  -> unref done.\n" );
-               }
-
+               
                data->detached = true;
-
+               data->destroyed = true;
+               
                return RS_REMOVE;
 
           case DWET_LEAVE:
