@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,15 +29,20 @@
 #ifndef __CORE__FBDEV_H__
 #define __CORE__FBDEV_H__
 
-#include <linux/fb.h>
+#include <asm/types.h>    /* Needs to be included before dfb_types.h */
 
 #include <core/coretypes.h>
 
+#include <core/layers_internal.h>
+
 #include <core/system.h>
 
 #include <fusion/call.h>
 #include <fusion/reactor.h>
 
+#include "agp.h"
+#include "fb.h"
+#include "surfacemanager.h"
 #include "vt.h"
 
 #ifndef FBIO_WAITFORVSYNC
@@ -56,6 +62,11 @@ typedef struct {
                                                 set by DirectFB */
      struct fb_var_screeninfo orig_var;      /* fbdev variable screeninfo
                                                 before DirectFB was started */
+
+     void                    *orig_cmap_memory;
+     void                    *temp_cmap_memory;
+     void                    *current_cmap_memory;
+
      struct fb_cmap           orig_cmap;     /* original palette */
 
      struct fb_cmap           current_cmap;  /* our copy of the cmap */
@@ -65,6 +76,8 @@ typedef struct {
      FusionCall               fbdev_ioctl;   /* ioctl rpc */
 
      unsigned long            page_mask;     /* PAGE_SIZE - 1 */
+
+     CoreSurfacePool          *pool;
      
      struct {
           int                 bus;
@@ -76,6 +89,15 @@ typedef struct {
           unsigned short      vendor;        /* Graphics device vendor id */
           unsigned short      model;         /* Graphics device model id */
      } device;
+
+     AGPShared               *agp;
+
+     FusionSHMPoolShared     *shmpool;
+     FusionSHMPoolShared     *shmpool_data;
+
+     CoreLayerRegionConfig    config;
+
+     SurfaceManager          *manager;
 } FBDevShared;
 
 typedef struct {
@@ -89,19 +111,32 @@ typedef struct {
      int                      fd;            /* file descriptor for /dev/fb */
 
      VirtualTerminal         *vt;
+
+     AGPDevice               *agp;
 } FBDev;
 
 /*
  * core init function, opens /dev/fb, get fbdev screeninfo
  * disables font acceleration, reads mode list
  */
-DFBResult dfb_fbdev_initialize();
-DFBResult dfb_fbdev_join();
+DFBResult dfb_fbdev_initialize( void );
+DFBResult dfb_fbdev_join( void );
 
 /*
  * deinitializes DirectFB fbdev stuff and restores fbdev settings
  */
 DFBResult dfb_fbdev_shutdown( bool emergency );
-DFBResult dfb_fbdev_leave( bool emergency );
+DFBResult dfb_fbdev_leave   ( bool emergency );
+
+const VideoMode *dfb_fbdev_find_mode( int                          width,
+                                      int                          height );
+DFBResult dfb_fbdev_test_mode       ( const VideoMode             *mode,
+                                      const CoreLayerRegionConfig *config );
+DFBResult dfb_fbdev_test_mode_simple( const VideoMode             *mode );
+
+DFBResult dfb_fbdev_set_mode        ( const VideoMode             *mode,
+                                      CoreSurface                 *surface,
+                                      unsigned int                 xoffset,
+                                      unsigned int                 yoffset );
 
 #endif
