@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -27,6 +28,8 @@
 
 #include <config.h>
 
+#include <asm/types.h>    /* Needs to be included before dfb_types.h */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -46,6 +49,7 @@
 #include <misc/conf.h>
 #include <misc/util.h>
 
+#include <direct/debug.h>
 #include <direct/mem.h>
 #include <direct/messages.h>
 #include <direct/thread.h>
@@ -60,6 +64,8 @@
 #include "fb.h"
 #include "vt.h"
 
+D_DEBUG_DOMAIN( VT, "FBDev/VT", "FBDev System Module VT Handling" );
+
 /*
  *  FIXME: the following looks like a bad hack.
  *
@@ -85,24 +91,28 @@ extern FBDev *dfb_fbdev;
 
 static VirtualTerminal *dfb_vt = NULL;
 
-static DFBResult vt_init_switching();
+static DFBResult vt_init_switching( void );
 static int       vt_get_fb( int vt );
 static void      vt_set_fb( int vt, int fb );
 static void     *vt_thread( DirectThread *thread, void *arg );
 
 DFBResult
-dfb_vt_initialize()
+dfb_vt_initialize( void )
 {
      DFBResult ret;
      struct vt_stat vs;
 
+     D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
+
      dfb_vt = D_CALLOC( 1, sizeof(VirtualTerminal) );
+     if (!dfb_vt)
+          return D_OOM();
 
      setsid();
-     dfb_vt->fd0 = open( "/dev/tty0", O_RDONLY );
+     dfb_vt->fd0 = open( "/dev/tty0", O_RDONLY | O_NOCTTY );
      if (dfb_vt->fd0 < 0) {
           if (errno == ENOENT) {
-               dfb_vt->fd0 = open( "/dev/vc/0", O_RDONLY );
+               dfb_vt->fd0 = open( "/dev/vc/0", O_RDONLY | O_NOCTTY );
                if (dfb_vt->fd0 < 0) {
                     if (errno == ENOENT) {
                          D_PERROR( "DirectFB/core/vt: Couldn't open "
@@ -141,22 +151,30 @@ dfb_vt_initialize()
 
 
      if (!dfb_config->vt_switch) {
-          dfb_vt->num = dfb_vt->prev;
+          if (dfb_config->vt_num != -1)
+               dfb_vt->num = dfb_config->vt_num;
+          else
+               dfb_vt->num = dfb_vt->prev;
 
           /* move vt to framebuffer */
           dfb_vt->old_fb = vt_get_fb( dfb_vt->num );
           vt_set_fb( dfb_vt->num, -1 );
      }
      else {
-          int n;
+          if (dfb_config->vt_num == -1) {
+               int n;
 
-          n = ioctl( dfb_vt->fd0, VT_OPENQRY, &dfb_vt->num );
-          if (n < 0 || dfb_vt->num == -1) {
-               D_PERROR( "DirectFB/core/vt: Cannot allocate VT!\n" );
-               close( dfb_vt->fd0 );
-               D_FREE( dfb_vt );
-               dfb_vt = NULL;
-               return DFB_INIT;
+               n = ioctl( dfb_vt->fd0, VT_OPENQRY, &dfb_vt->num );
+               if (n < 0 || dfb_vt->num == -1) {
+                    D_PERROR( "DirectFB/core/vt: Cannot allocate VT!\n" );
+                    close( dfb_vt->fd0 );
+                    D_FREE( dfb_vt );
+                    dfb_vt = NULL;
+                    return DFB_INIT;
+               }
+          }
+          else {
+               dfb_vt->num = dfb_config->vt_num;
           }
 
           /* move vt to framebuffer */
@@ -190,10 +208,10 @@ dfb_vt_initialize()
      ret = vt_init_switching();
      if (ret) {
           if (dfb_config->vt_switch) {
-               D_DEBUG( "switching back...\n" );
+               D_DEBUG_AT( VT, "  -> switching back...\n" );
                ioctl( dfb_vt->fd0, VT_ACTIVATE, dfb_vt->prev );
                ioctl( dfb_vt->fd0, VT_WAITACTIVE, dfb_vt->prev );
-               D_DEBUG( "...switched back\n" );
+               D_DEBUG_AT( VT, "  -> ...switched back\n" );
                ioctl( dfb_vt->fd0, VT_DISALLOCATE, dfb_vt->num );
           }
 
@@ -209,8 +227,10 @@ dfb_vt_initialize()
 }
 
 DFBResult
-dfb_vt_join()
+dfb_vt_join( void )
 {
+     D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
+
      dfb_vt_detach( true );
 
      return DFB_OK;
@@ -222,18 +242,17 @@ dfb_vt_shutdown( bool emergency )
      const char cursoron_str[] = "\033[?0;0;0c";
      const char blankon_str[] = "\033[9;10]";
 
+     D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
+
      if (!dfb_vt)
           return DFB_OK;
 
-     write( dfb_vt->fd, cursoron_str, sizeof(cursoron_str) );
-     write( dfb_vt->fd, blankon_str, sizeof(blankon_str) );
-
      if (dfb_config->vt_switching) {
           if (ioctl( dfb_vt->fd, VT_SETMODE, &dfb_vt->vt_mode ) < 0)
                D_PERROR( "DirectFB/fbdev/vt: Unable to restore VT mode!!!\n" );
 
-          sigaction( SIGUSR1, &dfb_vt->sig_usr1, NULL );
-          sigaction( SIGUSR2, &dfb_vt->sig_usr2, NULL );
+          sigaction( SIG_SWITCH_FROM, &dfb_vt->sig_usr1, NULL );
+          sigaction( SIG_SWITCH_TO, &dfb_vt->sig_usr2, NULL );
 
           direct_thread_cancel( dfb_vt->thread );
           direct_thread_join( dfb_vt->thread );
@@ -247,9 +266,13 @@ dfb_vt_shutdown( bool emergency )
           if (ioctl( dfb_vt->fd, KDSETMODE, KD_TEXT ) < 0)
                D_PERROR( "DirectFB/Keyboard: KD_TEXT failed!\n" );
      }
+     else {
+          write( dfb_vt->fd, blankon_str, sizeof(blankon_str) );
+     }
+     write( dfb_vt->fd, cursoron_str, sizeof(cursoron_str) );
 
      if (dfb_config->vt_switch) {
-          D_DEBUG( "switching back...\n" );
+          D_DEBUG_AT( VT, "  -> switching back...\n" );
 
           if (ioctl( dfb_vt->fd0, VT_ACTIVATE, dfb_vt->prev ) < 0)
                D_PERROR( "DirectFB/core/vt: VT_ACTIVATE" );
@@ -257,7 +280,7 @@ dfb_vt_shutdown( bool emergency )
           if (ioctl( dfb_vt->fd0, VT_WAITACTIVE, dfb_vt->prev ) < 0)
                D_PERROR( "DirectFB/core/vt: VT_WAITACTIVE" );
 
-          D_DEBUG( "switched back...\n" );
+          D_DEBUG_AT( VT, "  -> switched back...\n" );
 
           usleep( 40000 );
 
@@ -293,17 +316,21 @@ dfb_vt_shutdown( bool emergency )
 DFBResult
 dfb_vt_leave( bool emergency )
 {
+     D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
+
      return DFB_OK;
 }
 
 DFBResult
 dfb_vt_detach( bool force )
 {
+     D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
+
      if (dfb_config->vt_switch || force) {
           int            fd;
           struct vt_stat vt_state;
 
-          fd = open( "/dev/tty", O_RDONLY );
+          fd = open( "/dev/tty", O_RDONLY | O_NOCTTY );
           if (fd < 0) {
                if (errno == ENXIO)
                     return DFB_OK;
@@ -332,10 +359,12 @@ dfb_vt_detach( bool force )
 bool
 dfb_vt_switch( int num )
 {
+     D_DEBUG_AT( VT, "%s( %d )\n", __FUNCTION__, num );
+
      if (!dfb_config->vt_switching)
           return false;
 
-     D_DEBUG( "DirectFB/fbdev/vt: switching to vt %d...\n", num );
+     D_DEBUG_AT( VT, "  -> switching to vt %d...\n", num );
 
      if (ioctl( dfb_vt->fd0, VT_ACTIVATE, num ) < 0)
           D_PERROR( "DirectFB/fbdev/vt: VT_ACTIVATE failed\n" );
@@ -346,13 +375,14 @@ dfb_vt_switch( int num )
 static void *
 vt_thread( DirectThread *thread, void *arg )
 {
+     D_DEBUG_AT( VT, "%s( %p, %p )\n", __FUNCTION__, thread, arg );
+
      pthread_mutex_lock( &dfb_vt->lock );
 
      while (true) {
           direct_thread_testcancel( thread );
 
-          D_DEBUG( "DirectFB/fbdev/vt: %s (%d)\n",
-                    __FUNCTION__, dfb_vt->vt_sig);
+          D_DEBUG_AT( VT, "...%s (signal %d)\n", __FUNCTION__, dfb_vt->vt_sig);
 
           switch (dfb_vt->vt_sig) {
                default:
@@ -363,28 +393,31 @@ vt_thread( DirectThread *thread, void *arg )
                     pthread_cond_wait( &dfb_vt->wait, &dfb_vt->lock );
                     continue;
 
-               case SIGUSR1:
-                    if (ioctl( dfb_vt->fd, VT_RELDISP,
-                               dfb_core_suspend( NULL ) == DFB_OK ? 1 : 0 ) < 0)
-                         D_PERROR( "DirectFB/fbdev/vt: VT_RELDISP failed\n" );
+               case SIG_SWITCH_FROM:
+                    if (dfb_core_suspend( dfb_fbdev->core ) == DFB_OK) {
+                         if (ioctl( dfb_vt->fd, VT_RELDISP, VT_ACKACQ ) < 0)
+                              D_PERROR( "DirectFB/fbdev/vt: VT_RELDISP failed\n" );
+                    }
 
                     break;
 
-               case SIGUSR2:
-                    dfb_core_resume( NULL );
+               case SIG_SWITCH_TO:
+                    if (dfb_core_resume( dfb_fbdev->core ) == DFB_OK) {
+                         if (ioctl( dfb_vt->fd, VT_RELDISP, VT_ACKACQ ) < 0)
+                              D_PERROR( "DirectFB/fbdev/vt: VT_RELDISP failed\n" );
 
-                    if (ioctl( dfb_vt->fd, VT_RELDISP, 2 ) < 0)
-                         D_PERROR( "DirectFB/fbdev/vt: VT_RELDISP failed\n" );
-
-                    if (dfb_config->kd_graphics) {
-                         if (ioctl( dfb_vt->fd, KDSETMODE, KD_GRAPHICS ) < 0)
-                              D_PERROR( "DirectFB/fbdev/vt: KD_GRAPHICS failed!\n" );
+                         if (dfb_config->kd_graphics) {
+                              if (ioctl( dfb_vt->fd, KDSETMODE, KD_GRAPHICS ) < 0)
+                                   D_PERROR( "DirectFB/fbdev/vt: KD_GRAPHICS failed!\n" );
+                         }
                     }
 
                     break;
           }
 
           dfb_vt->vt_sig = -1;
+
+          pthread_cond_signal( &dfb_vt->wait );
      }
 
      return NULL;
@@ -393,10 +426,13 @@ vt_thread( DirectThread *thread, void *arg )
 static void
 vt_switch_handler( int signum )
 {
-     D_DEBUG( "DirectFB/fbdev/vt: %s (%d)\n", __FUNCTION__, signum);
+     D_DEBUG_AT( VT, "%s( %d )\n", __FUNCTION__, signum );
 
      pthread_mutex_lock( &dfb_vt->lock );
 
+     while (dfb_vt->vt_sig != -1)
+          pthread_cond_wait( &dfb_vt->wait, &dfb_vt->lock );
+
      dfb_vt->vt_sig = signum;
 
      pthread_cond_signal( &dfb_vt->wait );
@@ -405,18 +441,22 @@ vt_switch_handler( int signum )
 }
 
 static DFBResult
-vt_init_switching()
+vt_init_switching( void )
 {
+     const char cursoroff_str[] = "\033[?1;0;0c";
+     const char blankoff_str[] = "\033[9;0]";
      char buf[32];
 
+     D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
+
      /* FIXME: Opening the device should be moved out of this function. */
 
      snprintf(buf, 32, "/dev/tty%d", dfb_vt->num);
-     dfb_vt->fd = open( buf, O_RDWR );
+     dfb_vt->fd = open( buf, O_RDWR | O_NOCTTY );
      if (dfb_vt->fd < 0) {
           if (errno == ENOENT) {
                snprintf(buf, 32, "/dev/vc/%d", dfb_vt->num);
-               dfb_vt->fd = open( buf, O_RDWR );
+               dfb_vt->fd = open( buf, O_RDWR | O_NOCTTY );
                if (dfb_vt->fd < 0) {
                     if (errno == ENOENT) {
                          D_PERROR( "DirectFB/core/vt: Couldn't open "
@@ -437,6 +477,11 @@ vt_init_switching()
           }
      }
 
+     /* attach to the new TTY before doing anything like KDSETMODE with it,
+        otherwise we'd get access denied error: */
+     ioctl( dfb_vt->fd, TIOCSCTTY, 0 );
+
+     write( dfb_vt->fd, cursoroff_str, sizeof(cursoroff_str) );
      if (dfb_config->kd_graphics) {
           if (ioctl( dfb_vt->fd, KDSETMODE, KD_GRAPHICS ) < 0) {
                D_PERROR( "DirectFB/fbdev/vt: KD_GRAPHICS failed!\n" );
@@ -444,10 +489,8 @@ vt_init_switching()
                return DFB_INIT;
           }
      }
-
-     if (dfb_config->vt_switch) {
-          ioctl( dfb_vt->fd0, TIOCNOTTY, 0 );
-          ioctl( dfb_vt->fd, TIOCSCTTY, 0 );
+     else {
+          write( dfb_vt->fd, blankoff_str, sizeof(blankoff_str) );
      }
 
      if (dfb_config->vt_switching) {
@@ -456,10 +499,10 @@ vt_init_switching()
 
           memset( &sig_tty, 0, sizeof( sig_tty ) );
           sig_tty.sa_handler = vt_switch_handler;
-          sigemptyset( &sig_tty.sa_mask );
+          sigfillset( &sig_tty.sa_mask );
 
-          if (sigaction( SIGUSR1, &sig_tty, &dfb_vt->sig_usr1 ) ||
-              sigaction( SIGUSR2, &sig_tty, &dfb_vt->sig_usr2 )) {
+          if (sigaction( SIG_SWITCH_FROM, &sig_tty, &dfb_vt->sig_usr1 ) ||
+              sigaction( SIG_SWITCH_TO, &sig_tty, &dfb_vt->sig_usr2 )) {
                D_PERROR( "DirectFB/fbdev/vt: sigaction failed!\n" );
                close( dfb_vt->fd );
                return DFB_INIT;
@@ -468,14 +511,14 @@ vt_init_switching()
 
           vt.mode   = VT_PROCESS;
           vt.waitv  = 0;
-          vt.relsig = SIGUSR1;
-          vt.acqsig = SIGUSR2;
+          vt.relsig = SIG_SWITCH_FROM;
+          vt.acqsig = SIG_SWITCH_TO;
 
           if (ioctl( dfb_vt->fd, VT_SETMODE, &vt ) < 0) {
                D_PERROR( "DirectFB/fbdev/vt: VT_SETMODE failed!\n" );
 
-               sigaction( SIGUSR1, &dfb_vt->sig_usr1, NULL );
-               sigaction( SIGUSR2, &dfb_vt->sig_usr2, NULL );
+               sigaction( SIG_SWITCH_FROM, &dfb_vt->sig_usr1, NULL );
+               sigaction( SIG_SWITCH_TO, &dfb_vt->sig_usr2, NULL );
 
                close( dfb_vt->fd );
 
@@ -499,6 +542,8 @@ vt_get_fb( int vt )
 {
      struct fb_con2fbmap c2m;
 
+     D_DEBUG_AT( VT, "%s( %d )\n", __FUNCTION__, vt );
+
      c2m.console = vt;
 
      if (ioctl( dfb_fbdev->fd, FBIOGET_CON2FBMAP, &c2m )) {
@@ -507,6 +552,8 @@ vt_get_fb( int vt )
           return 0;
      }
 
+     D_DEBUG_AT( VT, "  -> %d\n", c2m.framebuffer );
+
      return c2m.framebuffer;
 }
 
@@ -516,6 +563,8 @@ vt_set_fb( int vt, int fb )
      struct fb_con2fbmap c2m;
      struct stat         sbf;
 
+     D_DEBUG_AT( VT, "%s( %d, %d )\n", __FUNCTION__, vt, fb );
+
      if (fstat( dfb_fbdev->fd, &sbf )) {
           D_PERROR( "DirectFB/FBDev/vt: Could not fstat fb device!\n" );
           return;
