@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -61,8 +61,8 @@ typedef struct {
 /*
  * allocates and switches to a new virtual terminal
  */
-DFBResult dfb_vt_initialize();
-DFBResult dfb_vt_join();
+DFBResult dfb_vt_initialize( void );
+DFBResult dfb_vt_join( void );
 
 /*
  * deallocates virtual terminal
