@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -63,13 +63,13 @@ typedef struct {
 } OSXInputData;
 
 static DFBInputEvent motionX = {
-     type:     DIET_UNKNOWN,
-     axisabs:  0
+     .type    = DIET_UNKNOWN,
+     .axisabs = 0,
 };
 
 static DFBInputEvent motionY = {
-     type:     DIET_UNKNOWN,
-     axisabs:  0
+     .type    = DIET_UNKNOWN,
+     .axisabs = 0,
 };
 
 static void
@@ -206,7 +206,7 @@ osxEventThread( DirectThread *thread, void *driver_data )
  * Called once during initialization of DirectFB.
  */
 static int
-driver_get_available()
+driver_get_available( void )
 {
      if (dfb_system_type() == CORE_OSX)
           return 1;
