@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -40,7 +40,7 @@
 #include <core/coretypes.h>
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
 
 #include <gfx/convert.h>
@@ -118,19 +118,19 @@ primaryGetScreenSize( CoreScreen *screen,
 
 ScreenFuncs osxPrimaryScreenFuncs = {
      .InitScreen    = primaryInitScreen,
-     .GetScreenSize = primaryGetScreenSize
+     .GetScreenSize = primaryGetScreenSize,
 };
 
 /******************************************************************************/
 
 static int
-primaryLayerDataSize()
+primaryLayerDataSize( void )
 {
      return 0;
 }
 
 static int
-primaryRegionDataSize()
+primaryRegionDataSize( void )
 {
      return 0;
 }
@@ -411,7 +411,7 @@ DisplayLayerFuncs osxPrimaryLayerFuncs = {
      .UpdateRegion      = primaryUpdateRegion,
 
      .AllocateSurface   = primaryAllocateSurface,
-     .ReallocateSurface = primaryReallocateSurface
+     .ReallocateSurface = primaryReallocateSurface,
 };
 
 /******************************************************************************/
@@ -436,11 +436,11 @@ update_screen( CoreSurface *surface, int x, int y, int w, int h )
 
      dst = CGDisplayBaseAddress( screen );
      dst_pitch = CGDisplayBytesPerRow( screen );
-     src += DFB_BYTES_PER_LINE( surface->format, x ) + y * pitch;
-     dst += DFB_BYTES_PER_LINE( surface->format, x ) + y * dst_pitch;
+     src += DFB_BYTES_PER_LINE( surface->config.format, x ) + y * pitch;
+     dst += DFB_BYTES_PER_LINE( surface->config.format, x ) + y * dst_pitch;
 
      for (i=0; i<h; ++i) {
-          direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->format, w ) );
+          direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->config.format, w ) );
 
           src += pitch;
           dst += dst_pitch;
@@ -501,7 +501,7 @@ dfb_osx_update_screen_handler( DFBRegion *region )
      fusion_skirmish_prevail( &dfb_osx->lock );
 
      if (!region)
-          ret = update_screen( surface, 0, 0, surface->width, surface->height );
+          ret = update_screen( surface, 0, 0, surface->config.size.w, surface->config.size.h );
      else
           ret = update_screen( surface,
                                region->x1,  region->y1,
@@ -524,28 +524,34 @@ dfb_osx_set_palette_handler( CorePalette *palette )
      return DFB_OK;
 }
 
-int
-dfb_osx_call_handler( int   caller,
-                      int   call_arg,
-                      void *call_ptr,
-                      void *ctx )
+FusionCallHandlerResult
+dfb_osx_call_handler( int           caller,
+                      int           call_arg,
+                      void         *call_ptr,
+                      void         *ctx,
+                      unsigned int  serial,
+                      int          *ret_val )
 {
      switch (call_arg) {
           case OSX_SET_VIDEO_MODE:
-               return dfb_osx_set_video_mode_handler( call_ptr );
+               *ret_val = dfb_osx_set_video_mode_handler( call_ptr );
+               break;
 
           case OSX_UPDATE_SCREEN:
-               return dfb_osx_update_screen_handler( call_ptr );
+               *ret_val = dfb_osx_update_screen_handler( call_ptr );
+               break;
 
           case OSX_SET_PALETTE:
-               return dfb_osx_set_palette_handler( call_ptr );
+               *ret_val = dfb_osx_set_palette_handler( call_ptr );
+               break;
 
           default:
                D_BUG( "unknown call" );
+               *ret_val = DFB_BUG;
                break;
      }
 
-     return 0;
+     return FCHR_RETURN;
 }
 
 static DFBResult
