@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -39,7 +40,8 @@
 #include <core/coretypes.h>
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
+#include <core/surface_buffer.h>
 #include <core/system.h>
 
 #include <gfx/convert.h>
@@ -57,27 +59,18 @@
 #include "sdl.h"
 #include "primary.h"
 
+D_DEBUG_DOMAIN( SDL_Screen,  "SDL/Screen",  "SDL System Screen" );
 D_DEBUG_DOMAIN( SDL_Updates, "SDL/Updates", "SDL System Screen Updates" );
 
-
-extern DFBSDL  *dfb_sdl;
-extern CoreDFB *dfb_sdl_core;
-
 /******************************************************************************/
 
-static DFBResult dfb_sdl_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config );
-static DFBResult dfb_sdl_update_screen( CoreDFB *core, DFBRegion *region );
-static DFBResult dfb_sdl_set_palette( CorePalette *palette );
-
 static DFBResult update_screen( int x, int y, int w, int h );
 
-static SDL_Surface *screen = NULL;
-
 /******************************************************************************/
 
 static DFBResult
 primaryInitScreen( CoreScreen           *screen,
-                   GraphicsDevice       *device,
+                   CoreGraphicsDevice   *device,
                    void                 *driver_data,
                    void                 *screen_data,
                    DFBScreenDescription *description )
@@ -102,8 +95,8 @@ primaryGetScreenSize( CoreScreen *screen,
      D_ASSERT( dfb_sdl != NULL );
 
      if (dfb_sdl->primary) {
-          *ret_width  = dfb_sdl->primary->width;
-          *ret_height = dfb_sdl->primary->height;
+          *ret_width  = dfb_sdl->primary->config.size.w;
+          *ret_height = dfb_sdl->primary->config.size.w;
      }
      else {
           if (dfb_config->mode.width)
@@ -122,7 +115,7 @@ primaryGetScreenSize( CoreScreen *screen,
 
 ScreenFuncs sdlPrimaryScreenFuncs = {
      .InitScreen    = primaryInitScreen,
-     .GetScreenSize = primaryGetScreenSize
+     .GetScreenSize = primaryGetScreenSize,
 };
 
 /******************************************************************************/
@@ -132,13 +125,13 @@ static void * ScreenUpdateLoop( DirectThread *thread, void *arg );
 /******************************************************************************/
 
 static int
-primaryLayerDataSize()
+primaryLayerDataSize( void )
 {
      return 0;
 }
 
 static int
-primaryRegionDataSize()
+primaryRegionDataSize( void )
 {
      return 0;
 }
@@ -243,14 +236,9 @@ primarySetRegion( CoreLayer                  *layer,
                   CoreLayerRegionConfig      *config,
                   CoreLayerRegionConfigFlags  updated,
                   CoreSurface                *surface,
-                  CorePalette                *palette )
+                  CorePalette                *palette,
+                  CoreSurfaceBufferLock      *lock )
 {
-     DFBResult ret;
-
-     ret = dfb_sdl_set_video_mode( dfb_sdl_core, config );
-     if (ret)
-          return ret;
-
      if (surface) {
           pthread_mutex_lock( &dfb_sdl->update.lock );
           dfb_sdl->primary = surface;
@@ -290,25 +278,27 @@ primaryRemoveRegion( CoreLayer             *layer,
 }
 
 static DFBResult
-primaryFlipRegion( CoreLayer           *layer,
-                   void                *driver_data,
-                   void                *layer_data,
-                   void                *region_data,
-                   CoreSurface         *surface,
-                   DFBSurfaceFlipFlags  flags )
+primaryFlipRegion( CoreLayer             *layer,
+                   void                  *driver_data,
+                   void                  *layer_data,
+                   void                  *region_data,
+                   CoreSurface           *surface,
+                   DFBSurfaceFlipFlags    flags,
+                   CoreSurfaceBufferLock *lock )
 {
-     dfb_surface_flip_buffers( surface, false );
+     dfb_surface_flip( surface, false );
 
      return dfb_sdl_update_screen( dfb_sdl_core, NULL );
 }
 
 static DFBResult
-primaryUpdateRegion( CoreLayer           *layer,
-                     void                *driver_data,
-                     void                *layer_data,
-                     void                *region_data,
-                     CoreSurface         *surface,
-                     const DFBRegion     *update )
+primaryUpdateRegion( CoreLayer             *layer,
+                     void                  *driver_data,
+                     void                  *layer_data,
+                     void                  *region_data,
+                     CoreSurface           *surface,
+                     const DFBRegion       *update,
+                     CoreSurfaceBufferLock *lock )
 {
      if (update) {
           DFBRegion region = *update;
@@ -319,87 +309,6 @@ primaryUpdateRegion( CoreLayer           *layer,
      return dfb_sdl_update_screen( dfb_sdl_core, NULL );
 }
 
-static DFBResult
-primaryAllocateSurface( CoreLayer              *layer,
-                        void                   *driver_data,
-                        void                   *layer_data,
-                        void                   *region_data,
-                        CoreLayerRegionConfig  *config,
-                        CoreSurface           **ret_surface )
-{
-     DFBSurfaceCapabilities caps = DSCAPS_SYSTEMONLY;
-
-     if (config->buffermode != DLBM_FRONTONLY)
-          caps |= DSCAPS_DOUBLE;
-
-     return dfb_surface_create( dfb_sdl_core, config->width, config->height,
-                                config->format, CSP_SYSTEMONLY,
-                                caps, NULL, ret_surface );
-}
-
-static DFBResult
-primaryReallocateSurface( CoreLayer             *layer,
-                          void                  *driver_data,
-                          void                  *layer_data,
-                          void                  *region_data,
-                          CoreLayerRegionConfig *config,
-                          CoreSurface           *surface )
-{
-     DFBResult ret;
-
-     /* FIXME: write surface management functions
-               for easier configuration changes */
-
-     switch (config->buffermode) {
-          case DLBM_BACKVIDEO:
-          case DLBM_BACKSYSTEM:
-               surface->caps |= DSCAPS_DOUBLE;
-
-               ret = dfb_surface_reconfig( surface,
-                                           CSP_SYSTEMONLY, CSP_SYSTEMONLY );
-               break;
-
-          case DLBM_FRONTONLY:
-               surface->caps &= ~DSCAPS_DOUBLE;
-
-               ret = dfb_surface_reconfig( surface,
-                                           CSP_SYSTEMONLY, CSP_SYSTEMONLY );
-               break;
-
-          default:
-               D_BUG("unknown buffermode");
-               return DFB_BUG;
-     }
-     if (ret)
-          return ret;
-
-     ret = dfb_surface_reformat( dfb_sdl_core, surface, config->width,
-                                 config->height, config->format );
-     if (ret)
-          return ret;
-
-
-     if (DFB_PIXELFORMAT_IS_INDEXED(config->format) && !surface->palette) {
-          DFBResult    ret;
-          CorePalette *palette;
-
-          ret = dfb_palette_create( dfb_sdl_core,
-                                    1 << DFB_COLOR_BITS_PER_PIXEL( config->format ),
-                                    &palette );
-          if (ret)
-               return ret;
-
-          if (config->format == DSPF_LUT8)
-               dfb_palette_generate_rgb332_map( palette );
-
-          dfb_surface_set_palette( surface, palette );
-
-          dfb_palette_unref( palette );
-     }
-
-     return DFB_OK;
-}
-
 DisplayLayerFuncs sdlPrimaryLayerFuncs = {
      .LayerDataSize     = primaryLayerDataSize,
      .RegionDataSize    = primaryRegionDataSize,
@@ -411,9 +320,6 @@ DisplayLayerFuncs sdlPrimaryLayerFuncs = {
      .RemoveRegion      = primaryRemoveRegion,
      .FlipRegion        = primaryFlipRegion,
      .UpdateRegion      = primaryUpdateRegion,
-
-     .AllocateSurface   = primaryAllocateSurface,
-     .ReallocateSurface = primaryReallocateSurface
 };
 
 /******************************************************************************/
@@ -421,20 +327,26 @@ DisplayLayerFuncs sdlPrimaryLayerFuncs = {
 static DFBResult
 update_screen( int x, int y, int w, int h )
 {
-     int          i;
-     void        *dst;
-     void        *src;
-     int          pitch;
-     DFBResult    ret;
-     CoreSurface *surface;
+#if 0
+     int                    i, n;
+     void                  *dst;
+     void                  *src;
+     DFBResult              ret;
+     CoreSurface           *surface;
+     CoreSurfaceBuffer     *buffer;
+     CoreSurfaceBufferLock  lock;
+     u16                   *src16, *dst16;
+     u8                    *src8;
+#endif
 
-     D_DEBUG_AT( SDL_Updates, "%s( %p, %d, %d, %d, %d )\n", __FUNCTION__, surface, x, y, w, h );
+     D_DEBUG_AT( SDL_Updates, "%s( %d, %d, %d, %d )\n", __FUNCTION__, x, y, w, h );
 
      D_DEBUG_AT( SDL_Updates, "  -> locking sdl lock...\n" );
 
      fusion_skirmish_prevail( &dfb_sdl->lock );
-
+#if 0
      surface = dfb_sdl->primary;
+     D_MAGIC_ASSERT_IF( surface, CoreSurface );
 
      D_DEBUG_AT( SDL_Updates, "  -> primary is %p\n", surface );
 
@@ -445,6 +357,10 @@ update_screen( int x, int y, int w, int h )
           return DFB_OK;
      }
 
+     buffer = dfb_surface_get_buffer( surface, CSBR_FRONT );
+
+     D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );
+
      D_DEBUG_AT( SDL_Updates, "  -> locking sdl surface...\n" );
 
      if (SDL_LockSurface( screen ) < 0) {
@@ -456,7 +372,7 @@ update_screen( int x, int y, int w, int h )
 
      D_DEBUG_AT( SDL_Updates, "  -> locking dfb surface...\n" );
 
-     ret = dfb_surface_soft_lock( surface, DSLF_READ, &src, &pitch, true );
+     ret = dfb_surface_buffer_lock( buffer, CSAF_CPU_READ, &lock );
      if (ret) {
           D_ERROR( "DirectFB/SDL: Couldn't lock layer surface: %s\n",
                    DirectFBErrorString( ret ) );
@@ -465,31 +381,36 @@ update_screen( int x, int y, int w, int h )
           return ret;
      }
 
+     src = lock.addr;
      dst = screen->pixels;
 
-     src += DFB_BYTES_PER_LINE( surface->format, x ) + y * pitch;
-     dst += DFB_BYTES_PER_LINE( surface->format, x ) + y * screen->pitch;
+     src += DFB_BYTES_PER_LINE( surface->config.format, x ) + y * lock.pitch;
+     dst += DFB_BYTES_PER_LINE( surface->config.format, x ) + y * screen->pitch;
 
      D_DEBUG_AT( SDL_Updates, "  -> copying pixels...\n" );
 
-     for (i=0; i<h; ++i) {
-          direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->format, w ) );
+     switch (screen->format->BitsPerPixel) {
+          case 16:
+               dfb_convert_to_rgb16( surface->config.format,
+                                     src, lock.pitch, surface->config.size.h,
+                                     dst, screen->pitch, w, h );
+               break;
 
-          src += pitch;
-          dst += screen->pitch;
+          default:
+               direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->config.format, w ) );
      }
 
      D_DEBUG_AT( SDL_Updates, "  -> unlocking dfb surface...\n" );
 
-     dfb_surface_unlock( surface, true );
+     dfb_surface_buffer_unlock( &lock );
 
      D_DEBUG_AT( SDL_Updates, "  -> unlocking sdl surface...\n" );
 
      SDL_UnlockSurface( screen );
-
+#endif
      D_DEBUG_AT( SDL_Updates, "  -> calling SDL_UpdateRect()...\n" );
 
-     SDL_UpdateRect( screen, x, y, w, h );
+     SDL_UpdateRect( dfb_sdl->screen, x, y, w, h );
 
      D_DEBUG_AT( SDL_Updates, "  -> unlocking sdl lock...\n" );
 
@@ -542,26 +463,50 @@ typedef enum {
      SDL_SET_PALETTE
 } DFBSDLCall;
 
-static DFBResult
-dfb_sdl_set_video_mode_handler( CoreLayerRegionConfig *config )
+static inline int
+get_pixelformat_target_depth( DFBSurfacePixelFormat format )
 {
+     switch (format) {
+          case DSPF_NV16:
+               return 16;
+
+          default:
+               break;
+     }
+
+     return DFB_BITS_PER_PIXEL( format );
+}
+
+static DFBResult
+dfb_sdl_set_video_mode_handler( CoreSurfaceConfig *config )
+{
+     int          depth = get_pixelformat_target_depth( config->format );
+     Uint32       flags = SDL_HWSURFACE | SDL_RESIZABLE;// | SDL_ASYNCBLIT | SDL_FULLSCREEN;
+     SDL_Surface *screen;
+
+     if (config->caps & DSCAPS_FLIPPING)
+          flags |= SDL_DOUBLEBUF;
+
      fusion_skirmish_prevail( &dfb_sdl->lock );
 
+     D_DEBUG_AT( SDL_Screen, "  -> SDL_SetVideoMode( %dx%d, %d, 0x%08x )\n",
+                 config->size.w, config->size.h, DFB_BITS_PER_PIXEL(config->format), flags );
+
      /* Set video mode */
-     if ( (screen=SDL_SetVideoMode(config->width, config->height,
-                                   DFB_BITS_PER_PIXEL(config->format),
-                                   SDL_HWSURFACE)) == NULL )
-     {
-             D_ERROR( "DirectFB/SDL: Couldn't set %dx%dx%d video mode: %s\n",
-                      config->width, config->height,
-                      DFB_COLOR_BITS_PER_PIXEL(config->format), SDL_GetError());
+     screen = SDL_SetVideoMode( config->size.w, config->size.h, depth, flags );
+     if (!screen) {
+          D_ERROR( "DirectFB/SDL: Couldn't set %dx%dx%d video mode: %s\n",
+                   config->size.w, config->size.h, depth, SDL_GetError());
 
-             fusion_skirmish_dismiss( &dfb_sdl->lock );
+          fusion_skirmish_dismiss( &dfb_sdl->lock );
 
-             return DFB_FAILURE;
+          return DFB_FAILURE;
      }
+
+     dfb_sdl->screen = screen;
+
      /* Hide SDL's cursor */
-     SDL_ShowCursor(SDL_DISABLE);
+     SDL_ShowCursor( SDL_DISABLE );
 
      fusion_skirmish_dismiss( &dfb_sdl->lock );
 
@@ -581,11 +526,11 @@ dfb_sdl_update_screen_handler( const DFBRegion *region )
      else {
           update.x1 = 0;
           update.y1 = 0;
-          update.x2 = surface->width - 1;
-          update.y2 = surface->height - 1;
+          update.x2 = surface->config.size.w - 1;
+          update.y2 = surface->config.size.h - 1;
      }
 
-
+#if 0
      pthread_mutex_lock( &dfb_sdl->update.lock );
 
      if (dfb_sdl->update.pending)
@@ -598,6 +543,12 @@ dfb_sdl_update_screen_handler( const DFBRegion *region )
      pthread_cond_signal( &dfb_sdl->update.cond );
 
      pthread_mutex_unlock( &dfb_sdl->update.lock );
+#else
+     if (surface->config.caps & DSCAPS_FLIPPING)
+          SDL_Flip( dfb_sdl->screen );
+     else
+          SDL_UpdateRect( dfb_sdl->screen, DFB_RECTANGLE_VALS_FROM_REGION(&update) );
+#endif
 
      return DFB_OK;
 }
@@ -616,44 +567,48 @@ dfb_sdl_set_palette_handler( CorePalette *palette )
 
      fusion_skirmish_prevail( &dfb_sdl->lock );
 
-     SDL_SetColors( screen, colors, 0, palette->num_entries );
+     SDL_SetColors( dfb_sdl->screen, colors, 0, palette->num_entries );
 
      fusion_skirmish_dismiss( &dfb_sdl->lock );
 
      return DFB_OK;
 }
 
-int
-dfb_sdl_call_handler( int   caller,
-                      int   call_arg,
-                      void *call_ptr,
-                      void *ctx )
+FusionCallHandlerResult
+dfb_sdl_call_handler( int           caller,
+                      int           call_arg,
+                      void         *call_ptr,
+                      void         *ctx,
+                      unsigned int  serial,
+                      int          *ret_val )
 {
      switch (call_arg) {
           case SDL_SET_VIDEO_MODE:
-               return dfb_sdl_set_video_mode_handler( call_ptr );
+               *ret_val = dfb_sdl_set_video_mode_handler( call_ptr );
+               break;
 
           case SDL_UPDATE_SCREEN:
-               dfb_sdl_update_screen_handler( call_ptr );
-               SHFREE( dfb_core_shmpool(dfb_sdl_core), call_ptr );
-               return 0;
+               *ret_val = dfb_sdl_update_screen_handler( call_ptr );
+               break;
 
           case SDL_SET_PALETTE:
-               return dfb_sdl_set_palette_handler( call_ptr );
+               *ret_val = dfb_sdl_set_palette_handler( call_ptr );
+               break;
 
           default:
                D_BUG( "unknown call" );
+               *ret_val = DFB_BUG;
                break;
      }
 
-     return 0;
+     return FCHR_RETURN;
 }
 
-static DFBResult
-dfb_sdl_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config )
+DFBResult
+dfb_sdl_set_video_mode( CoreDFB *core, CoreSurfaceConfig *config )
 {
-     int                    ret;
-     CoreLayerRegionConfig *tmp = NULL;
+     int                ret;
+     CoreSurfaceConfig *tmp = NULL;
 
      D_ASSERT( config != NULL );
 
@@ -661,11 +616,11 @@ dfb_sdl_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config )
           return dfb_sdl_set_video_mode_handler( config );
 
      if (!fusion_is_shared( dfb_core_world(core), config )) {
-          tmp = SHMALLOC( dfb_core_shmpool(core), sizeof(CoreLayerRegionConfig) );
+          tmp = SHMALLOC( dfb_core_shmpool(core), sizeof(CoreSurfaceConfig) );
           if (!tmp)
                return D_OOSHM();
 
-          direct_memcpy( tmp, config, sizeof(CoreLayerRegionConfig) );
+          direct_memcpy( tmp, config, sizeof(CoreSurfaceConfig) );
      }
 
      fusion_call_execute( &dfb_sdl->call, FCEF_NONE, SDL_SET_VIDEO_MODE,
@@ -677,7 +632,7 @@ dfb_sdl_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config )
      return ret;
 }
 
-static DFBResult
+DFBResult
 dfb_sdl_update_screen( CoreDFB *core, DFBRegion *region )
 {
      int        ret;
@@ -694,19 +649,20 @@ dfb_sdl_update_screen( CoreDFB *core, DFBRegion *region )
           direct_memcpy( tmp, region, sizeof(DFBRegion) );
      }
 
-     fusion_call_execute( &dfb_sdl->call, FCEF_ONEWAY, SDL_UPDATE_SCREEN,
-                          tmp ? tmp : region, &ret );
+     fusion_call_execute( &dfb_sdl->call, FCEF_NONE, SDL_UPDATE_SCREEN, tmp ? tmp : region, &ret );
+
+     if (tmp)
+          SHFREE( dfb_core_shmpool(core), tmp );
 
      return DFB_OK;
 }
 
-static DFBResult
+DFBResult
 dfb_sdl_set_palette( CorePalette *palette )
 {
      int ret;
 
-     fusion_call_execute( &dfb_sdl->call, FCEF_NONE, SDL_SET_PALETTE,
-                          palette, &ret );
+     fusion_call_execute( &dfb_sdl->call, FCEF_NONE, SDL_SET_PALETTE, palette, &ret );
 
      return ret;
 }
