@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -46,7 +46,7 @@
 #include <core/coretypes.h>
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
 
 #include <gfx/convert.h>
@@ -66,14 +66,16 @@ DFB_CORE_SYSTEM( sdl )
 DFBSDL  *dfb_sdl      = NULL;
 CoreDFB *dfb_sdl_core = NULL;
 
+extern const SurfacePoolFuncs sdlSurfacePoolFuncs;
 
-static DFBResult dfb_fbdev_read_modes();
+static DFBResult dfb_fbdev_read_modes( void );
 
 
 static void
 system_get_info( CoreSystemInfo *info )
 {
      info->type = CORE_SDL;
+     info->caps = CSCAPS_ACCELERATION;
 
      snprintf( info->name, DFB_CORE_SYSTEM_INFO_NAME_LENGTH, "SDL" );
 }
@@ -122,6 +124,8 @@ system_initialize( CoreDFB *core, void **data )
 
      fusion_arena_add_shared_field( dfb_core_arena( core ), "sdl", dfb_sdl );
 
+     dfb_surface_pool_initialize( core, &sdlSurfacePoolFuncs, &dfb_sdl->sdl_pool );
+
      *data = dfb_sdl;
 
      return DFB_OK;
@@ -144,6 +148,8 @@ system_join( CoreDFB *core, void **data )
 
      dfb_layers_register( screen, NULL, &sdlPrimaryLayerFuncs );
 
+     dfb_surface_pool_join( core, dfb_sdl->sdl_pool, &sdlSurfacePoolFuncs );
+
      *data = dfb_sdl;
 
      return DFB_OK;
@@ -169,6 +175,7 @@ system_shutdown( bool emergency )
           direct_thread_destroy( dfb_sdl->update.thread );
      }
 
+     dfb_surface_pool_destroy( dfb_sdl->sdl_pool );
 
      fusion_call_destroy( &dfb_sdl->call );
 
@@ -207,13 +214,13 @@ system_leave( bool emergency )
 }
 
 static DFBResult
-system_suspend()
+system_suspend( void )
 {
      return DFB_UNIMPLEMENTED;
 }
 
 static DFBResult
-system_resume()
+system_resume( void )
 {
      return DFB_UNIMPLEMENTED;
 }
@@ -232,25 +239,25 @@ system_unmap_mmio( volatile void  *addr,
 }
 
 static int
-system_get_accelerator()
+system_get_accelerator( void )
 {
      return -1;
 }
 
 static VideoMode *
-system_get_modes()
+system_get_modes( void )
 {
      return dfb_sdl->modes;
 }
 
 static VideoMode *
-system_get_current_mode()
+system_get_current_mode( void )
 {
      return NULL;
 }
 
 static DFBResult
-system_thread_init()
+system_thread_init( void )
 {
      return DFB_OK;
 }
@@ -275,7 +282,7 @@ system_video_memory_virtual( unsigned int offset )
 }
 
 static unsigned int
-system_videoram_length()
+system_videoram_length( void )
 {
      return 0;
 }
@@ -293,7 +300,7 @@ system_aux_memory_virtual( unsigned int offset )
 }
 
 static unsigned int
-system_auxram_length()
+system_auxram_length( void )
 {
      return 0;
 }
@@ -316,7 +323,7 @@ system_get_deviceid( unsigned int *ret_vendor_id,
  * parses video modes in /etc/fb.modes and stores them in dfb_fbdev->shared->modes
  * (to be replaced by DirectFB's own config system
  */
-static DFBResult dfb_fbdev_read_modes()
+static DFBResult dfb_fbdev_read_modes( void )
 {
      FILE *fp;
      char line[80],label[32],value[16];
