@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -32,11 +32,15 @@
 #include <fusion/call.h>
 #include <fusion/lock.h>
 
-typedef struct {
-     FusionSkirmish  lock;
-     FusionCall      call;
+#include <core/surface_pool.h>
+#include <core/system.h>
 
-     CoreSurface    *primary;
+typedef struct {
+     FusionSkirmish   lock;
+     FusionCall       call;
+
+     CoreSurface     *primary;
+     CoreSurfacePool *sdl_pool;
 
      struct {
           pthread_mutex_t  lock;
@@ -51,7 +55,16 @@ typedef struct {
      } update;
 
      VideoMode            *modes;        /* linked list of valid video modes */
+
+     SDL_Surface          *screen;
 } DFBSDL;
 
+extern DFBSDL  *dfb_sdl;
+extern CoreDFB *dfb_sdl_core;
+
+DFBResult dfb_sdl_set_video_mode( CoreDFB *core, CoreSurfaceConfig *config );
+DFBResult dfb_sdl_update_screen( CoreDFB *core, DFBRegion *region );
+DFBResult dfb_sdl_set_palette( CorePalette *palette );
+
 #endif
 
