@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -42,7 +42,7 @@
 #include <core/coretypes.h>
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
 #include <core/input.h>
 
@@ -88,7 +88,6 @@ static DFBResult update_screen( CoreSurface *surface,
                                 int x, int y, int w, int h );
 
 static void* vnc_server_thread( DirectThread *thread, void *data );
-static void* vnc_refresh_thread( DirectThread *thread, void *data );
 
 extern DFBVNC *dfb_vnc;
 extern CoreDFB *dfb_vnc_core;
@@ -99,7 +98,7 @@ static CoreInputDevice *vncInputDevice;
 
 static DFBResult
 primaryInitScreen( CoreScreen           *screen,
-                   GraphicsDevice       *device,
+                   CoreGraphicsDevice   *device,
                    void                 *driver_data,
                    void                 *screen_data,
                    DFBScreenDescription *description )
@@ -124,8 +123,8 @@ primaryGetScreenSize( CoreScreen *screen,
      D_ASSERT( dfb_vnc != NULL );
 
      if (dfb_vnc->primary) {
-          *ret_width  = dfb_vnc->primary->width;
-          *ret_height = dfb_vnc->primary->height;
+          *ret_width  = dfb_vnc->primary->config.size.w;
+          *ret_height = dfb_vnc->primary->config.size.h;
      }
      else {
           if (dfb_config->mode.width)
@@ -143,20 +142,20 @@ primaryGetScreenSize( CoreScreen *screen,
 }
 
 ScreenFuncs vncPrimaryScreenFuncs = {
-     .InitScreen   = primaryInitScreen,
-     .GetScreenSize = primaryGetScreenSize
+     .InitScreen    = primaryInitScreen,
+     .GetScreenSize = primaryGetScreenSize,
 };
 
 /******************************************************************************/
 
 static int
-primaryLayerDataSize()
+primaryLayerDataSize( void )
 {
      return 0;
 }
 
 static int
-primaryRegionDataSize()
+primaryRegionDataSize( void )
 {
      return 0;
 }
@@ -254,7 +253,8 @@ primarySetRegion( CoreLayer                  *layer,
                   CoreLayerRegionConfig      *config,
                   CoreLayerRegionConfigFlags  updated,
                   CoreSurface                *surface,
-                  CorePalette                *palette )
+                  CorePalette                *palette,
+                  CoreSurfaceBufferLock      *lock )
 {
      DFBResult ret;
 
@@ -287,25 +287,27 @@ primaryRemoveRegion( CoreLayer             *layer,
 }
 
 static DFBResult
-primaryFlipRegion( CoreLayer           *layer,
-                   void                *driver_data,
-                   void                *layer_data,
-                   void                *region_data,
-                   CoreSurface         *surface,
-                   DFBSurfaceFlipFlags  flags )
+primaryFlipRegion( CoreLayer             *layer,
+                   void                  *driver_data,
+                   void                  *layer_data,
+                   void                  *region_data,
+                   CoreSurface           *surface,
+                   DFBSurfaceFlipFlags    flags,
+                   CoreSurfaceBufferLock *lock )
 {
-     dfb_surface_flip_buffers( surface, false );
+     dfb_surface_flip( surface, false );
 
      return dfb_vnc_update_screen( dfb_vnc_core, NULL );
 }
 
 static DFBResult
-primaryUpdateRegion( CoreLayer           *layer,
-                     void                *driver_data,
-                     void                *layer_data,
-                     void                *region_data,
-                     CoreSurface         *surface,
-                     const DFBRegion           *update )
+primaryUpdateRegion( CoreLayer             *layer,
+                     void                  *driver_data,
+                     void                  *layer_data,
+                     void                  *region_data,
+                     CoreSurface           *surface,
+                     const DFBRegion       *update,
+                     CoreSurfaceBufferLock *lock )
 {
      if (update) {
           DFBRegion region = *update;
@@ -324,14 +326,18 @@ primaryAllocateSurface( CoreLayer              *layer,
                         CoreLayerRegionConfig  *config,
                         CoreSurface           **ret_surface )
 {
-     DFBSurfaceCapabilities caps = DSCAPS_SYSTEMONLY;
+     CoreSurfaceConfig conf;
+
+     conf.flags  = CSCONF_SIZE | CSCONF_FORMAT | CSCONF_CAPS;
+     conf.size.w = config->width;
+     conf.size.h = config->height;
+     conf.format = config->format;
+     conf.caps   = DSCAPS_SYSTEMONLY;
 
      if (config->buffermode != DLBM_FRONTONLY)
-          caps |= DSCAPS_DOUBLE;
+          conf.caps |= DSCAPS_DOUBLE;
 
-     return dfb_surface_create( dfb_vnc->core, config->width, config->height,
-                                config->format, CSP_SYSTEMONLY,
-                                caps, NULL, ret_surface );
+     return dfb_surface_create( dfb_vnc_core, &conf, CSTF_LAYER, DLID_PRIMARY, NULL, ret_surface );
 }
 
 static DFBResult
@@ -342,40 +348,22 @@ primaryReallocateSurface( CoreLayer             *layer,
                           CoreLayerRegionConfig *config,
                           CoreSurface           *surface )
 {
-     DFBResult ret;
+     DFBResult         ret;
+     CoreSurfaceConfig conf;
 
-     /* FIXME: write surface management functions
-               for easier configuration changes */
+     conf.flags  = CSCONF_SIZE | CSCONF_FORMAT | CSCONF_CAPS;
+     conf.size.w = config->width;
+     conf.size.h = config->height;
+     conf.format = config->format;
+     conf.caps   = DSCAPS_SYSTEMONLY;
 
-     switch (config->buffermode) {
-          case DLBM_BACKVIDEO:
-          case DLBM_BACKSYSTEM:
-               surface->caps |= DSCAPS_DOUBLE;
+     if (config->buffermode != DLBM_FRONTONLY)
+          conf.caps |= DSCAPS_DOUBLE;
 
-               ret = dfb_surface_reconfig( surface,
-                                           CSP_SYSTEMONLY, CSP_SYSTEMONLY );
-               break;
-
-          case DLBM_FRONTONLY:
-               surface->caps &= ~DSCAPS_DOUBLE;
-
-               ret = dfb_surface_reconfig( surface,
-                                           CSP_SYSTEMONLY, CSP_SYSTEMONLY );
-               break;
-
-          default:
-               D_BUG("unknown buffermode");
-               return DFB_BUG;
-     }
+     ret = dfb_surface_reconfig( surface, &conf );
      if (ret)
           return ret;
 
-     ret = dfb_surface_reformat( NULL, surface, config->width,
-                                 config->height, config->format );
-     if (ret)
-          return ret;
-
-
      if (DFB_PIXELFORMAT_IS_INDEXED(config->format) && !surface->palette) {
           DFBResult    ret;
           CorePalette *palette;
@@ -410,7 +398,7 @@ DisplayLayerFuncs vncPrimaryLayerFuncs = {
      .UpdateRegion      = primaryUpdateRegion,
 
      .AllocateSurface   = primaryAllocateSurface,
-     .ReallocateSurface = primaryReallocateSurface
+     .ReallocateSurface = primaryReallocateSurface,
 };
 
 /******************************************************************************/
@@ -418,48 +406,46 @@ DisplayLayerFuncs vncPrimaryLayerFuncs = {
 static DFBResult
 update_screen( CoreSurface *surface, int x, int y, int w, int h )
 {
-     int          i, j, k;
-     void        *dst, *p;
-     void        *src, *q;
-     int          pitch;
-     DFBResult    ret;
+     DFBResult              ret;
+     int                    i, j, k;
+     void                  *dst, *p;
+     void                  *src, *q;
+     CoreSurfaceBufferLock  lock;
 
      D_ASSERT( surface != NULL );
      D_ASSERT( rfb_screen != NULL );
      D_ASSERT( rfb_screen->frameBuffer != NULL );
 
-     ret = dfb_surface_soft_lock( dfb_vnc_core, surface, DSLF_READ, &src, &pitch, true );
+     ret = dfb_surface_lock_buffer( surface, CSBR_FRONT, CSAF_CPU_READ, &lock );
      if (ret) {
-          D_ERROR( "DirectFB/VNC: Couldn't lock layer surface: %s\n",
-                   DirectFBErrorString( ret ) );
+          D_DERROR( ret, "DirectFB/VNC: Couldn't lock layer surface!\n" );
           return ret;
      }
 
-     dst = rfb_screen->frameBuffer;
-     
-     src += DFB_BYTES_PER_LINE( surface->format, x ) + y * pitch;
-     dst += x * rfb_screen->depth/8 + y * rfb_screen->width * rfb_screen->depth/8;
+     src = lock.addr + DFB_BYTES_PER_LINE( surface->config.format, x ) + y * lock.pitch;
+
+     dst = rfb_screen->frameBuffer + x * rfb_screen->depth/8 + y * rfb_screen->width * rfb_screen->depth/8;
 
      for (i=0; i<h; ++i) {
-          /*direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->format, w ) );*/
-      for(j=0, p=src, q=dst; j<w; j++, 
-                    p += DFB_BYTES_PER_PIXEL(surface->format),
-                    q += rfb_screen->depth/8){
-         /*direct_memcpy( q, p, DFB_BYTES_PER_PIXEL(surface->format));*/
-         /**(char*) q = *(char*) (p+2);
-         *(char*) (q+1) = *(char*) (p+1);
-         *(char*) (q+2) = *(char*) p;*/
-         for(k=0; k<DFB_BYTES_PER_PIXEL(surface->format); k++)
-        *(char*) (q+k) = *(char*) (p+DFB_BYTES_PER_PIXEL(surface->format)-1-k);
+          /*direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->config.format, w ) );*/
+          for (j=0, p=src, q=dst; j<w; j++, 
+              p += DFB_BYTES_PER_PIXEL(surface->config.format),
+              q += rfb_screen->depth/8) {
+               /*direct_memcpy( q, p, DFB_BYTES_PER_PIXEL(surface->config.format));*/
+               /**(char*) q = *(char*) (p+2);
+               *(char*) (q+1) = *(char*) (p+1);
+               *(char*) (q+2) = *(char*) p;*/
+               for (k=0; k<DFB_BYTES_PER_PIXEL(surface->config.format); k++)
+                    *(char*) (q+k) = *(char*) (p+DFB_BYTES_PER_PIXEL(surface->config.format)-1-k);
           }
-      
-          src += pitch;
+
+          src += lock.pitch;
           dst += rfb_screen->width * rfb_screen->depth/8;
      }
 
      rfbMarkRectAsModified ( rfb_screen, x, y, x+w, y+h );
 
-     dfb_surface_unlock( surface, true );
+     dfb_surface_unlock_buffer( surface, &lock );
 
 
 #if 0 /* Mire test */
@@ -564,11 +550,6 @@ dfb_vnc_set_video_mode_handler( CoreLayerRegionConfig *config )
 
      direct_thread_create( DTT_OUTPUT, vnc_server_thread, rfb_screen, "VNC Output" ); 
     
-     /* Create a thread for refreshing if necessary */
-
-     if ( !(config->surface_caps & (DSCAPS_DOUBLE | DSCAPS_TRIPLE)) )
-          direct_thread_create( DTT_OUTPUT, vnc_refresh_thread, rfb_screen, "VNC Refresh" );
- 
      fusion_skirmish_dismiss( &dfb_vnc->lock );
 
      return DFB_OK;
@@ -581,17 +562,6 @@ vnc_server_thread( DirectThread *thread, void *data )
    return NULL;
 }
 
-static void*
-vnc_refresh_thread( DirectThread *thread, void *data )
-{
-   while(1) {
-      update_screen(dfb_vnc->primary, 0, 0, rfb_screen->width, rfb_screen->height);
-      /* refresh frequency of 50 Hz */
-      usleep(20000);
-   }
-   return NULL;
-}
-
 
 static DFBResult
 dfb_vnc_update_screen_handler( DFBRegion *region )
@@ -606,7 +576,7 @@ dfb_vnc_update_screen_handler( DFBRegion *region )
      fusion_skirmish_prevail( &dfb_vnc->lock );
 
      if (!region)
-          ret = update_screen( surface, 0, 0, surface->width, surface->height );
+          ret = update_screen( surface, 0, 0, surface->config.size.w, surface->config.size.h );
      else
           ret = update_screen( surface,
                                region->x1,  region->y1,
@@ -650,28 +620,34 @@ dfb_vnc_set_palette_handler( CorePalette *palette )
      return DFB_OK;
 }
 
-int
-dfb_vnc_call_handler( int   caller,
-                      int   call_arg,
-                      void *call_ptr,
-                      void *ctx )
+FusionCallHandlerResult
+dfb_vnc_call_handler( int           caller,
+                      int           call_arg,
+                      void         *call_ptr,
+                      void         *ctx,
+                      unsigned int  serial,
+                      int          *ret_val )
 {
      switch (call_arg) {
           case VNC_SET_VIDEO_MODE:
-               return dfb_vnc_set_video_mode_handler( call_ptr );
+               *ret_val = dfb_vnc_set_video_mode_handler( call_ptr );
+               break;
 
           case VNC_UPDATE_SCREEN:
-               return dfb_vnc_update_screen_handler( call_ptr );
+               *ret_val = dfb_vnc_update_screen_handler( call_ptr );
+               break;
 
           case VNC_SET_PALETTE:
-               return dfb_vnc_set_palette_handler( call_ptr );
+               *ret_val = dfb_vnc_set_palette_handler( call_ptr );
+               break;
 
           default:
                D_BUG( "unknown call" );
+               *ret_val = DFB_BUG;
                break;
      }
 
-     return 0;
+     return FCHR_RETURN;
 }
 
 static DFBResult
