@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,16 +29,21 @@
 #ifndef __SDL__PRIMARY_H__
 #define __SDL__PRIMARY_H__
 
+#include <fusion/call.h>
+
 #include <core/layers.h>
 #include <core/screens.h>
 
 extern ScreenFuncs       vncPrimaryScreenFuncs;
 extern DisplayLayerFuncs vncPrimaryLayerFuncs;
 
-int dfb_vnc_call_handler( int   caller,
-                          int   call_arg,
-                          void *call_ptr,
-                          void *ctx );
+FusionCallHandlerResult
+dfb_vnc_call_handler( int           caller,
+                      int           call_arg,
+                      void         *call_ptr,
+                      void         *ctx,
+                      unsigned int  serial,
+                      int          *ret_val );
 
 #endif
 
