@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -42,7 +42,7 @@
 #include <core/coretypes.h>
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/system.h>
 
 #include <gfx/convert.h>
@@ -154,32 +154,32 @@ system_leave( bool emergency )
 }
 
 static DFBResult
-system_suspend()
+system_suspend( void )
 {
      return DFB_UNIMPLEMENTED;
 }
 
 static DFBResult
-system_resume()
+system_resume( void )
 {
      return DFB_UNIMPLEMENTED;
 }
 
 
 static int
-system_get_accelerator()
+system_get_accelerator( void )
 {
      return -1;
 }
 
 static VideoMode *
-system_get_modes()
+system_get_modes( void )
 {
      return NULL;
 }
 
 static VideoMode *
-system_get_current_mode()
+system_get_current_mode( void )
 {
      return NULL;
 }
@@ -199,7 +199,7 @@ system_unmap_mmio( volatile void  *addr,
 }
 
 static DFBResult
-system_thread_init()
+system_thread_init( void )
 {
      return DFB_OK;
 }
@@ -226,7 +226,7 @@ system_video_memory_virtual( unsigned int offset )
 }
 
 static unsigned int
-system_videoram_length()
+system_videoram_length( void )
 {
      return 0;
 }
@@ -244,7 +244,7 @@ system_aux_memory_virtual( unsigned int offset )
 }
 
 static unsigned int
-system_auxram_length()
+system_auxram_length( void )
 {
      return 0;
 }
