@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -59,8 +59,6 @@
 
 #include <core/input_driver.h>
 
-extern DFBX11  *dfb_x11;
-extern CoreDFB *dfb_x11_core;
 
 DFB_INPUT_DRIVER( x11input )
 
@@ -71,19 +69,19 @@ DFB_INPUT_DRIVER( x11input )
 typedef struct {
      CoreInputDevice*    device;
      DirectThread*       thread;
-     DFBX11*             dfb_x11;
-     int                 stop;
+     DFBX11*             x11;
+     bool                stop;
 } X11InputData;
 
 
 static DFBInputEvent motionX = {
-     type:     DIET_UNKNOWN,
-     axisabs:  0
+     .type    = DIET_UNKNOWN,
+     .axisabs = 0,
 };
 
 static DFBInputEvent motionY = {
-     type:     DIET_UNKNOWN,
-     axisabs:  0
+     .type    = DIET_UNKNOWN,
+     .axisabs = 0,
 };
 
 static void
@@ -414,7 +412,7 @@ static void handleMouseEvent(XEvent* pXEvent, X11InputData* pData)
                case 5: /*down*/
                case 6: /*left*/
                case 7: /*right*/
-                    {
+                    if (pXEvent->type == ButtonPress) {
                          dfbEvent.type = DIET_AXISMOTION;
                          dfbEvent.flags = DIEF_AXISREL;
                          dfbEvent.axis = DIAI_Z;
@@ -436,8 +434,9 @@ static void handleMouseEvent(XEvent* pXEvent, X11InputData* pData)
                               dfbEvent.axis = DIAI_X;
                               dfbEvent.axisrel = 1;
                          }
-
                     }
+                    else
+                         return;
                     break;
                default:
                     break;
@@ -454,9 +453,9 @@ static void handleMouseEvent(XEvent* pXEvent, X11InputData* pData)
 static void
 handle_expose( const XExposeEvent *expose )
 {
-     CoreLayer         *layer = dfb_layer_at( DLID_PRIMARY );
-     DisplayLayerFuncs *funcs = layer->funcs;
-     CoreLayerContext  *context;
+     CoreLayer               *layer = dfb_layer_at( DLID_PRIMARY );
+     const DisplayLayerFuncs *funcs = layer->funcs;
+     CoreLayerContext        *context;
 
      D_ASSERT( funcs != NULL );
      D_ASSERT( funcs->UpdateRegion != NULL );
@@ -469,22 +468,17 @@ handle_expose( const XExposeEvent *expose )
           if (dfb_layer_context_get_primary_region( context,
                                                     false, &region ) == DFB_OK)
           {
-               CoreSurface *surface;
-
                /* Lock the region to avoid tearing due to concurrent updates. */
                dfb_layer_region_lock( region );
 
                /* Get the surface of the region. */
-               if (dfb_layer_region_get_surface( region, &surface ) == DFB_OK) {
+               if (region->surface && region->surface_lock.buffer) {
                     DFBRegion update = { expose->x, expose->y,
                                          expose->x + expose->width  - 1,
                                          expose->y + expose->height - 1 };
 
                     funcs->UpdateRegion( layer, layer->driver_data, layer->layer_data,
-                                         region->region_data, surface, &update );
-
-                    /* Release the surface. */
-                    dfb_surface_unref( surface );
+                                         region->region_data, region->surface, &update, &region->surface_lock );
                }
 
                /* Unlock the region. */
@@ -506,50 +500,32 @@ handle_expose( const XExposeEvent *expose )
 static void*
 x11EventThread( DirectThread *thread, void *driver_data )
 {
-     X11InputData *data    = driver_data;
-     DFBX11       *dfb_x11 = data->dfb_x11;
+     X11InputData *data   = driver_data;
+     DFBX11       *x11    = data->x11;
+     DFBX11Shared *shared = x11->shared;
 
      while (!data->stop) {
-          XEvent xEvent; 
+          unsigned int  pull = 23;
+          XEvent        xEvent; 
           DFBInputEvent dfbEvent;
 
           /* FIXME: Detect key repeats, we're receiving KeyPress, KeyRelease, KeyPress, KeyRelease... !!?? */
 
-#ifdef ___HELP___WHY_DOES_THIS_ALWAYS_BLOCK_THE_LAST_EVENT___HELP___
-          XNextEvent( dfb_x11->display, &xEvent );
+          if (!shared->xw || !shared->xw->window) {
+               /* no window, so no event */
+               usleep( 50000 );
+               continue;
+          }
 
-          do {
-               switch (xEvent.type) {
-                    case ButtonPress:
-                    case ButtonRelease:
-                         motion_realize( data );
-                    case MotionNotify:
-                         handleMouseEvent( &xEvent, data ); // crash ???
-                         break;
+          usleep( 10000 );
 
-                    case KeyPress:
-                    case KeyRelease: {
-                         motion_realize( data );
+          XLockDisplay( x11->display );
 
-                         dfbEvent.type     = (xEvent.type == KeyPress) ? DIET_KEYPRESS : DIET_KEYRELEASE;
-                         dfbEvent.flags    = DIEF_KEYCODE | DIEF_TIMESTAMP;
-                         dfbEvent.key_code = xEvent.xkey.keycode;
+          while (!data->stop && pull-- && XPending( x11->display )) {
+               XNextEvent( x11->display, &xEvent );
 
-                         dfbEvent.timestamp.tv_sec  =  xEvent.xkey.time / 1000;
-                         dfbEvent.timestamp.tv_usec = (xEvent.xkey.time % 1000) * 1000;
+               XUnlockDisplay( x11->display );
 
-                         dfb_input_dispatch( data->device, &dfbEvent );
-                         break;
-                    }
-
-                    default:
-                         break;
-               }
-          } while (XCheckMaskEvent( dfb_x11->display, ~0, &xEvent ));
-#else
-          usleep(10000);
-
-          while (XCheckMaskEvent( dfb_x11->display, ~0, &xEvent )) {
                switch (xEvent.type) {
                     case ButtonPress:
                     case ButtonRelease:
@@ -577,14 +553,21 @@ x11EventThread( DirectThread *thread, void *driver_data )
                          handle_expose( &xEvent.xexpose );
                          break;
 
+                    case DestroyNotify:
+                         /* this event is mainly to unblock XNextEvent. */
+                         break;
+
                     default:
                          break;
                }
-          }
-#endif
-          motion_realize( data );
 
-          direct_thread_testcancel( thread );
+               XLockDisplay( x11->display );
+          }
+
+          XUnlockDisplay( x11->display );
+
+          if (!data->stop)
+               motion_realize( data );
      }
 
      return NULL;
@@ -597,7 +580,7 @@ x11EventThread( DirectThread *thread, void *driver_data )
  * Called once during initialization of DirectFB.
  */
 static int
-driver_get_available()
+driver_get_available( void )
 {
      return dfb_system_type() == CORE_X11;
 }
@@ -629,11 +612,12 @@ driver_open_device( CoreInputDevice  *device,
                     void            **driver_data )
 {
      X11InputData *data;
-     DFBX11       *dfb_x11 = dfb_system_data();
+     DFBX11       *x11    = dfb_system_data();
+     DFBX11Shared *shared = x11->shared;
 
-     fusion_skirmish_prevail( &dfb_x11->lock );
+     fusion_skirmish_prevail( &shared->lock );
 
-     fusion_skirmish_dismiss( &dfb_x11->lock );
+     fusion_skirmish_dismiss( &shared->lock );
 
      /* set device vendor and name */
      snprintf( info->desc.vendor, DFB_INPUT_DEVICE_DESC_VENDOR_LENGTH, "XServer" );
@@ -656,8 +640,8 @@ driver_open_device( CoreInputDevice  *device,
      /* allocate and fill private data */
      data = D_CALLOC( 1, sizeof(X11InputData) );
 
-     data->device  = device;
-     data->dfb_x11 = dfb_x11;
+     data->device = device;
+     data->x11    = x11;
 
      /* start input thread */
      data->thread = direct_thread_create( DTT_INPUT, x11EventThread, data, "X11 Input" );
@@ -678,11 +662,13 @@ driver_get_keymap_entry( CoreInputDevice           *device,
                          DFBInputDeviceKeymapEntry *entry )
 {
      int           i;
-     X11InputData *data    = driver_data;
-     DFBX11       *dfb_x11 = data->dfb_x11;
+     X11InputData *data = driver_data;
+     DFBX11       *x11  = data->x11;
+
+     XLockDisplay( x11->display );
 
      for (i=0; i<4; i++) {
-          KeySym xSymbol = XKeycodeToKeysym( dfb_x11->display, entry->code, i );
+          KeySym xSymbol = XKeycodeToKeysym( x11->display, entry->code, i );
 
           if (i == 0)
                entry->identifier = xsymbol_to_id( xSymbol );
@@ -690,6 +676,8 @@ driver_get_keymap_entry( CoreInputDevice           *device,
           entry->symbols[i] = xsymbol_to_symbol( xSymbol );
      }
 
+     XUnlockDisplay( x11->display );
+
      /* is CapsLock effective? */
      if (entry->identifier >= DIKI_A && entry->identifier <= DIKI_Z)
           entry->locks |= DILS_CAPS;
@@ -707,13 +695,36 @@ driver_get_keymap_entry( CoreInputDevice           *device,
 static void
 driver_close_device( void *driver_data )
 {
-     X11InputData *data = driver_data;
+     X11InputData *data   = driver_data;
+     DFBX11       *x11    = data->x11;
+     DFBX11Shared *shared = x11->shared;
 
      /* stop input thread */
-     data->stop = 1;
+     data->stop = true;
 
-     direct_thread_join( data->thread );
-     direct_thread_destroy( data->thread );
+     XLockDisplay( x11->display );
+
+     if (shared->xw) {
+          XWindow *xw = shared->xw;
+
+          shared->xw = NULL;
+
+          /* the window must generate an event, otherwise the input thread will not end */
+          dfb_x11_close_window( x11, xw );
+     }
+
+     XSync( x11->display, False );
+
+     XUnlockDisplay( x11->display );
+
+     /* it is possible that this "close" function is called from the same
+      * thread that the input device is actually running on.
+      * This happens when you e.g. click the close box with your mouse.
+      * As a fix, we check if we are this thread. */
+     if (data->thread != direct_thread_self()) {
+          direct_thread_join( data->thread );
+          direct_thread_destroy( data->thread );
+     }
 
      /* free private data */
      D_FREE ( data );
