@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -53,6 +53,7 @@ static const DirectFBScreenEncoderTypeNames(encoder_type);
 static const DirectFBScreenEncoderTVStandardsNames(tv_standards);
 static const DirectFBScreenOutputCapabilitiesNames(output_caps);
 static const DirectFBScreenOutputConnectorsNames(connectors);
+static const DirectFBScreenOutputResolutionNames(resolutions);
 static const DirectFBScreenOutputSignalsNames(signals);
 static const DirectFBScreenMixerCapabilitiesNames(mixer_caps);
 
@@ -63,8 +64,8 @@ static IDirectFB *dfb = NULL;
 /*****************************************************************************/
 
 static DFBBoolean parse_command_line ( int argc, char *argv[] );
-static void       enum_input_devices ();
-static void       enum_screens ();
+static void       enum_input_devices ( void );
+static void       enum_screens ( void );
 
 /*****************************************************************************/
 
@@ -166,6 +167,10 @@ input_device_callback( DFBInputDeviceID           id,
      
      
      /* Details */
+     if (desc.caps & DICAPS_KEYS)
+          printf( "   Min. Keycode: %d\n", desc.min_keycode );
+     if (desc.caps & DICAPS_KEYS)
+          printf( "   Max. Keycode: %d\n", desc.max_keycode );
      if (desc.caps & DICAPS_AXES)
           printf( "   Max. Axis: %d\n", desc.max_axis );
      if (desc.caps & DICAPS_BUTTONS)
@@ -178,7 +183,7 @@ input_device_callback( DFBInputDeviceID           id,
 }
 
 static void
-enum_input_devices()
+enum_input_devices( void )
 {
      DFBResult ret;
 
@@ -406,6 +411,18 @@ dump_encoders( IDirectFBScreen *screen,
 
                printf( "\n" );
           }
+          
+          /* Output Resolutions */
+          if (descs[i].caps & DSECAPS_RESOLUTION) {
+               printf( "     Output Resolutions: " );
+
+               for (n=0; resolutions[n].resolution; n++) {
+                    if (descs[i].all_resolutions & resolutions[n].resolution)
+                         printf( "%s ", resolutions[n].name );
+               }
+
+               printf( "\n" );
+          }
 
           /* Output connectors */
           if (descs[i].caps & DSECAPS_CONNECTORS) {
@@ -453,17 +470,29 @@ dump_outputs( IDirectFBScreen *screen,
 
           printf( "\n" );
 
+	  /* Connectors */
+          if (descs[i].caps & DSOCAPS_CONNECTORS) {
+              printf( "     Connectors: " );
 
-          /* Connectors */
-          printf( "     Connectors: " );
+              for (n=0; connectors[n].connector; n++) {
+                   if (descs[i].all_connectors & connectors[n].connector)
+                        printf( "%s ", connectors[n].name );
+              }
 
-          for (n=0; connectors[n].connector; n++) {
-               if (descs[i].all_connectors & connectors[n].connector)
-                    printf( "%s ", connectors[n].name );
+              printf( "\n" );
           }
 
-          printf( "\n" );
+          /* Output Resolutions */
+          if (descs[i].caps & DSOCAPS_RESOLUTION) {
+               printf( "     Resolutions: " );
 
+               for (n=0; resolutions[n].resolution; n++) {
+                    if (descs[i].all_resolutions & resolutions[n].resolution)
+                         printf( "%s ", resolutions[n].name );
+               }
+
+               printf( "\n" );
+          }
 
           /* Signals */
           printf( "     Signals:    " );
@@ -539,7 +568,7 @@ screen_callback( DFBScreenID           id,
 }
 
 static void
-enum_screens()
+enum_screens( void )
 {
      DFBResult ret;
 
