@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -52,14 +52,17 @@ static DFBDisplayLayerDescription  desc;
 
 /*****************************************************************************/
 
-static DFBDisplayLayerID         id         = DLID_PRIMARY;
-static int                       width      = 0;
-static int                       height     = 0;
-static DFBSurfacePixelFormat     format     = DSPF_UNKNOWN;
-static DFBDisplayLayerBufferMode buffermode = -1;
-static int                       opacity    = -1;
-static int                       level      = 0;
-static DFBBoolean                set_level  = DFB_FALSE;
+static DFBDisplayLayerID         id           = DLID_PRIMARY;
+static int                       width        = 0;
+static int                       height       = 0;
+static DFBSurfacePixelFormat     format       = DSPF_UNKNOWN;
+static DFBDisplayLayerBufferMode buffermode   = -1;
+static int                       opacity      = -1;
+static int                       level        = 0;
+static DFBBoolean                set_level    = DFB_FALSE;
+static int                       rotation     = 0;
+static DFBBoolean                set_rotation = DFB_FALSE;
+static DFBSurfaceLockFlags       test_lock    = 0;
 
 /*****************************************************************************/
 
@@ -123,6 +126,32 @@ main( int argc, char *argv[] )
      /* Show/change the configuration. */
      set_configuration();
 
+     /* Test Lock() on layer surface? */
+     if (test_lock) {
+          IDirectFBSurface *surface;
+
+          fprintf( stderr, "\nGetting layer surface...\n" );
+
+          ret = layer->GetSurface( layer, &surface );
+          if (ret)
+               DirectFBError( "IDirectFBDisplayLayer::GetSurface() failed", ret );
+          else {
+               void *data;
+               int   pitch;
+
+               fprintf( stderr, "\nTesting Lock( %s ) on layer surface...\n",
+                        test_lock == DSLF_READ ? "read only" : test_lock == DSLF_WRITE ? "write only" : "read/write" );
+
+               ret = surface->Lock( surface, test_lock, &data, &pitch );
+               if (ret)
+                    DirectFBError( "IDirectFBSurface::Lock() failed", ret );
+               else
+                    fprintf( stderr, "  => OK\n\n" );
+
+               surface->Release( surface );
+          }
+     }
+
      /* Release the display layer. */
      layer->Release( layer );
 
@@ -142,14 +171,18 @@ print_usage (const char *prg_name)
      fprintf (stderr, "\nDirectFB Layer Configuration (version %s)\n\n", DIRECTFB_VERSION);
      fprintf (stderr, "Usage: %s [options]\n\n", prg_name);
      fprintf (stderr, "Options:\n");
-     fprintf (stderr, "   -l, --layer   <id>              Use the specified layer, default is primary\n");
-     fprintf (stderr, "   -m, --mode    <width>x<height>  Change the resolution (pixels)\n");
-     fprintf (stderr, "   -f, --format  <pixelformat>     Change the pixel format\n");
-     fprintf (stderr, "   -b, --buffer  <buffermode>      Change the buffer mode (single/video/system)\n");
-     fprintf (stderr, "   -o, --opacity <opacity>         Change the layer's opacity (0-255)\n");
-     fprintf (stderr, "   -L, --level   <level>           Change the layer's level\n");
-     fprintf (stderr, "   -h, --help                      Show this help message\n");
-     fprintf (stderr, "   -v, --version                   Print version information\n");
+     fprintf (stderr, "   -l,  --layer            <id>              Use the specified layer, default is primary\n");
+     fprintf (stderr, "   -m,  --mode             <width>x<height>  Change the resolution (pixels)\n");
+     fprintf (stderr, "   -f,  --format           <pixelformat>     Change the pixel format\n");
+     fprintf (stderr, "   -b,  --buffer           <buffermode>      Change the buffer mode (single/video/system)\n");
+     fprintf (stderr, "   -o,  --opacity          <opacity>         Change the layer's opacity (0-255)\n");
+     fprintf (stderr, "   -L,  --level            <level>           Change the layer's level\n");
+     fprintf (stderr, "   -R,  --rotate           <degree>          Change the layer rotation\n");
+     fprintf (stderr, "   -t,  --test-lock                          Get layer surface and Lock() it (read/write)\n");
+     fprintf (stderr, "   -tr, --test-lock-read                     Get layer surface and Lock() it (read  only)\n");
+     fprintf (stderr, "   -tw, --test-lock-write                    Get layer surface and Lock() it (write only)\n");
+     fprintf (stderr, "   -h,  --help                               Show this help message\n");
+     fprintf (stderr, "   -v,  --version                            Print version information\n");
      fprintf (stderr, "\n");
 
      fprintf (stderr, "Known pixel formats:\n");
@@ -285,6 +318,20 @@ parse_level( const char *arg )
      return DFB_TRUE;
 }
 
+static DFBBoolean
+parse_rotation( const char *arg )
+{
+     if (sscanf( arg, "%d", &rotation ) != 1) {
+          fprintf (stderr, "\nInvalid rotation specified!\n\n");
+
+          return DFB_FALSE;
+     }
+
+     set_rotation = DFB_TRUE;
+
+     return DFB_TRUE;
+}
+
 static DFBBoolean
 parse_command_line( int argc, char *argv[] )
 {
@@ -375,6 +422,36 @@ parse_command_line( int argc, char *argv[] )
                continue;
           }
 
+          if (strcmp (arg, "-R") == 0 || strcmp (arg, "--rotate") == 0) {
+               if (++n == argc) {
+                    print_usage (argv[0]);
+                    return DFB_FALSE;
+               }
+
+               if (!parse_rotation( argv[n] ))
+                    return DFB_FALSE;
+
+               continue;
+          }
+
+          if (strcmp (arg, "-t") == 0 || strcmp (arg, "--test-lock") == 0) {
+               test_lock = DSLF_READ | DSLF_WRITE;
+
+               continue;
+          }
+
+          if (strcmp (arg, "-tr") == 0 || strcmp (arg, "--test-lock-read") == 0) {
+               test_lock = DSLF_READ;
+
+               continue;
+          }
+
+          if (strcmp (arg, "-tw") == 0 || strcmp (arg, "--test-lock-write") == 0) {
+               test_lock = DSLF_WRITE;
+
+               continue;
+          }
+
           print_usage (argv[0]);
 
           return DFB_FALSE;
@@ -384,7 +461,7 @@ parse_command_line( int argc, char *argv[] )
 }
 
 static void
-set_configuration()
+set_configuration( void )
 {
      DFBResult             ret;
      DFBDisplayLayerConfig config;
@@ -455,6 +532,15 @@ set_configuration()
                DirectFBError( "IDirectFBDisplayLayer::SetLevel() failed", ret );
      }
 
+     /* Set the rotation if requested. */
+     if (set_rotation) {
+          ret = layer->SetRotation( layer, rotation );
+          if (ret == DFB_UNSUPPORTED)
+               fprintf( stderr, "Rotation (%d) not supported!\n\n", level );
+          else if (ret)
+               DirectFBError( "IDirectFBDisplayLayer::SetRotation() failed", ret );
+     }
+
 
      if (config.flags & DLCONF_WIDTH)
           printf( "Width       %d\n", config.width );
