@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -181,10 +181,11 @@ int main (int         argc,
           struct stat statbuf; 
           if (0 == stat(filename[0], &statbuf))
           {
+               FILE *f;
                unsigned char *data = alloca(statbuf.st_size);
                memset(data, 0, statbuf.st_size);
 
-               FILE *f = fopen(filename[0], "r");
+               f = fopen(filename[0], "r");
                if (f)
                {
                     fread(data, statbuf.st_size, 1, f);
@@ -627,10 +628,10 @@ static inline void save_uchar (CSourceData   *csource,
      return;
 }
 
-static DFBResult dump_data(CSourceData         *csource,
-                           const char          *name,
-                           const unsigned char *data,
-                           unsigned int        len)
+static void dump_data(CSourceData         *csource,
+                      const char          *name,
+                      const unsigned char *data,
+                      unsigned int         len)
 {
      fprintf (csource->fp,
               "static const unsigned char %s_data[] =\n", name);
@@ -642,8 +643,6 @@ static DFBResult dump_data(CSourceData         *csource,
      while (--len);
 
      fprintf (csource->fp, "\";\n\n");
-
-     return DFB_OK;
 }
 
 static DFBResult dump_raw_data(const char            *name,
