@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -226,7 +226,7 @@ foo_update( StretRegion     *region,
 }
 
 const StretRegionClass unique_foo_region_class = {
-     GetInput: foo_get_input,
-     Update:   foo_update
+     .GetInput = foo_get_input,
+     .Update   = foo_update,
 };
 
