@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -44,7 +44,7 @@
 #include <core/layers.h>
 #include <core/layers_internal.h>      /* FIXME */
 #include <core/state.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/windowstack.h>
 #include <core/windows_internal.h>     /* FIXME */
 
@@ -71,7 +71,7 @@ static const ReactionFunc unique_context_globals[] = {
 /**************************************************************************************************/
 
 static void
-context_destructor( FusionObject *object, bool zombie )
+context_destructor( FusionObject *object, bool zombie, void *ctx )
 {
      int            i;
      UniqueContext *context = (UniqueContext*) object;
@@ -119,7 +119,7 @@ FusionObjectPool *
 unique_context_pool_create( const FusionWorld *world )
 {
      return fusion_object_pool_create( "UniQuE Context Pool", sizeof(UniqueContext),
-                                       sizeof(UniqueContextNotification), context_destructor, world );
+                                       sizeof(UniqueContextNotification), context_destructor, NULL, world );
 }
 
 /**************************************************************************************************/
@@ -394,7 +394,7 @@ unique_draw_cursor( CoreWindowStack *stack, UniqueContext *context, CardState *s
 
      /* Different compositing methods depending on destination format. */
      if (flags & DSBLIT_BLEND_ALPHACHANNEL) {
-          if (DFB_PIXELFORMAT_HAS_ALPHA( state->destination->format )) {
+          if (DFB_PIXELFORMAT_HAS_ALPHA( state->destination->config.format )) {
                /*
                 * Always use compliant Porter/Duff SRC_OVER,
                 * if the destination has an alpha channel.
@@ -418,7 +418,7 @@ unique_draw_cursor( CoreWindowStack *stack, UniqueContext *context, CardState *s
                dfb_state_set_src_blend( state, DSBF_ONE );
 
                /* Need to premultiply source with As*Ac or only with Ac? */
-               if (! (stack->cursor.surface->caps & DSCAPS_PREMULTIPLIED))
+               if (! (stack->cursor.surface->config.caps & DSCAPS_PREMULTIPLIED))
                     flags |= DSBLIT_SRC_PREMULTIPLY;
                else if (flags & DSBLIT_BLEND_COLORALPHA)
                     flags |= DSBLIT_SRC_PREMULTCOLOR;
@@ -443,7 +443,7 @@ unique_draw_cursor( CoreWindowStack *stack, UniqueContext *context, CardState *s
                 * cx = Cd * (1-As*Ac) + Cs*As * Ac  (still same effect as above)
                 * ax = Ad * (1-As*Ac) + As*As * Ac  (wrong, but discarded anyways)
                 */
-               if (stack->cursor.surface->caps & DSCAPS_PREMULTIPLIED) {
+               if (stack->cursor.surface->config.caps & DSCAPS_PREMULTIPLIED) {
                     /* Need to premultiply source with Ac? */
                     if (flags & DSBLIT_BLEND_COLORALPHA)
                          flags |= DSBLIT_SRC_PREMULTCOLOR;
