@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -56,7 +57,7 @@ static const ReactionFunc unique_decoration_globals[] = {
 /**************************************************************************************************/
 
 static void
-decoration_destructor( FusionObject *object, bool zombie )
+decoration_destructor( FusionObject *object, bool zombie, void *ctx )
 {
      UniqueDecoration *decoration = (UniqueDecoration*) object;
 
@@ -78,7 +79,7 @@ unique_decoration_pool_create( const FusionWorld *world )
      return fusion_object_pool_create( "UniQuE Decoration Pool",
                                        sizeof(UniqueDecoration),
                                        sizeof(UniqueDecorationNotification),
-                                       decoration_destructor, world );
+                                       decoration_destructor, NULL, world );
 }
 
 /**************************************************************************************************/
