@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -346,9 +346,9 @@ void      unique_wm_module_deinit( WMData   *data,
                                    bool      master,
                                    bool      emergency );
 
-UniqueContext    *unique_wm_create_context();
-UniqueDecoration *unique_wm_create_decoration();
-UniqueWindow     *unique_wm_create_window();
+UniqueContext    *unique_wm_create_context( void );
+UniqueDecoration *unique_wm_create_decoration( void );
+UniqueWindow     *unique_wm_create_window( void );
 
 /* HACK: temporary, will move into cursor class */
 void unique_draw_cursor( CoreWindowStack *stack, UniqueContext *context, CardState *state, DFBRegion *region );
