@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -64,14 +65,14 @@ default_update( StretRegion     *region,
 }
 
 static const StretRegionClass default_class = {
-     Update:   default_update
+     .Update = default_update,
 };
 
 /**************************************************************************************************/
 
 static const StretRegionClass *classes[MAX_CLASSES] = { &default_class, NULL };
 
-static pthread_mutex_t         classes_lock  = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t         classes_lock  = PTHREAD_MUTEX_INITIALIZER;
 static int                     classes_count = 1;
 
 /**************************************************************************************************/
@@ -149,6 +150,7 @@ stret_region_create( StretRegionClassID   class_id,
                      int                  height,
                      StretRegion         *parent,
                      int                  level,
+                     FusionSHMPoolShared *pool,
                      StretRegion        **ret_region )
 {
      int          i;
@@ -176,10 +178,10 @@ stret_region_create( StretRegionClassID   class_id,
      D_ASSERT( ret_region != NULL );
 
      /* Allocate region data. */
-     region = SHCALLOC( 1, sizeof(StretRegion) + sizeof(FusionVector) * levels );
+     region = SHCALLOC( pool, 1, sizeof(StretRegion) + sizeof(FusionVector) * levels );
      if (!region) {
           D_WARN( "out of (shared) memory" );
-          return DFB_NOSYSTEMMEMORY;
+          return D_OOSHM();
      }
 
      /* Initialize region data. */
@@ -192,10 +194,11 @@ stret_region_create( StretRegionClassID   class_id,
      region->clazz    = class_id;
      region->data     = data;
      region->arg      = arg;
+     region->shmpool  = pool;
 
      /* Initialize levels. */
      for (i=0; i<levels; i++)
-          fusion_vector_init( &region->children[i], 4 );
+          fusion_vector_init( &region->children[i], 4, pool );
 
 
      /* Add the region to its parent. */
@@ -206,15 +209,15 @@ stret_region_create( StretRegionClassID   class_id,
 
           if (fusion_vector_add( children, region )) {
                D_WARN( "out of (shared) memory" );
-               SHFREE( region );
-               return DFB_NOSYSTEMMEMORY;
+               SHFREE( pool, region );
+               return D_OOSHM();
           }
      }
 
 
      D_MAGIC_SET( region, StretRegion );
 
-#if DIRECT_BUILD_DEBUG
+#if D_DEBUG_ENABLED
 {
      DFBRegion bounds;
 
@@ -281,7 +284,7 @@ stret_region_destroy( StretRegion *region )
 
      D_MAGIC_CLEAR( region );
 
-     SHFREE( region );
+     SHFREE( region->shmpool, region );
 
      return DFB_OK;
 }
