@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -192,7 +192,7 @@ parse_command_line( int argc, char *argv[] )
 }
 
 static void
-set_color()
+set_color( void )
 {
      DFBResult  ret;
      char      *error;
