@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -55,7 +55,7 @@
 #include <core/layer_context.h>
 #include <core/layer_region.h>
 #include <core/layers_internal.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/palette.h>
 #include <core/windows.h>
 #include <core/windows_internal.h>
@@ -275,6 +275,14 @@ wm_resume( void *wm_data, void *shared_data )
      return DFB_OK;
 }
 
+static DFBResult
+wm_post_init( void *wm_data, void *shared_data )
+{
+     D_DEBUG_AT( WM_Unique, "wm_post_init()\n" );
+
+     return DFB_OK;
+}
+
 /**************************************************************************************************/
 
 static DFBResult
@@ -534,12 +542,6 @@ wm_enum_windows( CoreWindowStack      *stack,
 
 /**************************************************************************************************/
 
-static DFBResult
-wm_start_desktop( CoreWindowStack *stack)
-{
-     return DFB_OK;
-}
-
 static DFBResult
 wm_get_insets( CoreWindowStack *stack,
                CoreWindow      *window,
@@ -922,13 +924,18 @@ wm_update_cursor( CoreWindowStack       *stack,
      D_ASSERT( surface != NULL );
 
      if (flags & CCUF_ENABLE) {
-          CoreSurface *cursor_bs;
+          CoreSurface            *cursor_bs;
+          DFBSurfaceCapabilities  caps = DSCAPS_NONE;
+
+          dfb_surface_caps_apply_policy( stack->cursor.policy, &caps );
 
           D_ASSERT( context->cursor_bs == NULL );
 
           /* Create the cursor backing store surface. */
-          ret = dfb_surface_create( wmdata->core, stack->cursor.size.w, stack->cursor.size.h,
-                                    DSPF_RGB16, stack->cursor.policy, DSCAPS_NONE, NULL, &cursor_bs );
+          ret = dfb_surface_create_simple( wmdata->core, stack->cursor.size.w, stack->cursor.size.h,
+                                           region->config.format, caps, CSTF_SHARED | CSTF_CURSOR,
+                                           0, /* FIXME: no shared cursor objects, no cursor id */
+                                           NULL, &cursor_bs );
           if (ret) {
                D_ERROR( "WM/Default: Failed creating backing store for cursor!\n" );
                return ret;
@@ -974,9 +981,9 @@ wm_update_cursor( CoreWindowStack       *stack,
           }
 
           if (flags & CCUF_SIZE) {
-               ret = dfb_surface_reformat( wmdata->core, context->cursor_bs,
+               ret = dfb_surface_reformat( context->cursor_bs,
                                            stack->cursor.size.w, stack->cursor.size.h,
-                                           context->cursor_bs->format );
+                                           context->cursor_bs->config.format );
                if (ret) {
                     D_ERROR( "WM/Default: Failed resizing backing store for cursor!\n" );
                     return ret;
