@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -35,7 +35,7 @@
 #include <fusion/object.h>
 
 
-bool         unique_wm_running      ();
+bool         unique_wm_running      ( void );
 
 DirectResult unique_wm_enum_contexts( FusionObjectCallback  callback,
                                       void                 *ctx );
