@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -37,7 +38,7 @@
 
 #include <core/coretypes.h>
 #include <core/layers_internal.h>      /* FIXME */
-#include <core/surfaces.h>
+#include <core/surface.h>
 #include <core/windows.h>
 #include <core/windows_internal.h>     /* FIXME */
 #include <core/windowstack.h>
@@ -107,7 +108,7 @@ static DFBResult  update_window   ( UniqueWindow        *window,
 static void       update_flags    ( UniqueWindow  *window );
 
 static void       set_opacity     ( UniqueWindow  *window,
-                                    __u8           opacity );
+                                    u8             opacity );
 
 static DFBResult  move_window     ( UniqueWindow  *window,
                                     int            dx,
@@ -129,7 +130,7 @@ static inline int get_priority( DFBWindowCapabilities caps, DFBWindowStackingCla
 /**************************************************************************************************/
 
 static void
-window_destructor( FusionObject *object, bool zombie )
+window_destructor( FusionObject *object, bool zombie, void *ctx )
 {
      int            i;
      UniqueContext *context;
@@ -189,7 +190,7 @@ FusionObjectPool *
 unique_window_pool_create( const FusionWorld *world )
 {
      return fusion_object_pool_create( "UniQuE Window Pool", sizeof(UniqueWindow),
-                                       sizeof(UniqueWindowNotification), window_destructor, world );
+                                       sizeof(UniqueWindowNotification), window_destructor, NULL, world );
 }
 
 /**************************************************************************************************/
@@ -227,10 +228,10 @@ unique_window_create( CoreDFB                 *core,
 
      bounds = config->bounds;
 
-     if (bounds.x == 0 && bounds.y == 0) {
+     /*if (bounds.x == 0 && bounds.y == 0) {
           bounds.x = (context->width  - bounds.w) / 2;
           bounds.y = (context->height - bounds.h) / 2;
-     }
+     }*/
 
      /* Initialize window data. */
      uniwin->shared    = context->shared;
@@ -414,6 +415,9 @@ unique_window_set_config( UniqueWindow           *window,
      if (flags & CWCF_EVENTS)
           window->events = config->events;
 
+     if (flags & CWCF_COLOR)
+          return DFB_UNSUPPORTED;
+
      if (flags & CWCF_COLOR_KEY)
           window->color_key = config->color_key;
 
@@ -822,9 +826,6 @@ _unique_window_input_channel_listener( const void *msg_data,
 static inline int
 get_priority( DFBWindowCapabilities caps, DFBWindowStackingClass stacking )
 {
-     if (caps & DWHC_TOPMOST)
-          return 2;
-
      switch (stacking) {
           case DWSC_UPPER:
                return  1;
@@ -943,7 +944,7 @@ insert_window( UniqueContext *context,
      fusion_vector_foreach (other, index, context->windows) {
           D_MAGIC_ASSERT( other, UniqueWindow );
 
-          if (window->caps & DWHC_TOPMOST || other->priority > window->priority)
+          if (other->priority > window->priority)
                break;
      }
 
@@ -1051,8 +1052,7 @@ resize_window( UniqueWindow *window,
           return DFB_LIMITEXCEEDED;
 
      if (window->surface) {
-          ret = dfb_surface_reformat( NULL, window->surface,
-                                      width, height, window->surface->format );
+          ret = dfb_surface_reformat( window->surface, width, height, window->surface->config.format );
           if (ret)
                return ret;
      }
@@ -1197,11 +1197,11 @@ restack_window( UniqueWindow           *window,
 
           if (index < 0)
                index = 0;
-          else if (index > context->windows.count - 1)
-               index = context->windows.count - 1;
+          else if (index > fusion_vector_size( &context->windows ) - 1)
+               index = fusion_vector_size( &context->windows ) - 1;
      }
      else if (relation)
-          index = context->windows.count - 1;
+          index = fusion_vector_size( &context->windows ) - 1;
      else
           index = 0;
 
@@ -1219,7 +1219,7 @@ restack_window( UniqueWindow           *window,
      }
 
      /* Assure window won't be below any window with a lower priority. */
-     while (index < context->windows.count - 1) {
+     while (index < fusion_vector_size( &context->windows ) - 1) {
           int           above = (old > index) ? index : index + 1;
           UniqueWindow *other = fusion_vector_at( &context->windows, above );
 
@@ -1250,9 +1250,9 @@ restack_window( UniqueWindow           *window,
 
 static void
 set_opacity( UniqueWindow *window,
-             __u8          opacity )
+             u8            opacity )
 {
-     __u8           old;
+     u8             old;
      UniqueContext *context;
 
      D_MAGIC_ASSERT( window, UniqueWindow );
@@ -1410,7 +1410,7 @@ update_flags( UniqueWindow *window )
 
      D_MAGIC_ASSERT( window, UniqueWindow );
 
-     if (! (window->options & DWOP_GHOST) && ! (window->caps & DWHC_TOPMOST)) {
+     if (! (window->options & DWOP_GHOST)) {
           if (window->foos[0]) {
                for (i=0; i<8; i++)
                     window->foos[i]->flags |= SRF_INPUT;
@@ -1484,7 +1484,7 @@ create_regions( UniqueWindow *window )
 
      D_MAGIC_ASSERT( shared, WMShared );
 
-     if (! (window->options & DWOP_GHOST) && ! (window->caps & DWHC_TOPMOST))
+     if (! (window->options & DWOP_GHOST))
           flags |= SRF_INPUT;
 
      if (! (window->caps & DWCAPS_INPUTONLY))
